#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2006-2007 NTT
#/**
# * @file    actmonitor.sh
# * @brief   Clusterコマンド（ACT側監視要求）処理
# *
# * ACT状態用PREXプロセスの監視を行う。
# *
# *   詳細は以下のとおり。
# *     -# コマンドパラメータのチェックを行う。以下に示すパラメータ指定は、正常なパラメータとみなす。
# *        - 第1パラメータが指定されている、第3パラメータ以降は指定されていない。
# *     -# 正常なパラメータとみなせない場合には、エラーとして、syslogにエラーメッセージを出力後（prexsyslogコマンド）、
# *        戻り値としてRET_NGを返却して終了する。
# *     -# 第2パラメータの値が「-N」の場合は、以降の処理でのエラーメッセージ出力を行なわない。
# *     -# メッセージキューの存在確認を行う（ipcs -q | grep [キー値]）。grepコマンドのエラーの場合、syslogにメッセージを出力後（prexsyslogコマンド）、戻り値としてRET_NGを返却して終了する。
# *     -# メッセージキューが存在しない場合、syslogにメッセージを出力後（prexsyslogコマンド）、戻り値としてRET_NGを返却して終了する。
# *     -# postmasterプロセスの存在確認を行う（ps --pid [PID]）。postmasterのPIDはpostmaster.pidファイルより取得する。postmasterプロセスが存在しない場合、syslogにメッセージを出力後（prexsyslogコマンド）、戻り値としてRET_NGを返却して終了する。
# *     -# postmasterプロセスが存在しない場合、syslogにメッセージを出力後（prexsyslogコマンド）、戻り値としてRET_NGを返却して終了する。
# *     -# postgres（BGwriter）プロセスの存在確認を行う（ps x | grep [プロセス名]）。grepコマンドのエラーの場合、syslogにメッセージを出力後（prexsyslogコマンド）、戻り値としてRET_NGを返却して終了する。
# *     -# postgres（BGwriter）プロセスが存在しない場合、syslogにメッセージを出力後（prexsyslogコマンド）、戻り値としてRET_NGを返却して終了する。
# *     -# データベーステーブルへのチェック処理を行う（acttestsql.shコマンド）。
# *        エラーがあった場合、syslogにメッセージを出力後（prexsyslogコマンド）、戻り値としてRET_NGを返却して終了する。
# *     -# 戻り値としてRET_OKを返却して終了する。 
# *
# * @note        postgres（BGwriter）は、PostgreSQLがACTモードのときに動作している「postgres: writer process」というプロセスである。
# *              PostgreSQLがACTモードかどうかの判断を、このプロセスの存在確認で行う。
# * @param[in]   msgkey    メッセージキューのキー値
# * @retval      RET_OK(0) 正常終了
# * @retval      RET_NG(1) 異常終了
# *
# * @version 1.1.0
# * @date    08/02/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

##
## 引数チェック
##
if [ ${#} -lt 1 -o ${#} -gt 2 ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_NUMOFPARAM}
	exit ${RET_NG}
fi

if [ ! -z "${2}" -a "${2}" != '-N' ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_INVALIDPARAM}
	exit ${RET_NG}
fi

##
## 変数宣言
##
key=${1}                                        # メッセージキューのキー値
keyhex=`echo "obase=16; ibase=10; ${key}" | bc` # メッセージキューのキー値(16進)
log_mode="${2}"                                 # エラーメッセージ出力有無

##
## メッセージキューの存在確認
##
ipcs -q | grep -i "^0x[0]*${keyhex}" > /dev/null 2>&1
ret=${?}

if [ ${ret} -eq ${RET_GREP_NOTFOUND} ] ; then
	# 存在しない場合、エラー
	[ "${log_mode}" != '-N' ] && ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_FAILEDTOFOUNDMSGQ} ${0} ${LINENO}
	exit ${RET_NG}
elif [ ${ret} -ne ${RET_OK} ]; then
	# grepコマンドエラー
	[ "${log_mode}" != '-N' ] && ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_ERRORGREP} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## postmasterプロセスの存在確認
##
# postmaster.pidファイルより、postmasterプロセスのPIDを取得
pid_postmaster=`head -n 1 ${postmaster_pid_file}`

if [ ${?} -ne ${RET_OK} -o -z "${pid_postmaster}" ] ; then
	[ "${log_mode}" != '-N' ] && ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_FAILEDTOFOUNDPROCESS} ${0} ${LINENO} ${MSGWD_POSTMASTER}
	exit ${RET_NG}
fi

# postmasterプロセスの存在を確認
ps --pid ${pid_postmaster} > /dev/null

if [ ${?} -ne ${RET_OK} ] ; then
	# 存在しない場合、エラー
	[ "${log_mode}" != '-N' ] && ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_FAILEDTOFOUNDPROCESS} ${0} ${LINENO} ${MSGWD_POSTMASTER}
	exit ${RET_NG}
fi

##
## postgres(BGwriter)の存在確認
##
ps --ppid ${pid_postmaster} -f | grep -q "${PNAME_POSTGRES_BGWRITER}"
ret=${?}

if [ ${ret} -eq ${RET_ERR_GREP} ] ; then
	[ "${log_mode}" != '-N' ] && ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_ERRORGREP} ${0} ${LINENO}
	exit ${RET_NG}
elif [ ${ret} -ne ${RET_OK} ] ; then
	# 存在しない場合、エラー
	[ "${log_mode}" != '-N' ] && ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_FAILEDTOFOUNDPROCESS} ${0} ${LINENO} ${MSGWD_POSTGRES_BGWRITER}
	exit ${RET_NG}
fi

##
## データベーステーブルへのチェック処理
##
${PREX_BIN}/acttestsql.sh

if [ ${?} -ne ${RET_OK} ] ; then
	[ "${log_mode}" != '-N' ] && ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_ERRORSQLCHECK} ${0} ${LINENO}
	exit ${RET_NG}
fi

exit ${RET_OK}
