#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2006-2007 NTT
#/**
# * @file    toact.sh
# * @brief   Clusterコマンド（ACT化要求）処理
# *
# * PREXをSBY状態からACT状態に遷移させる。
# *
# *   詳細は以下のとおり。
# *     -# コマンドパラメータのチェックを行う。以下に示すパラメータ指定は、正常なパラメータとみなす。
# *        - 第1パラメータが設定されている、第2パラメータ以降は指定されていない。
# *     -# 正常なパラメータとみなせない場合には、エラーとして、syslogにエラーメッセージを出力後（prexsyslogコマンド）、
# *        戻り値としてRET_NGを返却して終了する。
# *     -# 最終WALセグメントの先頭LSN経由での終端LSNを取得を行なう。終端LSNの取得に失敗した場合は、リトライを行なう。
# *     -# メッセージキューに「ACT化要求」メッセージを送信し、メッセージキューから「ACT化応答」メッセージを受信する（msgactコマンド）。
# *        エラーがあった場合、syslogにメッセージを出力後（prexsyslogコマンド）、戻り値としてRET_NGを返却して終了する。
# *     -# 戻り値としてRET_OKを返却して終了する。
# *
# * @param[in]   key        メッセージキューのキー値
# * @param[in]   waldir     WALセグメント格納ディレクトリ
# * @param[in]   archivedir アーカイブログ格納ディレクトリ
# * @retval      RET_OK(0)  ACT化成功
# * @retval      RET_NG(1)  ACT化失敗
# * @note      「ACT化要求」メッセージは、WALWriterが受信する。
# * @note      「ACT化応答」メッセージは、postmasterもしくはWALWriterから送信される。
# *
# * @version 1.1.0
# * @date    08/07/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

# 引数チェック
if [ ${#} -ne 3 ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_NUMOFPARAM}
	exit ${RET_NG}
fi 

##
## 変数宣言
##
key=${1}          # メッセージキューのキー値
waldir="${2}"     # WALセグメント格納ディレクトリ
archivedir="${3}" # アーカイブログ格納ディレクトリ
last_lsn=         # 終端LSN
logid=            # 終端LSNのログID
offset=           # 終端LSNのオフセット

##
## 最終WALセグメントより、終端LSNを取得
## 終端LSNの取得に失敗した場合、規定回数のリトライを行なう
##
retry_cnt=0
ret=${RET_NG}
while [ ${retry_cnt} -lt ${LASTWAL_RETRY} ]
do
#	last_lsn="`${PREX_BIN}/LastWAL "${waldir}" "${archivedir}"`"
	last_lsn="`${PREX_BIN}/LastLSN`"
	
	if [ ${?} -eq ${RET_OK} ] ; then
		ret=${RET_OK}
		break
	fi
	
	sleep ${LASTWAL_RETRY_INTERVAL}
	retry_cnt=`expr ${retry_cnt} + 1`
done

if [ ${ret} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_FAILEDTOGET} ${0} ${LINENO} ${MSGWD_LASTLSN_LOCAL}
	exit ${RET_NG}
fi

# 終端LSNよりログIDおよびオフセットを取得
logid="`echo "${last_lsn}" | cut -d '/' -f 1`"
offset="`echo "${last_lsn}" | cut -d '/' -f 2`"

##
##「ACT化要求」メッセージ送信、「ACT化応答」メッセージ受信
##
${PREX_BIN}/msgact ${key} "${logid}" "${offset}"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_FAILEDTOACTMSGSENDRECV} ${0} ${LINENO}
	exit ${RET_NG}
fi 

exit ${RET_OK}
