#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2006-2007 NTT
#/**
# * @file    tostop_immediate.sh
# * @brief   Clusterコマンド（強制停止要求）処理
# *
# * PostgreSQLの即時シャットダウンとPREXプロセスの停止を行う。
# *
# *   詳細は以下のとおり。
# *     -# コマンドパラメータのチェックを行う。以下に示すパラメータ指定は、正常なパラメータとみなす。
# *        - 第1パラメータが設定されている、第3パラメータ以降は指定されていない。
# *     -# 正常なパラメータとみなせない場合には、エラーとして、syslogにエラーメッセージを出力する（prexsyslogコマンド）。
# *     -# 第2パラメータの値が「-N」の場合は、以降の処理でのエラーメッセージ出力を行なわない。
# *     -# WALWriterプロセスの存在確認を行う（ps x | grep [プロセス名]）。grepコマンドのエラーの場合、syslogにメッセージを出力後（prexsyslogコマンド）、戻り値にRET_NGを設定し、以降の処理を継続する。
# *     -# WALWriterプロセスが存在する場合、WALWriterプロセスを停止する（kill [プロセスID]）。
# *        エラーがあった場合、syslogにメッセージを出力する（prexsyslogコマンド）。
# *     -# postmasterプロセスの存在確認を行う（ps x | grep [プロセス名]）。grepコマンドのエラーの場合、syslogにメッセージを出力後（prexsyslogコマンド）、戻り値にRET_NGを設定し、以降の処理を継続する。
# *     -# postmasterプロセスが存在する場合、PostgreSQLの即時シャットダウンを行う（pg_ctl stop -m immediate）。
# *     -# 即時シャットダウンに失敗した場合、syslogに警告メッセージを出力し（prexsyslogコマンド）、以下8～9の処理を行う。
# *     -# postmasterプロセスを強制終了する（kill -KILL [プロセスID]）。
# *     -# 強制終了したことによって残ってしまう不要なファイルを削除する（rm -f [ファイルパス]）。
# *     -# メッセージキューの存在確認を行う（ipcs -q | grep [キー値]）。grepコマンドのエラーの場合、syslogにメッセージを出力後（prexsyslogコマンド）、戻り値にRET_NGを設定し、以降の処理を継続する。
# *     -# メッセージキューが存在する場合、メッセージキューの削除を行う（ipcrm -Q [キー値]）。
# *        エラーがあった場合、syslogにメッセージを出力する（prexsyslogコマンド）。
# *     -# 上記のいずれかの停止処理でエラーがあった場合は、戻り値としてRET_NGを返却して終了する。
# *        エラーがなかった場合は戻り値としてRET_OKを返却して終了する。
# *
# * @attention  存在確認や停止処理でエラーがあった場合でも、以降の処理を継続して行う。
# * @attention  不要なファイルとは、${PGDATA}/postmaster.pidと/tmp/.s.PGSQL.xxxx.lockである（xxxxは強制終了したpostmasterが使用していたポート番号の値）
# *
# * @param[in]   key       メッセージキューのキー値
# * @param[in]   log_mode  ログ出力有無
# * @retval      RET_OK(0) 停止成功
# * @retval      RET_NG(1) 停止失敗
# *
# * @version 1.1.0
# * @date    08/07/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

# 本スクリプトの戻り値
tostopret=${RET_OK}

##
## 引数チェック
##
if [ ${#} -lt 1 -o ${#} -gt 2 ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_NUMOFPARAM}
	exit ${RET_NG}
fi

if [ ! -z "${2}" -a "${2}" != '-N' ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_INVALIDPARAM}
	exit ${RET_NG}
fi

key=${1}
log_mode="${2}"

# WALWriterプロセスの存在確認
tmp=`ps --user "${PREX_USER}" | grep -P "${PNAME_WALWriter}\$"`
ret=${?}
if [ ${ret} -eq ${RET_ERR_GREP} ] ; then
	[ "${log_mode}" != '-N' ] && ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_ERRORGREP} ${0} ${LINENO}
	tostopret=${RET_NG}
elif [ ${ret} -eq ${RET_OK} ]; then
	# 存在したら強制終了
	pid=`echo ${tmp} | awk '{print $1}'`
	
	if [ ${?} -ne ${RET_OK} ] ; then
		[ "${log_mode}" != '-N' ] && ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_FAILEDTOSTOPPROCESS} ${0} ${LINENO} ${MSGWD_WALWRITER}
		tostopret=${RET_NG}
	fi 
	kill ${pid}
fi 

# postmaster.pidファイルより、postmasterプロセスのPIDを取得
pid_postmaster=`head -n 1 ${postmaster_pid_file}`

if [ ! -z "${pid_postmaster}" ] ; then
	# postmasterプロセスの存在を確認
	ps --pid ${pid_postmaster} > /dev/null
	
	if [ ${?} -eq ${RET_OK} ] ; then
		# postmasterが存在する場合、postmaster即時シャットダウン
		pg_ctl stop -m immediate
		
		if [ ${?} -ne ${RET_OK} ] ; then
			# 停止しなかったら強制終了
			[ "${log_mode}" != '-N' ] && ${PREX_BIN}/prexsyslog ${LOG_WARNING} ${MSGID_CC_WARNINGFAILEDTOPGSTOP} ${0} ${LINENO}
			kill -KILL ${pid_postmaster}
			
			# postmaster不要ファイル削除
			rm -f ${postmaster_pid_file} ${postmaster_lock_file} > /dev/null 2>&1
			
			tostopret=${RET_NG}
		fi
	fi
fi

# メッセージキューの存在確認
if [ -n "${key}" ] ; then
	# キー値を16進数に変換し検索
	keyhex=`echo "obase=16; ibase=10; ${key}" | bc`
	ipcs -q | grep -i "^0x[0]*${keyhex}" > /dev/null 2>&1
	ret=${?}
	if [ ${ret} -eq ${RET_ERR_GREP} ] ; then
		[ "${log_mode}" != '-N' ] && ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_ERRORGREP} ${0} ${LINENO}
		tostopret=${RET_NG}
	elif [ ${ret} -eq ${RET_OK} ]; then
		# 存在したら強制終了
		ipcrm -Q ${key}
		if [ ${?} -ne ${RET_OK} ] ; then
			[ "${log_mode}" != '-N' ] && ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CC_FAILEDTODELETEMSGQ} ${0} ${LINENO}
			tostopret=${RET_NG}
		fi 
	fi 
fi 

exit ${tostopret}
