/*****************************************************************************
   Copyright (C) 2006 NTT
  *****************************************************************************/
/**
 * @file    CompRemoteLSN.c
 * @brief   ローカル・リモートLSN比較コマンドのインプリメンテーション
 *
 * ローカル・リモートLSN比較コマンドのインプリメンテーションを行う。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */

#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include <unistd.h>
#include "PREXDefine.h"
#include "PREXMessage.h"
#include "GetCheckPointLSN.h"
#include "CompRemoteLSN.h"

/**
 * @fn      int CompRemoteLSN_main(int argc, char *argv[])
 * @brief   ローカル・リモートLSN比較コマンド・メイン処理
 *
 * ローカルLSNとリモートLSNの比較を行う。
 *
 *   詳細は以下のとおり。
 *     -# コマンドパラメータのチェックを行う。以下に示すパラメータ指定は、正常なパラメータとみなす。
 *        - 第1パラメータが指定されている、第2パラメータが指定されている、第3パラメータ以降は指定されていない。
 *     -# 正常なパラメータとみなせない場合には、エラーとして、syslogにエラーメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *     -# 相手側サーバのPostgreSQLに接続する。
 *     -# 接続できた場合、相手側サーバのPostgreSQLとの接続を切断しRET_OKを返却して終了する。
 *     -# 接続できなかった場合、6以降の処理を行う。
 *     -# LSNローカル値を取得する。
 *        エラーが発生した場合、syslogにメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *     -# LSNリモート値を取得する(ssh接続)。
 *        エラーが発生した場合、スリープ後に再試行する。
 *     -# LSN値の比較を行う。
 *     -# ローカルのトランザクションが先行していた場合、戻り値としてRET_LOCAL_ADVANCEDを返却して終了する。
 *     -# リモートのトランザクションが先行していた場合、戻り値としてRET_REMOTE_ADVANCEDを返却して終了する。
 *     -# ローカルとリモートが同一であった場合、戻り値としてRET_SAME_XLOGを返却して終了する。
 *
 * @remarks     実際のメイン関数の名称は「main」であるが、ドキュメント生成ツールの仕様で他のソースに
 *              同名の関数があると正しく出力されないため、ここでは変更して記述している。
 * @param[in]   argc          引数の数
 * @param[in]   argv[0]       コマンド名
 * @param[in]   argv[1]       相手側サーバのホスト名
 * @param[in]   argv[2]       相手側サーバの接続ユーザ名
 * @retval      RET_OK(0)     相手側サーバACT状態（初期起動でない）
 * @retval      RET_NG(1)     異常終了
 * @retval      RET_LOCAL_ADVANCED(2)      ローカルの方がトランザクション先行
 * @retval      RET_REMOTE_ADVANCED(3)     リモートの方がトランザクション先行
 * @retval      RET_SAME_XLOG(4)           ローカルとリモートのトランザクションが同一
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
int main(int argc, char *argv[])
{
	int rcode;
	char sCommand[CC_COMMANDMAX_PSQL]; /* コマンド文字列作成用 */
	LSNInfo_t LocalLSN, RemoteLSN;

	/* 引数チェック */
	if (argc != 3) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, MSGWD_NUMOFPARAM);
		return RET_NG;
	}
	
	/* 相手側サーバのPostgreSQL接続 */
	sprintf(sCommand, CC_CMDFMT_PSQL, argv[ARGINDEX_HOST]);
	rcode = system(sCommand);
	if (WIFEXITED(rcode) != FALSE && WEXITSTATUS(rcode) == RET_OK) {
		return RET_OK;
	}

	/* LSNローカル値を取得 (コマンドを実行し標準出力に出力されたものを取得する) */
	rcode = GetCheckPointLSN(&LocalLSN, NULL);
	if (rcode == RET_NG) {
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOGET, MSGWD_LOCALLSN);
		return RET_NG;
	}
	
	/* LSNリモート値を取得 (コマンドを実行し標準出力に出力されたものを取得する) */
	while (1) {
		/* 取得できるまでポーリング */
		rcode = GetCheckPointLSN(&RemoteLSN, argv);
		if (rcode == RET_OK) {
			break;
		} 
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOGET, MSGWD_REMOTELSN);
		sleep(CC_GETREMOTELSN_POLLING_INTERVAL);
	}
	
	/* どちらが先行しているかを返却 */
	if (CompLSN_IsLessThan(LocalLSN , RemoteLSN)) {
		return RET_REMOTE_ADVANCED;
	} else if (CompLSN_IsLessThan(RemoteLSN, LocalLSN)) {
		return RET_LOCAL_ADVANCED;
	}
	/* XLOG同一 */
	return RET_SAME_XLOG;
}
