/*****************************************************************************
   Copyright (C) 2006-2007 NTT
  *****************************************************************************/
/**
 * @file    ReadConfig.c
 * @brief   設定ファイル（WALWriter.conf）読み込みコマンドのインプリメンテーション
 *
 * 設定ファイル（WALWriter.conf）読み込みコマンドのインプリメンテーションを行う。
 *
 * @version 1.1.0
 * @date    08/02/2007
 * @since   1.0
 */
#include <string.h>
#include <stdlib.h>
#include "PREXMessage.h"
#include "PREXDefine.h"
#include "ReadWWConfFile.h"
#include "ReadConfig.h"

void PrintParameters(WWSettingInfo_t* pInfo);

/**
 * @fn      int ReadConfig_main(int argc, char *argv[])
 * @brief   設定ファイル（WALWriter.conf）読み込みコマンド・メイン処理
 *
 * 設定ファイル（WALWriter.conf）を読み込み、環境変数へ設定を行う。
 *
 *   詳細は以下のとおり。
 *     -# コマンドパラメータのチェックを行う。以下に示すパラメータ指定は、正常なパラメータとみなす。
 *        - 第1が指定されている、第3パラメータ以降は指定されていない。
 *     -# 正常なパラメータとみなせない場合には、エラーとして、syslogにエラーメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *     -# 設定ファイルを読み込み、各項目の取得とチェックを行う（ReadConfigFile 関数）。
 *        エラーが発生した場合、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 取得した値をシェル変数設定用文字列に変換し、標準出力に出力する。
 *     -# 呼出元に RET_OKを返却する。
 *
 * @param[in]   argc          引数の数
 * @param[in]   argv[0]       コマンド名
 * @param[in]   argv[1]       設定ファイルパス（WALWriter.conf）
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
int main(int argc, char *argv[])
{
	int rcode;
	WWSettingInfo_t setInfo;

	/* パラメータチェック */
	if (argc != 2) {
		/* 引数の数が不正 */
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, MSGWD_NUMOFPARAM);
		return RET_NG;
	}
	
	/* 設定ファイル項目の初期化 */
	rcode = InitSettingInfo(&setInfo);
	if (rcode != RET_OK) {
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOREADCONFIG);
		return RET_NG;
	}
	
	/* 設定ファイル項目の読み込み */
	rcode = ReadConfigFile(argv[ARGINDEX_WWCONF], &setInfo);
	if (rcode == RET_NG) {
		TermSettingInfo(&setInfo);
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOREADCONFIG);
		return RET_NG;
	}

	/* 標準出力に出力 */
	PrintParameters(&setInfo);

	/* 設定ファイル項目の解放 */
	TermSettingInfo(&setInfo);

	return RET_OK;
}

/**
 * @fn      void PrintParameters(WWSettingInfo_t* pInfo)
 * @brief   シェル変数設定用文字列生成・出力
 *
 * 設定ファイル情報の各項目をシェル変数設定用文字列に変換し、標準出力に出力する。
 *
 *   詳細は以下のとおり。
 *     -# シェル変数設定用文字列に変換し、標準出力に出力する。
 *
 * @param[in]   pInfo         設定ファイル情報
 *
 * @version 1.1.0
 * @date    08/02/2007
 * @since   1.0
 */
void PrintParameters(WWSettingInfo_t* pInfo)
{
	printf("ws_host='%s' ", pInfo->ws_host);
	printf("prex_port='%d' ", pInfo->prex_port);
	printf("ws_polling_interval='%d' ", pInfo->ws_polling_interval);
	printf("prex_msgkey='%d' ", pInfo->prex_msgkey);
	printf("actwalhost='%s' ", pInfo->actwalhost);
	printf("actarchivehost='%s' ", pInfo->actarchivehost);
	printf("actbackuphost='%s' ", pInfo->actbackuphost);
	printf("actwaluser='%s' ", pInfo->actwaluser);
	printf("actarchiveuser='%s' ", pInfo->actarchiveuser);
	printf("actbackupuser='%s' ", pInfo->actbackupuser);
	printf("actwaldir='%s' ", pInfo->actwaldir);
	printf("actarchivedir='%s' ", pInfo->actarchivedir);
	printf("actbackupdir='%s' ", pInfo->actbackupdir);
	printf("sbywaldir='%s' ", pInfo->sbywaldir);
	printf("sbyarchivedir='%s' ", pInfo->sbyarchivedir);
	printf("sbybackupdir='%s' ", pInfo->sbybackupdir);
	printf("acthahost='%s'", pInfo->acthahost);
	printf("\n");
	
	return;
}
