/*****************************************************************************
   Copyright (C) 2006 NTT
  *****************************************************************************/
/**
 * @file    SyncXLOG.c
 * @brief   XLOG同期コマンドのインプリメンテーション
 *
 * XLOG同期コマンドのインプリメンテーションを行う。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include <unistd.h>
#include "SyncXLOG.h"
#include "PREXMessage.h"
#include "PREXDefine.h"

/**
 * @fn      int SyncXLOG_main(int argc, char *argv[])
 * @brief   XLOG同期コマンド・メイン処理
 *
 * XLOG同期を行う。
 *
 *   詳細は以下のとおり。
 *     -# コマンドパラメータのチェックを行う。以下に示すパラメータ指定は、正常なパラメータとみなす。
 *        - 第1～第11パラメータが指定されている、第2,3パラメータに正数値が指定されている、第12パラメータ以降は指定されていない。
 *     -# 正常なパラメータとみなせない場合には、エラーとして、syslogにエラーメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *     -# ソケットを作成する。ソケットの作成でエラーが発生した場合には、
 *        syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 接続要求を行なう。エラーが発生した場合は、パラメータで指定された秒数スリープする。
 *     -# 接続を確立するまで 4 を繰り返す。
 *     -# XLOG同期処理を行う（SyncXLOG 関数）。エラーが発生した場合には、syslogにエラーメッセージを出力し、
 *        ソケットをクローズし、呼出元に RET_NGを返却する。
 *     -# 「同期完了」の送信を行う。エラーが発生した場合には、syslogにエラーメッセージを出力し、ソケットをクローズし、
 *        呼出元に RET_NGを返却する。
 *     -# ソケットをクローズし、呼出元に RET_OKを返却する。
 *
 * @remarks     実際のメイン関数の名称は「main」であるが、ドキュメント生成ツールの仕様で他のソースに
 *              同名の関数があると正しく出力されないため、ここでは変更して記述している。
 * @param[in]   argc          引数の数
 * @param[in]   argv[0]       コマンド名
 * @param[in]   argv[1]       相手側サーバへの接続ホスト名
 * @param[in]   argv[2]       相手側サーバへの接続ポート番号
 * @param[in]   argv[3]       相手側サーバへの接続要求ポーリング間隔
 * @param[in]   argv[4]       転送対象WALディレクトリを持つリモートホスト：actwalhost
 * @param[in]   argv[5]       actwalhostへのログインユーザ名：actwaluser
 * @param[in]   argv[6]       リモート側WALディレクトリ：actwaldir
 * @param[in]   argv[7]       ローカル側WALディレクトリ：sbywaldir
 * @param[in]   argv[8]       転送対象アーカイブログディレクトリを持つリモート側ホスト：actarchivehost
 * @param[in]   argv[9]       actarchivehostへのログインユーザ名：actarchiveuser
 * @param[in]   argv[10]       リモート側アーカイブディレクトリ：actarchivedir
 * @param[in]   argv[11]      ローカル側アーカイブディレクトリ：sbyarchivedir
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
int main(int argc, char *argv[])
{
	int rcode;
	int nPort;
	int nPollingInterval;
	CommHeader_t       sendInfo;
	struct hostent*    connect_host;
	int  sock;                          /* ソケット */
	struct sockaddr_in sock_addr;       /* アドレスフォーマット */
	
	/* パラメータチェック */
	if (argc != 12) {
		/* 引数の数が不正 */
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, MSGWD_NUMOFPARAM);
		return RET_NG;
	}

	/* 正数値か */
	nPort = atoi(argv[ARGINDEX_PORT]);
	if (nPort <= 0) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, MSGWD_CONN_PORTNO);
		return RET_NG;
	}
	nPollingInterval = atoi(argv[ARGINDEX_POLLING_INTERVAL]);
	if (nPollingInterval <= 0) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, MSGWD_CONN_POLLING_INTERVAL);
		return RET_NG;
	}
	
	/* ソケット生成の準備 */
	bzero((char*)&sock_addr, sizeof(sock_addr));
	sock_addr.sin_family = AF_INET;
	sock_addr.sin_port = htons(nPort);

	/* 接続先サーバ情報の取得 */
	connect_host = gethostbyname(argv[ARGINDEX_HOST]);
	if (connect_host == NULL) {
		in_addr_t addr;
		addr = inet_addr(argv[ARGINDEX_HOST]);
		connect_host = gethostbyaddr((char*)&addr, sizeof(addr), AF_INET);
		if (connect_host == NULL) {
			OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, MSGWD_CONN_HOSTNAME);
			return RET_NG;
		}
	}
	memcpy(&sock_addr.sin_addr, connect_host->h_addr, connect_host->h_length);

	/* ソケットの生成 */
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock == INVALID_SOCKET) {
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOOPENSOCKET);
		return RET_NG;
	}

	/* 相手側サーバへの接続 */
	do {
		rcode = connect(sock, (struct sockaddr *)&sock_addr, sizeof(sock_addr));
		if (rcode == SOCKET_ERROR) {
			sleep(nPollingInterval);
		}
	} while (rcode == SOCKET_ERROR);

	/* XLOG同期 */
	rcode = SyncXLOG(argv[ARGINDEX_WALHOST], argv[ARGINDEX_WALUSER], argv[ARGINDEX_REMOTEWALDIR], argv[ARGINDEX_LOCALWALDIR],
		argv[ARGINDEX_ARCHIVEHOST], argv[ARGINDEX_ARCHIVEUSER], argv[ARGINDEX_REMOTEARCHIVEDIR], argv[ARGINDEX_LOCALARCHIVEDIR]);
	if (rcode == RET_NG) {
		shutdown(sock, SHUT_RDWR);
		close(sock);
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOSYNCXLOG);
		return RET_NG;
	}

	sendInfo.nMessageID   = MSGID_ENDSYNCXLOG;
	sendInfo.uMessageSize = 0;

	/* 「同期完了」送信 */
	rcode = send(sock, (char*)&(sendInfo), sizeof(CommHeader_t), MSG_NOSIGNAL);
	if (rcode == SOCKET_ERROR) {
		shutdown(sock, SHUT_RDWR);
		close(sock);
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOSEND, MSGWD_TCPMSG_ENDSYNCXLOG);
		return RET_NG;
	}
	
	shutdown(sock, SHUT_RDWR);
	close(sock);
	return RET_OK;
}

/**
 * @fn      int SyncXLOG(char* remotewalhost, char* remotewaluser, char* remotewaldir, char* localwaldir, 
                char* remotearchivehost, char* remotearchiveuser, char* remotearchivedir, char* localarchivedir)
 * @brief   XLOG同期処理
 *
 * XLOG同期処理を行う。
 *
 *   詳細は以下のとおり。
 *     -# コマンドパラメータのチェックを行う。以下に示すパラメータ指定は、正常なパラメータとみなす。
 *        - 第1～第8パラメータが指定されている。
 *     -# 正常なパラメータとみなせない場合には、エラーとして、syslogにエラーメッセージを出力後、戻り値としてRET_NGを呼出元に返却して終了する。
 *     -# rsyncコマンドでWALディレクトリの転送を行う。
 *        エラーが発生した場合、syslogにエラーメッセージを出力し、戻り値としてRET_NGを呼出元に返却して終了する。
 *     -# rsyncコマンドでアーカイブログの転送を行う。
 *        エラーが発生した場合、syslogにエラーメッセージを出力し、戻り値としてRET_NGを呼出元に返却して終了する。
 *     -# 呼出元に RET_OKを返却する。
 *
 * @remarks     実際のメイン関数の名称は「main」であるが、ドキュメント生成ツールの仕様で他のソースに
 *              同名の関数があると正しく出力されないため、ここでは変更して記述している。
 * @param[in]   remotewalhost      転送対象WALディレクトリを持つリモートホスト：actwalhost
 * @param[in]   remotewaluser      actwalhostへのログインユーザ名：actwaluser
 * @param[in]   remotewaldir       リモート側WALディレクトリ：actwaldir
 * @param[in]   localwaldir        ローカル側WALディレクトリ：sbywaldir
 * @param[in]   remotearchivehost  転送対象アーカイブログディレクトリを持つリモート側ホスト：actarchivehost
 * @param[in]   remotearchiveuser  actarchivehostへのログインユーザ名：actarchiveuser
 * @param[in]   remotearchivedir   リモート側アーカイブディレクトリ：actarchivedir
 * @param[in]   localarchivedir    ローカル側アーカイブディレクトリ：sbyarchivedir
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
int SyncXLOG(char* remotewalhost, char* remotewaluser, char* remotewaldir, char* localwaldir, 
             char* remotearchivehost, char* remotearchiveuser, char* remotearchivedir, char* localarchivedir)
{
	int rcode;
	char sCommand[CC_COMMANDMAX_SYNCXLOG]; /* コマンド文字列作成用 */

	/* パラメータチェック */
	if (remotewalhost == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, ENVITEM_ACTWALHOST);
		return RET_NG;
	}
	if (remotewaluser == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, ENVITEM_ACTWALUSER);
		return RET_NG;
	}
	if (remotewaldir == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, ENVITEM_ACTWALDIR);
		return RET_NG;
	}
	if (localwaldir == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, ENVITEM_SBYWALDIR);
		return RET_NG;
	}
	if (remotearchivehost == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, ENVITEM_ACTARCHIVEHOST);
		return RET_NG;
	}
	if (remotearchiveuser == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, ENVITEM_ACTARCHIVEUSER);
		return RET_NG;
	}
	if (remotearchivedir == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, ENVITEM_ACTARCHIVEDIR);
		return RET_NG;
	}
	if (localarchivedir == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, ENVITEM_SBYARCHIVEDIR);
		return RET_NG;
	}
	
	/* WALディレクトリ同期用コマンド文字列の作成 */
	sprintf(sCommand, CC_CMDFMT_SYNCXLOG, remotewaluser, remotewalhost, remotewaldir, localwaldir);
	rcode = system(sCommand);
	if (WIFEXITED(rcode) == FALSE || WEXITSTATUS(rcode) != RET_OK) {
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTORSYNC, MSGWD_WALDIR);
		return RET_NG;
	}

	/* アーカイブログディレクトリ同期用コマンド文字列の作成 */
	sprintf(sCommand, CC_CMDFMT_SYNCXLOG, remotearchiveuser, remotearchivehost, remotearchivedir, localarchivedir);
	rcode = system(sCommand);
	if (WIFEXITED(rcode) == FALSE || WEXITSTATUS(rcode) != RET_OK) {
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTORSYNC, MSGWD_ARCHIVEDIR);
		return RET_NG;
	}

	return RET_OK;
}
