/*****************************************************************************
   Copyright (C) 2006 NTT
  *****************************************************************************/
/**
 * @file    WaitSyncXLOG.c
 * @brief   XLOG同期完了待ちコマンドのインプリメンテーション
 *
 * XLOG同期完了待ちコマンドのインプリメンテーションを行う。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include "PREXMessage.h"
#include "PREXDefine.h"
#include "WaitSyncXLOG.h"

/**
 * @fn      int WaitSyncXLOG_main(int argc, char *argv[])
 * @brief   XLOG同期完了待ちコマンド・メイン処理
 *
 * XLOG同期完了待ちを行う。
 *
 *   詳細は以下のとおり。
 *     -# コマンドパラメータのチェックを行う。以下に示すパラメータ指定は、正常なパラメータとみなす。
 *        - 第1パラメータのみが指定されている。
 *     -# 正常なパラメータとみなせない場合には、エラーとして、syslogにエラーメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *     -# ソケットを作成する。ソケットの作成でエラーが発生した場合には、
 *        syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 「同期完了」の受信を待つ。エラーが発生した場合には、syslogにエラーメッセージを出力し、ソケットをクローズし、呼出元に RET_NGを返却する。
 *     -# ソケットをクローズし、呼出元に RET_OKを返却する。
 *
 * @remarks     実際のメイン関数の名称は「main」であるが、ドキュメント生成ツールの仕様で他のソースに
 *              同名の関数があると正しく出力されないため、ここでは変更して記述している。
 * @param[in]   argc          引数の数
 * @param[in]   argv[0]       コマンド名
 * @param[in]   argv[1]       待ち受けポート番号
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
int main(int argc, char *argv[])
{
	int nPort;
	int rcode;
	int acc = FD_INACTIVE;
	int sock = FD_INACTIVE;          /* ソケット */
	struct sockaddr_in sock_addr;    /* アドレスフォーマット */
	size_t nAddrLen;
	CommHeader_t recvInfo;
	
	/* 引数チェック */
	if (argc != 2) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, MSGWD_NUMOFPARAM);
		return RET_NG;
	}
	/* 正数値か */
	nPort = atoi(argv[ARGINDEX_PORT]);
	if (nPort <= 0) {
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, MSGWD_CONN_PORTNO);
		return RET_NG;
	}

	/* ソケットの生成 */
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock == INVALID_SOCKET) {
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOOPENSOCKET);
		return RET_NG;
	}

	bzero((char*)&sock_addr, sizeof(sock_addr));
	sock_addr.sin_family = AF_INET;
	sock_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	sock_addr.sin_port = htons(nPort);
	nAddrLen   = sizeof(sock_addr);

	/* バインド */
	rcode = bind(sock, (struct sockaddr*)&sock_addr, nAddrLen);
	if (rcode == SOCKET_ERROR) {
		WaitSyncXLOG_FREE();
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOSETTINGRECVWAIT);
		return RET_NG;
	}

	rcode = listen(sock, 1);
	if (rcode == SOCKET_ERROR) {
		WaitSyncXLOG_FREE();
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOSETTINGRECVWAIT);
		return RET_NG;
	}

	/*  同期を行っているサーバからの接続を受け付ける */
	acc = accept(sock, (struct sockaddr*)&sock_addr, (socklen_t*)(&nAddrLen));
	if (acc == INVALID_SOCKET) {
		WaitSyncXLOG_FREE();
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOACCEPT);
		return RET_NG;
	}

	/* listen ポートのクローズ */
	shutdown(sock, SHUT_RDWR);
	close(sock);
	sock = FD_INACTIVE;

	/* 「同期完了」の受信 */
	rcode = recv(acc, (char*)&(recvInfo), sizeof(CommHeader_t), 0);
	if (rcode == SOCKET_ERROR) {
		WaitSyncXLOG_FREE();
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTORECV, MSGWD_TCPMSG_ENDSYNCXLOG);
		return RET_NG;
	}

	/* メッセージ ID が MSGWD_TCPMSG_ENDSYNCXLOG かどうかの確認 */
	if (recvInfo.nMessageID != MSGID_ENDSYNCXLOG) {
		WaitSyncXLOG_FREE();
		OutputSyslog(LOG_ERR, MSGID_CC_RECVILLEGALMSG, MSGWD_TCPMSG_ENDSYNCXLOG);
		return RET_NG;
	}
	
	/* ソケットクローズ */
	WaitSyncXLOG_FREE();

	return RET_OK;

}
