/*****************************************************************************
   Copyright (C) 2006-2007 NTT
  *****************************************************************************/
/**
 * @file    msgact.c
 * @brief   ACT化要求／応答メッセージ送受信コマンドのインプリメンテーション
 *
 * ACT化要求／応答メッセージ送受信コマンドのインプリメンテーションを行う。
 *
 * @version 1.1.0
 * @date    08/07/2007
 * @since   1.0
 */

#include <sys/time.h>
#include <signal.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include "PREXDefine.h"
#include "PREXMessage.h"
#include "msgact.h"

/**
 * @fn		void sig_alrm()
 * @brief	割り込み用関数
 *	
 * 割り込み処理のシグナルハンドラ
 *
 *   詳細は以下のとおり。
 *     -# syslogにメッセージを出力する。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
void sig_alrm()
{
	OutputSyslog(LOG_WARNING, MSGID_CC_WARNINGWAITTOACT, DELAY_SEC);
}


/**
 * @fn      int msgact_main(int argc , char *argv[])
 * @brief   ACT化要求／応答メッセージ送受信コマンド・メイン処理
 *
 * ACT化要求メッセージの送信およびACT化応答メッセージの受信を行う。
 *
 *   詳細は以下のとおり。
 *     -# コマンドパラメータのチェックを行う。以下に示すパラメータ指定は、正常なパラメータとみなす。
 *        - 第1～第3パラメータが指定されている、第4パラメータ以降は指定されていない。
 *     -# 正常なパラメータとみなせない場合には、エラーとして、syslogにエラーメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *     -# メッセージキューの取得を行う。
 *        エラーがあった場合、syslogにメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *     -# 「ACT化要求」メッセージの送信を行う。
 *        エラーがあった場合、syslogにメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *     -# メッセージの受信を行う。
 *     -# 受信したメッセージが、「ACT化応答」メッセージ以外であった場合、またはシグナルを受信した場合、5の処理に戻る。
 *     -# メッセージキューが削除されている場合、syslogにメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *     -# 「ACT化応答」メッセージの内容が成功だった場合、戻り値としてRET_OKを返却して終了する。内容が失敗だった場合、戻り値としてRET_NGを返却して終了する。
 *
 * @remarks     実際のメイン関数の名称は「main」であるが、ドキュメント生成ツールの仕様で他のソースに
 *              同名の関数があると正しく出力されないため、ここでは変更して記述している。
 * @param[in]   argc        引数の数
 * @param[in]   argv[0]     コマンド名
 * @param[in]   argv[1]     メッセージキューのキー値
 * @param[in]   argv[2]     終端LSNのログID
 * @param[in]   argv[3]     終端LSNのオフセット
 * @retval      RET_OK(0)   正常終了
 * @retval      RET_NG(1)   異常終了
 *
 * @version 1.1.0
 * @date    08/07/2007
 * @since   1.0
 */
int main(int argc, char *argv[])
{
	
	int breakflag = FALSE;
	int retval = RET_OK;
	
	/* 割り込み関数用の変数 */
	struct	sigaction	act,oldact;
	/* タイマー関数用の変数 */
	struct	itimerval	value,oldvalue;

	/* メッセージ */
	ssize_t        		scode;                	/* 諸関数の戻り値 */
	MessageQueItem_t   	msgInfo;              	/* 受信メッセージ格納用領域 */
	LSNInfo_t			lsn;					/* 終端LSN */
	
	/* メッセージキューのID */
	int nMsgQueID = 0;
	int msgkey;
	
	/* パラメータチェック */
	if (argc != 4) {
		/* パラメータエラー(パラメータ過不足) */
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, MSGWD_NUMOFPARAM);
		exit(RET_NG);
	}

	/* ACT化要求 CC->WalWriter */
	msgInfo.nAddressID = MSGTO_WW;
	msgInfo.nMissionID = MSNID_TOACT;
	
	/* ログIDおよびオフセットを16進から10進に変換 */
	lsn.uXLogID = (uint32) strtoll(argv[ARGINDEX_LOGID], NULL, 16);
	lsn.uXRecOff = (uint32) strtoll(argv[ARGINDEX_OFFSET], NULL, 16);
	msgInfo.LSN = lsn;
	
	/* メッセージキューの取得 */
	msgkey = atoi(argv[ARGINDEX_MSGQKEY]);
	if ((nMsgQueID = msgget((key_t)msgkey, MSGGET_FLG_RW )) == MSGQ_ERROR) {
		/* メッセージキューが取得できない */
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOGETMSGQ);
		exit(RET_NG);
	}

	/* ACT化要求メッセージを送信 */
	if (msgsnd(nMsgQueID, &msgInfo, MESSAGE_LEN, 0) == -1) {
		/* ACT化要求メッセージを送信できない */
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOSEND, MSGWD_QMSG_TOACT);
		exit(RET_NG);
	}
	
	/* 割り込み関数の設定 */
	act.sa_handler = sig_alrm;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	sigaction(SIGALRM, &act , &oldact);

	/* 割り込みの設定 */
	value.it_value.tv_usec = DELAY_USEC;
	value.it_value.tv_sec = DELAY_SEC;
	value.it_interval.tv_usec = DELAY_USEC;
	value.it_interval.tv_sec = DELAY_SEC;
	setitimer(ITIMER_REAL, &value , &oldvalue);

	/* ACT化応答メッセージを受信 */
	do {
		errno = 0;
		scode = msgrcv(nMsgQueID, &msgInfo, MESSAGE_LEN_NO_LSN, MSGTO_CCTOACT, 0);
		if (scode >= 0) {
			if (msgInfo.nMissionID == MSNID_ACTOK) {
				breakflag = TRUE;
				retval = RET_OK;
			}
			else if (msgInfo.nMissionID == MSNID_ACTNG) {
				OutputSyslog(LOG_ERR, MSGID_CC_RECVMSGRETISNG, MSGWD_QMSG_REPLYTOACT);
				breakflag = TRUE;
				retval = RET_NG;
			}
		} else {
			if (errno == EINTR) {
				continue;
			}
			OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTORECV, MSGWD_QMSG_REPLYTOACT);
			breakflag = TRUE;
			retval = RET_NG;
		}
	} while (breakflag != TRUE);

	/* 割り込みの解除 */
	setitimer(ITIMER_REAL, &oldvalue, &value);
	/* 割り込み関数の解除 */
	sigaction(SIGALRM, &oldact, NULL);

	exit(retval);
}
