/*****************************************************************************
   Copyright (C) 2006 NTT
  *****************************************************************************/
/**
 * @file    msgsbyrecv.c
 * @brief   SBY化応答メッセージ受信コマンドのインプリメンテーション
 *
 * SBY化応答メッセージ受信コマンドのインプリメンテーションを行う。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */

#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include "PREXDefine.h"
#include "PREXMessage.h"
#include "msgsbyrecv.h"

/**
 * @fn      int msgsbyrecv_main(int argc, char *argv[])
 * @brief   SBY化応答メッセージ受信コマンド・メイン処理
 *
 * SBY化応答メッセージの受信を行う。
 *
 *   詳細は以下のとおり。
 *     -# コマンドパラメータのチェックを行う。以下に示すパラメータ指定は、正常なパラメータとみなす。
 *        - 第1パラメータが指定されている、第2パラメータ以降は指定されていない。
 *     -# メッセージキューの取得を行う。
 *        エラーがあった場合、syslogにメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *     -# メッセージの受信を行う。
 *     -# 受信したメッセージが、「SBY化応答」メッセージ以外であった場合、またはシグナルを受信した場合、3の処理に戻る。
 *     -# メッセージキューが削除されている場合、syslogにメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *     -# 「SBY化応答」メッセージの内容が成功の場合、戻り値としてRET_OKを返却して終了する。内容が失敗の場合、戻り値としてRET_NGを返却して終了する。
 *
 * @remarks     実際のメイン関数の名称は「main」であるが、ドキュメント生成ツールの仕様で他のソースに
 *              同名の関数があると正しく出力されないため、ここでは変更して記述している。
 * @param[in]   argc          引数の数
 * @param[in]   argv[0]       コマンド名
 * @param[in]   argv[1]       メッセージキューのキー値
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
int main(int argc, char *argv[])
{
	int FlgToSBY;
	int retval = RET_OK;
	
	/* メッセージ */
	ssize_t				scode;
	MessageQueItem_t 	msgInfo;
	
	/* メッセージキューのID */
	int nMsgQueID = 0;
	int msgkey;
	
	/* パラメータチェック */
	if (argc != 2) {
		/* メッセージキューのキーが指定されていない */ 
		OutputSyslog(LOG_ERR, MSGID_CC_ILLEGALPARAM, MSGWD_NUMOFPARAM);
		exit(RET_NG);
	}
	
	msgkey = atoi(argv[ARGINDEX_MSGQKEY]);
	
	/* メッセージキューの取得 */
	if ((nMsgQueID = msgget((key_t)msgkey, MSGGET_FLG_RW )) == MSGQ_ERROR) {
		/* メッセージキューが取得できない */
		OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTOGETMSGQ);
		exit(RET_NG);
	}
	
	
	/* SBY化応答メッセージを受信 */
	FlgToSBY = FALSE;
	do{
		errno = 0;
		scode = msgrcv(nMsgQueID, &msgInfo, MESSAGE_LEN_NO_LSN, MSGTO_CCTOSBY, 0);
		if (scode >= 0) {
			if (msgInfo.nMissionID == MSNID_SBYOK) {
				FlgToSBY = TRUE;
				retval = RET_OK;
			}
			else if (msgInfo.nMissionID == MSNID_SBYNG){
				OutputSyslog(LOG_ERR, MSGID_CC_RECVMSGRETISNG, MSGWD_QMSG_REPLYTOSBY);
				FlgToSBY = TRUE;
				retval = RET_NG;
			}
		} else {
			if (errno == EINTR) {
				continue;
			}
			OutputSyslog(LOG_ERR, MSGID_CC_FAILEDTORECV, MSGWD_QMSG_REPLYTOSBY);
			exit(RET_NG);
		}
	} while (FlgToSBY != TRUE);
	
	exit(retval);

}
