/*****************************************************************************
   Copyright (C) 2006-2008 NTT
  *****************************************************************************/
/**
 * @file    PREXDefine.h
 * @brief   PREX共通定義
 *
 * PREXにおけるモジュール間で、共通に使用するテーブル及び定数を定義する。
 *
 * @version 1.3.0
 * @date    06/17/2008
 * @since   1.0
 */
#ifndef PREXDEFINE_H_
#define PREXDEFINE_H_

#include <stdio.h>
#include <syslog.h>

#ifndef C_H
typedef unsigned short uint16;         ///< PostgreSQL における uint16 の定義を流用
typedef unsigned int   uint32;         ///< PostgreSQL における uint32 の定義を流用
#endif /* C_H */

/* テーブル定義 */

/**
 * @struct  LSNInfo_t
 * @brief   LSN情報
 *
 * @note    PostgreSQLにおける XLogRecPtrと同じ構造である。
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct LSNInfo_t {
	uint32 uXLogID;                         ///< ログファイルID
	uint32 uXRecOff;                        ///< ログファイル上のオフセット
} LSNInfo_t;

/**
 * @struct  CommHeader_t
 * @brief   TCP送受信情報ヘッダ部
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct CommHeader_t {
	int          nMessageID;                       ///< メッセージID
	unsigned int uMessageSize;                     ///< メッセージサイズ
} CommHeader_t;

/**
 * @struct  CommWALInfo_t
 * @brief   「WAL転送」情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct CommWALInfo_t {
	CommHeader_t  header;                           ///< ヘッダ部
	LSNInfo_t     LSN;                              ///< LSN情報
	unsigned int  orgMessageSize;                   ///< ログ圧縮前のメッセージサイズ
	char*         pWAL;                             ///< データ領域（WAL）の先頭アドレス
} CommWALInfo_t;

/**
 * @struct  CommSendWALInfo_t
 * @brief   「WAL転送」情報（送信用）
 *
 * @version 1.1.0
 * @date    08/01/2007
 * @since   1.0
 */
typedef struct CommSendWALInfo_t {
	CommHeader_t  header;                           ///< ヘッダ部
	LSNInfo_t     LSN;                              ///< LSN情報
	unsigned int  orgMessageSize;                   ///< ログ圧縮前のメッセージサイズ
	char          pWAL[0];                          ///< データ領域（WAL）の先頭
} CommSendWALInfo_t;

/**
 * @struct  CommReplyOfSendingWALInfo_t
 * @brief   「WAL転送応答」情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct CommReplyOfSendingWALInfo_t {
	CommHeader_t  header;      ///< ヘッダ部
	int           result;      ///< WAL転送結果（RSLT_SENDWAL_OK または RSLT_SENDWAL_NG）
} CommReplyOfSendingWALInfo_t;

/* 200806 -> */
/**
 * @struct  CommSendTLIInfo_t
 * @brief   「TLI通知」情報
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
 */
typedef struct CommSendTLIInfo_t {
	CommHeader_t  header;      ///< ヘッダ部
	unsigned int  TLI;         ///< TLI
} CommSendTLIInfo_t;

/**
 * @struct  CommHistoryInfo_t
 * @brief   「history」情報
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
 */
typedef struct CommHistoryInfo_t {
	CommHeader_t  header;        ///< ヘッダ部
	char*         pHistory;     ///< データ領域（History）の先頭アドレス
} CommHistoryInfo_t;

/**
 * @struct  CommSendHistoryInfo_t
 * @brief   「history」情報（送信用）
 *
 * @version 1.3.0
 * @date    06/05/2008
 * @since   1.3
 */
typedef struct CommSendHistoryInfo_t {
	CommHeader_t  header;        ///< ヘッダ部
	char         pHistory[0];   ///< データ領域（History）の先頭アドレス
} CommSendHistoryInfo_t;
/* <- 200806 */

/**
 * @struct  CommLastLSNInfo_t
 * @brief   「ファイル転送情報通知」情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct CommLastLSNInfo_t {
	CommHeader_t  header;      ///< ヘッダ部
	LSNInfo_t     LSN;         ///< ファイル転送終了LSN
} CommLastLSNInfo_t;

/**
 * @struct  MessageQueItem_t
 * @brief   メッセージキューへの送受信に使用するメッセージ情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct MessageQueItem_t {
	long       nAddressID;                           ///< 宛先ID
	int        nMissionID;                           ///< 指示ID
	LSNInfo_t  LSN;                                  ///< LSN情報
} MessageQueItem_t;

/* 200806 -> */
/**
 * @struct  MessageQueItemTLI_t
 * @brief   メッセージキューへのTLI送信に使用するメッセージ情報
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
 */
typedef struct MessageQueItemTLI_t {
	long         nAddressID;       ///< 宛先ID
	int          nMissionID;       ///< 指示ID
	LSNInfo_t    LSN;              ///< LSN情報
	unsigned int TLI;              ///< TLI
} MessageQueItemTLI_t;
/* <- 200806 */

/*****************
 * 定数定義
 ****************/
/* フラグ */
/**
 * @def     TRUE
 * @brief   真（フラグの値）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define TRUE             1

/**
 * @def     FALSE
 * @brief   偽（フラグの値）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define FALSE            0

/* 戻り値 */
/**
 * @def     RET_OK
 * @brief   戻り値（正常終了／成功）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define RET_OK             0

/**
 * @def     RET_NG
 * @brief   戻り値（異常終了／失敗）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define RET_NG             1

/**
 * @def     RET_ECONNREFUSED
 * @brief   戻り値（接続断）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define RET_ECONNREFUSED   2

/**
 * @def     RET_TIMEOUT
 * @brief   戻り値（受信タイムアウト）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define RET_TIMEOUT        3

/**
 * @def     RET_CONNECT
 * @brief   戻り値（接続異常）
 *
 * @version 1.0.0
 * @date    08/06/2007
 * @since   1.1
 */
#define RET_CONNECT        4


/**
 * @def     RET_RSYNC_ERR_FILE_NOT_FOUND
 * @brief   戻り値（rsync 異常終了／失敗）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define RET_RSYNC_ERR_FILE_NOT_FOUND             23

/**
 * @def     RET_LOCAL_ADVANCED
 * @brief   戻り値（ローカルLSN先行)
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define RET_LOCAL_ADVANCED              2

/**
 * @def     RET_REMOTE_ADVANCED
 * @brief   戻り値（リモートLSN先行)
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define RET_REMOTE_ADVANCED             3

/**
 * @def     RET_SAME_XLOG
 * @brief   戻り値（XLOG同一)
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define RET_SAME_XLOG                   4

/**
 * @def     RET_GREP_NOT_FOUND
 * @brief   戻り値（指定のものが見つからない）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define RET_GREP_NOT_FOUND             1

/* 宛先ID */
/**
 * @def     MSGTO_WW
 * @brief   宛先ID（WALWriter）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGTO_WW          1

/**
 * @def     MSGTO_CCTOACT
 * @brief   宛先ID（Clusterコマンド（ACT化要求））
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGTO_CCTOACT     2

/**
 * @def     MSGTO_WS
 * @brief   宛先ID（WALSender）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGTO_WS          3

/**
 * @def     MSGTO_PGRCV
 * @brief   宛先ID（postgres（リカバリ処理））
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGTO_PGRCV       4

/**
 * @def     MSGTO_PG
 * @brief   宛先ID（postgres）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGTO_PG          5

/**
 * @def     MSGTO_CCTOSBY
 * @brief   宛先ID（Clusterコマンド（SBY化要求））
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGTO_CCTOSBY     6

/* 指示ID */
/**
 * @def     MSNID_ACTOK
 * @brief   指示ID（ACT化成功）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSNID_ACTOK       0

/**
 * @def     MSNID_ACTNG
 * @brief   指示ID（ACT化失敗）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSNID_ACTNG       1

/**
 * @def     MSNID_TOACT
 * @brief   指示ID（ACT化要求）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSNID_TOACT       2

/**
 * @def     MSNID_PITR
 * @brief   指示ID（WAL反映）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSNID_PITR        3

/**
 * @def     MSNID_SENDWAL
 * @brief   指示ID（WAL転送指示）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSNID_SENDWAL     4

/**
 * @def     MSNID_SENDWALFIN
 * @brief   指示ID（WAL転送終了通知）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSNID_SENDWALFIN  5

/**
 * @def     MSNID_SBYOK
 * @brief   指示ID（SBY化成功）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSNID_SBYOK       6

/**
 * @def     MSNID_SBYNG
 * @brief   指示ID（SBY化失敗）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSNID_SBYNG       7

/* 200806 -> */
/**
 * @def     MSNID_TLI
 * @brief   指示ID（TLI更新）
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
 */
#define MSNID_TLI     8

/**
 * @def     MSNID_RECEIVETLI
 * @brief   指示ID（TLI受信）
 *
 * @version 1.3.0
 * @date    07/25/2008
 * @since   1.3
 */
#define MSNID_RECEIVETLI     9
/* <- 200806 */


/* TCP通信におけるメッセージID */
/**
 * @def     MSGID_REQSTARTSEND
 * @brief   メッセージID（送信開始要求）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGID_REQSTARTSEND       10000


/**
 * @def     MSGID_SENDLASTLSN
 * @brief   メッセージID（ファイル転送情報通知）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGID_SENDLASTLSN          101

/**
 * @def     MSGID_SENDWAL
 * @brief   メッセージID（WAL転送）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGID_SENDWAL              102


/**
 * @def     MSGID_SENDCOMPWAL
 * @brief   メッセージID（圧縮WAL転送）
 *
 * @version 1.0.0
 * @date    08/06/2007
 * @since   1.1
 */
#define MSGID_SENDCOMPWAL          105

/* 200806 -> */
/**
 * @def     MSGID_SENDTLI
 * @brief   メッセージID（TLI通知）
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
 */
#define MSGID_SENDTLI              107

/**
 * @def     MSGID_SENDHISTORY
 * @brief   メッセージID（history情報通知）
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
 */
#define MSGID_SENDHISTORY          109
/* <- 200806 */

/**
 * @def     MSGID_BKUPFINISHED
 * @brief   メッセージID（バックアップ取得完了通知）
 *
 * @version 1.0.0
 * @date    08/06/2007
 * @since   1.1
 */
#define MSGID_BKUPFINISHED        10006


/**
 * @def     MSGID_REPLYSENDWAL
 * @brief   メッセージID（WAL転送応答）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGID_REPLYSENDWAL       10003

/**
 * @def     MSGID_ENDSYNCXLOG
 * @brief   メッセージID（同期完了）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGID_ENDSYNCXLOG       20204

/* 200806 -> */
/**
 * @def     MSGID_REPLYSENDTLI
 * @brief   メッセージID（TIL通知応答）
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
 */
#define MSGID_REPLYSENDTLI       10008

/**
 * @def     MSGID_REPLYSENDHISTORY
 * @brief   メッセージID（history情報通知応答）
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
 */
#define MSGID_REPLYSENDHISTORY   10010
/* <- 200806 */

/* PGクラスタモード */
/**
 * @def     PG_MODE_SBY
 * @brief   postmasterの状態（SBY）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define PG_MODE_SBY     0

/**
 * @def     PG_MODE_ACT_SA
 * @brief   postmasterの状態（ACT：スタンドアローン）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define PG_MODE_ACT_SA	1

/**
 * @def     PG_MODE_ACT_REP
 * @brief   postmasterの状態（ACT：レプリケーション）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define PG_MODE_ACT_REP  2

/**
 * @def     MESSAGE_LEN
 * @brief   メッセージキュー に送信するメッセージ（LSN情報付き）のサイズ
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MESSAGE_LEN (sizeof(MessageQueItem_t) - sizeof(long))

/**
 * @def     MESSAGE_LEN_NO_LSN
 * @brief   メッセージキュー に送信するメッセージ（LSN情報無し）のサイズ
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MESSAGE_LEN_NO_LSN (sizeof(MessageQueItem_t) - sizeof(long) - sizeof(LSNInfo_t))

/* 200806 -> */
/**
 * @def     MESSAGE_TLI_LEN
 * @brief   メッセージキュー に送信するTLIメッセージのサイズ
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
 */
#define MESSAGE_TLI_LEN (sizeof(MessageQueItemTLI_t) - sizeof(long))
/* <- 200806 */

/**
 * @def     PREX_MAX_HOSTNAME_LEN
 * @brief   ホスト名の最大バイトサイズ
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define PREX_MAX_HOSTNAME_LEN  128

/**
 * @def     PREX_MAX_USERNAME_LEN
 * @brief   ユーザ名の最大バイトサイズ
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define PREX_MAX_USERNAME_LEN  64

/**
 * @def     PREX_MAX_PATH_LEN
 * @brief   ディレクトリパスの最大バイトサイズ
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define PREX_MAX_PATH_LEN      256

/**
 * @def     RSLT_SENDWAL_OK
 * @brief   WAL転送応答の結果（成功）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define RSLT_SENDWAL_OK    0

/**
 * @def     RSLT_SENDWAL_NG
 * @brief   WAL転送応答の結果（失敗）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define RSLT_SENDWAL_NG    -1

/**
 * @def     MSGGET_FLG_RW
 * @brief   メッセージキューID取得時のフラグ（read/write）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGGET_FLG_RW    0600

/**
 * @def     MAX_MSGLEN
 * @brief   syslog に出力するメッセージの最大長さ
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MAX_MSGLEN    512

/**
 * @def     MSGCODE_FUNC_LEN
 * @brief   メッセージコード中の発生箇所（機能番号）の桁数
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGCODE_FUNC_LEN    2

/**
 * @def     MSGCODE_TYPE_LEN
 * @brief   メッセージコード中のメッセージ種別の桁数
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGCODE_TYPE_LEN    1

/**
 * @def     MSGCODE_ID_LEN
 * @brief   メッセージコード中のメッセージ個別番号の桁数
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGCODE_ID_LEN    5

/**
 * @def     PREX_MSGFMT_SRCINFO
 * @brief   呼出元情報（ファイル名及び行番号）付きログテキストのフォーマット
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define PREX_MSGFMT_SRCINFO    "%s[%s]: %s %s (%s,%d)"

/**
 * @def     PREX_MSGFMT_SRCINFO_PGSTYLE
 * @brief   呼出元情報（ファイル名及び行番号）付きログテキストのフォーマット
 *
 * @attention  このフォーマットは PostgreSQL内プロセス（WALSender, postmaster,
 *             postgres, postgres（リカバリ処理））から出力する際に使用する
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define PREX_MSGFMT_SRCINFO_PGSTYLE "%s (%s,%d)"

/**
 * @def     TIMELINENO_STR_LEN
 * @brief   タイムライン番号文字列の長さ
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define TIMELINENO_STR_LEN   8

/**
 * @def     XLOGFILEID_STR_LEN
 * @brief   ログファイルID文字列の長さ
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define XLOGFILEID_STR_LEN   8

/**
 * @def     SEGMENT_NO_STR_LEN
 * @brief   セグメント番号文字列の長さ
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define SEGMENT_NO_STR_LEN   8

/**
 * @def     XLOGFILENAME_LEN
 * @brief   WALセグメント名の長さ
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define XLOGFILENAME_LEN    (TIMELINENO_STR_LEN+XLOGFILEID_STR_LEN+SEGMENT_NO_STR_LEN)

/**
 * @def     FD_INACTIVE
 * @brief   未使用ソケット
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define FD_INACTIVE  (-1)

/**
 * @def     SOCKET_ERROR
 * @brief   ソケット関連関数のエラー時の戻り値
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define SOCKET_ERROR    (-1)

/**
 * @def     INVALID_SOCKET
 * @brief    ソケット関連関数のエラー時の戻り値
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define INVALID_SOCKET  (-1)

/**
 * @def     TRGTSOCK_SHUTDOWN
 * @brief   接続先ソケットのシャットダウン（recv()の戻り値として使用）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define TRGTSOCK_SHUTDOWN  0

/**
 * @def     MSGQ_ERROR
 * @brief   メッセージキュー関連関数のエラー時の戻り値
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define MSGQ_ERROR    (-1)

/**
 * @def     UNSET_MSGQID
 * @brief   未設定のメッセージキューID
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define UNSET_MSGQID    (-1)

/* PostgreSQL 関連項目 */
/**
 * @def     WAL_SEG_FILESIZE
 * @brief   WALセグメントのファイルサイズ
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define WAL_SEG_FILESIZE    ((uint32)(16*1024*1024))

/**
 * @def     XRECOFF_MAX
 * @brief   xrecoffの最大値
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define XRECOFF_MAX    (uint32)0xffffffff / WAL_SEG_FILESIZE * WAL_SEG_FILESIZE -1

/**
 * @def     INVALID_XLOGID
 * @brief   不正な xlogid の値
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define INVALID_XLOGID   0xFFFFFFFF

/**
 * @def     INVALID_XRECOFF
 * @brief   不正な xrecoff の値
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define INVALID_XRECOFF   0xFFFFFFFF

/**
 * @def     PREX_SYSLOG_ID
 * @brief   syslog に出力する名称
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define PREX_SYSLOG_ID          "prex"


/**
 * @def     ERR_RETRY_MAX_COUNT
 * @brief   send/recvにおいてリトライを行うエラーの最大許容数
 *
 * @version 1.1.0
 * @date    10/02/2007
 * @since   1.0
 */
#define ERR_RETRY_MAX_COUNT      5

/* 200806 -> */
/**
 * @def     HISTORYFILENAME
 * @brief   LastLSN用historyファイル名
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
 */
#define HISTORYFILENAME "LASTLSNHISTORYFILE"
/* <- 200806 */

/*****************
 * マクロ関数定義
 ****************/


/**
 * @def     OutputSyslogCore(priority__, msgcode__, file__, line__, ...)
 * @brief   syslog へのメッセージ出力
 *
 * @param[in]  priority__  syslog 出力に関するプライオリティ
 * @param[in]  msgcode__   メッセージコード（文字列）
 * @param[in]  file__      ファイル名（文字列）
 * @param[in]  line__      行番号（数値）
 * @param[in]  ...         フォーマット文字列への埋め込み文字列（可変個数）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define OutputSyslogCore(priority__, msgcode__, file__, line__, ...) {\
	char msg__[MAX_MSGLEN + 1]; \
	char msgfull__[MAX_MSGLEN + 1]; \
	char funcid__[MSGCODE_FUNC_LEN + 1]; \
	int funcindex__; \
	char msgid__[MSGCODE_ID_LEN + 1]; \
	int msgindex__; \
	strncpy(funcid__, msgcode__, MSGCODE_FUNC_LEN); \
	strncpy(msgid__, msgcode__ + MSGCODE_FUNC_LEN + MSGCODE_TYPE_LEN, MSGCODE_ID_LEN); \
	funcid__[MSGCODE_FUNC_LEN] = '\0'; \
	funcindex__ = atoi(funcid__); \
	msgid__[MSGCODE_ID_LEN] = '\0'; \
	msgindex__ = atoi(msgid__); \
	snprintf (msg__, MAX_MSGLEN + 1, PREXMsgInfo[msgindex__].format, ## __VA_ARGS__); \
	snprintf (msgfull__, MAX_MSGLEN + 1, PREX_MSGFMT_SRCINFO, PREXFuncInfo[funcindex__].name, msgcode__, \
			 PREXMsgInfo[msgindex__].level, msg__, file__, line__); \
	openlog(PREX_SYSLOG_ID , LOG_NDELAY, LOG_LOCAL0); \
	syslog(priority__,  msgfull__); \
	closelog(); \
}


/**
 * @def     DebugOutputSyslogCore(priority__, format__, file__, line__, ...)
 * @brief   syslog へのメッセージ出力(デバッグ出力用)
 *
 * @param[in]  priority__  syslog 出力に関するプライオリティ
 * @param[in]  format__    メッセージフォーマット（文字列）
 * @param[in]  file__      ファイル名（文字列）
 * @param[in]  line__      行番号（数値）
 * @param[in]  ...         フォーマット文字列への埋め込み文字列（可変個数）
 *
 * @version 1.0.0
 * @date    09/01/2007
 * @since   1.1
 */
#define DebugOutputSyslogCore(priority__, format__, file__, line__, ...) {\
    int myerr = errno; \
	char msg__[MAX_MSGLEN + 1]; \
	char msgfull__[MAX_MSGLEN + 1]; \
	snprintf (msg__, MAX_MSGLEN + 1, format__, ## __VA_ARGS__); \
	snprintf (msgfull__, MAX_MSGLEN + 1, "WALWriter (Debug) %s (%s,%d)", \
			 msg__, file__, line__); \
	openlog(PREX_SYSLOG_ID , LOG_NDELAY, LOG_LOCAL0); \
	syslog(LOG_ERR,  msgfull__); \
	closelog(); \
	errno = myerr; \
}



/**
 * @def     OutputSyslog(priority__, msgcode__, ...)
 * @brief   syslog へのメッセージ出力
 *
 * @param[in]  priority__  syslog 出力に関するプライオリティ
 * @param[in]  msgcode__   メッセージコード（文字列）
 * @param[in]  ...         フォーマット文字列への埋め込み文字列（可変個数）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define OutputSyslog(priority__, msgcode__, ...) \
	OutputSyslogCore(priority__, msgcode__, __FILE__, __LINE__, ## __VA_ARGS__)


/**
 * @def     DebugOutputSyslog(priority__, format__, ...)
 * @brief   syslog へのメッセージ出力(デバッグ用)
 *
 * @param[in]  priority__  syslog 出力に関するプライオリティ
 * @param[in]  format__    メッセージフォーマット（文字列）
 * @param[in]  ...         フォーマット文字列への埋め込み文字列（可変個数）
 *
 * @version 1.0.0
 * @date    09/01/2007
 * @since   1.1
 */

#ifdef DEBUG
#define DebugOutputSyslog(priority__, format__, ...) \
	DebugOutputSyslogCore(priority__, format__, __FILE__, __LINE__, ## __VA_ARGS__)
#else
#define DebugOutputSyslog(priority__, format__, ...) 
#endif



/**
 * @def     DebugOutputSyslog(priority__, format__, ...)
 * @brief   syslog へのメッセージ出力(デバッグ用)
 *
 * @param[in]  priority__  syslog 出力に関するプライオリティ
 * @param[in]  format__    メッセージフォーマット（文字列）
 * @param[in]  ...         フォーマット文字列への埋め込み文字列（可変個数）
 *
 * @version 1.0.0
 * @date    09/01/2007
 * @since   1.1
 */

#ifdef DEBUG_MESSAGE
#define DebugOutputMesSyslog(priority__, format__, ...) \
	DebugOutputSyslogCore(priority__, format__, __FILE__, __LINE__, ## __VA_ARGS__)
#else
#define DebugOutputMesSyslog(priority__, format__, ...) 
#endif




/**
 * @def     SetLSNInfo(LSNInfo_t* pLSN__, uint32 xlogid__, uint32 xrecoff__)
 * @brief   LSN情報への値設定
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define SetLSNInfo(pLSN__, xlogid__, xrecoff__) { \
	(pLSN__)->uXLogID  = xlogid__; \
	(pLSN__)->uXRecOff = xrecoff__; \
}

/**
 * @def     InitLSNInfo(LSNInfo_t* pLSN__)
 * @brief   LSN情報の初期化
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define InitLSNInfo(pLSN__) SetLSNInfo(pLSN__, 0, 0)

/**
 * @def     ADDSegSizeToLSNInfo(LSNInfo_t LSN__)
 * @brief   LSN情報の値を1セグメントサイズ分進める
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define ADDSegSizeToLSNInfo(LSN__) \
{ \
	if ((LSN__).uXRecOff <= XRECOFF_MAX - WAL_SEG_FILESIZE) { \
		(LSN__).uXRecOff += WAL_SEG_FILESIZE; \
	} else { \
		(LSN__).uXLogID++; \
		(LSN__).uXRecOff = (LSN__).uXRecOff % WAL_SEG_FILESIZE; \
	} \
}

/**
 * @def     CompLSN_IsLessThan(LSNInfo_t LSN1_, LSNInfo_t LSN2_)
 * @brief   LSNの比較（LSN1 ＜ LSN2）
 *
 *   指定された２つの LSN情報が持つ LSNの比較を行なう。
 *
 *   詳細は以下のとおり。
 *     -# LSN1_ が LSN2_ より小さい場合、非0 を返却する。
 *
 * @param[in]  LSN1_  比較対象の LSN情報
 * @param[in]  LSN2_  比較対象の LSN情報
 * @retval     非0  LSN1_ が LSN2_ より小さい。
 * @retval     0    LSN1_ が LSN2_ と同じか、あるいは大きい。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define CompLSN_IsLessThan(a, b) \
	((a).uXLogID < (b).uXLogID || \
	((a).uXLogID == (b).uXLogID && (a).uXRecOff < (b).uXRecOff))

/**
 * @def     CompLSN_IsLessEqual(LSNInfo_t LSN1_, LSNInfo_t LSN2_)
 * @brief   LSNの比較（LSN1 ≦ LSN2）
 *
 *   指定された２つの LSN情報が持つ LSNの比較を行なう。
 *
 *   詳細は以下のとおり。
 *     -# LSN1_ が LSN2_ より小さいか、または同じ場合、非0 を返却する。
 *
 * @param[in]  LSN1_  比較対象の LSN情報
 * @param[in]  LSN2_  比較対象の LSN情報
 * @retval     非0  LSN1_ が LSN2_ より小さいか、あるいは同じ。
 * @retval     0    LSN1_ が LSN2_ より大きい。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define CompLSN_IsLessEqual(a, b) \
	((a).uXLogID < (b).uXLogID || \
	((a).uXLogID == (b).uXLogID && (a).uXRecOff <= (b).uXRecOff))

#endif /* PREXDEFINE_H_ */
