#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2007 NTT
#/**
# * @file    check_enable_toact.sh
# * @brief   フェールオーバによるACT化可否のチェック項目を実行する。
# *
# * フェールオーバによるACT化可否のチェックを行なう。
# *  - リモートサーバの状態確認(ACT状況確認)
# *  - フェールオーバ抑止ファイルのステータス確認
# *  - ネットワーク通信可否
# *
# *   詳細は以下のとおり。
# *     -# リモートサーバのACT状況確認を呼び出し、リモートサーバのPREXがACTでないことをチェック ※1
# *       - ACT状況確認の戻り値が「0」(ACTとして動作中)の場合、ACT化可否確認のリトライ間隔の秒数待機した後、リトライする。
# *       - ACT状況確認の戻り値が「1」(ACTとして動作中ではない)の場合、2の処理を行なう。
# *     -# リモートサーバのフェールオーバー抑止ファイルのステータスをチェック ※1
# *       - ステータスが「1」(フェールオーバ抑止)の場合、ACT化可否確認のリトライ間隔の秒数待機した後、リトライする。
# *       - ステータスが「0」(フェールオーバ許可)の場合、戻り値にACT化OKを返却する。
# *       - フェールオーバ抑止ファイルが存在しない場合、戻り値にACT化OKを返却する。
# *     -# 1または2で、リモートサーバにアクセスできなかった場合、ACT化可否確認のリトライ間隔の秒数待機した後、リトライする。
# *     -# 1～3でのリトライ回数がACT化可否確認のリトライ回数を超えた場合、戻り値にACT化NG(1)を返却する。
# *        最後のACT化可否の判定がリモートサーバにアクセスできないことによるものである場合、戻り値にACT化OKを返却する。
# *
# * ※1 リモートサーバへのアクセスに使用するノードは、以下の優先順序で行なう。
# *     (1)Heartbeat LAN (複数指定されていた場合は、指定順とする)
# *     (2)データ転送LAN
# *
# * @param[in] msgkey          メッセージキューのキー値
# * @param[in] acthahosts      Heartbeat LANを経由する相手ノード宛のIPアドレス(複数指定可)
# * @param[in] actwalhost      データ転送LANを経由する相手ノード宛のIPアドレス
# * @param[in] retry_count     ACT化可否確認のリトライ回数
# * @param[in] retry_interval  ACT化可否確認のリトライ間隔
# * @retval    RET_OK(0)  ACT化OK
# * @retval    RET_NG(1)  ACT化NG
# *
# * @version 1.1.0
# * @date    10/30/2007
# * @since   1.1
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

##
## 定数
##
DEFAULT_IFS="${IFS}" # IFS(デフォルト値)

##
## 変数宣言
##
msgkey=${1}         # メッセージキューのキー値
acthahosts="${2}"   # Heartbeat LANを経由する相手ノード宛のIPアドレス(複数指定可)
actwalhost="${3}"   # データ転送LANを経由する相手ノード宛のIPアドレス
retry_count=${4}    # ACT化可否確認のリトライ回数
retry_interval=${5} # ACT化可否確認のリトライ間隔
ret=
status=
retry_current=
network_err=
actcheck_done=0


# acthahostsは配列に変換
IFS=',' && acthahosts=(`echo "${acthahosts}"`) && IFS="${DEFAULT_IFS}"

##
## ACT化可否チェック
## ・リモートサーバの状態確認(ACT状況確認)
## ・フェールオーバ抑止ファイルのステータス確認
## ・ネットワーク通信可否
##
retry_current=-1

while [ ${retry_current} -lt ${retry_count} ]
do
	retry_current=`expr ${retry_current} + 1`
	network_err=0
	
	##
	## リモートサーバのACT状況確認を呼び出し、
	## ・リモートサーバのPREXがACT以外の場合、戻り値にACT化OKを返却
	## ・リモートサーバのPREXがACTの場合、リトライする
	##
	for host in "${acthahosts[@]}" "${actwalhost}"
	do
		# ネットワーク接続可否を判定
		# 接続できない場合、次のIPアドレスで試行
		ping -w 1 -r ${host} > /dev/null 2>&1 || continue
		
		# リモートサーバのACT状況確認を呼び出し、PREXの状態を確認
		# ACTであった場合、戻り値にACT化NGを返却
		ssh "${PREX_USER}@${host}" "${PREX_BIN}/actmonitor.sh ${msgkey} -N"
		
		if [ ${?} -eq ${RET_OK} ] ; then
			# ACTの場合、リトライ間隔の秒数待機した後、リトライする
			[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_CMN_DEBUG} ${0} ${LINENO} "the status of PREX of remote host is ACT (retry=${retry_current})"
			[ ${retry_current} -lt ${retry_count} ] && sleep ${retry_interval}
			continue 2
		else
			# ACT以外の場合、以降の処理を行なう
			actcheck_done=1
			break
		fi
	done
	
	##
	## フェールオーバー抑止ファイルのステータスをチェック
	## ・フェールオーバ抑止ファイルが存在しない場合、戻り値にACT化OKを返却
	## ・ステータスが「ACT化許可」の場合、戻り値にACT化OKを返却
	## ・ステータスが「ACT化抑止」の場合、リトライする
	##
	if [ ${actcheck_done} -eq 1 ] ; then
		for host in "${acthahosts[@]}" "${actwalhost}"
		do
			# ネットワーク接続可否を判定
			# 接続できない場合、次のIPアドレスで試行
			ping -w 1 -r ${host} > /dev/null 2>&1 || continue
			
			# リモートサーバのフェールオーバ抑止ファイルが存在有無を確認
			ssh "${PREX_USER}@${host}" "[ ! -f \"${FILE_NAME_FO}\" ]"
			
			if [ ${?} -eq ${RET_OK} ] ; then
				# フェールオーバ抑止ファイルが存在しない場合、戻り値にACT化OKを返却
				exit ${RET_OK}
			fi
			
			# リモートサーバのフェールオーバ抑止ファイルのステータスを取得
			status=`ssh "${PREX_USER}@${host}" "${PREX_BIN}/exclusion.sh \"${FO_LOCK_COMMANDID}\" \"${FILE_GET}\""`
			ret=${?}
			
			# フェールオーバ抑止ファイルが存在しない場合、戻り値にACT化OKを返却
			if [ ${ret} -eq ${RET_NOTEXISTS} ] ; then
				# フェールオーバ抑止ファイルが存在しない
				exit ${RET_OK}
			elif [ ${ret} -eq ${RET_NG} ] ; then
				# ステータス取得に失敗した場合、リトライ間隔の秒数待機した後、リトライする
				[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_CMN_DEBUG} ${0} ${LINENO} "exclusion command returned error (retry=${retry_current})"
				[ ${retry_current} -lt ${retry_count} ] && sleep ${retry_interval}
				continue 2
			fi
			
			# ステータスの確認
			if [ "${status}" = "${FO_TYPEID_LOCK}" ] ; then
				# ステータスが「ACT化抑止」の場合、リトライ間隔の秒数待機した後、リトライする
				[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_CMN_DEBUG} ${0} ${LINENO} "the status of the FO control file is not allow change to ACT (retry=${retry_current})"
				[ ${retry_current} -lt ${retry_count} ] && sleep ${retry_interval}
				continue 2
			else
				# ステータスが「ACT化抑止」以外の場合、戻り値にACT化OKを返却
				exit ${RET_OK}
			fi
		done
	fi
	
	##
	## ネットワーク通信不可の場合、リトライ間隔の秒数待機した後、リトライする
	##
	network_err=1
	[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_CMN_DEBUG} ${0} ${LINENO} "impossible to connect a remote host (retry=${retry_current})"
	[ ${retry_current} -lt ${retry_count} ] && sleep ${retry_interval}
done

##
## リトライ回数がACT化可否確認のリトライ回数を超えた場合、
## 最後のチェック結果の種類により以下の処理を行なう
## ・ネットワーク通信不可：戻り値にACT化OKを返却
## ・上記以外：戻り値にACT化NGを返却
##
if [ ${network_err} -eq 1 ] ; then
	exit ${RET_OK}
else
	exit ${RET_NG}
fi
