#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2007 NTT
#/**
# * @file    exclusion.sh
# * @brief   抑止ファイル操作処理
# *
# * 各抑止ファイルに対する操作処理を行う。
# *
# *   詳細は以下のとおり。
# *     -# セマフォコマンドIDに対応するロックファイル(ACT監視抑止ファイル、SBY監視抑止ファイル、フェールオーバ監視抑止ファイル)と対応するセマフォをロックする。
# *     -# 操作内容(ファイルへの値設定(作成)、ファイル内容の取得、ファイルの削除)に対応する処理を実行する。
# *        なお、ファイル内容の取得操作時に対象ファイルが存在しない場合は
# *        戻り値としてRET_NOTRUNを返却する。
# *     -# セマフォのロックを解除する。
# *
# *        エラーがあった場合、syslogにメッセージを出力後（prexsyslogコマンド）、戻り値としてRET_NGを返却して終了する。
# *
# * @param[in]   command_id   セマフォコマンドID
# * @param[in]   command      操作内容
# * @param[in]   set_value    ファイルへの設定値
# * @retval      RET_OK(0)    正常終了
# * @retval      RET_NG(1)    異常終了
# * @retval      RET_NOTRUN(7)    対象ファイル無し
# *
# * @version 1.0.0
# * @date    08/01/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

##
## 定数宣言
##
RET_NOTRUN=7 # ファイル内容取得操作実行時にファイルが無い場合の戻り値

##
## 変数宣言
##
command_id=${1} # セマフォコマンドID
command=${2}   # 操作内容
set_value=${3} # ファイルへの設定値
file_name=0   # セマフォ操作ロックファイル名格納用
ret_val=${RET_OK}

##
## ファイルロック
##
# 引数に対応するロックファイル名を設定
case ${command_id} in
	${ACT_LOCK_COMMANDID})
		file_name=${FILE_NAME_ACT}
		;;
	${SBY_LOCK_COMMANDID})
		file_name=${FILE_NAME_SBY}
		;;
	${FO_LOCK_COMMANDID})
		file_name=${FILE_NAME_FO}
		;;
	*)
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CMN_ILLEGALPARAM} ${0} ${LINENO} ${command_id}
		exit ${RET_NG}
		;;
esac

# セマフォコマンドを実行
${PREX_BIN}/Semaphore ${command_id}

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CMN_FAILEDTOCTLLOCK}  ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## ファイル操作
##
# 引数に対応するファイル操作を実行
case "${command}" in
	"${FILE_SET}")
		if [ "${set_value}" != "${RET_OK}" -a "${set_value}" != "${RET_NG}" ] ; then
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CMN_ILLEGALPARAM} ${0} ${LINENO} "${set_value}"
			ret_val=${RET_NG}
		fi
		if [ ${ret_val} -eq ${RET_OK} ] ; then
			echo ${set_value} > ${file_name}
			if [ ${?} -ne ${RET_OK} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CMN_FAILEDTOWRITEFILE} ${0} ${LINENO} ${file_name}
				ret_val=${RET_NG}
			fi
		fi
		;;

	"${FILE_GET}")
		# 対象ファイルが存在しない場合、戻り値にステータス(未起動)を返却
		if [ ! -f "${file_name}"  ] ; then
			ret_val=${RET_NOTRUN}
		else
			# ファイル内容取得
			cat ${file_name}
			if [ ${?} -ne ${RET_OK} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CMN_FAILEDTOACCESSFILE} ${0} ${LINENO} ${file_name}
				ret_val=${RET_NG}
			fi
		fi
		;;
	"${FILE_RM}")
		rm -f ${file_name}
		if [ ${?} -ne ${RET_OK} ] ; then
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CMN_FAILEDTODELETEFILE} ${0} ${LINENO} ${file_name}
			ret_val=${RET_NG}
		fi
		;;
	*)
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CMN_ILLEGALPARAM} ${0} ${LINENO} ${command}
		ret_val=${RET_NG}
		;;
esac

##
## ファイルロック解除
##
# ロックのセマフォコマンドIDに対応するセマフォコマンドIDを設定
command_id=`expr ${command_id} + 1`

# セマフォコマンドを実行
${PREX_BIN}/Semaphore ${command_id}

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_CMN_FAILEDTOCTLLOCK} ${0} ${LINENO}
	ret_val=${RET_NG}
fi

exit ${ret_val}
