/*****************************************************************************
   Copyright (C) 2006 NTT
  *****************************************************************************/
/**
 * @file    GetCheckPointLSN.c
 * @brief   チェックポイントLSN取得に関する関数の定義
 *
 * チェックポイントLSNの取得を行なう際に使用する関数を定義する。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "PREXDefine.h"
#include "PREXMessage.h"
#include "GetCheckPointLSN.h"

/**
 * @fn      int GetCheckPointLSN(LSNInfo_t* pCheckPointLSN, char** argv)
 * @brief   チェックポイントLSN取得
 *
 *   pg_controlよりチェックポイントLSNを取得する。
 *
 *   詳細は以下のとおり。
 *     -# パラメータチェックを実施する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NG を返却する。
 *     -# リモートホスト名の指定がある場合、sshで接続し、リモートホストのpg_controlから
 *       「Latest checkpoint location」の値を取得する。指定がない場合は、ローカルのpg_controlから
 *       「Latest checkpoint location」の値を取得する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NG を返却する。
 *     -# 取得した文字列を「/」区切りで分割する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NG を返却する。
 *     -# 分割した文字列を数値に変換しLSNのxlogidとxrecoffとして設定する
 *     -# 呼出元に RET_OKを返却する。
 *
 * @attention ローカル値を取得する際は、第2引数にNULLを指定すること。
 * @param[out] pCheckPointLSN   チェックポイントLSN格納用のアドレス
 * @param[in]  argv             リモート接続用情報
 * @retval  RET_OK    正常終了
 * @retval  RET_NG    異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int GetCheckPointLSN(LSNInfo_t* pCheckPointLSN, char** argv)
{
	char sCommand[CMN_COMMANDMAX]; /* コマンド文字列作成用 */
	char sCheckpoint[XLOGFILEID_STR_LEN + SEGMENT_NO_STR_LEN + 2];
	char sXlogid[XLOGFILEID_STR_LEN + 1];
	char sXrecoff[SEGMENT_NO_STR_LEN + 1];
	char* pTmp;
	uint32 nXlogid;
	uint32 nXrecoff;
	FILE *fp;

	/* 引数チェック */
	if (pCheckPointLSN == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, MSGWD_CHECKPOINTLSN);
		return RET_NG;
	}
	
	/* pg_controlから値を取得 (コマンドを実行し標準出力に出力されたものを取得する) */
	if (argv == NULL) {
		/* リモート接続用情報指定なしの場合、ローカル値取得 */
		sprintf(sCommand, CMN_CMD_GETCHECKPOINTLSN);
	} else {
		/* リモート接続用情報指定ありの場合、リモート値取得 */
		sprintf(sCommand, CMN_CMDFMT_GETREMOTECHECKPOINTLSN, argv[ARGINDEX_USER], argv[ARGINDEX_HOST], CMN_CMD_GETCHECKPOINTLSN);
	}
	
	if ((fp=popen(sCommand,"r"))==NULL) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOGET, MSGWD_CHECKPOINTLSN);
		return RET_NG;
	}
	fgets(sCheckpoint, XLOGFILEID_STR_LEN + SEGMENT_NO_STR_LEN + 2, fp);
	if (feof(fp)) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOGET, MSGWD_CHECKPOINTLSN);
		pclose(fp);
		return RET_NG;
	}
	pclose(fp);
	
	/* 取得した文字列を「/」区切りで分割する */
	pTmp = strchr(sCheckpoint, '/');
	if (pTmp==NULL) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOGET, MSGWD_CHECKPOINTLSN);
		return RET_NG;
	} 
	/* 「/」の右側がxrecoff */
	strcpy(sXrecoff, pTmp+1);
	/* 「/」の左側がxlogid */
	*pTmp = '\0';	
	strcpy(sXlogid, sCheckpoint);
	
	/* 取得した結果を16進から10進に変換 */
	nXlogid = (uint32)strtoll(sXlogid, NULL, 16);
	nXrecoff = (uint32)strtoll(sXrecoff, NULL, 16);

	/* LSNに設定 */
	SetLSNInfo(pCheckPointLSN, nXlogid, nXrecoff);
	
	return RET_OK;
}
