/*****************************************************************************
   Copyright (C) 2006 NTT
  *****************************************************************************/
/**
 * @file    GetCheckPointLSNFromBackupLabel.c
 * @brief   backup_labelからのチェックポイントLSN取得に関する関数の定義
 *
 * backup_labelからのチェックポイントLSNの取得を行なう際に使用する関数を定義する。
 *
 * @version 1.0.0
 * @date    10/27/2006
 * @since   1.0
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "PREXDefine.h"
#include "PREXMessage.h"
#include "WWCommon.h"
#include "GetCheckPointLSNFromBackupLabel.h"

/**
 * @fn      int GetCheckPointLSNFromBackupLabel(LSNInfo_t* pCheckPointLSN)
 * @brief   backup_labelからのチェックポイントLSN取得
 *
 *   backup_labelよりチェックポイントLSNを取得する。
 *
 *   詳細は以下のとおり。
 *     -# パラメータチェックを実施する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NG を返却する。
 *     -# ローカルの$sbybackupdir/backup_labelが存在するか確認する。
 *        エラー発生時は、syslogにワーニングメッセージを出力し、呼出元に RET_NG を返却する。
 *     -# ローカルの$sbybackupdir/backup_labelから「START WAL LOCATION:」の値を取得する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NG を返却する。
 *     -# 取得した文字列を「/」区切りで分割する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NG を返却する。
 *     -# 分割した文字列を数値に変換しLSNのxlogidとxrecoffとして設定する。
 *     -# 呼出元に RET_OKを返却する。
 *
 * @attention 特に無し
 * @param[out] pCheckPointLSN   チェックポイントLSN格納用のアドレス
 * @retval  RET_OK    正常終了
 * @retval  RET_NG    異常終了
 *
 * @version 1.0.0
 * @date    10/27/2006
 * @since   1.0
*/
int GetCheckPointLSNFromBackupLabel(LSNInfo_t* pCheckPointLSN)
{
	char sCommand[CMN_COMMANDMAX_GETCHECKPOINTLSNFROMBACKUPLABEL]; /* コマンド文字列作成用 */
	char sPath[PREX_MAX_PATH_LEN]; /* backup_labelパス文字列作成用 */
	char sCheckpoint[XLOGFILEID_STR_LEN + SEGMENT_NO_STR_LEN + 2];
	char sXlogid[XLOGFILEID_STR_LEN + 1];
	char sXrecoff[SEGMENT_NO_STR_LEN + 1];
	char* pTmp;
	uint32 nXlogid;
	uint32 nXrecoff;
	FILE *fp;

	/* 引数チェック */
	if (pCheckPointLSN == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, MSGWD_CHECKPOINTLSN);
		return RET_NG;
	}
	
	/* $sbybackupdir/backup_labelがあるかどうか確認 */
	/* $sbybackupdirの値は、グローバル変数 g_WWEnvInfo.sbybackupdir から取得 */
	sprintf(sPath, CMN_PATH_BACKUPLABEL, g_WWEnvInfo.sbybackupdir);
	if (access(sPath, F_OK) == -1) {
		OutputSyslog(LOG_WARNING, MSGID_CMN_FAILEDTOGET, MSGWD_CHECKPOINTLSN);
		return RET_NG;
	}

	/* $sbybackupdir/backup_labelから値を取得 (コマンドを実行し標準出力に出力されたものを取得する) */
	sprintf(sCommand, CMN_CMD_GETCHECKPOINTLSNFROMBACKUPLABEL, sPath);
	
	if ((fp=popen(sCommand,"r"))==NULL) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOGET, MSGWD_CHECKPOINTLSN);
		return RET_NG;
	}
	fgets(sCheckpoint, XLOGFILEID_STR_LEN + SEGMENT_NO_STR_LEN + 2, fp);
	if (feof(fp)) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOGET, MSGWD_CHECKPOINTLSN);
		pclose(fp);
		return RET_NG;
	}
	pclose(fp);
	
	/* 取得した文字列を「/」区切りで分割する */
	pTmp = strchr(sCheckpoint, '/');
	if (pTmp==NULL) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOGET, MSGWD_CHECKPOINTLSN);
		return RET_NG;
	} 
	/* 「/」の右側がxrecoff */
	strcpy(sXrecoff, pTmp+1);
	/* 「/」の左側がxlogid */
	*pTmp = '\0';	
	strcpy(sXlogid, sCheckpoint);
	
	/* 取得した結果を16進から10進に変換 */
	nXlogid = (uint32)strtoll(sXlogid, NULL, 16);
	nXrecoff = (uint32)strtoll(sXrecoff, NULL, 16);

	/* LSNに設定 */
	SetLSNInfo(pCheckPointLSN, nXlogid, nXrecoff);
	
	return RET_OK;
}
