/*****************************************************************************
   Copyright (C) 2007 NTT
  *****************************************************************************/
/**
 * @file    LastWAL.c
 * @brief   終端LSN取得コマンド (最終WALセグメント経由)
 *
 * 最終WALセグメントを特定し、最終WALセグメントより、終端LSNを取得する。
 *
 * 最終WALセグメントは、WALセグメント格納ディレクトリ配下に存在する
 * WALセグメントファイル名より特定し、最終WALセグメントより、終端LSNの
 * 取得を行なう。
 *
 * WALセグメント格納ディレクトリ配下にWALセグメントファイルが存在しない場合、
 * アーカイブログ格納ディレクトリ配下の最新WALセグメント+1の先頭を示すLSNを
 * 終端LSNとして返却する。
 *
 * @version 1.0.0
 * @date    08/07/2007
 * @since   1.0
 */
 
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>

#include "postgres.h"
#include "access/xlog_internal.h"
#include "utils/elog.h"

#include "PREXMessage.h"
#include "PREXDefine.h"

#define FILE_NAME_MAX_LEN (256 + 1)

#define PAGEHEADER_NONE	2 // ページヘッダが存在しない
#define PAGEHEADER_INVA	3 // ページヘッダの情報が不正

static int filter_walfile(const struct dirent*);
static XLogRecPtr* getXLogRecPtr(const char*);
static int getLastLSN(const char* filename, const char* waldir, XLogRecPtr* lastLSN);
static int nextRecIsExists(char* buf, FILE* fp, const char* filename, XLogRecPtr* lastLSN);

extern int LastLSN_main(int argc, char *argv[], XLogRecPtr *lastLSN);

#define freeEntries(entries__, num__) \
{ \
    int counter; \
    if((entries__) != NULL) { \
        for(counter = 0; counter < (num__); counter++) { \
            if((entries__)[counter] != NULL) { \
                free((entries__)[counter]); \
            } \
        } \
        free((entries__)); \
    } \
}



/**
 * @fn      int LastWAL_main(int argc, char *argv[])
 * @brief   LSN取得(最終WALセグメント)・メイン処理
 *
 * 最終WALセグメントを特定し、最終WALセグメントより、終端LSNを取得する。
 *
 *  詳細は以下の通り。
 *   -# コマンドパラメータのチェックを行う。以下に示すパラメータ指定は、正常なパラメータとみなす。
 *     - 第1～第2パラメータが指定されている、第3パラメータ以降は指定されていない。
 *   -# 正常なパラメータとみなせない場合には、エラーとして、syslogにエラーメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *   -# WALセグメント格納ディレクトリ配下のファイル一覧をファイル名でソートした状態で取得する。
 *     - WALセグメント格納ディレクトリ配下にファイルが存在する場合、4の処理を行なう。
 *     - WALセグメント格納ディレクトリ配下にファイルが存在しない場合、8の処理を行なう。
 *   -# ファイル一覧の順に5～7の処理を行なう。
 *   -# WALセグメントファイルのページヘッダの有無および正当性を確認する。ページヘッダが存在しない、または不正な場合、次のファイルを処理する。
 *   -# ページヘッダが正常に存在する場合、当該WALセグメントファイルより、終端LSNを取得する。
 *   -# 取得した終端LSNを「ログID/オフセット」の形式で返却(標準出力に出力)する。
 *   -# ファイル一覧の全ファイルにページヘッダが存在しない場合は、の処理を行なう。
 *   -# アーカイブログ格納ディレクトリ配下の最新WALセグメントファイル名(ファイル名ソートした先頭)を取得する。
 *   -# 最新WALセグメントのファイル名より、ログID(ファイル名の9文字目から16文字目まで)およびセグメントID(ファイル名の17文字目から末尾まで)を取得する。
 *   -# 次のWALセグメント(セグメントID+1)からオフセットを算出(オフセット = (セグメントID * 0x01000000)し、当該WALセグメントの先頭を示すLSNを取得する。
 *   -# 取得したLSNより、終端LSNを追跡し、取得した終端LSNを「ログID/オフセット」の形式で返却(標準出力に出力)する。
 *
 * @param[in]   argc      引数の数
 * @param[in]   argv[0]   コマンド名
 * @param[in]   argv[1]   WALセグメント格納ディレクトリ名
 * @param[in]   argv[2]   アーカイブログ格納ディレクトリ名
 * @param[out]  lastLSN   LSN文字列 (出力形式:「logid/offset」)
 * @retval      RET_OK(0) 正常終了
 * @retval      RET_NG(1) 異常終了
 *
 * @version 1.0.0
 * @date    08/07/2007
 * @since   1.0
 */
int main(int argc, char* argv[])
{
	/*
	 * 変数宣言
	 */
	char* waldir;                      /* WALセグメント格納ディレクトリ名  */
	char* archivedir;                  /* アーカイブログ格納ディレクトリ名 */
	char filename[FILE_NAME_MAX_LEN];  /* 最終WALセグメント名              */
	XLogRecPtr* startLSN;              /* 最終WALセグメントの先頭LSN       */
	XLogRecPtr lastLSN;                /* 終端LSN                          */
	int ret;                           /* 諸関数の戻り値                   */
	int flag = 0;                      /* 終端LSN取得済みフラグ            */
	
	/*
	 * パラメータチェック
	 */
	if ( argc != 3 ) {
		/* 引数の数が不正 */
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, MSGWD_NUMOFPARAM);
		return RET_NG;
	}
	
	waldir = argv[1];
	archivedir = argv[2];
	startLSN = NULL;
	struct dirent** entries = NULL;
	int num = 0;
	memset(filename, '\0', FILE_NAME_MAX_LEN);
	
	/*
	 * WALセグメントファイルの一覧をファイル名でソートした状態で取得
	 */
	if ( (num = scandir(waldir, &entries, filter_walfile, alphasort)) < 0 ) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOACCESSDIR, waldir, strerror(errno));
		return RET_NG;
	}
	
	/*
	 * WALセグメント格納ディレクトリ配下にファイルが存在する場合、以下の処理を行なう
	 * ・ファイル一覧の順に、ページヘッダの存在有無を確認する。存在しない(無効)場合は次のファイルを処理
	 * ・ページヘッダが存在する場合、当該ファイルより終端LSNを取得する
	 */
	if ( num > 0 ) {
		/* ファイルが存在する場合 */
		int i;
		for ( i = (num - 1) ; i >= 0 ; i-- ) {
			/* 終端LSNを取得 */
			ret = getLastLSN(entries[i]->d_name, waldir, &lastLSN);
			
			switch (ret) {
				case RET_OK:
					/* 終端LSNの取得に成功  */
					/* ループ終了条件を設定 */
					i = 0;
					flag = 1;
					break;
				
				case PAGEHEADER_NONE:
				case PAGEHEADER_INVA:
					/* ページヘッダが存在しない(無効)なWALセグメントの場合は、次のファイルを処理 */
					continue;
				
				default:
					/* 異常終了 */
					freeEntries(entries, num);
				    OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDLASTLSN, MSGWD_INVALIDHEADER);
					return RET_NG;
			}
		}
		
		freeEntries(entries, num);
	}
	
	/*
	 * WAL格納ディレクトリ配下にファイルが存在しない、またはWALセグメント格納ディレクトリに
	 * 該当ファイル(ページヘッダが存在するファイル)が存在しない場合、以下の処理を行なう
	 * ・アーカイブログ格納ディレクトリ配下の最新WALセグメントファイル名を取得
	 * ・ファイル名より、ログIDおよびセグメントIDを取得
	 * ・セグメントIDを「1」を加算し、セグメントIDとログIDより、当該セグメントの先頭LSNを取得
	 */
	if ( num == 0 || flag == 0 ) {
		/* アーカイブログ格納ディレクトリ配下のファイル一覧を取得 */
		if ( (num = scandir(archivedir, &entries, filter_walfile, alphasort)) < 0 ) {
    		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOACCESSDIR, archivedir, strerror(errno));
			return RET_NG;
		}
		
		if(num == 0) {
			OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDLASTLSN, MSGWD_NOWALSEGMENT);
            freeEntries(entries, num);
			return RET_NG;
        }
		
		/* 最新WALセグメントファイル名を取得 */
		strcpy(filename, entries[num - 1]->d_name);
		freeEntries(entries, num);
		
		/*
		 * 最終WALセグメントのファイル名より、当該セグメントの先頭LSNを取得
		 */
		if ( (startLSN = getXLogRecPtr(filename)) == NULL ) {
			/* LSN取得に失敗 */
			OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDLASTLSN, strerror(errno));
			return RET_NG;
		}
		
		/* 次のWALセグメントを示すLSNに設定 */
		if ( startLSN->xrecoff >= (0xFE *  (uint32) 0x01000000) ) {
			/* セグメントIDが繰り上がる場合、               */
			/* ログIDを「1」加算し、オフセットを「0」に設定 */
			lastLSN.xlogid  = startLSN->xlogid + 1;
			lastLSN.xrecoff = SizeOfXLogLongPHD;
		} else {
			// 次のWALセグメントの先頭LSNに設定 (1セグメント分(16M)のオフセットを加算)
			lastLSN.xlogid  = startLSN->xlogid;
			lastLSN.xrecoff = startLSN->xrecoff + (uint32) 0x01000000 + SizeOfXLogLongPHD;
		}
	}
	
	/*
	 * 終端LSNを標準出力に出力
	 */
	fprintf(stdout, "%X/%X", lastLSN.xlogid, lastLSN.xrecoff);
	
	free(startLSN);
	
	return RET_OK;
}

/**
 * @fn      static XLogRecPtr* getXLogRecPtr(const char*)
 * @brief   WALセグメントファイル名より、当該WALセグメントの先頭LSNを取得する。
 *
 *  詳細は以下の通り。
 *   -# ファイル名より、ログIDおよびセグメントIDを取得する。
 *   -# ログIDおよびセグメントIDを16進->10進に変換する。
 *   -# セグメントIDよりオフセットを取得する。
 *   -# ログIDおよびオフセットより、LSNを構築し返却する。
 *
 * @param[in]   filename    WALセグメントファイル名
 * @retval      XLogRecPtr* 当該WALセグメントのLSN
 * @retval      NULL        異常終了
 * @pre     filenameの長さは事前に24文字であることが確認済みである
 *
 * @version 1.0.0
 * @date    08/07/2007
 * @since   1.0
 */
static XLogRecPtr* getXLogRecPtr(const char* filename) {
	
	/*
	 * 変数宣言
	 */
	XLogRecPtr* lsn = NULL; // LSN
	uint32 logid = 0;       // ログID
	uint32 offset = 0;      // オフセット
	uint32 segid = 0;       // セグメントID
	
	char _logid[9];     // 作業用領域
	char _segid[9];     // 作業用領域
	memset(_logid, '\0', 9);
	memset(_segid, '\0', 9);
	
	/* ファイル名より、ログIDおよびセグメントIDを取得        */
	/* ・ログID = ファイル名の9文字目から16文字目まで        */
	/* ・セグメントID = ファイル名の17文字目から24文字目まで */
	strncpy(_logid, filename + 8, 8);
	strncpy(_segid, filename + 16, 8);
	
	/* 16進->10進変換 */
	logid = (uint32) strtoll(_logid, NULL, 16);
	segid = (uint32) strtoll(_segid, NULL, 16);
	
	// セグメントIDより当該セグメントの先頭LSN(オフセット)を算出
	offset = (uint32) (segid * 0x01000000);
	
	/* LSN格納領域を確保 */
	lsn = malloc(sizeof(XLogRecPtr));
	if(lsn == NULL) {
        return NULL;
    }
	
	lsn->xlogid = logid;
	lsn->xrecoff = offset;
	
	return lsn;
}

/**
 * @fn      static int getLastLSN(const char* filename, const char* waldir, XLogRecPtr* lastLSN)
 * @brief   WALセグメントファイルのページヘッダの種別を取得する。
 *
 *  詳細は以下の通り。
 *   -# ファイルサイズがXLogSegSize(16777216[byte])であることを確認する。
 *   -# 先頭のページヘッダを取得し、マジックナンバがXLOG_PAGE_MAGIC(0xD05E)と一致することを確認する。マジックナンバが一致しない場合、ページヘッダではないと判断し、戻り値にPAGEHEADER_NONEを返却する。
 *   -# ページヘッダが存在する場合、ページヘッダの情報より、セグメントファイル名(タイムラインID、ログID、セグメントID)を取得する。
 *   -# 3で取得したセグメントファイル名が現在対象となっているWALセグメントファイル名と同一であることを確認する。同一でなかった場合、戻り値にPAGEHEADER_INVAを返却する。
 *   -# 終端LSNを取得する。
 *
 * @param[in]  filename           WALセグメントファイル名
 * @param[in]  waldir             WALセグメント格納ディレクトリ名
 * @param[in]  lastLSN            終端LSN格納領域
 * @retval     RET_OK(0)          終端LSNの取得に成功
 * @retval     RET_NG(1)          終端LSNの取得に失敗
 * @retval     PAGEHEADER_NONE(2) ページヘッダが存在しない
 * @retval     PAGEHEADER_INVA(3) ページヘッダの情報が不正
 *
 * @version 1.0.0
 * @date    08/08/2007
 * @since   1.0
 */
static int getLastLSN(const char* filename, const char* waldir, XLogRecPtr* lastLSN) {
	/*
	 * 変数宣言
	 */
	struct stat sb;
	FILE* fp = NULL;
	char buf[XLOG_BLCKSZ];
	char targetfile[FILE_NAME_MAX_LEN + 1];
	XLogPageHeader pageHeaderTop;
	
	char argv0[64];
	char argv1[64];
	char argv2[64];
	char* ArgvForLastLSN[3] = {argv0, argv1, argv2};
	
	/*
	 * ファイル名を作成する
	 */
	snprintf(targetfile, FILE_NAME_MAX_LEN, "%s/%s", waldir, filename);
	
	/*
	 * ファイルサイズをチェック
	 */
	if ( stat(targetfile, &sb) == -1 ) {
		// 当該ファイルが存在しない
		OutputSyslog(LOG_ERR, MSGID_CMN_NOFILEEXISTS, targetfile);
		return RET_NG;
	}
	
	if ( sb.st_size != XLogSegSize ) {
		// ファイルサイズ不一致
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALFILE, targetfile, MSGWD_INVALIDSIZE);
		return RET_NG;
	}
	
	if ( (fp = fopen(targetfile, "rb")) == NULL ) {
		// ファイルオープンに失敗
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOOPENFILE, targetfile);
		return RET_NG;
	}
	
	/* 先頭ページを読み込む */
	if ( fread(buf, XLOG_BLCKSZ, 1, fp) < 1 ) {
		// ファイル読み込みに失敗
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOREADFILE, targetfile);
		fclose(fp);
		return RET_NG;
	}
	
	/* 先頭ページのページヘッダを取得 */
	pageHeaderTop = (XLogPageHeader) buf;
	
	/* ページヘッダのマジックナンバをチェック */
	if ( pageHeaderTop->xlp_magic != XLOG_PAGE_MAGIC ) {
		// マジックナンバが一致しない
		fclose(fp);
		return PAGEHEADER_NONE;
	}
	
	uint32 pageHeaderSize = XLogPageHeaderSize(pageHeaderTop);
	uint32 logid = ((XLogRecPtr) pageHeaderTop->xlp_pageaddr).xlogid;
	uint32 offset = ((XLogRecPtr) pageHeaderTop->xlp_pageaddr).xrecoff;
	uint32 segid = offset / XLogSegSize;
	
	/* ページヘッダの情報より、WALセグメントファイル名を取得 */
	char walfilename[24 + 1];
	sprintf(walfilename, "%08X%08X%08X", pageHeaderTop->xlp_tli, logid, segid);
	
	/* 取得したWALセグメントファイル名が現在対象となっている */
	/* WALセグメントファイルと同一であるかを確認             */
	if ( strcmp(filename, walfilename) != 0 ) {
		fclose(fp);
		return PAGEHEADER_INVA;
	}
	
	/* 終端LSNを取得 */
	if ( pageHeaderTop->xlp_info & XLP_FIRST_IS_CONTRECORD ) {
		// ページヘッダ(継続)の場合
		int ret = nextRecIsExists(buf, fp, targetfile, lastLSN);
		fclose(fp);
		return ret;
	}
	
	fclose(fp);
	
	// ページヘッダ(通常)の場合
	/* ・先頭レコードヘッダのデータサイズが0の場合      */
	/*   先頭レコードのLSNを終端LSNに設定する           */
	/* ・先頭レコードヘッダのデータサイズが0以外の場合  */
	/*   先頭レコード位置のLSNより、終端LSN追跡を行なう */
	XLogRecord* recordHeader = (XLogRecord*) (buf + pageHeaderSize);
	
	if ( recordHeader->xl_len == 0 ) {
		// データサイズが0の場合
		lastLSN->xlogid = logid;
		lastLSN->xrecoff = offset + pageHeaderSize;
	} else {
		// データサイズが0以外の場合、終端LSNを追跡
		argv0[0] = '\0';
		sprintf(argv1, "%u", logid);
		sprintf(argv2, "%u", (offset + pageHeaderSize));
		
		LastLSN_main(3, ArgvForLastLSN, lastLSN);
	}
	
	return RET_OK;
}

/**
 * @fn      static int nextRecIsExists(char* pageData, FILE* fp, const char* filename, XLogRecPtr* lastLSN)
 * @brief   継続レコード以降を解析し、終端LSNを取得する。
 *
 *  詳細は以下の通り。
 *   -# バッファより、ページヘッダおよびレコードヘッダを取得する。
 *   -# レコードヘッダより、残りのデータサイズを取得する。残りデータサイズより、レコードが次のページに継続されるかを判定する。
 *   -# レコードが次のページに継続する場合、ファイルポインタより、1ページ分読み込みバッファに格納し、当関数の再起呼び出しを行なう。
 *   -# レコードが現在のページ内に収まる場合、終端LSNを取得する。
 *
 * @param[in]   pageData     ページデータ
 * @param[in]   fp           ファイルポインタ
 * @param[in]   filename     WALセグメントファイル名
 * @param[in]   lastLSN      終端LSN格納領域
 * @retval      RET_OK(0)    正常終了
 * @retval      RET_NG(1)    異常終了
 *
 * @version 1.0.0
 * @date    08/07/2007
 * @since   1.0
 */
static int nextRecIsExists(char* pageData, FILE* fp, const char* filename, XLogRecPtr* lastLSN) {
	
	XLogPageHeader pageHeader;        // ページヘッダ
	uint32 pageHeaderSize;            // ページヘッダサイズ
	XLogContRecord* recordHeaderCont; // レコードヘッダ(継続)
	XLogRecord* recordHeaderNext;     // 次のレコードヘッダ
	XLogRecPtr currLSN;               // 現在のページの先頭LSN
	XLogRecPtr nextLSN;               // 次のページの先頭LSN
	
	char argv0[64];
	char argv1[64];
	char argv2[64];
	char* ArgvForLastLSN[3] = {argv0, argv1, argv2};
	
	/* ページヘッダ、ページヘッダサイズ、レコードヘッダ(継続)、LSNを取得 */
	pageHeader = (XLogPageHeader) pageData;
	pageHeaderSize = XLogPageHeaderSize(pageHeader);
	recordHeaderCont = (XLogContRecord*) (pageData + pageHeaderSize);
	currLSN = pageHeader->xlp_pageaddr;
	
	/* レコードが現在のページで終了しない場合                                    */
	/* ・レコードデータ長 > 未使用ページ領域                                     */
	/*   次のページを読み込み、再起呼び出しにより終端LSNを取得                   */
	/* ・レコードデータ長 > (未使用ページ領域 - レコードヘッダ(通常)サイズ)      */
	/*   次のページを読み込み、ページヘッダの存在(正当性)を確認し、終端LSNを取得 */
	uint32 rem_len = recordHeaderCont->xl_rem_len;
	
	if ( rem_len > (XLOG_BLCKSZ - pageHeaderSize - SizeOfXLogContRecord - SizeOfXLogRecord) ) {
		/* 次のページを読み込む */
		if ( fread(pageData, XLOG_BLCKSZ, 1, fp) < 1 ) {
			// ファイル読み込みに失敗
			OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOREADFILE, filename);
			return RET_NG;
		}
		
		/* 次のページのページヘッダ、LSNを取得 */
		XLogPageHeader pageHeaderNext = (XLogPageHeader) pageData;
		nextLSN = pageHeaderNext->xlp_pageaddr;
		
		/* レコードデータ長 > 未使用ページ領域の場合 */
		/* 再起呼び出しにより、終端LSNを取得         */
		if ( rem_len > (XLOG_BLCKSZ - pageHeaderSize - SizeOfXLogContRecord) ) {
			return nextRecIsExists(pageData, fp, filename, lastLSN);
		}
		
		/* レコードデータ長 > (未使用ページ領域 - レコードヘッダ(通常)サイズ)の場合 */
		/* ・次のページのページヘッダが存在しない or ダミーデータの場合             */
		/*   ページヘッダの先頭レコードのLSNを終端LSNに設定                         */
		/* ・次のページのページヘッダが存在する場合                                 */
		/*   ページヘッダの先頭レコード位置から終端LSNを追跡                        */
		if ( pageHeaderNext->xlp_magic != XLOG_PAGE_MAGIC || XLByteLE(nextLSN, currLSN) ) {
			// 次のページのページヘッダが存在しない
			lastLSN->xlogid = currLSN.xlogid;
			lastLSN->xrecoff = (currLSN.xrecoff + XLOG_BLCKSZ + SizeOfXLogLongPHD);
		} else {
			// 次のページのページヘッダが存在する場合
			argv0[0] = '\0';
			sprintf(argv1, "%u", nextLSN.xlogid);
			sprintf(argv2, "%u", (uint32) (nextLSN.xrecoff + SizeOfXLogLongPHD));
			
			LastLSN_main(3, ArgvForLastLSN, lastLSN);
		}
		return RET_OK;
	}
	
	/* レコードが現在のページで終了する場合、次のレコードヘッダの有無を確認 */
	recordHeaderNext = (XLogRecord*) (pageData + pageHeaderSize + MAXALIGN(rem_len + SizeOfXLogContRecord));
	
	if ( recordHeaderNext->xl_len == 0 ) {
		lastLSN->xlogid = currLSN.xlogid;
 		lastLSN->xrecoff = (currLSN.xrecoff + (pageHeaderSize + MAXALIGN(rem_len + SizeOfXLogContRecord)));
	} else {
		argv0[0] = '\0';
		sprintf(argv1, "%u", currLSN.xlogid);
		sprintf(argv2, "%u", (uint32) (currLSN.xrecoff + (pageHeaderSize + MAXALIGN(rem_len + SizeOfXLogContRecord))));
		
		LastLSN_main(3, ArgvForLastLSN, lastLSN);
	}
	
	return RET_OK;
}

/**
 * @fn      static int filter_walfile(const struct dirent* entry)
 * @brief   WALセグメントファイルを抽出するためのフィルタリング
 *
 *  詳細は以下の通り。
 *   -# エントリ種別が「ファイル」であることを確認する。
 *   -# ファイル名が24文字であることを確認する。
 *
 * @param[in]   entry     エントリ
 * @retval      TRUE(1)   WALセグメントファイルである
 * @retval      FALSE(0)  WALセグメントファイルではない
 *
 * @version 1.0.0
 * @date    08/07/2007
 * @since   1.0
 */
static int filter_walfile(const struct dirent* entry) {
	/*
	 * 以下の条件に一致することを確認
	 * ・ファイル種別が「ファイル」であること
	 * ・ファイル名の文字列長が24文字であること
	 */
	if ( entry->d_type == DT_REG && strlen(entry->d_name) == 24 ) {
		return TRUE;
	}
	return FALSE;
}
