/*****************************************************************************
   Copyright (C) 2007 NTT
  *****************************************************************************/
/**
 * @file    PREXUtil.c
 * @brief   WALSender/WALWriter共通で利用するユーティリティ
 *
 * WALSender/WALWriter共通で利用するユーティリティのインプリメンテーションを行う。
 *
 * @version 1.0.0
 * @date    09/21/2007
 * @since   1.1
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <errno.h>
#include <sys/stat.h>

#include "PREXUtil.h"


/**
 * @fn      ssize_t recvallWithRetry(int fd, void *vptr, size_t n, int flags)
 * @brief   全データの受信処理
 *
 * 送信されたデータを全て受信する。受信に失敗した場合、そのエラー原因が
 * EINTERもしくはERESTARTSYSの場合は、再度受信処理を行う。
 * EAGAINの場合は、リトライ回数分、再度受信処理を行う。
 * 
 *
 * @param[in]   fd            ファイルディスクリプタ
 * @param[out]  vptr          受信したデータの格納領域
 * @param[in]   n             受信するデータのサイズ
 * @param[in]   flags         受信オプション
 * @param[in]   retry         EAGAINが返却された場合のrecvリトライ回数
 * @retval      ret           受信したバイト数。エラーの場合は-1、接続先が正しくシャットダウンした
 *                            場合は0を返却する。
 *
 * @version 1.1.0
 * @date    08/09/2007
 * @since   1.0
 */
ssize_t recvallWithRetry(int fd, void *vptr, size_t n, int flags, int retry) {
    size_t  nleft;       /* 受信残バイト数       */
    ssize_t nread;       /* 受信済みバイト数     */
    char    *ptr;        /* 受信領域へのポインタ */
    int     counter;     /* 現在のリトライ回数   */

    counter = 0;
    ptr = vptr;
    nleft = n;
    while(nleft > 0) {
        if((nread = recv(fd, ptr, nleft, flags)) < 0) {
            if(errno == EINTR || errno == 512) { /* 512 = ERESTARTSYS */
                nread = 0;
            } else if(errno == EAGAIN) {
                if(retry != 0) {
                    if(counter >= retry) {
                        return -1;
                    }
                    counter++;
                }
                nread = 0;
            } else {
                return -1;
            }
        } else if(nread == 0) {
            /* 全データを受信する前にシャットダウンを検知した */
            /* 場合は、0を返却する                            */
            if(nleft > 0) {
                nleft = n;
            }
            break;
        }
        
        nleft -= nread;
        ptr   += nread;
    }
    
    return (n - nleft);
}



/**
 * @fn      ssize_t recvall(int fd, void *vptr, size_t n, int flags)
 * @brief   全データの受信処理
 *
 * 送信されたデータを全て受信する。受信に失敗した場合、そのエラー原因が
 * EINTERもしくはERESTARTSYSの場合は、再度受信処理を行う。
 *
 * @param[in]   fd            ファイルディスクリプタ
 * @param[out]  vptr          受信したデータの格納領域
 * @param[in]   n             受信するデータのサイズ
 * @param[in]   flags         受信オプション
 * @retval      ret           受信したバイト数。エラーの場合は-1、接続先が正しくシャットダウンした
 *                            場合は0を返却する。
 *
 * @version 1.1.0
 * @date    08/09/2007
 * @since   1.0
 */
ssize_t recvall(int fd, void *vptr, size_t n, int flags) {
    return recvallWithRetry(fd, vptr, n, flags, 0);
}



/**
 * @fn      ssize_t sendall(int fd, const void *vptr, size_t n, int flags)
 * @brief   全データの送信処理
 *
 * 送信バッファに指定されたデータを全て送信する。
 *
 * @param[in]   fd            ファイルディスクリプタ
 * @param[out]  vptr          送信バッファ
 * @param[in]   n             送信するデータのサイズ
 * @param[in]   flags         送信オプション
 * @retval      ret           送信したバイト数。エラーの場合は-1を返却する。
 *
 * @note    sendコールの仕様としては、非ブロッキングモードでない限り、指定された
 *          データをすべて送信するまでブロックするが、システム状態によってはすべ
 *          てのデータを送信する前にブロックがとかれるため、本関数で確実にデータ
 *          を送信する必要がある。
 *
 * @version 1.1.0
 * @date    09/20/2007
 * @since   1.0
 */
ssize_t sendall(int fd, const void *vptr, size_t n, int flags) {
    size_t     nleft;        /* 送信残のバイト数         */
    ssize_t    nwritten;     /* 送信済みバイト数         */
    const char *ptr;         /* 送信バッファへのポインタ */
    
    ptr = vptr;
    nleft = n;
    while (nleft > 0) {
        if( (nwritten = send(fd, ptr, nleft, flags)) <= 0) {
            if(errno == EINTR) {
                nwritten = 0;
            } else {
                return -1;
            }
        }
        nleft -= nwritten;
        ptr += nwritten;
    }
    return n;
}

