/*****************************************************************************
   Copyright (C) 2006-2007 NTT
  *****************************************************************************/
/**
 * @file    ReadWWConfFile.c
 * @brief   WALWriter用設定ファイルの読み込みに関する関数の定義
 *
 * WALWriter用設定ファイルの読み込みを行なう際に使用する関数を定義する。
 *
 * @version 1.1.0
 * @date    10/30/2007
 * @since   1.0
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "PREXDefine.h"
#include "PREXMessage.h"
#include "ReadWWConfFile.h"

WWSettingItem_t WW_ENVITEM_LIST[] = {    ///< 設定項目情報リスト
	{WW_ITEMID_WS_HOST,                  WW_ITEMTYPE_CHAR,         WW_ITEM_WS_HOST},
	{WW_ITEMID_PREX_PORT,                WW_ITEMTYPE_NUM_UNSIGNED, WW_ITEM_PREX_PORT},
	{WW_ITEMID_WS_POLLING_INTERVAL,      WW_ITEMTYPE_NUM_UNSIGNED, WW_ITEM_WS_POLLING_INTERVAL},
	{WW_ITEMID_WW_BUFFERS,               WW_ITEMTYPE_NUM_UNSIGNED, WW_ITEM_WW_BUFFERS},
	{WW_ITEMID_PREX_MSGKEY,              WW_ITEMTYPE_NUM_UNSIGNED, WW_ITEM_PREX_MSGKEY},
	{WW_ITEMID_ACTWALHOST,               WW_ITEMTYPE_CHAR,         WW_ITEM_ACTWALHOST},
	{WW_ITEMID_ACTARCHIVEHOST,           WW_ITEMTYPE_CHAR,         WW_ITEM_ACTARCHIVEHOST},
	{WW_ITEMID_ACTBACKUPHOST,            WW_ITEMTYPE_CHAR,         WW_ITEM_ACTBACKUPHOST},
	{WW_ITEMID_ACTWALUSER,               WW_ITEMTYPE_CHAR,         WW_ITEM_ACTWALUSER},
	{WW_ITEMID_ACTARCHIVEUSER,           WW_ITEMTYPE_CHAR,         WW_ITEM_ACTARCHIVEUSER},
	{WW_ITEMID_ACTBACKUPUSER,            WW_ITEMTYPE_CHAR,         WW_ITEM_ACTBACKUPUSER},
	{WW_ITEMID_ACTWALDIR,                WW_ITEMTYPE_CHAR,         WW_ITEM_ACTWALDIR},
	{WW_ITEMID_ACTARCHIVEDIR,            WW_ITEMTYPE_CHAR,         WW_ITEM_ACTARCHIVEDIR},
	{WW_ITEMID_ACTBACKUPDIR,             WW_ITEMTYPE_CHAR,         WW_ITEM_ACTBACKUPDIR},
	{WW_ITEMID_SBYWALDIR,                WW_ITEMTYPE_CHAR,         WW_ITEM_SBYWALDIR},
	{WW_ITEMID_SBYARCHIVEDIR,            WW_ITEMTYPE_CHAR,         WW_ITEM_SBYARCHIVEDIR},
	{WW_ITEMID_SBYBACKUPDIR,             WW_ITEMTYPE_CHAR,         WW_ITEM_SBYBACKUPDIR},
	{WW_ITEMID_PREX_VIRTUAL_IP,          WW_ITEMTYPE_CHAR,         WW_ITEM_PREX_VIRTUAL_IP},
	{WW_ITEMID_ACTHAHOST,                WW_ITEMTYPE_CHAR,         WW_ITEM_ACTHAHOST},
	{WW_ITEMID_KEEPALIVES_IDLE,          WW_ITEMTYPE_NUM_UNSIGNED, WW_ITEM_KEEPALIVES_IDLE},
	{WW_ITEMID_KEEPALIVES_INTERVAL,      WW_ITEMTYPE_NUM_UNSIGNED, WW_ITEM_KEEPALIVES_INTERVAL},
	{WW_ITEMID_KEEPALIVES_COUNT,         WW_ITEMTYPE_NUM_UNSIGNED, WW_ITEM_KEEPALIVES_COUNT},
	{WW_ITEMID_ENABLETOACTRETRYCOUNT,    WW_ITEMTYPE_NUM_UNSIGNED, WW_ITEM_ENABLETOACTRETRYCOUNT},
	{WW_ITEMID_ENABLETOACTRETRYINTERVAL, WW_ITEMTYPE_NUM_UNSIGNED, WW_ITEM_ENABLETOACTRETRYINTERVAL},
	{WW_ITEMID_INVALID,                  WW_ITEMTYPE_INVALID,      NULL}
};

/**
 * @fn      int ReadConfigFile(char* pPath, WWSettingInfo_t* pEnvInfo)
 * @brief   WALWriter用設定ファイルからの設定項目値取得
 *
 *   指定された設定ファイルの各行を読み込み、設定ファイル項目情報に値を設定する。
 *
 *   詳細は以下のとおり。
 *     -# 引数 pPath が NULL、または pEnvInfo が NULL の場合は、syslog
 *        にエラーメッセージを出力し、呼出元に RET_NG を返却する。
 *     -# 指定されたファイルをオープンする。エラー発生時は、syslog
 *        にエラーメッセージを出力し、呼出元に RET_NG を返却する。
 *     -# 指定されたファイルの終端に到達するまで、4 ～ 6 を繰り返す。
 *     -# 設定ファイルから１行を読み込む。エラー発生時は、syslogにエラーメッセージ
 *        を出力し、呼出元に RET_NG を返却する。
 *     -# 読み込んだ行を４種類（空行、コメント行、設定行、それ以外）に分類する
 *        （GetConfLineType 関数）。分類した結果、設定行以外の場合は、4 に戻り
 *        処理を続行する。
 *     -# 設定行から、該当する設定項目の値を取得し、設定ファイル項目情報に設定する
 *        （SetEnvInfoFromLine 関数）。エラー発生時は、何もせず 4 に戻り処理を続行する。
 *     -# 指定されたファイルをクローズする。
 *     -# 設定ファイル項目情報のすべてのメンバ変数をチェックする
 *        （CheckConfigurationStatus 関数）。エラー発生時は、呼出元に RET_NG を返却する。
 *
 * @attention  設定ファイルにおける記述形式について、以下に注意事項を示す。
 *             - 行頭に '#' が存在する場合は、コメント行とみなす。
 *             - １行に '=' が複数存在する場合はエラーとする。
 *             - 行において '=' より左側に長さ１以上の文字列が存在しない場合は
 *               エラーとする。
 *             - 行において '=' より左側に空白文字以外の文字が存在しない場合は
 *               エラーとする。
 *             - 行頭及び行末の連続した空白文字は、すべて無視する。
 *             - '=' の前後の連続した空白文字は、すべて無視する。
 *
 * @param[in]  pPath      WALWriter用設定ファイルのパス
 * @param[out] pEnvInfo   設定ファイル項目情報のアドレス
 *
 * @retval  RET_OK    正常終了
 * @retval  RET_NG    異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int ReadConfigFile(char* pPath, WWSettingInfo_t* pEnvInfo)
{
	int         rcode;
	char*       ccode;
	LineType_t  ltype;
	FILE*       fpWWConf;
	char        bufRead[READ_FILE_SIZE];

	/* 引数の検査 */
	if ((pPath == NULL) || (pEnvInfo == NULL)) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, INTERNAL_PARAMETERS);
		return RET_NG;
	}

	/* WALWriter用設定ファイルのオープン */
	fpWWConf = fopen(pPath, "r");
	if (fpWWConf == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOOPENFILE, pPath);
		return RET_NG;
	}

	/* 行読み込み */
	ccode = fgets(bufRead, READ_FILE_SIZE, fpWWConf);
	while (ccode != NULL) {
		ltype = GetConfLineType(bufRead);
		switch (ltype) {
			case LINETYPE_EMPTY:    /* 空行       */
			case LINETYPE_COMMENT:  /* コメント行 */
				break;
			case LINETYPE_CONF:     /* 設定行 */
				/* 設定項目の取得 */
				rcode = SetEnvInfoFromLine(pEnvInfo, bufRead);
				if (rcode != RET_OK) {
					fclose(fpWWConf);
					return RET_NG;
				}
				break;
			default:                /* その他の行 */
				break;
		};

		/* 行読み込み*/
		ccode = fgets(bufRead, READ_FILE_SIZE, fpWWConf);
	}

	/* 設定ファイルの読込みに失敗した場合 */
	if (feof(fpWWConf) == 0) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOREADFILE, pPath);
		fclose(fpWWConf);
		return RET_NG;
	}

	/* 設定ファイルのクローズ */
	fclose(fpWWConf);

	/* 設定ファイル項目情報の各項目値の検査 */
	rcode = CheckConfigurationStatus(pEnvInfo);
	if (rcode != RET_OK) {
		return RET_NG;
	}

	return RET_OK;
}

/**
 * @fn      LineType_t GetConfLineType(char* bufLine)
 * @brief   設定ファイル行の種類判別
 *
 *   指定された文字列（設定ファイルから読み込んだ行）の種類を判別する。
 *
 *   詳細は以下のとおり。
 *     -# 指定された文字列が、空行（改行文字のみ）であるかどうかを判定する。
 *        空行である場合、呼出元に LINETYPE_EMPTY を返却する。
 *     -# 指定された文字列が、コメント行（行頭が "#"）であるかどうかを判定する。
 *        コメント行である場合、呼出元に LINETYPE_COMMENT を返却する。
 *     -# 指定された文字列が、設定行（2文字目以降に "=" を含む）であるかどうかを判定する。
 *        設定行である場合、呼出元に LINETYPE_CONF を返却する。
 *     -# 上記以外と判断した場合、呼出元に LINETYPE_ERROR を返却する。
 *
 * @param[in]   bufLine   判別対象の行文字列
 *
 * @retval      LINETYPE_EMPTY    空行
 * @retval      LINETYPE_COMMENT  コメント行
 * @retval      LINETYPE_CONF     設定行
 * @retval      LINETYPE_OTHER    その他の行
 * @retval      LINETYPE_ERROR    エラー
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
LineType_t GetConfLineType(char* bufLine)
{
	char*  posFirst;
	char*  posLast;

	if (bufLine == NULL) {
		return LINETYPE_ERROR;
	}

	/* 空行かどうかを判定 */
	if (*bufLine == CHAR_RETURN) {
		return LINETYPE_EMPTY;
	}

	/* コメント行かどうかを判定 */
	if (*bufLine == CHAR_SHARP) {
		return LINETYPE_COMMENT;
	}

	/* 設定行かどうかを判定 */
	posFirst = strchr(bufLine, CHAR_EQUAL);
	if ((posFirst != NULL) && (posFirst != bufLine)) {
		posLast  = strrchr(bufLine, CHAR_EQUAL);
		if (posFirst == posLast) {
			return LINETYPE_CONF;
		}
	}

	return LINETYPE_OTHER;
}

/**
 * @fn      int SetEnvInfoFromLine(WWSettingInfo_t* pEnvInfo, char* bufLine)
 * @brief   設定行からの値取得
 *
 *   指定された設定行の記述に従い、設定項目に値を設定する。
 *
 *   詳細は以下のとおり。
 *   -# 指定された文字列において '=' より前の文字列を切り出し、前後の空白文字を削除する。
 *   -# 1 で切り出した文字列に該当する設定項目名が存在するかどうかを確認する。
 *      該当する設定項目が存在する場合は 3～4 を実行する。存在しない場合は、
 *      何もせず呼出元に RET_OK を返却する。
 *   -# 指定された文字列において '=' より後の文字列を切り出し、前後の空白文字を削除して、
 *      該当する設定項目の値域に関するチェックを行う。値域の範囲外の場合は、syslog
 *      にエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *      値域チェックに問題がなければ、その設定値を設定ファイル情報に設定する。
 *      この時、設定値が文字列のものは、格納用の領域をメモリ確保する。メモリ確保に
 *      失敗した場合は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *   -# 3 で切り出した値を設定値として、設定ファイル項目情報の該当する設定項目に設定する。
 *
 * @pre     bufLine には１つの '=' が含まれていること。
 * @post    bufLine の内容は更新される。
 *
 * @note 以下に、設定ファイルから取得する設定値及び設定値の制限について示す。
 *     - ws_host：WALSenderが稼動するホストのホスト名またはIPアドレス。最大128バイト。
 *       未設定時はエラー。
 *     - prex_port：WALSender接続ポート番号。未設定時はエラー。
 *     - ws_polling_interval：WALSenderへの接続要求ポーリング間隔（秒）。正数値。
 *       未設定時は 1 を適用。
 *     - ww_buffers：WALWriter用WALバッファにおけるブロック数。8192以上の整数値。
 *       未設定時は 8192 を適用。
 *     - prex_msgkey：使用するメッセージキューのキー値。他のIPCリソースで未使用の正数値。
 *       未設定時は 7890 を適用。
 *     - actwalhost：転送対象WALセグメント格納ディレクトリを持つACT側ホストのホスト名
 *       または IPアドレス。最大128バイト。未設定時はエラー。
 *     - actarchivehost：転送対象アーカイブログ格納ディレクトリを持つACT側ホストのホスト名
 *       または IPアドレス。最大128バイト。未設定時はエラー。
 *     - actbackuphost：転送対象ベースバックアップ格納ディレクトリを持つACT側ホストの
 *       ホスト名または IPアドレス。最大128バイト。未設定時はエラー。
 *     - actwaluser：actwalhostで指定したホストへのログインユーザ名。最大64バイト。
 *       未設定時はエラー。
 *     - actarchiveuser：actarchivehostで指定したホストへのログインユーザ名。
 *       最大64バイト。未設定時はエラー。
 *     - actbackupuser：actbackuphostで指定したホストへのログインユーザ名。最大64バイト。
 *       未設定時はエラー。
 *     - actwaldir：ACT側WALセグメント格納ディレクトリ。最大256バイト。
 *       未設定時は /usr/local/pgsql/data/pg_xlog を適用。
 *     - actarchivedir：ACT側アーカイブログ格納ディレクトリ。最大256バイト。
 *       未設定時はエラー。
 *     - actbackupdir：ACT側ベースバックアップ格納ディレクトリ。最大256バイト。
 *       未設定時はエラー。
 *     - sbywaldir：SBY側WALセグメント格納ディレクトリ。最大256バイト。
 *       未設定時は /usr/local/pgsql/data/pg_xlog を適用。
 *     - sbyarchivedir：SBY側アーカイブログ格納ディレクトリ。最大256バイト。
 *       未設定時はエラー。
 *     - sbybackupdir：SBY側ベースバックアップ格納ディレクトリ。最大256バイト。
 *       未設定時はエラー。
 *     - prex_virtual_ip：バーチャルホスト用のIPアドレスまたはホスト名。最大256バイト。
 *       未設定時はエラー。
 *     - acthahost：ACT側ホストの高可用化ソフトウェア用通信ノードの
 *       ホスト名または IPアドレス。最大128バイト。未設定時はエラー。
 *     - keepalives_idle：WALSenderとのTCP通信における通信断の確認を開始するまでの間隔（秒）。正数値。
 *       未設定時は 0 を適用。
 *     - keepalives_interval：WALSenderとのTCP通信において、通信断の確認を行う間隔（秒）。正数値。
 *       未設定時は 0 を適用。
 *     - keepalives_count：WALSenderとのTCP通信において、通信断の確認を行う回数。正数値。
 *       未設定時は 0 を適用。
 *     - enabletoactretrycount：WALWriterで通信異常を検知した時のACT化可否チェックの試行回数。正数値。
 *       未設定時は 0 を適用。
 *     - enabletoactretryinterval：WALWriterで通信異常を検知した時のACT化可否チェックの試行間隔。正数値。
 *       未設定時は 0 を適用。
 *
 * @param[out]   pEnvInfo   設定ファイル項目情報のアドレス
 * @param[in]    bufLine    設定項目行
 *
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.1.0
 * @date    10/30/2007
 * @since   1.0
*/
int SetEnvInfoFromLine(WWSettingInfo_t* pEnvInfo, char* bufLine)
{
	int    rcode;
	char*  pItemName;
	char*  pItemValue;
	char*  posEqual;
	char*  posItemNameEnd;
	char*  posItemValueEnd;
	int    nItemPos;
	int    nItemID=0;
	int    nItemType=0;
	int    nItemValue=0;

	/* パラメータチェック */
	if (bufLine == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, INTERNAL_PARAMETERS);
		return RET_NG;
	}

	/* bufLine に '=' が含まれているかをチェック */
	posEqual = strchr(bufLine, CHAR_EQUAL);
	if (posEqual == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, INTERNAL_PARAMETERS);
		return RET_NG;
	}

	/* 初期化 */
	pItemName  = bufLine;
	pItemValue = posEqual + 1;
	posItemValueEnd = bufLine + strlen(bufLine);
	posItemNameEnd  = posEqual;

	/* 設定項目名（左辺文字列）の取得 */
	/*   - 手前の空白文字を無視する */
	while ((*pItemName == CHAR_SPACE) || (*pItemName == CHAR_TAB)) {
		pItemName++;
	}

	/*   - 後方の空白文字を '\0' で上書きする */
	*posItemNameEnd = CHAR_NUL;
	while ((posItemNameEnd > pItemName) &&
		((*(posItemNameEnd-1) == CHAR_SPACE) || (*(posItemNameEnd-1) == CHAR_TAB))) {
		*(posItemNameEnd-1) = CHAR_NUL;
		posItemNameEnd--;
	}

	if (pItemName == posItemNameEnd) { /* 左辺が無い場合 */
		return RET_OK;
	}

	/* 該当する設定項目名の有無の検査 */
	rcode = -1;
	nItemPos = 0;
	while (WW_ENVITEM_LIST[nItemPos].nID != WW_ITEMID_INVALID) {
		rcode = strcmp(WW_ENVITEM_LIST[nItemPos].pName, pItemName);
		if (rcode == 0) {
			nItemID   = WW_ENVITEM_LIST[nItemPos].nID;
			nItemType = WW_ENVITEM_LIST[nItemPos].nType;
			break;
		}
		nItemPos++;
	}

	if (rcode != 0) {  /* 該当設定項目が無い場合 */
		return RET_OK;
	}

	/* 設定値（右辺文字列）の取得 */
	/*   - 手前の空白文字を無視する */
	while ((*pItemValue == CHAR_SPACE) || (*pItemValue == CHAR_TAB)) {
		pItemValue++;
	}

	/*   - 後方の空白文字を '\0' で上書きする */
	while ((posItemValueEnd > pItemValue) &&
		  ((*(posItemValueEnd-1) == CHAR_SPACE) ||
		   (*(posItemValueEnd-1) == CHAR_TAB)   ||
		   (*(posItemValueEnd-1) == CHAR_RETURN))) {
		*(posItemValueEnd-1) = CHAR_NUL;
		posItemValueEnd--;
	}

	if (pItemValue == posItemValueEnd) { /* 右辺が無い場合 */
		return RET_OK;
	}

	/* 設定項目種別が数値の場合 */
	if (nItemType == WW_ITEMTYPE_NUM_UNSIGNED) {
		nItemValue = atoi(pItemValue);
	}

	/* 設定項目別の値域検査 */
	switch (nItemID) {
		case WW_ITEMID_WS_HOST:                  /* ws_host     */
			SetStringEnvItem(&rcode, &(pEnvInfo->ws_host), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_HOSTNAME_LEN, WW_ITEM_WS_HOST);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_PREX_PORT:                  /* prex_port     */
			SetIntegerEnvItemGE(&rcode, &(pEnvInfo->prex_port), nItemValue,
					MIN_WW_POSNUM_ITEMVAL, WW_ITEM_PREX_PORT);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_WS_POLLING_INTERVAL:        /* ws_polling_interval     */
			SetIntegerEnvItemGE(&rcode, &(pEnvInfo->ws_polling_interval), nItemValue,
					MIN_WW_POSNUM_ITEMVAL, WW_ITEM_WS_POLLING_INTERVAL);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_WW_BUFFERS:                  /* ww_buffers     */
			SetIntegerEnvItemGE(&rcode, &(pEnvInfo->ww_buffers), nItemValue,
					MIN_WW_BUFFERS, WW_ITEM_WW_BUFFERS);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_PREX_MSGKEY:                 /* prex_msgkey     */
			SetIntegerEnvItemGE(&rcode, &(pEnvInfo->prex_msgkey), nItemValue,
					MIN_WW_UINT_ITEMVAL, WW_ITEM_PREX_MSGKEY);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_ACTWALHOST:                 /* actwalhost     */
			SetStringEnvItem(&rcode, &(pEnvInfo->actwalhost), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_HOSTNAME_LEN, WW_ITEM_ACTWALHOST);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_ACTARCHIVEHOST:                 /* actarchivehost     */
			SetStringEnvItem(&rcode, &(pEnvInfo->actarchivehost), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_HOSTNAME_LEN, WW_ITEM_ACTARCHIVEHOST);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_ACTBACKUPHOST:                 /* actbackuphost     */
			SetStringEnvItem(&rcode, &(pEnvInfo->actbackuphost), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_HOSTNAME_LEN, WW_ITEM_ACTBACKUPHOST);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_ACTWALUSER:                 /* actwaluser     */
			SetStringEnvItem(&rcode, &(pEnvInfo->actwaluser), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_USERNAME_LEN, WW_ITEM_ACTWALUSER);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_ACTARCHIVEUSER:                 /* actarchiveuser     */
			SetStringEnvItem(&rcode, &(pEnvInfo->actarchiveuser), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_USERNAME_LEN, WW_ITEM_ACTARCHIVEUSER);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_ACTBACKUPUSER:                 /* actbackupuser     */
			SetStringEnvItem(&rcode, &(pEnvInfo->actbackupuser), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_USERNAME_LEN, WW_ITEM_ACTBACKUPUSER);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_ACTWALDIR:                 /* actwaldir     */
			SetStringEnvItem(&rcode, &(pEnvInfo->actwaldir), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_PATH_LEN, WW_ITEM_ACTWALDIR);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_ACTARCHIVEDIR:                 /* actarchivedir     */
			SetStringEnvItem(&rcode, &(pEnvInfo->actarchivedir), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_PATH_LEN, WW_ITEM_ACTARCHIVEDIR);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_ACTBACKUPDIR:                 /* actbackupdir     */
			SetStringEnvItem(&rcode, &(pEnvInfo->actbackupdir), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_PATH_LEN, WW_ITEM_ACTBACKUPDIR);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_SBYWALDIR:                 /* sbywaldir     */
			SetStringEnvItem(&rcode, &(pEnvInfo->sbywaldir), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_PATH_LEN, WW_ITEM_SBYWALDIR);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_SBYARCHIVEDIR:                 /* sbyarchivedir     */
			SetStringEnvItem(&rcode, &(pEnvInfo->sbyarchivedir), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_PATH_LEN, WW_ITEM_SBYARCHIVEDIR);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_SBYBACKUPDIR:                 /* sbybackupdir     */
			SetStringEnvItem(&rcode, &(pEnvInfo->sbybackupdir), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_PATH_LEN, WW_ITEM_SBYBACKUPDIR);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_PREX_VIRTUAL_IP:                 /* prex_virtual_ip     */
			SetStringEnvItem(&rcode, &(pEnvInfo->prex_virtual_ip), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_IP_LEN, WW_ITEM_PREX_VIRTUAL_IP);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_ACTHAHOST:                    /* acthahost     */
			SetStringEnvItem(&rcode, &(pEnvInfo->acthahost), pItemValue,
					MIN_WW_ITEMVAL_LEN, MAX_WW_HOSTNAME_LEN, WW_ITEM_ACTHAHOST);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;

		case WW_ITEMID_KEEPALIVES_IDLE:             /* keepalives_idle        */
			SetIntegerEnvItemGE(&rcode, &(pEnvInfo->keepalives_idle), nItemValue,
					MIN_WW_UINT_ITEMVAL, WW_ITEM_KEEPALIVES_IDLE);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;

		case WW_ITEMID_KEEPALIVES_INTERVAL:         /* keepalives_interval    */
			SetIntegerEnvItemGE(&rcode, &(pEnvInfo->keepalives_interval), nItemValue,
					MIN_WW_UINT_ITEMVAL, WW_ITEM_KEEPALIVES_INTERVAL);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;

		case WW_ITEMID_KEEPALIVES_COUNT:             /* keepalives_count        */
			SetIntegerEnvItemGE(&rcode, &(pEnvInfo->keepalives_count), nItemValue,
					MIN_WW_UINT_ITEMVAL, WW_ITEM_KEEPALIVES_COUNT);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_ENABLETOACTRETRYCOUNT:        /* enabletoactretrycount   */
			SetIntegerEnvItemGE(&rcode, &(pEnvInfo->enabletoactretrycount), nItemValue,
					MIN_WW_UINT_ITEMVAL, WW_ITEM_ENABLETOACTRETRYCOUNT);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;
		case WW_ITEMID_ENABLETOACTRETRYINTERVAL:     /* enabletoactretryinterval */
			SetIntegerEnvItemGE(&rcode, &(pEnvInfo->enabletoactretryinterval), nItemValue,
					MIN_WW_UINT_ITEMVAL, WW_ITEM_ENABLETOACTRETRYINTERVAL);
			if (rcode != RET_OK) {
				return RET_NG;
			}
			break;


	}

	return RET_OK;
}

/**
 * @fn      int CheckConfigurationStatus(WWSettingInfo_t* pEnvInfo)
 * @brief   設定ファイル項目情報の値設定状況検査
 *
 *   指定された設定ファイル項目情報における値設定状況を検査する。
 *
 *   詳細は以下のとおり。
 *     -# 指定された設定ファイル項目情報における各項目について、値が未設定
 *        （項目種別が WW_ITEMTYPE_CHAR の場合は DEF_CHAR_VALUE、
 *        WW_ITEMTYPE_NUM_UNSIGNED の場合は DEF_UNSIGNED_NUM_VALUE）のものが
 *        存在する場合は、syslog にエラーメッセージを出力し、呼出元に
 *        RET_NG を返却する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @param[in]   pEnvInfo   設定ファイル項目情報のアドレス
 *
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.1.0
 * @date    08/02/2007
 * @since   1.0
*/
int CheckConfigurationStatus(WWSettingInfo_t* pEnvInfo)
{
	if (pEnvInfo->ws_host == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_WS_HOST);
		return RET_NG;
	}
	if (pEnvInfo->prex_port == DEF_UNSIGNED_NUM_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_PREX_PORT);
		return RET_NG;
	}
	if (pEnvInfo->ws_polling_interval == DEF_UNSIGNED_NUM_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_WS_POLLING_INTERVAL);
		return RET_NG;
	}
	if (pEnvInfo->ww_buffers == DEF_UNSIGNED_NUM_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_WW_BUFFERS);
		return RET_NG;
	}
	if (pEnvInfo->prex_msgkey == DEF_UNSIGNED_NUM_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_PREX_MSGKEY);
		return RET_NG;
	}
	if (pEnvInfo->actwalhost == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_ACTWALHOST);
		return RET_NG;
	}
	if (pEnvInfo->actarchivehost == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_ACTARCHIVEHOST);
		return RET_NG;
	}
	if (pEnvInfo->actbackuphost == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_ACTBACKUPHOST);
		return RET_NG;
	}
	if (pEnvInfo->actwaluser == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_ACTWALUSER);
		return RET_NG;
	}
	if (pEnvInfo->actarchiveuser == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_ACTARCHIVEUSER);
		return RET_NG;
	}
	if (pEnvInfo->actbackupuser == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_ACTBACKUPUSER);
		return RET_NG;
	}
	if (pEnvInfo->actwaldir == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_ACTWALDIR);
		return RET_NG;
	}
	if (pEnvInfo->actarchivedir == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_ACTARCHIVEDIR);
		return RET_NG;
	}
	if (pEnvInfo->actbackupdir == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_ACTBACKUPDIR);
		return RET_NG;
	}
	if (pEnvInfo->sbywaldir == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_SBYWALDIR);
		return RET_NG;
	}
	if (pEnvInfo->sbyarchivedir == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_SBYARCHIVEDIR);
		return RET_NG;
	}
	if (pEnvInfo->sbybackupdir == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_SBYBACKUPDIR);
		return RET_NG;
	}
	if (pEnvInfo->prex_virtual_ip == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_PREX_VIRTUAL_IP);
		return RET_NG;
	}
	if (pEnvInfo->acthahost == DEF_CHAR_VALUE) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, WW_ITEM_ACTHAHOST);
		return RET_NG;
	}

	return RET_OK;
}

/**
 * @fn      int InitSettingInfo(WWSettingInfo_t* pEnvInfo)
 * @brief   設定ファイル項目情報の初期化
 *
 *   指定された設定ファイル項目情報がの各項目の値を初期化する。
 *
 *   詳細は以下のとおり。
 *     -# 指定された設定ファイル項目情報における各項目について、初期値が
 *        既定されているものには初期値を設定する。その他の項目については、
 *        文字列項目は DEF_CHAR_VALUE、数値項目は DEF_SIGNED_NUM_VALUE を
 *        設定する。
 *
 * @param[in]   pEnvInfo   設定ファイル項目情報のアドレス
 *
 * @retval  RET_OK  正常終了
 * @retval  RET_NG  異常終了
 *
 * @version 1.1.0
 * @date    10/30/2007
 * @since   1.0
*/
int InitSettingInfo(WWSettingInfo_t* pEnvInfo)
{
	pEnvInfo->ws_host                  = DEF_CHAR_VALUE;
	pEnvInfo->prex_port                = DEF_SIGNED_NUM_VALUE;
	pEnvInfo->ws_polling_interval      = DEF_WS_POLLING_INTERVAL;
	pEnvInfo->ww_buffers               = DEF_WW_BUFFERS;
	pEnvInfo->prex_msgkey              = DEF_PREX_MSGKEY;
	pEnvInfo->actwalhost               = DEF_CHAR_VALUE;
	pEnvInfo->actarchivehost           = DEF_CHAR_VALUE;
	pEnvInfo->actbackuphost            = DEF_CHAR_VALUE;
	pEnvInfo->actwaluser               = DEF_CHAR_VALUE;
	pEnvInfo->actarchiveuser           = DEF_CHAR_VALUE;
	pEnvInfo->actbackupuser            = DEF_CHAR_VALUE;

	pEnvInfo->actarchivedir            = DEF_CHAR_VALUE;
	pEnvInfo->actbackupdir             = DEF_CHAR_VALUE;

	pEnvInfo->sbyarchivedir            = DEF_CHAR_VALUE;
	pEnvInfo->sbybackupdir             = DEF_CHAR_VALUE;
	pEnvInfo->prex_virtual_ip          = DEF_CHAR_VALUE;
	pEnvInfo->acthahost                = DEF_CHAR_VALUE;
	pEnvInfo->keepalives_idle          = DEF_SIGNED_NUM_VALUE;
	pEnvInfo->keepalives_interval      = DEF_SIGNED_NUM_VALUE;
	pEnvInfo->keepalives_count         = DEF_SIGNED_NUM_VALUE;
	pEnvInfo->enabletoactretrycount    = DEF_ENABLETOACTRETRYCOUNT;
	pEnvInfo->enabletoactretryinterval = DEF_ENABLETOACTRETRYINTERVAL;

	/* actwaldir の初期値を登録 */
	pEnvInfo->actwaldir = strdup(DEF_ACTWALDIR);
	if (pEnvInfo->actwaldir == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOALLOCMEM);
		return RET_NG;
	}

	/* sbywaldir の初期値を登録 */
	pEnvInfo->sbywaldir = strdup(DEF_SBYWALDIR);
	if (pEnvInfo->sbywaldir == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOALLOCMEM);
		free(pEnvInfo->actwaldir);
		pEnvInfo->actwaldir = DEF_CHAR_VALUE;
		return RET_NG;
	}

	return RET_OK;
}

/**
 * @fn      void TermSettingInfo(WWSettingInfo_t* pEnvInfo)
 * @brief   設定ファイル項目情報のクリア
 *
 *   指定された設定ファイル項目情報が使用する領域を解放する。
 *
 *   詳細は以下のとおり。
 *     -# 指定された設定ファイル項目情報における文字列項目について、値が未設定
 *        （DEF_CHAR_VALUE）以外のものは、領域を解放する。
 *
 * @param[in]   pEnvInfo   設定ファイル項目情報のアドレス
 *
 * @return   なし
 *
 * @version 1.1.0
 * @date    08/02/2007
 * @since   1.0
*/
void TermSettingInfo(WWSettingInfo_t* pEnvInfo)
{
	if (pEnvInfo->ws_host != DEF_CHAR_VALUE) {
		free(pEnvInfo->ws_host);
		pEnvInfo->ws_host = DEF_CHAR_VALUE;
	}
	pEnvInfo->prex_port           = DEF_SIGNED_NUM_VALUE;
	pEnvInfo->ws_polling_interval = DEF_SIGNED_NUM_VALUE;
	pEnvInfo->ww_buffers          = DEF_SIGNED_NUM_VALUE;
	pEnvInfo->prex_msgkey         = DEF_SIGNED_NUM_VALUE;
	if (pEnvInfo->actwalhost != DEF_CHAR_VALUE) {
		free(pEnvInfo->actwalhost);
		pEnvInfo->actwalhost = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->actarchivehost != DEF_CHAR_VALUE) {
		free(pEnvInfo->actarchivehost);
		pEnvInfo->actarchivehost = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->actbackuphost != DEF_CHAR_VALUE) {
		free(pEnvInfo->actbackuphost);
		pEnvInfo->actbackuphost = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->actwaluser != DEF_CHAR_VALUE) {
		free(pEnvInfo->actwaluser);
		pEnvInfo->actwaluser = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->actarchiveuser != DEF_CHAR_VALUE) {
		free(pEnvInfo->actarchiveuser);
		pEnvInfo->actarchiveuser = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->actbackupuser != DEF_CHAR_VALUE) {
		free(pEnvInfo->actbackupuser);
		pEnvInfo->actbackupuser = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->actwaldir != DEF_CHAR_VALUE) {
		free(pEnvInfo->actwaldir);
		pEnvInfo->actwaldir = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->actarchivedir != DEF_CHAR_VALUE) {
		free(pEnvInfo->actarchivedir);
		pEnvInfo->actarchivedir = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->actbackupdir != DEF_CHAR_VALUE) {
		free(pEnvInfo->actbackupdir);
		pEnvInfo->actbackupdir = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->sbywaldir != DEF_CHAR_VALUE) {
		free(pEnvInfo->sbywaldir);
		pEnvInfo->sbywaldir = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->sbyarchivedir != DEF_CHAR_VALUE) {
		free(pEnvInfo->sbyarchivedir);
		pEnvInfo->sbyarchivedir = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->sbybackupdir != DEF_CHAR_VALUE) {
		free(pEnvInfo->sbybackupdir);
		pEnvInfo->sbybackupdir = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->prex_virtual_ip != DEF_CHAR_VALUE) {
		free(pEnvInfo->prex_virtual_ip);
		pEnvInfo->prex_virtual_ip = DEF_CHAR_VALUE;
	}
	if (pEnvInfo->acthahost != DEF_CHAR_VALUE) {
		free(pEnvInfo->acthahost);
		pEnvInfo->acthahost = DEF_CHAR_VALUE;
	}

	return;
}

