/*****************************************************************************
   Copyright (C) 2007 NTT
  *****************************************************************************/
/**
 * @file    Semaphore.c
 * @brief   セマフォ管理コマンド
 *
 *セマフォの準備、ロック、ロック解除、解放のコマンドを提供する。
 *
 * @version 1.0.0
 * @date    08/01/2007
 * @since   1.0
 */

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <semaphore.h>

#include "PREXMessage.h"
#include "PREXDefine.h"
#include "Semaphore.h"

/************************************************
 * プロトタイプ宣言
 ***********************************************/
static int smg_open();
static int smg_unlink(void);
static int smg_lock(int command_id);
static int smg_unlock(int command_id);


/************************************************
 * グローバル変数定義
 ***********************************************/
	sem_t *act_sid = NULL;
	sem_t *sby_sid = NULL;
	sem_t *fo_sid = NULL;

/**
 * @fn      int smg_open()
 * @brief   セマフォ生成
 *
 * 各ファイルに対応するセマフォを作成する。
 *
 *   詳細は以下のとおり。
 *     -# セマフォが作成されていない場合は作成して、
 *        セマフォのsidをグローバル変数に格納する。
 *
 * @param[in]   open_mode     動作モードID
 *
 * @version 1.0.0
 * @date    08/01/2007
 * @since   1.0
 */
static int smg_open()
{
	if((act_sid = sem_open(SEM_CREATE_ACT_WORD,O_CREAT,S_IRWXU,SEM_INIT_VALUE)) == SEM_FAILED) {
		/* ACT監視抑止用セマフォの作成失敗 */
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOSMGOPEN, SEM_CREATE_ACT_WORD, strerror(errno));
        /* 後処理 */
		smg_unlink();
		return RET_NG;
	}
	if((sby_sid = sem_open(SEM_CREATE_SBY_WORD,O_CREAT,S_IRWXU,SEM_INIT_VALUE)) == SEM_FAILED) {
		/* SBY監視抑止用セマフォの作成失敗 */
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOSMGOPEN, SEM_CREATE_SBY_WORD, strerror(errno));
        /* 後処理 */
		smg_unlink();
		return RET_NG;
	}
	if((fo_sid = sem_open(SEM_CREATE_FO_WORD,O_CREAT,S_IRWXU,SEM_INIT_VALUE)) == SEM_FAILED) {
		/* フェールオーバ抑止用セマフォの作成失敗 */
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOSMGOPEN, SEM_CREATE_FO_WORD, strerror(errno));
        /* 後処理 */
		smg_unlink();
		return RET_NG;
	}
	return RET_OK;
}

/**
 * @fn      int smg_unlink()
 * @brief   セマフォ解放
 *
 * 各ファイルに対応するセマフォを解放する。
 *
 *   詳細は以下のとおり。
 *     -# 各ファイルに対応するセマフォを解放する。
 *
 * @version 1.0.0
 * @date    08/01/2007
 * @since   1.0
 */
static int smg_unlink(void)
{
	int ret_val = 0;
	
	if(act_sid != NULL) {
		if(sem_close(act_sid) == SEM_ERROR) {
			/* ACT監視抑止用セマフォのクローズ失敗 */
			OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOSMGCLOSE, SEM_CREATE_ACT_WORD, strerror(errno));
			ret_val = 1;
		}
		if(sem_unlink(SEM_CREATE_ACT_WORD) == SEM_ERROR) {
			/* ACT監視抑止用セマフォの削除失敗 */
			OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOSMGUNLINK, SEM_CREATE_ACT_WORD, strerror(errno));
			ret_val = 1;
		}
		else{
			act_sid = NULL;
		}
	}
	
	if(sby_sid != NULL) {
		if(sem_close(sby_sid) == SEM_ERROR) {
			/* SBY監視抑止用セマフォのクローズ失敗 */
			OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOSMGCLOSE, SEM_CREATE_SBY_WORD, strerror(errno));
			ret_val = 1;
		}
		if(sem_unlink(SEM_CREATE_SBY_WORD) == SEM_ERROR) {
			/* ACT監視抑止用セマフォの削除失敗 */
			OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOSMGUNLINK, SEM_CREATE_SBY_WORD, strerror(errno));
			ret_val = 1;
		}
		else{
			sby_sid = NULL;
		}
	}
	
	if(fo_sid != NULL) {
		if(sem_close(fo_sid) == SEM_ERROR) {
			/* フェールオーバ抑止用セマフォのクローズ失敗 */
			OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOSMGCLOSE, SEM_CREATE_FO_WORD, strerror(errno));
			ret_val = 1;
		}
		if(sem_unlink(SEM_CREATE_FO_WORD) == SEM_ERROR) {
			/* ACT監視抑止用セマフォの削除失敗 */
			OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOSMGUNLINK, SEM_CREATE_FO_WORD, strerror(errno));
			ret_val = 1;
		}
		else{
			fo_sid = NULL;
		}
	}
	
	if(ret_val) {
		return RET_NG;
	}
	
	return RET_OK;
}

/**
 * @fn      int smg_lock(int command_id)
 * @brief   セマフォのロック
 *
 * 引数に対応するセマフォをロックする。
 *
 *   詳細は以下のとおり。
 *     -# 引数に対応するセマフォのアドレスを設定する。
 *     -# アドレスに対応するセマフォをロックする。
 *
 * @param[in]   command_id     コマンドID
 *
 * @version 1.0.0
 * @date    08/01/2007
 * @since   1.0
 */
static int smg_lock(int command_id)
{
	sem_t *sid=NULL;
	
	switch (command_id) {
		case SMG_ACT_LOCK_COMMAND:
			sid=act_sid;
			break;
		case SMG_SBY_LOCK_COMMAND:
			sid=sby_sid;
			break;
		case SMG_FO_LOCK_COMMAND:
		    sid=fo_sid;
		    break;
		default:
		    /* sid=NULLのまま */
		    break;
	}
	
	if(sem_wait(sid) == SEM_ERROR) {
		/* セマフォのロック失敗 */
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOSMGLOCK, command_id, strerror(errno));
		return RET_NG;
	}
	return RET_OK;
}

/**
 * @fn      int smg_unlock(int command_id)
 * @brief   セマフォのロック解除
 *
 * 引数に対応するセマフォをロック解除する。
 *
 *   詳細は以下のとおり。
 *     -# 引数に対応するセマフォのアドレスを設定する。
 *     -# アドレスに対応するセマフォをロック解除する。
 *
 * @param[in]   command_id     コマンドID
 *
 * @version 1.0.0
 * @date    08/01/2007
 * @since   1.0
 */
static int smg_unlock(int command_id)
{
	sem_t *sid=NULL;
	
	switch (command_id) {
		case SMG_ACT_UNLOCK_COMMAND:
			sid=act_sid;
			break;
		case SMG_SBY_UNLOCK_COMMAND:
			sid=sby_sid;
			break;
		case SMG_FO_UNLOCK_COMMAND:
		    sid=fo_sid;
		    break;
		default:
		    /* sid=NULLのまま */
		    break;
	}
	
	if(sem_post(sid) == SEM_ERROR) {
		/* セマフォのロック解除失敗 */
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOSMGUNLOCK, command_id, strerror(errno));
		return RET_NG;
	}
	return RET_OK;
}

/**
 * @fn      int Semaphore_main(int argc, char *argv[])
 * @brief   セマフォ管理コマンド・メイン処理
 *
 * セマフォの準備、ロック、ロック解除、解放のコマンドを提供する。
 *
 *   詳細は以下のとおり。
 *     -# コマンドパラメータのチェックを行う。以下に示すパラメータ指定は、正常なパラメータとみなす。
 *        - 第1が指定されている、第2パラメータ以降は指定されていない。
 *     -# 正常なパラメータとみなせない場合には、エラーとして、syslogにエラーメッセージを出力後、戻り値としてRET_NGを返却して終了する。
 *     -# 引数に指定されたコマンドを実行する。
 *        エラーが発生した場合、呼出元に RET_NGを返却する。
 *     -# コマンド完了後、呼出元に RET_OKを返却する。
 *     -# セマフォの準備の場合、セマフォが存在しない場合のみ作成する
 *
 * @param[in]   argc          引数の数
 * @param[in]   argv[0]       コマンドID
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    08/01/2007
 * @since   1.0
 */
int main(int argc, char *argv[])
{
	int command_id = 0;
	
	/* パラメータチェック */
	if (argc != 2) {
		/* 引数の数が不正 */
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, MSGWD_NUMOFPARAM);
		return RET_NG;
	}
	
	command_id = atoi(argv[1]);
	
	switch (command_id) {
		case SMG_OPEN_COMMAND:
			if(smg_open() == RET_NG) {
				return RET_NG;
			}
			break;
		case SMG_UNLINK_COMMAND:
			if(smg_open() == RET_NG) {
				return RET_NG;
			}
			if(smg_unlink() == RET_NG) {
				return RET_NG;
			}
			break;
		case SMG_ACT_LOCK_COMMAND:
		case SMG_SBY_LOCK_COMMAND:
		case SMG_FO_LOCK_COMMAND:
			if(smg_open() == RET_NG) {
				return RET_NG;
			}
			if(smg_lock(command_id) == RET_NG) {
				return RET_NG;
			}
			break;
		case SMG_ACT_UNLOCK_COMMAND:
		case SMG_SBY_UNLOCK_COMMAND:
		case SMG_FO_UNLOCK_COMMAND:
			if(smg_open() == RET_NG) {
				return RET_NG;
			}
			if(smg_unlock(command_id) == RET_NG) {
				return RET_NG;
			}
			break;
		default:
			/* 引数エラー */
			OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, argv[1]);
			return RET_NG;
	}
	
	return RET_OK;
}


