/*****************************************************************************
   Copyright (C) 2006-2007 NTT
  *****************************************************************************/
/**
 * @file    exclude.c
 * @brief   二重起動抑止コマンドのインプリメンテーション
 *
 * 二重起動抑止コマンドのインプリメンテーションを行う。
 *
 * @version 1.1.0
 * @date    08/09/2007
 * @since   1.0
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <unistd.h>
#include <sys/wait.h>

#include "PREXDefine.h"
#include "PREXMessage.h"

#define ACCESS_FLAG O_WRONLY|O_CREAT
#define ACCESS_MODE S_IWUSR|S_IWGRP|S_IWOTH

/**
 * @def     RET_LOCK
 * @brief   戻り値（二重起動）
 *
 * @version 1.0.0
 * @date    08/09/2007
 * @since   1.0
 */
#define RET_LOCK           2

/**
 * @def     RET_ERROR
 * @brief   戻り値（二重起動抑止に失敗）
 *
 * @version 1.0.0
 * @date    08/09/2007
 * @since   1.1
 */
#define RET_ERROR          3



/**
 * @def     FREE_MAC
 * @brief   確保したメモリの解放
 *
 *   確保したメモリを解放する。
 *
 *   詳細は以下のとおり。
 *     -# 確保したメモリを解放する。
 *
 * @return      なし
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define FREE_MAC {\
  if(buf != NULL)\
  {\
    free(buf);\
    buf = NULL;\
  }\
}

/**
 * @fn      int exclude_main(int argc, char *argv[])
 * @brief   二重起動抑止コマンドコマンド・メイン処理
 *
 * 起動するコマンドの二重起動を抑止する。
 *
 *   詳細は以下のとおり。
 *     -# コマンドパラメータのチェックを行う。
 *        エラーが発生した場合、syslogにメッセージを出力後、戻り値としてRET_ERRORを返却して終了する。
 *     -# 二重起動抑止ロックファイルのオープンを行う。
 *        エラーが発生した場合、syslogにメッセージを出力後、戻り値としてRET_ERRORを返却して終了する。
 *     -# 二重起動抑止ロックファイルのアクセス権を変更する。
 *     -# 二重起動抑止ロックファイルのロックを取得する。
 *        他のプロセス等によって既にロックが取得されている場合、ファイルをクローズし、戻り値としてRET_LOCKを返却して終了する。
 *        エラーが発生した場合、syslogにメッセージを出力後、ファイルをクローズし、戻り値としてRET_ERRORを返却して終了する。
 *     -# 実行するコマンド文字列分の領域確保を行う。
 *        エラーが発生した場合、syslogにメッセージを出力後、ファイルのロック解除およびクローズを行い、戻り値としてRET_ERRORを返却して終了する。
 *     -# 実行するコマンド文字列の作成を行う。
 *     -# コマンドの実行を行う。
 *        コマンドの戻り値が-1の場合、確保したメモリの解放、ファイルのロック解除およびクローズを行い、戻り値としてRET_ERRORを返却して終了する。
 *     -# 確保したメモリの解放を行う。
 *     -# ファイルのロック解除を行う。
 *        エラーが発生した場合、syslogにメッセージを出力後、戻り値としてRET_ERRORを返却して終了する。
 *     -# ファイルのクローズを行う。
 *        エラーが発生した場合、syslogにメッセージを出力後、戻り値としてRET_ERRORを返却して終了する。
 *     -# 戻り値として、7で実行したコマンドの戻り値を返却して終了する。
 *
 * @remarks     実際のメイン関数の名称は「main」であるが、ドキュメント生成ツールの仕様で他のソースに
 *              同名の関数があると正しく出力されないため、ここでは変更して記述している。
 * @param[in]   argc          引数の数
 * @param[in]   argv[0]       コマンド名
 * @param[in]   argv[1]       二重起動抑止ロックファイル名
 * @param[in]   argv[2]       実行するコマンド名
 * @param[in]   argv[n]       コマンド実行時のオプション（任意）
 * @retval      ret           コマンドの実行結果(正常終了)
 * @retval      RET_LOCK(2)   二重起動
 * @retval      RET_ERROR(3)  二重起動抑止に失敗
 *
 * @version 1.1.0
 * @date    08/09/2007
 * @since   1.0
 */
int main(int argc, char *argv[])
{

	int fd;
	int len=0;
	int i;
	char *buf = NULL;
	char *lockfile = NULL; /* ロックファイル名 */
	int cmd_ret;  /* コマンドの戻り値 */
	int ret = RET_OK;      /* 戻り値 */

	if (argc < 3) {
		OutputSyslog(LOG_ERR, MSGID_CMN_ILLEGALPARAM, MSGWD_NUMOFPARAM);
		return RET_ERROR;
	}

	/* ロックファイル名を取得する */
	lockfile = argv[1];
	/* 排他制御ファイルのオープン */
	if((fd = open(lockfile, ACCESS_FLAG, S_IWRITE)) <= 0) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOOPENFILE, MSGWD_EXCLUSIVELOCK);
		return RET_ERROR;
	}

	/* アクセス権を変更 */
	fchmod(fd , ACCESS_MODE);

	/* ファイルロック開始 */
	if (flock(fd, LOCK_EX | LOCK_NB) != 0) {
		if(errno == EWOULDBLOCK ){
			/* 既にロックが取得されている */
			close(fd);
			return RET_LOCK;
		}
		
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOCRTLLOCK, MSGWD_EXCLUSIVELOCK);
		/* ファイルのクローズ */
		close(fd);
		return RET_ERROR;
	}

	/* コマンド文字列長を取得します。 */
	for (i=2 ; i < argc ; i++) {
		len += strlen(argv[i]);
		len ++;	/* パラメータ間の空白 */
	}

	/* コマンド文字列の領域を確保する */
	buf = malloc(len + 1);	/* \0分の領域も確保する */
	if (buf == NULL) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOALLOCMEM);
		/* ファイルロック解除 */
		flock(fd, LOCK_UN);
		/* ファイルのクローズ */
		close(fd);
		return RET_ERROR;
	}

	/* コマンド文字列を作成します。 */
	strcpy(buf , "");
	for (i=2 ; i < argc ; i++) {
		strcat(buf , argv[i]);
		strcat(buf , " ");
	}

	/* コマンド実行 */
	cmd_ret = system(buf);

	if (cmd_ret == -1) {
		/* 確保したメモリを解放する。 */
		FREE_MAC;

		/* ファイルロック解除 */
		flock(fd, LOCK_UN);

		/* ファイルのクローズ */
		close(fd);
		return RET_ERROR;
	}
	
	ret = WEXITSTATUS(cmd_ret);

	/* 確保したメモリを解放する。 */
	FREE_MAC;

	/* ファイルロック解除 */
	if (flock(fd, LOCK_UN) != 0) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOCRTLLOCK, MSGWD_EXCLUSIVELOCK);
		ret = RET_ERROR;
	}

	/* ファイルのクローズ */
	if (close(fd) != 0) {
		OutputSyslog(LOG_ERR, MSGID_CMN_FAILEDTOREADFILE, MSGWD_EXCLUSIVELOCK);
		ret = RET_ERROR;
	}
	return ret;
}
