#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2007 NTT
#/**
# * @file    demote.sh
# * @brief   FMS連携機能 - SBY化コマンド(demote)
# *
# * ACT状態のPREXをSBY状態に遷移させる。
# *
# *   詳細は以下のとおり。
# *     -# 設定ファイル(WALWriter.conf)からメッセージキューのキー値を取得する。
# *     -# 状態遷移機能のACT状況確認を呼び出し、PREXの状態がACTであることを確認する。
# *     -# SBY監視抑止ファイルおよびACT監視抑止ファイルを「監視抑止状態」に更新する。
# *     -# 状態遷移機能の強制停止要求を呼び出し、postmasterを強制停止(immediate)する。
# *     -# 状態遷移機能のSBY化要求を呼び出し、メッセージキューの再作成およびpostmaster、WALWriterを起動する。
# *     -# ACT監視抑止ファイルを削除し、SBY監視抑止ファイルを「監視状態」に更新する。
# *
# * @retval      RET_OK(0) SBY化成功
# * @retval      RET_NG(1) SBY化失敗
# * 
# * @version 1.0.0
# * @date    08/06/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

#/**
# * @fn      function reset_monitor_control()
# * @brief   監視抑止を元の状態に復元する。
# *
# * 当コマンドでエラーが発生した場合に呼ばれる。
# * 監視抑止を元の状態に戻す。戻す状態は以下の通り。
# *  - SBY監視：「監視抑止状態」- SBY監視抑止ファイルを「監視抑止状態」に更新
# *  - ACT監視：「監視状態」- ACT監視抑止ファイルを「監視状態」に更新
# *
# *   詳細は以下のとおり。
# *     -# SBY監視抑止ファイルを「監視抑止状態」に更新する。
# *     -# ACT監視抑止ファイルを「監視状態」に更新する。
# *
# * @retval    RET_OK(0) 正常終了
# * @retval    RET_NG(1) 異常終了
# *
# * @version 1.0.0
# * @date    08/02/2007
# * @since   1.0
# */
function reset_monitor_control()
{
	##
	## 監視抑止を設定
	## ・SBY監視：「監視抑止状態」- SBY監視抑止ファイルを「監視抑止状態」に更新
	## ・ACT監視：「監視抑止状態」- ACT監視抑止ファイルを「監視状態」に更新
	##
	${PREX_BIN}/exclusion.sh "${SBY_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_LOCK}" && \
	${PREX_BIN}/exclusion.sh "${ACT_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_UNLOCK}"

	if [ ${?} -ne ${RET_OK} ] ; then
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
		return ${RET_NG}
	fi
	
	return ${RET_OK}
}

[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[start] demote.sh"

##
## 変数宣言
##
msgkey=  # メッセージキューのキー値

##
## 設定ファイル(WALWriter.conf)より、メッセージキューのキー値を取得
##
msgkey="`${PREX_BIN}/getmsgkey.sh`"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORGETMSGKEY} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## 状態遷移機能のACT状況確認を呼び出し、PREXの状態がACTであることを確認
##
${PREX_BIN}/actmonitor.sh ${msgkey} -N

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORSTATENOTMATCH} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## 監視抑止を設定
## ・SBY監視：「監視抑止状態」- SBY監視抑止ファイルを「監視抑止状態」に更新
## ・ACT監視：「監視抑止状態」- ACT監視抑止ファイルを「監視抑止状態」に更新
##
${PREX_BIN}/exclusion.sh "${SBY_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_LOCK}" && \
${PREX_BIN}/exclusion.sh "${ACT_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_LOCK}"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## 状態遷移機能の強制停止要求を実行
## ・WALWriterを停止
## ・postmasterを強制停止(immediate)
## ・メッセージキューを削除
##
${PREX_BIN}/tostop_immediate.sh ${msgkey}

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORSTOPIMMEDIATE} ${0} ${LINENO}
	reset_monitor_control
	exit ${RET_NG}
fi

##
## リカバリ設定ファイル(recovery.conf)のファイル名を更新
##
# recovery.confファイルの存在確認
if [ ! -e ${recoveryconf_file} ]; then
	# recovery.doneファイルの存在確認
	if [ ! -e ${recoverydone_file} ]; then
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORRECOVERYFILENOTEXISTS} ${0} ${LINENO}
		reset_monitor_control
		exit ${RET_NG}
	else
		# recovery.doneが存在する場合、ファイル名をrecovery.confに書き換える
		mv -f ${recoverydone_file} ${recoveryconf_file}
		
		if [ ${?} -ne ${RET_OK} ] ; then
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORRECOVERYFILERENAME} ${0} ${LINENO}
			reset_monitor_control
			exit ${RET_NG}
		fi
	fi
fi

##
## 状態遷移機能のSBY化要求を実行
## ・WALWriterを起動
## ・postmasterを起動
## ・メッセージキューを作成
##
${PREX_BIN}/tosby.sh ${msgkey} -i

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORTOSBY} ${0} ${LINENO}
	reset_monitor_control
	exit ${RET_NG}
fi

##
## 監視抑止を設定
## ・SBY監視：「監視状態」- SBY監視抑止ファイルを「監視状態」に更新
## ・ACT監視：「not runnning」- ACT監視抑止ファイルを削除
##
${PREX_BIN}/exclusion.sh "${SBY_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_UNLOCK}" && \
${PREX_BIN}/exclusion.sh "${ACT_LOCK_COMMANDID}" "${FILE_RM}"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
	exit ${RET_NG}
fi

[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[end]   demote.sh (ret=${RET_OK})"

exit ${RET_OK}
