#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2007 NTT
#/**
# * @file    monitor.sh
# * @brief   FMS連携機能 - 監視コマンド(monitor)
# *
# * PREXの状態を監視する。
# * PREXの監視パターンを以下に示す。
# *  - ACT監視：PREXの状態がACTであることを確認する。
# *  - SBY監視：PREXの状態がSBYであることを確認する。
# * 
# *   詳細は以下のとおり。
# *     -# 設定ファイル(WALWriter.conf)からメッセージキューのキー値を取得する。
# *     -# 監視モードが「act」の場合、以下の処理を行なう。
# *     -# ACT監視抑止ファイルより、ステータスを取得する。
# *       - ステータスが「監視抑止」の場合、戻り値に異常なしを返却する。
# *       - ステータスが「not running」の場合、戻り値に未起動を返却する。
# *       - ステータスが「監視状態」の場合、以降の処理を行なう。
# *     -# 状態監視機能のACT状況確認を実行し、PREXの状態がACTであることを確認する。
# *     -# ACT状況確認の結果、PREXの状態がACT以外であった場合、戻り値に異常ありを返却する。
# *     -# 再度、ACT監視抑止ファイルより、ステータスを取得する。
# *     -# 監視モードが「sby」の場合、以下の処理を行なう。
# *     -# SBY監視抑止ファイルより、ステータスを取得する。
# *       - ステータスが「監視抑止」の場合、戻り値に異常なしを返却する。
# *       - ステータスが「not running」の場合、戻り値に未起動を返却する。
# *       - ステータスが「監視状態」の場合、以降の処理を行なう。
# *     -# 状態監視機能のSBY状況確認を実行し、PREXの状態がSBYであることを確認する。
# *     -# SBY状況確認の結果、PREXの状態がSBY以外であった場合、戻り値に異常ありを返却する。
# *
# * @param[in] mode_monitor       監視モード
# * @retval    OCF_SUCCESS(0)     異常なし
# * @retval    OCF_ERR_GENERIC(1) 異常あり
# * @retval    OCF_NOT_RUNNING(7) 未起動
# * 
# * @version 1.0.0
# * @date    08/06/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

##
## 定数
##
ENABLE_MONITOR=2  # 監視抑止ファイルのステータス(監視状態)
DISABLE_MONITOR=3 # 監視抑止ファイルのステータス(監視抑止状態)
FILE_NOT_EXISTS=4 # 監視抑止ファイルのステータス(未起動)

#/**
# * @fn      function get_status()
# * @brief   監視抑止のステータスを返却する。
# *
# * 指定された監視種別の監視抑止ファイルよりステータスを取得する。
# *
# *   詳細は以下のとおり。
# *     -# 監視抑止ファイルの有無を確認し、存在しない場合は、ステータス(未起動)を返却する。
# *     -# 監視抑止ファイルの内容を取得し、取得した値(ステータス)を返却する。
# *
# * @param[in] mode               監視モード
# * @retval    ENABLE_MONITOR(2)  ステータス(監視状態)
# * @retval    DISABLE_MONITOR(3) ステータス(監視抑止状態)
# * @retval    FILE_NOT_EXISTS(4) ステータス(未起動)
# * @retval    RET_NG(1)          異常終了
# *
# * @version 1.0.0
# * @date    08/06/2007
# * @since   1.0
# */
function get_status()
{
	##
	## 変数宣言
	##
	local mode="${1}" # 監視モード
	local status=     # ステータス
	local ret=        # 戻り値
	
	case "${mode}" in
		'act')
			filename="${FILE_NAME_ACT}"
			lockcid="${ACT_LOCK_COMMANDID}"
			;;
		'sby')
			filename="${FILE_NAME_SBY}"
			lockcid="${SBY_LOCK_COMMANDID}"
			;;
	esac
	
	# 監視抑止ファイルが存在しない場合、戻り値にステータス(未起動)を返却
	[ ! -f "${filename}"  ] && return ${FILE_NOT_EXISTS}
	
	# 監視抑止ファイルよりステータスを取得
	status=`${PREX_BIN}/exclusion.sh "${lockcid}" "${FILE_GET}"`
	ret=${?}
	
	# 監視抑止ファイルが存在しない場合、戻り値にステータス(未起動)を返却
	if [ ${ret} -eq ${RET_NOTEXISTS} ] ; then
		# 監視抑止ファイルが存在しない
		return ${FILE_NOT_EXISTS}
	elif [ ${ret} -eq ${RET_NG} ] ; then
		# 監視抑止ファイルのステータス取得に失敗
		return ${RET_NG}
	fi
	
	# 監視抑止ファイルの内容を確認
	case ${status} in
		${MONITOR_TYPEID_LOCK})
			# ステータスが監視抑止状態の場合
			return ${DISABLE_MONITOR}
			;;
		${MONITOR_TYPEID_UNLOCK})
			# ステータスが監視状態の場合
			return ${ENABLE_MONITOR}
			;;
		*)
			# 規定外の値の場合、戻り値に異常終了を返却
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_INVALIDSTATUS} ${0} ${LINENO} "${mode}" "${status}"
			return ${RET_NG}
			;;
	esac
}

#######################################
## メインルート
##

[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[start] monitor.sh (mode=${1})"

##
## 変数宣言
##
mode_monitor="${1}" # 監視モード
msgkey=             # メッセージキューのキー値
ret=                # 戻り値

##
## 設定ファイル(WALWriter.conf)より、メッセージキューのキー値を取得
##
msgkey="`${PREX_BIN}/getmsgkey.sh`"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORGETMSGKEY} ${0} ${LINENO}
	exit ${OCF_ERR_GENERIC}
fi

##
## 監視抑止ファイルの確認
##
get_status "${mode_monitor}"

case ${?} in
	${FILE_NOT_EXISTS})
		# ステータス(未起動)の場合
		# 戻り値に未起動を返却
		
		[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[end]   monitor.sh (ret=${OCF_NOT_RUNNING})"
		
		exit ${OCF_NOT_RUNNING}
		;;
	${DISABLE_MONITOR})
		# ステータス(監視抑止状態)の場合
		# 戻り値に異常なしを返却
		
		[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[end]   monitor.sh (ret=${OCF_SUCCESS})"
		
		exit ${OCF_SUCCESS}
		;;
	${ENABLE_MONITOR})
		# ステータス(監視状態)の場合
		# ACT/SBY状況確認を実行
		case "${mode_monitor}" in
			'act')
				# 監視モード(ACT)の場合
				${PREX_BIN}/actmonitor.sh "${msgkey}"
			;;
			'sby')
				# 監視モード(SBY)の場合
				${PREX_BIN}/sbymonitor.sh "${msgkey}"
			;;
		esac
		
		# ACT/SBY状況確認の実行結果を保持
		if [ ${?} -eq ${RET_OK} ] ; then
			ret=${OCF_SUCCESS}
		else
			ret=${OCF_ERR_GENERIC}
		fi
		;;
	${RET_NG})
		# 異常終了の場合
		
		[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[end]   monitor.sh (ret=${OCF_ERR_GENERIC})"
		
		exit ${OCF_ERR_GENERIC}
		;;
esac

##
## 再度、監視抑止ファイルのステータスを確認
## (A)ステータス(監視状態)が変更されている場合、変更されたステータスに従った戻り値を返却
## (B)ステータス(監視状態)が変更されていない場合、戻り値にACT/SBY状況確認の結果を返却
##
get_status "${mode_monitor}"

case ${?} in
	${FILE_NOT_EXISTS})
		# ステータス(未起動)の場合
		# 戻り値に未起動を返却
		
		[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[end]   monitor.sh (ret=${OCF_NOT_RUNNING})"
		
		exit ${OCF_NOT_RUNNING}
		;;
	${DISABLE_MONITOR})
		# ステータス(監視抑止状態)の場合
		# 戻り値に異常なしを返却
		
		[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[end]   monitor.sh (ret=${OCF_SUCCESS})"
		
		exit ${OCF_SUCCESS}
		;;
	${ENABLE_MONITOR})		# ステータス(監視状態)の場合
		# ACT/SBY状況確認の実行結果を返却
		
		[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[end]   monitor.sh (ret=${ret})"
		
		exit ${ret}
		;;
	${RET_NG})
		# 異常終了の場合
		
		[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[end]   monitor.sh (ret=${OCF_ERR_GENERIC})"
		
		exit ${OCF_ERR_GENERIC}
		;;
esac
