#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2006-2007 NTT
#/**
# * @file    prex_resource_agent.sh
# * @brief   FMS対応PREX用リソースエージェント
# *
# * パラメタに応じてPREXシステムの起動、停止、ACT化、SBY化、監視を行う。
# *
# *   詳細は以下のとおり。
# *     -# 与えられたパラメータにより以下2～7の処理を行う。
# *     -# パラメータがstartだった場合、状態遷移機能(FMS-IF)の開始コマンドを呼び出す。
# *     -# パラメータがstopだった場合、状態遷移機能(FMS-IF)の停止コマンドを呼び出す。
# *     -# パラメータがpromoteだった場合、状態遷移機能(FMS-IF)のACT化コマンドを呼び出す。
# *     -# パラメータがdemoteだった場合、状態遷移機能(FMS-IF)のSBY化コマンドを呼び出す。
# *     -# パラメータがmonitorだった場合、状態監視機能(FMS-IF)の監視コマンドを呼び出す。
# *     -# パラメータが上記以外だった場合、syslogにメッセージを出力し(prexsyslog コマンド)、戻り値としてOCF_ERR_GENERICを返却して終了する。
# *     -# 2,4の結果が正常の場合、戻り値をOCF_SUCCESSとして呼び出し元に返却する。異常の場合、戻り値をOCF_ERR_GENERICとして呼び出し元に返却する。
# *     -# 3,5の結果が正常／異常に関わらず、戻り値をOCF_ERR_GENERICとして呼び出し元にを返却する。
# *     -# 6の結果が正常の場合、戻り値をOCF_SUCCESSとして呼び出し元に返却する。異常の場合、戻り値をOCF_ERR_GENERICとして呼び出し元に返却する。未起動の場合、戻り値をOCF_NOT_RUNNINGとして呼び出し元に返却する。
# *
# * @param[in]   start            start実行オプション
# * @param[in]   stop             stop実行オプション
# * @param[in]   promote          promote実行オプション
# * @param[in]   demote           demote実行オプション
# * @param[in]   monitor          monitor実行オプション
# * @retval    OCF_SUCCESS(0)     正常終了
# * @retval    OCF_ERR_GENERIC(1) 異常終了
# * @retval    OCF_NOT_RUNNING(7) 未起動
# *
# * @version 1.1.0
# * @date    08/01/2007
# * @since   1.0
# */

# DEFINE値取り込み
. /home/prexuser/prex/bin/prexdefine.sh

# switching ocf-shellfuncs path
if [ -f ${OCF_ROOT}/resource.d/heartbeat/.ocf-shellfuncs ]; then
        FUNCTION_FILE="${OCF_ROOT}/resource.d/heartbeat/.ocf-shellfuncs"
elif [ -f /usr/lib64/heartbeat/ocf-shellfuncs ]; then
        FUNCTION_FILE="/usr/lib64/heartbeat/ocf-shellfuncs"
else
	echo "ocf-shellfuncs file doesn't exist." >&2
	exit 1
fi

. ${FUNCTION_FILE}

#. /usr/lib64/heartbeat/ocf-shellfuncs

#/**
# * @fn      function start()
# * @brief   FMS対応PREX用リソースエージェント - start
# *
# * PREXを起動させる。
# *
# *   詳細は以下のとおり。
# *     -# 環境変数START_MODE(起動モード)の値を取得し、取得した値により以下の処理を行なう。
# *     -# START_MODE = NCの場合、状態遷移機能(FMS-IF)の開始コマンド(通常起動)を実行する。
# *     -# START_MODE = SCの場合、状態遷移機能(FMS-IF)の開始コマンド(単独起動)をXLOG同期を行なうモードで実行する。
# *     -# START_MODE = SFの場合、状態遷移機能(FMS-IF)の開始コマンド(単独起動)をXLOG同期を行なわないモードで実行する。
# *     -# 上記で実行した開始コマンドの実行結果を返却する。
# *
# * @retval    OCF_SUCCESS(0)     正常終了
# * @retval    OCF_ERR_GENERIC(1) 異常終了
# *
# * @version 1.1.0
# * @date    08/01/2007
# * @since   1.0
# */
function start()
{
	##
	## 変数宣言
	##
	local ret=     # 戻り値
	local mode=    # 起動モード
	local command= # コマンド文字列
	
	##
	## 環境変数START_MODE(起動モード)の値を取得
	##
	mode="${OCF_RESKEY_START_MODE}"
	
	if [ -z "${mode}" ] ; then
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::ParamErr Line=${LINENO}"
		return ${OCF_ERR_GENERIC}
	fi
	
	##
	## 起動モードを判定し、開始コマンド(通常起動 or 単独起動)を実行
	## ・起動モードが「NC」の場合、開始コマンド(通常起動)を実行
	## ・起動モードが「SC」の場合、開始コマンド(単独起動)を実行 - XLOG同期を行なうモード
	## ・起動モードが「SF」の場合、開始コマンド(単独起動)を実行 - XLOG同期を行なわないモード
	##
	case "${mode}" in
		'NC') # 開始コマンド(通常起動)
			command="${PREX_BIN}/start.sh -NC"
			;;
		'SC') # 開始コマンド(単独起動) - XLOG同期を行なうモード
			command="${PREX_BIN}/start.sh -SC"
			;;
		'SF') # 開始コマンド(単独起動) - XLOG同期を行なわないモード
			command="${PREX_BIN}/start.sh -SF"
			;;
		*)    # 規定外の値である場合、エラーを返却
			ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::ParamErr Line=${LINENO}"
			return ${OCF_ERR_GENERIC}
			;;
	esac
	
	su - ${PREX_USER} -c "${PREX_BIN}/exclude \"${FILE_NAME_EXCLUSIVE}\" \"${command}\""
	ret=${?}
	
	if [ ${ret} -eq ${RET_NG} ] ; then
		# 開始コマンドが異常終了した場合
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::CommandErr(${command}) ErrCode=${RET_NG} Line=${LINENO}"
		ret=${OCF_ERR_GENERIC}
	elif [ ${ret} -eq ${RET_LOCK} ] ; then
		# 多重起動の場合
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::CommandErr(${command}) ErrCode=${RET_LOCK} Line=${LINENO}"
		ret=${OCF_ERR_GENERIC}
	elif [ ${ret} -eq ${RET_ERROR} ] ; then
		# 多重起動防止に失敗した場合
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::InternalErr Line=${LINENO}"
		ret=${OCF_ERR_GENERIC}
	fi
	
	return ${ret}
}

#/**
# * @fn      function stop()
# * @brief   FMS対応PREX用リソースエージェント - stop
# *
# * PREXを停止させる。
# *
# *   詳細は以下のとおり。
# *     -# 設定ファイル(WALWriter.conf)からメッセージキューのキー値を取得する。
# *     -# 停止コマンドを実行する。
# *     -# 上記で実行した停止コマンドの実行結果を返却する。
# *
# * @retval    OCF_SUCCESS(0)     正常終了
# * @retval    OCF_ERR_GENERIC(1) 異常終了
# *
# * @version 1.1.0
# * @date    08/01/2007
# * @since   1.0
# */
function stop()
{
	##
	## 停止コマンドを実行
	##
	su - ${PREX_USER} -c "${PREX_BIN}/stop.sh"
	
	[ ${?} -ne ${RET_OK} ] && return ${OCF_ERR_GENERIC}
	return ${OCF_SUCCESS}
}

#/**
# * @fn      function promote()
# * @brief   FMS対応PREX用リソースエージェント - promote
# *
# * SBY状態のPREXをACT状態に遷移させる。
# *
# *   詳細は以下のとおり。
# *     -# 環境変数START_MODE(起動モード)および環境変数ENABLE_FAILED_OVER(マシン障害によるフェールオーバー時の動作オプション)の値を取得し、取得した値により以下の処理を行なう。
# *     -# START_MODE = NC, ENABLE_FAILED_OVER = OFFの場合、通常起動＋マシン障害時のフェールオーバーを抑止するモードでACT化コマンドを実行する。
# *     -# START_MODE = NC, ENABLE_FAILED_OVER = ONの場合、通常起動＋マシン障害時のフェールオーバーを許可するモードでACT化コマンドを実行する。
# *     -# START_MODE = SC, ENABLE_FAILED_OVER = OFFの場合、単独起動＋マシン障害時のフェールオーバーを抑止するモードでACT化コマンドを実行する。
# *     -# START_MODE = SF, ENABLE_FAILED_OVER = OFFの場合、単独起動＋マシン障害時のフェールオーバーを抑止するモードでACT化コマンドを実行する。
# *     -# START_MODE = SC, ENABLE_FAILED_OVER = ONの場合、単独起動＋マシン障害時のフェールオーバーを許可するモードでACT化コマンドを実行する。
# *     -# START_MODE = SF, ENABLE_FAILED_OVER = ONの場合、単独起動＋マシン障害時のフェールオーバーを許可するモードでACT化コマンドを実行する。
# *     -# 上記で実行したACT化コマンドの実行結果を返却する。
# *
# * @retval    OCF_SUCCESS(0)     正常終了
# * @retval    OCF_ERR_GENERIC(1) 異常終了
# *
# * @version 1.1.0
# * @date    08/01/2007
# * @since   1.0
# */
function promote()
{
	##
	## 変数宣言
	##
	local ret=          # 戻り値
	local mode_startup= # 起動モード
	local mode_fo=      # マシン障害によるフェールオーバー時の動作オプション
	local command=      # コマンド文字列
	
	##
	## 環境変数START_MODE(起動モード)の値を取得
	##
	mode_startup="${OCF_RESKEY_START_MODE}"
	
	if [ -z "${mode_startup}" ] ; then
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::ParamErr Line=${LINENO}"
		return ${OCF_ERR_GENERIC}
	fi
	
	##
	## 環境変数ENABLE_FAILED_OVER(マシン障害によるフェールオーバー時の動作オプション)の値を取得
	##
	mode_fo="${OCF_RESKEY_ENABLE_FAILED_OVER}"
	
	if [ -z "${mode_fo}" ] ; then
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::ParamErr Line=${LINENO}"
		return ${OCF_ERR_GENERIC}
	fi
	
	##
	## 起動モードおよびマシン障害によるフェールオーバー時の動作オプションを判定し、ACT化コマンドを実行
    ## ・起動オプション = NC, マシン障害によるフェールオーバー時の動作オプション = OFF
    ##   ->通常起動＋マシン障害時のフェールオーバーを抑止するモードでACT化コマンドを実行する。
    ## ・起動オプション = NC, マシン障害によるフェールオーバー時の動作オプション = ON
    ##   ->通常起動＋マシン障害時のフェールオーバーを許可するモードでACT化コマンドを実行する。
    ## ・起動オプション = SC, マシン障害によるフェールオーバー時の動作オプション = OFF
    ##   ->単独起動＋マシン障害時のフェールオーバーを抑止するモードでACT化コマンドを実行する。
    ## ・起動オプション = SF, マシン障害によるフェールオーバー時の動作オプション = OFF
    ##   ->単独起動＋マシン障害時のフェールオーバーを抑止するモードでACT化コマンドを実行する。
    ## ・起動オプション = SC, マシン障害によるフェールオーバー時の動作オプション = ON
    ##   ->単独起動＋マシン障害時のフェールオーバーを許可するモードでACT化コマンドを実行する。
    ## ・起動オプション = SF, マシン障害によるフェールオーバー時の動作オプション = ON
    ##   ->単独起動＋マシン障害時のフェールオーバーを許可するモードでACT化コマンドを実行する。
	##
	command="${PREX_BIN}/promote.sh"
	
	# ACT化コマンドの引数を構築 (起動オプション)
	case "${mode_startup}" in
		'NC') # 通常起動
			command="${command} NC"
			;;
		'SC') # 単独起動(XLOG同期を行なうモード)
			command="${command} SC"
			;;
		'SF') # 単独起動(XLOG同期を行なわないモード)
			command="${command} SF"
			;;
		*)    # 規定外の値である場合、エラーを返却
			ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::ParamErr Line=${LINENO}"
			return ${OCF_ERR_GENERIC}
			;;
	esac
	
	# ACT化コマンドの引数を構築 (マシン障害によるフェールオーバー時の動作オプション)
	case "${mode_fo}" in
		'ON')  # マシン障害時のフェールオーバーを許可するモード
			command="${command} ON"
			;;
		'OFF') # マシン障害時のフェールオーバーを抑止するモード
			command="${command} OFF"
			;;
		*)     # 規定外の値である場合、エラーを返却
			ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::ParamErr Line=${LINENO}"
			return ${OCF_ERR_GENERIC}
			;;
	esac
	
	# ACT化コマンド実行
	su - ${PREX_USER} -c "${PREX_BIN}/exclude \"${FILE_NAME_EXCLUSIVE}\" \"${command}\""
	ret=${?}
	
	if [ ${ret} -eq ${RET_NG} ] ; then
		# ACT化コマンドが異常終了した場合
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::CommandErr(${command}) ErrCode=${RET_NG} Line=${LINENO}"
		ret=${OCF_ERR_GENERIC}
	elif [ ${ret} -eq ${RET_LOCK} ] ; then
		# 多重起動の場合
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::CommandErr(${command}) ErrCode=${RET_LOCK} Line=${LINENO}"
		ret=${OCF_ERR_GENERIC}
	elif [ ${ret} -eq ${RET_ERROR} ] ; then
		# 多重起動防止に失敗した場合
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::InternalErr Line=${LINENO}"
		ret=${OCF_ERR_GENERIC}
	fi
	
	return ${ret}
}

#/**
# * @fn      function demote()
# * @brief   FMS対応PREX用リソースエージェント - demote
# *
# * ACT状態のPREXをSBY状態に遷移させる。
# *
# *   詳細は以下のとおり。
# *     -# SBY化コマンドを実行する。
# *     -# 上記で実行したSBY化コマンドの実行結果を返却する。
# *
# * @retval    OCF_SUCCESS(0)     正常終了
# * @retval    OCF_ERR_GENERIC(1) 異常終了
# *
# * @version 1.1.0
# * @date    08/02/2007
# * @since   1.0
# */
function demote()
{
	##
	## 変数宣言
	##
	local ret=       # 戻り値
	local command=   # コマンド文字列
	
	##
	## SBY化コマンドを実行
	##
	command="${PREX_BIN}/demote.sh"
	
	su - ${PREX_USER} -c "${PREX_BIN}/exclude \"${FILE_NAME_EXCLUSIVE}\" \"${command}\""
	ret=${?}
	
	if [ ${ret} -eq ${RET_NG} ] ; then
		# SBY化コマンドが異常終了した場合
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::CommandErr(${command}) ErrCode=${RET_NG} Line=${LINENO}"
		ret=${OCF_ERR_GENERIC}
	elif [ ${ret} -eq ${RET_LOCK} ] ; then
		# 多重起動の場合
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::CommandErr(${command}) ErrCode=${RET_LOCK} Line=${LINENO}"
		ret=${OCF_ERR_GENERIC}
	elif [ ${ret} -eq ${RET_ERROR} ] ; then
		# 多重起動防止に失敗した場合
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::InternalErr Line=${LINENO}"
		ret=${OCF_ERR_GENERIC}
	fi
	
	return ${ret}
}

#/**
# * @fn      function monitor()
# * @brief   FMS対応PREX用リソースエージェント - monitor
# *
# * 指定されたパラメタに応じてACTもしくはSBYの監視を行う。
# *
# *   詳細は以下のとおり。
# *     -# 環境変数MONITOR_MODE(監視モード)の値を取得し、取得した値により以下の処理を行なう。
# *     -# MONITOR_MODE = actの場合、監視コマンド(ACT)を実行する。
# *     -# MONITOR_MODE = sbyの場合、監視コマンド(SBY)を実行する。
# *     -# 上記で実行した監視コマンドの実行結果を返却する。
# *
# * @retval    OCF_SUCCESS(0)     正常終了
# * @retval    OCF_ERR_GENERIC(1) 異常終了
# * @retval    OCF_NOT_RUNNING(7) 未起動
# *
# * @version 1.1.0
# * @date    08/01/2007
# * @since   1.0
# */
function monitor()
{
	##
	## 変数宣言
	##
	local ret=    # 戻り値
	local mode=   # 監視モード
	
	##
	## 環境変数MONITOR_MODE(監視モード)の値を取得
	##
	mode="${OCF_RESKEY_MONITOR_MODE}"
	
	if [ -z "${mode}" ] ; then
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::ParamErr Line=${LINENO}"
		return ${OCF_ERR_GENERIC}
	fi
	
	##
	## 監視モードを判定し、監視コマンド(ACT or SBY)を実行
	## ・監視モードが「act」の場合、監視コマンド(ACT)を実行
	## ・監視モードが「sby」の場合、監視コマンド(SBY)を実行
	##
	case "${mode}" in
		'act') # 監視コマンド(ACT)を実行
			su - ${PREX_USER} -c "${PREX_BIN}/monitor.sh act"
			ret=${?}
			;;
		'sby') # 監視コマンド(SBY)を実行
			su - ${PREX_USER} -c "${PREX_BIN}/monitor.sh sby"
			ret=${?}
			;;
		*)     # 規定外の値である場合、エラーを返却
			ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::ParamErr Line=${LINENO}"
			return ${OCF_ERR_GENERIC}
			;;
	esac
	
	return ${ret}
}

#/**
# * @fn      function metadata()
# * @brief   FMS対応PREX用リソースエージェント - metadata
# *
# * メタデータを表示する。
# *
# *   詳細は以下のとおり。
# *     -# メタデータを標準出力に出力する。
# *
# * @retval    OCF_SUCCESS(0) 正常終了
# *
# * @version 1.1.0
# * @date    08/09/2007
# * @since   1.0
# */
function metadata()
{
	cat <<END
<?xml version="1.0"?>
<!DOCTYPE resource-agent SYSTEM "ra-api-1.dtd">
<resource-agent name="Prex" version="1.0">
<version>1.0</version>

<longdesc lang="en">
This is an example resource agent that impliments two states
</longdesc>
<shortdesc lang="en">Example prex resource agent</shortdesc>

<parameters>

<parameter name="state" unique="1">
<longdesc lang="en">
Location to store the resource state in
</longdesc>
<shortdesc lang="en">State file</shortdesc>
<coontent type="string" default="/var/run/heartbeat/rsctmp/Prex-{OCF_RESOURCE_INSTANCE}.state"/>
</parameter>

</parameters>

<actions>
<action name="start" timeout="90"/>
<action name="stop" timeout="100"/>
<action name="monitor" depth="0" timeout="20" interval="10"/>
<action name="meta-data" timeout="5"/>
<action name="varify-all" timeout="30"/>
</actions>
</resource-agent>
END

	exit ${OCF_SUCCESS}
}

#/**
# * @fn      function usage()
# * @brief   FMS対応PREX用リソースエージェント - usage
# *
# * usageを表示する。
# *
# *   詳細は以下のとおり。
# *     -# usageを標準出力に出力する。
# *
# * @retval    OCF_SUCCESS(0) 正常終了
# *
# * @version 1.1.0
# * @date    08/09/2007
# * @since   1.0
# */
function usage()
{
	cat <<END
usage: prex_resource_agent.sh {start|stop|promote|demote|monitor|validate-all|meta-data}

Expects to have a fully populated OCF RA-compliant environment set
END
	exit ${OCF_SUCCESS}
}

#######################################
## メインルート
##

##
## 変数宣言
##
ret=         # 戻り値
param="${1}" # 起動オプション

if [ -z "${param}" ] ; then
	ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::ParamErr Line=${LINENO}"
	exit ${OCF_ERR_GENERIC}
fi

##
## 1.パラメータがstartだった場合、状態遷移機能(FMS-IF)の開始コマンドを呼び出す。
## 2.パラメータがstopだった場合、状態遷移機能(FMS-IF)の停止コマンドを呼び出す。
## 3.パラメータがpromoteだった場合、状態遷移機能(FMS-IF)のACT化コマンドを呼び出す。
## 4.パラメータがdemoteだった場合、状態遷移機能(FMS-IF)のSBY化コマンドを呼び出す。
## 5.パラメータがmonitorだった場合、状態監視機能(FMS-IF)の監視コマンドを呼び出す。
## パラメータが上記以外だった場合、syslogにメッセージを出力し、OCF_ERR_GENERICを返却して終了する。
## 
## 1,3の結果が正常の場合、戻り値をOCF_SUCCESSとして呼び出し元に返却する。異常の場合、戻り値をOCF_ERR_GENERICとして呼び出し元に返却する。
## 2,4の結果が正常／異常に関わらず、戻り値をOCF_ERR_GENERICとして呼び出し元にを返却する。
## 5の結果が正常の場合、戻り値をOCF_SUCCESSとして呼び出し元に返却する。異常の場合、戻り値をOCF_ERR_GENERICとして呼び出し元に返却する。未起動の場合、戻り値をOCF_NOT_RUNNINGとして呼び出し元に返却する。
##
case "${param}" in
	'start')
		start
		ret=${?}
		;;
	'stop')
		stop
		ret=${OCF_SUCCESS}
		;;
	'promote')
		promote
		ret=${?}
		;;
	'demote')
		demote
		ret=${OCF_SUCCESS}
		;;
	'monitor')
		monitor
		ret=${?}
		;;
	'meta-data')
		metadata
		exit ${OCF_SUCCESS}
		;;
	'validate-all')
		exit ${OCF_SUCCESS}
		;;
	'usage'|'help')
		usage
		exit ${OCF_SUCCESS}
		;;
	*)
		ocf_log err "Prex ${__OCF_ACTION} NG return=${OCF_ERR_GENERIC} Info::ParamErr Line=${LINENO}"
		exit ${OCF_ERR_GENERIC}
		;;
esac

exit ${ret}
