#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2007 NTT
#/**
# * @file    promote.sh
# * @brief   FMS連携機能 - ACT化コマンド(promote)
# *
# * SBY状態のPREXシステムをACT状態に遷移させる。
# *
# *   詳細は以下のとおり。
# *     -# 設定ファイル(WALWriter.conf)より、パラメータを取得する。取得するパラメータを以下に示す。
# *       - prex_msgkey (メッセージキューのキー値)
# *       - acthahost (Heartbeat LANを経由する相手ノード宛のIPアドレス)
# *       - actwalhost (データ転送LANを経由する相手ノード宛のIPアドレス)
# *       - sbywaldir (ローカルサーバのWALセグメント格納ディレクトリ)
# *       - sbyarchivedir (ローカルサーバのアーカイブログ格納ディレクトリ)
# *     -# 状態遷移機能のSBY状況確認を呼び出し、PREXの状態がSBYであることを確認する。
# *     -# SBY監視抑止ファイルおよびACT監視抑止ファイルを「監視抑止状態」に更新する。
# *     -# 起動オプションの値が「通常起動」の場合、4～5の処理を行ないACT化可否を判定する。
# *     -# Heartbeat LAN経由でリモートサーバのACT状況確認を呼び出し、リモートサーバのPREXがACTでないことを確認する。ACTであった場合は、戻り値にACT化失敗を返却する。Heartbeat LAN経由でのリモートサーバへのアクセスに失敗した場合は、データ転送LAN経由で同処理を行なう。
# *     -# Heartbeat LAN経由でリモートサーバのフェールオーバー抑止ファイルからステータスを取得する。ステータスが「1」(ACT化抑止)であった場合は、戻り値にACT化失敗を返却する。Heartbeat LAN経由でのリモートサーバへのアクセスに失敗した場合は、データ転送LAN経由で同処理を行なう。
# *       - 4または5の処理で、Heartbeat LAN経由、データ転送LAN経由の両者でリモートサーバへのアクセスに失敗した場合は、マシン障害時のフェールオーバー動作オプションを確認する。OFF(フェールオーバー抑止)の場合は、戻り値にACT化失敗を返却する。
# *     -# 状態遷移機能のACT化要求を呼び出し、PREXをACTに遷移させる。
# *     -# ACT監視抑止ファイルを「監視状態」に更新する。
# *
# * @param[in]   mode_starup 起動モード
# * @param[in]   mode_fo     マシン障害によるフェールオーバー時の動作オプション
# * @retval      RET_OK(0)   ACT化成功
# * @retval      RET_NG(1)   ACT化失敗
# *
# * @version 1.0.0
# * @date    08/01/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

##
## 定数
##
DEFAULT_IFS="${IFS}" # IFS(デフォルト値)

#/**
# * @fn      function reset_monitor_control()
# * @brief   監視抑止を元の状態に復元する。
# *
# * 当コマンドでエラーが発生した場合に呼ばれる。
# * 監視抑止を元の状態に戻す。戻す状態は以下の通り。
# *  - SBY監視：「監視状態」- SBY監視抑止ファイルを「監視状態」に更新
# *  - ACT監視：「not running」- ACT監視抑止ファイルを削除
# *
# *   詳細は以下のとおり。
# *     -# SBY監視抑止ファイルを「監視状態」に更新する。
# *     -# ACT監視抑止ファイルを削除する。
# *
# * @retval    RET_OK(0) 正常終了
# * @retval    RET_NG(1) 異常終了
# *
# * @version 1.0.0
# * @date    08/02/2007
# * @since   1.0
# */
function reset_monitor_control()
{
	##
	## 監視抑止を設定
	## ・SBY監視：「監視状態」- SBY監視抑止ファイルを「監視状態」に更新
	## ・ACT監視：「not running」- ACT監視抑止ファイルを削除
	##
	${PREX_BIN}/exclusion.sh "${SBY_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_UNLOCK}" && \
	${PREX_BIN}/exclusion.sh "${ACT_LOCK_COMMANDID}" "${FILE_RM}"

	if [ ${?} -ne ${RET_OK} ] ; then
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
		return ${RET_NG}
	fi
	
	return ${RET_OK}
}

#/**
# * @fn      function is_enable_toact()
# * @brief   フェールオーバによるACT化可否のチェック項目を実行する。
# *
# * 当コマンドでエラーが発生した場合に呼ばれる。
# * 監視抑止を元の状態に戻す。戻す状態は以下の通り。
# *  - SBY監視：「監視抑止状態」- SBY監視抑止ファイルを「監視抑止状態」に更新する。
# *  - ACT監視：「監視状態」- ACT監視抑止ファイルを「監視状態」に更新する。
# *
# *   詳細は以下のとおり。
# *     -# リモートサーバのACT状況確認を呼び出し、リモートサーバのPREXがACTでないことをチェック ※1
# *       - ACT状況確認の戻り値が「0」(ACTとして動作中)の場合、戻り値にACT化NGを返却する。
# *       - ACT状況確認の戻り値が「1」(ACTとして動作中ではない)の場合、2の処理を行なう。
# *     -# リモートサーバのフェールオーバー抑止ファイルのステータスをチェック ※1
# *       - ステータスが「1」(フェールオーバ抑止)の場合、戻り値にACT化NGを返却する。
# *       - ステータスが「0」(フェールオーバ許可)の場合、以降の処理を行なう。(3の処理は行なわない)
# *       - フェールオーバ抑止ファイルが存在しない場合、以降の処理を行なう。(3の処理は行なわない)
# *     -# 1または2で、リモートサーバにアクセスできなかった場合は、マシン障害時のフェールオーバー動作オプションをチェック
# *       - マシン障害時のフェールオーバー動作オプションが「OFF」(ACT化抑止)の場合、戻り値にACT化NGを返却する。
# *       - マシン障害時のフェールオーバー動作オプションが「ON」(ACT化許可)の場合、戻り値にACT化OKを返却する。
# *
# * ※1 リモートサーバへのアクセスに使用するノードは、以下の優先順序で行なう。
# *     (1)Heartbeat LAN (複数指定されていた場合は、指定順とする)
# *     (2)データ転送LAN
# *
# * @param[in] msgkey     メッセージキューのキー値
# * @param[in] acthahosts Heartbeat LANを経由する相手ノード宛のIPアドレス(複数指定可)
# * @param[in] actwalhost データ転送LANを経由する相手ノード宛のIPアドレス
# * @param[in] mode_fo    マシン障害によるフェールオーバー時の動作オプション
# * @retval    RET_OK(0)  ACT化OK
# * @retval    RET_NG(1)  ACT化NG
# *
# * @version 1.0.0
# * @date    08/02/2007
# * @since   1.0
# */
function is_enable_toact()
{
	##
	## 変数宣言
	##
	local msgkey=${1}       # メッセージキューのキー値
	local acthahosts="${2}" # Heartbeat LANを経由する相手ノード宛のIPアドレス(複数指定可)
	local actwalhost="${3}" # データ転送LANを経由する相手ノード宛のIPアドレス
	local mode_fo="${4}"    # マシン障害によるフェールオーバー時の動作オプション
	local ret=
	local status=
	local actcheck_done=0
	
	# acthahostsは配列に変換
	IFS=',' && acthahosts=(`echo "${acthahosts}"`) && IFS="${DEFAULT_IFS}"
	
	##
	## リモートサーバのACT状況確認を呼び出し、
	## リモートサーバのPREXがACT以外であることをチェック
	##
	for host in "${acthahosts[@]}" "${actwalhost}"
	do
		# ネットワーク接続可否を判定
		# 接続できない場合、次のIPアドレスで試行
		ping -w 1 -r ${host} > /dev/null 2>&1
		
		if [ ${?} -ne ${RET_OK} ] ; then
			${PREX_BIN}/prexsyslog ${LOG_WARNING} ${MSGID_FMSIF_ERRORNETWORK} ${0} ${LINENO} ${host}
			continue
		fi
		
		# リモートサーバのACT状況確認を呼び出し、PREXの状態を確認
		# ACTであった場合、戻り値にACT化NGを返却
		ssh "${PREX_USER}@${host}" "${PREX_BIN}/actmonitor.sh ${msgkey} -N"
		
		if [ ${?} -eq ${RET_OK} ] ; then
			# ACTの場合、ACT化NGを返却
			return ${RET_NG}
		else
			# ACT以外の場合、以降の処理を行なう
			actcheck_done=1
			break
		fi
	done
	
	##
	## フェールオーバー抑止ファイルのステータスをチェック
	## ・フェールオーバ抑止ファイルが存在しない場合、戻り値にACT化OKを返却
	## ・ステータスが「ACT化許可」の場合、戻り値にACT化OKを返却
	## ・ステータスが「ACT化抑止」の場合、戻り値にACT化NGを返却
	##
	if [ ${actcheck_done} -eq 1 ] ; then
		for host in "${acthahosts[@]}" "${actwalhost}"
		do
			# ネットワーク接続可否を判定
			# 接続できない場合、次のIPアドレスで試行
			ping -w 1 -r ${host} > /dev/null 2>&1
			
			if [ ${?} -ne ${RET_OK} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_WARNING} ${MSGID_FMSIF_ERRORNETWORK} ${0} ${LINENO} ${host}
				continue
			fi
			
			# リモートサーバのフェールオーバ抑止ファイルが存在有無を確認
			ssh "${PREX_USER}@${host}" "[ ! -f \"${FILE_NAME_FO}\" ]"
			
			if [ ${?} -eq ${RET_OK} ] ; then
				# フェールオーバ抑止ファイルが存在しない場合、戻り値にACT化OKを返却
				return ${RET_OK}
			fi
			
			# リモートサーバのフェールオーバ抑止ファイルのステータスを取得
			status=`ssh "${PREX_USER}@${host}" "${PREX_BIN}/exclusion.sh \"${FO_LOCK_COMMANDID}\" \"${FILE_GET}\""`
			ret=${?}
			
			# フェールオーバ抑止ファイルが存在しない場合、戻り値にACT化OKを返却
			if [ ${ret} -eq ${RET_NOTEXISTS} ] ; then
				# フェールオーバ抑止ファイルが存在しない
				return ${RET_OK}
			elif [ ${ret} -eq ${RET_NG} ] ; then
				# ステータス取得に失敗
				${PREX_BIN}/prexsyslog ${LOG_WARNING} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
				return ${RET_NG}
			fi
			
			# ステータスの確認
			if [ "${status}" = "${FO_TYPEID_LOCK}" ] ; then
				# ステータスが「ACT化抑止」の場合、戻り値にACT化NGを返却
				return ${RET_NG}
			else
				# ステータスが「ACT化抑止」以外の場合、戻り値にACT化OKを返却
				return ${RET_OK}
			fi
		done
	fi
	
	##
	## リモートサーバにアクセスできなかった場合
	## マシン障害時のフェールオーバー動作オプションをチェック
	## ・オプションが「ON」の場合、戻り値にACT化OKを返却
	## ・オプションが「OFF」の場合、戻り値にACT化NGを返却
	##
	[ "${mode_fo}" = 'OFF' ] && return ${RET_NG}
	
	return ${RET_OK}
}

#######################################
## メインルート
##

[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[start] promote.sh (mode=${1}, enable_fo=${2})"

##
## 変数宣言
##
mode_startup="${1}" # 起動モード
mode_fo="${2}"      # マシン障害によるフェールオーバー時の動作オプション
msgkey=             # メッセージキューのキー値
param_list=()       # 設定ファイルの設定値一覧

##
## 設定ファイル(WALWriter.conf)より、以下のパラメータを取得
## ・acthahost (Heartbeat LANを経由する相手ノード宛のIPアドレス)
## ・actwalhost (データ転送LANを経由する相手ノード宛のIPアドレス)
## ・sbywaldir (ローカルサーバのWALセグメント格納ディレクトリ)
## ・sbyarchivedir (ローカルサーバのアーカイブログ格納ディレクトリ)
##
IFS='
'
param_list=(`${PREX_BIN}/ReadConfig "${wwconffile}"`)

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOREADCONFIG} ${0} ${LINENO}
	exit ${RET_NG}
fi

for param in "${param_list[@]}"
do
    eval "${param}"
done
IFS="${DEFAULT_IFS}"

##
## 設定ファイル(WALWriter.conf)より、メッセージキューのキー値を取得
##
msgkey="`${PREX_BIN}/getmsgkey.sh`"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORGETMSGKEY} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## 状態遷移機能のSBY状況確認を呼び出し、PREXの状態がSBYであることを確認
##
${PREX_BIN}/sbymonitor.sh ${msgkey} -N

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORSTATENOTMATCH} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## 監視抑止を設定
## ・SBY監視：「監視抑止状態」- SBY監視抑止ファイルを「監視抑止状態」に更新
## ・ACT監視：「監視抑止状態」- ACT監視抑止ファイルを「監視抑止状態」に更新
##
${PREX_BIN}/exclusion.sh "${SBY_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_LOCK}" && \
${PREX_BIN}/exclusion.sh "${ACT_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_LOCK}"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## 起動モードが「NC」(通常起動)の場合、ACT化可否を判定
##
if [ "${mode_startup}" = 'NC' ] ; then
	is_enable_toact ${msgkey} "${acthahost}" "${actwalhost}" "${mode_fo}"

	if [ ${?} -ne ${RET_OK} ] ; then
		${PREX_BIN}/prexsyslog ${LOG_WARNING} ${MSGID_FMSIF_FOCONTROLDONE} ${0} ${LINENO}
		reset_monitor_control
		exit ${RET_NG}
	fi
fi

##
## 状態遷移機能のACT化要求を実行
## ・WALWriterを起動
## ・postmasterを起動
## ・メッセージキューを作成
##
${PREX_BIN}/toact.sh ${msgkey} "${sbywaldir}" "${sbyarchivedir}"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORTOACT} ${0} ${LINENO}
	reset_monitor_control
	exit ${RET_NG}
fi

##
## 監視抑止を設定
## ・ACT監視：「監視状態」- ACT監視抑止ファイルを「監視状態」に更新
##
${PREX_BIN}/exclusion.sh "${ACT_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_UNLOCK}"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
	exit ${RET_NG}
fi

[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[end]   promote.sh (ret=${RET_OK})"

exit ${RET_OK}
