#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2007-2008 NTT
#/**
# * @file    start.sh
# * @brief   FMS連携機能 - 開始コマンド(start)
# *
# * OUS状態のPREXシステムをSBY状態に遷移させる。
# *
# *   詳細は以下のとおり。
# *     -# 設定ファイル(WALWriter.conf)より、パラメータを取得する。取得するパラメータを以下に示す。
# *       - prex_msgkey (メッセージキューのキー値)
# *       - ws_host (リモートサーバへの接続ホスト名)
# *       - prex_port (リモートサーバへの接続ポート番号)
# *       - ws_polling_interval (リモートサーバへの接続要求ポーリング間隔)
# *       - actwalhost (転送対象WALディレクトリを持つリモートホスト)
# *       - actwaluser (転送対象WALディレクトリを持つリモートホストへのログインユーザ名)
# *       - actwaldir (リモートサーバのWAL格納ディレクトリ)
# *       - sbywaldir (ローカルサーバのWAL格納ディレクトリ)
# *       - actarchivehost (転送対象アーカイブログディレクトリを持つリモートホスト)
# *       - actarchiveuser (転送対象アーカイブログディレクトリを持つリモートホストへのログインユーザ名)
# *       - actarchivedir (リモートサーバのアーカイブログ格納ディレクトリ)
# *       - sbyarchivedir (ローカルサーバのアーカイブログ格納ディレクトリ)
# *     -# 状態遷移機能のACT状況確認およびSBY状況確認を呼び出し、PREXの状態がOUSであることを確認する。
# *     -# アクセス管理のセマフォを準備する。
# *     -# SBY監視抑止ファイルを「監視抑止状態」に更新する。
# *     -# ACT監視抑止ファイルが存在する場合は、削除する。
# *     -# LastLSN用historyファイルが存在する場合は、削除する。
# *     -# recoveryファイルに「recovery_target_timeline」が設定されている場合は、削除する。
# *     -# 起動モードにより、以下の処理を行なう。
# *       - 起動モードが「-NC」(通常起動)の場合、初期起動 or 組み込みの判定を行ない、状態遷移機能のSBY化要求を呼び出す。
# *       - 起動モードが「-SC」(単独起動 XLOG同期あり)の場合、ローカルサーバのトランザクションが先行していることを確認し、状態遷移機能のSBY化要求を呼び出す。
# *       - 起動モードが「-SF」(単独起動 XLOG同期なし)の場合、状態遷移機能のSBY化要求を呼び出す。
# *     -# SBY監視抑止ファイルを「監視状態」に更新する。
# *
# * @param[in]   mode_starup 起動モード
# * @retval      RET_OK(0)   起動成功
# * @retval      RET_NG(1)   起動失敗
# *
# * @version 1.0.0
# * @date    08/06/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

##
## 定数
##
DEFAULT_IFS="${IFS}" # IFS(デフォルト値)

#/**
# * @fn      function reset_monitor_control()
# * @brief   監視抑止を元の状態に復元する。
# *
# * 当コマンドでエラーが発生した場合に呼ばれる。
# * 監視抑止を元の状態に戻す。戻す状態は以下の通り。
# *  - ACT監視：「not running」- ACT監視抑止ファイルを削除
# *  - SBY監視：「not running」- SBY監視抑止ファイルを削除
# *
# *   詳細は以下のとおり。
# *     -# ACT監視抑止ファイルを削除する。
# *     -# SBY監視抑止ファイルを削除する。
# *
# * @retval    RET_OK(0) 正常終了
# * @retval    RET_NG(1) 異常終了
# *
# * @version 1.0.0
# * @date    08/02/2007
# * @since   1.0
# */
function reset_monitor_control()
{
	##
	## 監視抑止を設定
	## ・ACT監視：「nut running」- ACT監視抑止ファイルを削除
	## ・SBY監視：「nut running」- SBY監視抑止ファイルを削除
	##
	${PREX_BIN}/exclusion.sh "${ACT_LOCK_COMMANDID}" "${FILE_RM}" && \
	${PREX_BIN}/exclusion.sh "${SBY_LOCK_COMMANDID}" "${FILE_RM}"

	if [ ${?} -ne ${RET_OK} ] ; then
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
		return ${RET_NG}
	fi
	
	return ${RET_OK}
}

#/**
# * @fn      function start_normal()
# * @brief   開始コマンド(通常起動)相当の処理を行なう。
# *
# *   詳細は以下のとおり。
# *     -# リモートサーバにLastLSN用historyファイルが存在する場合は、削除する。
# *     -# リモートサーバのrecoveryファイルに「recovery_target_timeline」が設定されている場合は、削除する。
# *     -# 初期起動であるか否かを判定。リモートサーバのPostgreSQLへの接続を試行する。
# *       - PostgreSQLへの接続に失敗した場合、初期起動と判定する。
# *       - PostgreSQLへの接続に成功した場合、組み込みと判定する。
# *     -# 初期起動以外の場合は、以下の処置を行なう。
# *       - WALセグメント格納ディレクトリ配下のファイルを削除する。
# *       - 状態遷移機能のSBY化要求を呼び出す。(WALWriterをバックグランドで起動、「SBY化応答」メッセージを受信)
# *     -# 初期起動の場合は、ローカルサーバとリモートサーバのトランザクション状態を比較する。
# *     -# 3の結果、リモートサーバのトランザクションが先行している場合、以下の処理を行なう。
# *       - XLOG同期を行う。
# *       - 相手のTLIをrecovoryファイルに設定する。
# *       - 状態遷移機能のSBY化要求を初期起動オプションで呼び出す。(WALWriterを初期起動オプションにてバックグラウンドで起動)
# *     -# 3の結果、ローカルサーバのトランザクションが先行している場合、以下の処理を行なう。
# *       - リモートサーバのXLOG同期の完了を待つ。
# *       - 状態遷移機能のSBY化要求を初期起動オプションで呼び出す。(WALWriterを初期起動オプションにてバックグラウンドで起動)
# *     -# 3の結果、ローカルサーバとリモートサーバのトランザクションが同一の場合、以下の処理を行なう。
# *       - 状態遷移機能のSBY化要求を初期起動オプションで呼び出す。(WALWriterを初期起動オプションにてバックグラウンドで起動)
# *     -# フェールオーバー抑止ファイルが存在する場合、削除する。
# *
# * @param[in]   msgkey              メッセージキューのキー値 
# * @param[in]   ws_host             リモートサーバへの接続ホスト名
# * @param[in]   prex_port           リモートサーバへの接続ポート番号
# * @param[in]   ws_polling_interval リモートサーバへの接続要求ポーリング間隔
# * @param[in]   actwalhost          転送対象WALディレクトリを持つリモートホスト
# * @param[in]   actwaluser          転送対象WALディレクトリを持つリモートホストへのログインユーザ名
# * @param[in]   actwaldir           リモートサーバのWAL格納ディレクトリ
# * @param[in]   sbywaldir           ローカルサーバのWAL格納ディレクトリ
# * @param[in]   actarchivehost      転送対象アーカイブログディレクトリを持つリモートホスト
# * @param[in]   actarchiveuser      転送対象アーカイブログディレクトリを持つリモートホストへのログインユーザ名
# * @param[in]   actarchivedir       リモートサーバのアーカイブディレクトリ
# * @param[in]   sbyarchivedir       ローカルサーバのアーカイブディレクトリ
# * @retval      RET_OK(0)           正常終了
# * @retval      RET_NG(1)           異常終了
# *
# * @version 1.3.0
# * @date    06/17/2008
# * @since   1.0
# */
function start_normal()
{
	##
	## 変数宣言
	##
	local msgkey="${1}"              # メッセージキューのキー値 
	local ws_host="${2}"             # リモートサーバへの接続ホスト名
	local prex_port="${3}"           # リモートサーバへの接続ポート番号
	local ws_polling_interval="${4}" # リモートサーバへの接続要求ポーリング間隔
	local actwalhost="${5}"          # 転送対象WALディレクトリを持つリモートホスト
	local actwaluser="${6}"          # 転送対象WALディレクトリを持つリモートホストへのログインユーザ名
	local actwaldir="${7}"           # リモートサーバのWAL格納ディレクトリ
	local sbywaldir="${8}"           # ローカルサーバのWAL格納ディレクトリ
	local actarchivehost="${9}"      # 転送対象アーカイブログディレクトリを持つリモートホスト
	local actarchiveuser="${10}"     # 転送対象アーカイブログディレクトリを持つリモートホストへのログインユーザ名
	local actarchivedir="${11}"      # リモートサーバのアーカイブディレクトリ
	local sbyarchivedir="${12}"      # ローカルサーバのアーカイブディレクトリ
	##->200806
	local tli=                       # 相手サーバのTLI
	##<-200806
	
        ##->200806
        ssh "${actwaluser}@${actwalhost}" "rm -f ${FILE_NAME_HISTORY}"

        ##
        ## recovoryファイルからrecovery_target_timelineを削除
        ##

        # recovoryファイル名を取得

        remote_recovery_file_name=`ssh ${actwaluser}@${actwalhost} ls ${PGDATA}|grep recovery`

        if [ ${?} -ne ${RET_OK} ] ; then
                ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOGET} ${0} ${LINENO} ${remote_recovery_file_name}
                return ${RET_NG}
        fi

        # recovoryファイルからrecovery_target_timelineを除いた内容を格納
        remote_recovery_file_tmp=`ssh ${actwaluser}@${actwalhost} sed s/^\s*recovery_target_timeline.*\$//g ${PGDATA}/${remote_recovery_file_name}`

        if [ ${?} -ne ${RET_OK} ] ; then
                ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOGET} ${0} ${LINENO} ${remote_recovery_file_name}
                return ${RET_NG}
        fi

        # recovoryファイルを格納した内容で新規作成
        ssh ${actwaluser}@${actwalhost} "echo \"${remote_recovery_file_tmp}\" > ${PGDATA}/${remote_recovery_file_name}"

        if [ ${?} -ne ${RET_OK} ] ; then
                ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOCREATE} ${0} ${LINENO} ${remote_recovery_file_name}
                return ${RET_NG}
        fi
        ##<-200806

	##
	## 下記の条件別に、SBY化要求を実行
	## (A)初期起動以外の場合
	## (B)初期起動、かつリモートサーバのトランザクションが先行
	## (C)初期起動、かつローカルサーバのトランザクションが先行
	## (D)初期起動、かつローカルサーバとリモートサーバのトランザクションが同じ
	##
	${PREX_BIN}/CompRemoteLSN "${actwalhost}" "${actwaluser}"
	
	case ${?} in
		${RET_OK})
			# 初期起動以外
			# WALセグメント格納ディレクトリ配下のファイルを削除
			for filename in `find "${walsegment_dir}" -type f`
			do
				rm -f "${filename}"
				
				if [ ${?} -ne ${RET_OK} ] ; then
					${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORDELETEFILE} ${0} ${LINENO} "${filename}"
					return ${RET_NG}
				fi
			done
			
			# SBY化要求を実行
			${PREX_BIN}/tosby.sh ${msgkey}
			
			if [ ${?} -ne ${RET_OK} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORTOSBY} ${0} ${LINENO}
				return ${RET_NG}
			fi
			;;
		${RET_REMOTE_ADVANCED})
			# リモートサーバのトランザクションが先行
			# XLOG同期
			${PREX_BIN}/SyncXLOG ${ws_host} ${prex_port} ${ws_polling_interval} ${actwalhost} ${actwaluser} ${actwaldir} ${sbywaldir} ${actarchivehost} ${actarchiveuser} ${actarchivedir} ${sbyarchivedir}
			
			if [ ${?} -ne ${RET_OK} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOSYNCXLOG} ${0} ${LINENO}
				return ${RET_NG}
			fi

			##->200806
			# 
			# 相手のTLIをrecovoryファイルに設定する
			# 

			# 相手のTLIを取得			
			tli=`ssh "${actwaluser}"@"${actwalhost}"  pg_controldata | grep '^Latest checkpoint.s TimeLineID:' | sed 's/^Latest checkpoint.s TimeLineID:[ ]*//g'`
			if [ -z ${tli} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOGET} ${0} ${LINENO} "TimeLineID"
				return ${RET_NG}
			fi

			# recovoryファイルに相手のTLIを設定したrecovery_target_timelineを追記
			echo "recovery_target_timeline='${tli}'" >> ${recoveryconf_file}

			if [ ${?} -ne ${RET_OK} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOCREATE} ${0} ${LINENO} ${recoveryconf_file}
				return ${RET_NG}
			fi
			##<-200806


			
			# SBY化要求を実行 (初期起動オプション指定)
			${PREX_BIN}/tosby.sh ${msgkey} -i
			
			if [ ${?} -ne ${RET_OK} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORTOSBY} ${0} ${LINENO}
				return ${RET_NG}
			fi
			;;
		${RET_LOCAL_ADVANCED})
			# ローカルのトランザクションが先行
			# XLOG同期待ち
			${PREX_BIN}/WaitSyncXLOG "${prex_port}"
			
			if [ ${?} -ne ${RET_OK} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOWAITSYNCXLOG} ${0} ${LINENO}
				return ${RET_NG}
			fi
			
			# SBY化要求を実行 (初期起動オプション指定)
			${PREX_BIN}/tosby.sh ${msgkey} -i
			
			if [ ${?} -ne ${RET_OK} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORTOSBY} ${0} ${LINENO}
				return ${RET_NG}
			fi
			;;
		${RET_SAME_XLOG})
			# ローカルとリモートのトランザクションが同一
			# SBY化要求を実行 (初期起動オプション指定)
			${PREX_BIN}/tosby.sh ${msgkey} -i
			
			if [ ${?} -ne ${RET_OK} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORTOSBY} ${0} ${LINENO}
				return ${RET_NG}
			fi
			;;
		*)
			# 異常終了の場合
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOCOMPREMOTELSN} ${0} ${LINENO}
			return ${RET_NG}
			;;
	esac
	
	##
	## フェールオーバ抑止ファイルが存在する場合は削除
	##
	${PREX_BIN}/exclusion.sh "${FO_LOCK_COMMANDID}" "${FILE_RM}"

	if [ ${?} -ne ${RET_OK} ] ; then
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
		return ${RET_NG}
	fi
	
	return ${RET_OK}
}

#/**
# * @fn      function start_single()
# * @brief   開始コマンド(単独起動)相当の処理を行なう。
# *
# *   詳細は以下のとおり。
# *     -# 起動モードにより、以下の処理を行なう。
# *       - 起動モードが「-SF」(XLOG同期を行なわない)の場合、状態遷移機能のSBY化要求を初期起動オプションで呼び出す。(WALWriterを初期起動オプションにてバックグラウンドで起動)
# *       - 起動モードが「-SC」(XLOG同期を行なう)の場合、
# *         リモートサーバにLastLSN用historyファイルが存在する場合は、削除する。
# *         リモートサーバのrecoveryファイルに「recovery_target_timeline」が設定されている場合は、削除する。
# *         ローカルサーバとリモートサーバのトランザクション状態を比較する。
# *     -# トランザクション状態比較の結果、リモートサーバのトランザクションが先行している場合、以下の処理を行なう。
# *       - 起動失敗を返却して終了する。
# *     -# トランザクション状態比較の結果、ローカルサーバのトランザクションが先行している場合、以下の処理を行なう。
# *       - 状態遷移機能のSBY化要求を初期起動オプションで呼び出す。(WALWriterを初期起動オプションにてバックグラウンドで起動)
# *
# * @param[in]   mode_startup 起動モード (-SF or -SC)
# * @param[in]   msgkey       メッセージキューのキー値 
# * @param[in]   actwalhost   リモートサーバのホスト名
# * @param[in]   actwaluser   リモートサーバのログインユーザ名
# * @retval      RET_OK(0)    正常終了
# * @retval      RET_NG(1)    異常終了
# *
# * @version 1.0.0
# * @date    08/06/2007
# * @since   1.0
# */
function start_single()
{
	##
	## 変数宣言
	##
	local mode_startup="${1}" # 起動モード
	local msgkey="${2}"       # メッセージキューのキー値
	local actwalhost="${3}"   # リモートサーバのホスト名
	local actwaluser="${4}"   # リモートサーバのログインユーザ名
	local ret=                # 戻り値
	
	##
	## 起動オプション別に、以下の処理を実行
	## (A)「-SF」の場合、SBY化要求を初期起動オプションで実行
	## (B)「-SC」の場合、ローカルサーバとリモートサーバのトランザクション状態を比較
	##  (B-1)リモートサーバのトランザクションが先行している場合、起動失敗を返却して終了
	##  (B-2)ローカルサーバのトランザクションが先行している場合、SBY化要求を初期起動オプションで実行
	##
	case "${mode_startup}" in
		'-SF')
			# 単独起動 (XLOG同期なし)
			# SBY化要求を実行
			${PREX_BIN}/tosby.sh ${msgkey} -i
			
			if [ ${?} -ne ${RET_OK} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORTOSBY} ${0} ${LINENO}
				return ${RET_NG}
			fi
			;;
		'-SC')
		        ##->200806
		        ssh "${actwaluser}@${actwalhost}" "rm -f ${FILE_NAME_HISTORY}"

		        ##
		        ## recovoryファイルからrecovery_target_timelineを削除
		        ##

		        # recovoryファイル名を取得

		        remote_recovery_file_name=`ssh ${actwaluser}@${actwalhost} ls ${PGDATA}|grep recovery`

		        if [ ${?} -ne ${RET_OK} ] ; then
		                ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOGET} ${0} ${LINENO} ${remote_recovery_file_name}
		                return ${RET_NG}
		        fi

		        # recovoryファイルからrecovery_target_timelineを除いた内容を格納
		        remote_recovery_file_tmp=`ssh ${actwaluser}@${actwalhost} sed s/^\s*recovery_target_timeline.*\$//g ${PGDATA}/${remote_recovery_file_name}`

		        if [ ${?} -ne ${RET_OK} ] ; then
		                ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOGET} ${0} ${LINENO} ${remote_recovery_file_name}
		                return ${RET_NG}
		        fi

		        # recovoryファイルを格納した内容で新規作成
		        ssh ${actwaluser}@${actwalhost} "echo \"${remote_recovery_file_tmp}\" > ${PGDATA}/${remote_recovery_file_name}"

		        if [ ${?} -ne ${RET_OK} ] ; then
		                ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOCREATE} ${0} ${LINENO} ${remote_recovery_file_name}
		                return ${RET_NG}
		        fi
		        ##<-200806

			# 単独起動 (XLOG同期あり)
			# ローカル・リモートLSN比較を実行
			${PREX_BIN}/CompRemoteLSN "${actwalhost}" "${actwaluser}"
			ret=${?}
			
			# ローカルサーバのトランザクションが先行の場合、SBY化要求を初期起動オプションで実行
			if [ ${ret} -eq ${RET_LOCAL_ADVANCED} -o ${ret} -eq ${RET_SAME_XLOG} ] ; then
				${PREX_BIN}/tosby.sh ${msgkey} -i
				
				if [ ${?} -ne ${RET_OK} ] ; then
					${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORTOSBY} ${0} ${LINENO}
					return ${RET_NG}
				fi
			elif [ ${ret} -eq ${RET_REMOTE_ADVANCED} ] ; then
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORLSNREMOTEADVANTAGE} ${0} ${LINENO}
				return ${RET_NG}
			else
				# LSN比較に失敗の場合、異常終了を返却
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOCOMPREMOTELSN} ${0} ${LINENO}
				return ${RET_NG}
			fi
		;;
	esac
	
	return ${RET_OK}
}

#######################################
## メインルート
##

[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[start] start.sh (mode=${1})"

##
## 変数宣言
##
mode_startup="${1}" # 起動モード
param_list=()       # 設定ファイルの設定値一覧
prex_msgkey=        # メッセージキューのキー値         (設定ファイル項目)
actwalhost=         # リモートサーバのホスト名         (設定ファイル項目)
actwaluser=         # リモートサーバのログインユーザ名 (設定ファイル項目)

##<-200806
recovery_file_tmp=  # recovery.confファイルの一時格納領域
##->200806

##
## 設定ファイル(WALWriter.conf)より、以下のパラメータを取得
## ・prex_msgkey (メッセージキューのキー値)
## ・actwalhost  (リモートサーバのホスト名)
## ・actwaluser  (リモートサーバのログインユーザ名)
##
IFS='
'
param_list=(`${PREX_BIN}/ReadConfig "${wwconffile}"`)

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOREADCONFIG} ${0} ${LINENO}
	exit ${RET_NG}
fi

for param in "${param_list[@]}"
do
    eval "${param}"
done
IFS="${DEFAULT_IFS}"

##
## 状態遷移機能のACT状況確認およびSBY状況確認を呼び出し、PREXの状態がOUSであることを確認
##
${PREX_BIN}/actmonitor.sh ${prex_msgkey} -N

if [ ${?} -eq ${RET_OK} ] ; then
	# PREXの状態がACTの場合
	exit ${RET_OK}
fi

${PREX_BIN}/sbymonitor.sh ${prex_msgkey} -N

if [ ${?} -eq ${RET_OK} ] ; then
	# PREXの状態がSBYの場合
	exit ${RET_OK}
fi

##
## アクセス管理のセマフォを作成する。
##
${PREX_BIN}/Semaphore ${SMG_OPEN_COMMAND}

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORSEMAPHORE} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## 監視抑止を設定
## ・SBY監視：「監視抑止状態」- SBY監視抑止ファイルを「監視抑止状態」に更新
## ・ACT監視：「not running」- ACT監視抑止ファイルを削除
##
${PREX_BIN}/exclusion.sh "${SBY_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_LOCK}" && \
${PREX_BIN}/exclusion.sh "${ACT_LOCK_COMMANDID}" "${FILE_RM}"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## リカバリ設定ファイル(recovery.conf)のファイル名を更新
##
# recovery.confファイルの存在確認
if [ ! -e ${recoveryconf_file} ]; then
	# recovery.doneファイルの存在確認
	if [ ! -e ${recoverydone_file} ]; then
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORRECOVERYFILENOTEXISTS} ${0} ${LINENO}
		reset_monitor_control
		exit ${RET_NG}
	else
		# recovery.doneが存在する場合、ファイル名をrecovery.confに書き換える
		mv -f ${recoverydone_file} ${recoveryconf_file}
		
		if [ ${?} -ne ${RET_OK} ] ; then
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORRECOVERYFILERENAME} ${0} ${LINENO}
			reset_monitor_control
			exit ${RET_NG}
		fi
	fi
fi

##<-200806

##
## LastLSN用historyファイルを削除
##
rm -f ${FILE_NAME_HISTORY}
if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTODELETEFILE} ${0} ${LINENO} ${FILE_NAME_HISTORY}
	exit ${RET_NG}
fi

##
## recovoryファイルからrecovery_target_timelineを削除
##

# recovoryファイルからrecovery_target_timelineを除いた内容を格納
recovery_file_tmp=`sed s/^\s*recovery_target_timeline.*\$//g ${recoveryconf_file}`

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOGET} ${0} ${LINENO} ${recoveryconf_file}
	exit ${RET_NG}
fi

# recovoryファイルを格納した内容で新規作成
echo "${recovery_file_tmp}" > ${recoveryconf_file}

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_FAILEDTOCREATE} ${0} ${LINENO} ${recoveryconf_file}
	exit ${RET_NG}
fi


##->200806

##
## 起動モードの判定
## (A)「-NC」の場合、開始コマンド(通常起動)相当の処理を行なう
## (B)「-SC」or「-SF」の場合、開始コマンド(単独起動)相当の処理を行なう
##
case "${mode_startup}" in
	'-NC')
		# 通常起動の場合
		start_normal "${prex_msgkey}" "${ws_host}" "${prex_port}" "${ws_polling_interval}" "${actwalhost}" "${actwaluser}" "${actwaldir}" "${sbywaldir}" "${actarchivehost}" "${actarchiveuser}" "${actarchivedir}" "${sbyarchivedir}"
		;;
	'-SC'|'-SF')
		# 単独起動の場合
		start_single "${mode_startup}" "${prex_msgkey}" "${actwalhost}" "${actwaluser}"
		;;
esac

if [ ${?} -ne ${RET_OK} ] ; then
	reset_monitor_control
	exit ${RET_NG}
fi

##
## 監視抑止を設定
## ・SBY監視：「監視抑止状態」- SBY監視抑止ファイルを「監視状態」に更新
##
${PREX_BIN}/exclusion.sh "${SBY_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_UNLOCK}"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
	exit ${RET_NG}
fi

# 開始コマンド完了メッセージを出力
${PREX_BIN}/prexsyslog ${LOG_INFO} ${MSGID_FMSIF_PREXONSTANDBY} ${0} ${LINENO} ${mode_startup}

[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[end]   start.sh (ret=${RET_OK})"

exit ${RET_OK}
