#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2007 NTT
#/**
# * @file    stop.sh
# * @brief   FMS連携機能 - 停止コマンド(stop)
# *
# * SBY状態のPREXをOUS状態に遷移させる。
# *
# *   詳細は以下のとおり。
# *     -# 以降の処理でエラーが発生した場合、後続の処理を継続する。
# *     -# 設定ファイル(WALWriter.conf)からメッセージキューのキー値を取得する。
# *     -# SBY監視抑止ファイルを「監視抑止状態」に更新する。
# *     -# ACT監視抑止ファイルが存在する場合は、削除する。
# *     -# 状態遷移機能の強制停止要求を実行する。(postmaster終了、WALWriter終了、メッセージキュー削除)
# *     -# SBY監視抑止ファイルを削除する。
# *     -# アクセス管理のセマフォを解放する。
# *     -# ACT化要求／応答の送受信プロセスが存在する場合は、当該プロセスを強制終了する。
# *     -# ACT化可否判定プロセスが存在する場合は、当該プロセスを強制終了する。
# *
# * @retval      RET_OK(0) 停止成功
# * @retval      RET_NG(1) 停止失敗
# * 
# * @version 1.0.0
# * @date    10/30/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[start] stop.sh"

##
## 変数宣言
##
msgkey=   # メッセージキューのキー値
errflg=0  # エラー発生認識フラグ (0:エラーなし, 1:エラーあり)

##
## 設定ファイル(WALWriter.conf)より、メッセージキューのキー値を取得
##
msgkey="`${PREX_BIN}/getmsgkey.sh`"

if [ ${?} -ne ${RET_OK} ] ; then
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORGETMSGKEY} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## 監視抑止を設定
## ・SBY監視：「監視抑止状態」- SBY監視抑止ファイルを「監視抑止状態」に更新
## ・ACT監視：「not running」- ACT監視抑止ファイルを削除
##
${PREX_BIN}/exclusion.sh "${SBY_LOCK_COMMANDID}" "${FILE_SET}" "${MONITOR_TYPEID_LOCK}" && \
${PREX_BIN}/exclusion.sh "${ACT_LOCK_COMMANDID}" "${FILE_RM}"

if [ ${?} -ne ${RET_OK} ] ; then
	errflg=1
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
fi

##
## 状態遷移機能の強制停止要求を実行
##
${PREX_BIN}/tostop_immediate.sh ${msgkey}

if [ ${?} -ne ${RET_OK} ] ; then
	errflg=1
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORSTOPIMMEDIATE} ${0} ${LINENO}
fi

##
## 監視抑止を設定
## ・SBY監視：「not running」- SBY監視抑止ファイルを削除
##
${PREX_BIN}/exclusion.sh "${SBY_LOCK_COMMANDID}" "${FILE_RM}"

if [ ${?} -ne ${RET_OK} ] ; then
	errflg=1
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERROREXCLUSION} ${0} ${LINENO}
fi

##
## アクセス管理のセマフォを解放する。
##
${PREX_BIN}/Semaphore ${SMG_UNLINK_COMMAND}

if [ ${?} -ne ${RET_OK} ] ; then
	errflg=1
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_FMSIF_ERRORSEMAPHORE} ${0} ${LINENO}
fi

##
## ACT化要求／応答の送受信プロセスを強制終了
##
# ACT化要求／応答の送受信プロセスの存在確認
tmp=`ps --user "${PREX_USER}" | grep -P "${PNAME_MSGACT} ${msgkey}"`

if [ ${?} -eq ${RET_OK} ]; then
	# プロセスが存在したら強制終了
	pid=`echo ${tmp} | awk '{print $1}'`
	kill -9 ${pid}
fi

##
## ACT化可否判定プロセスを強制終了
##
# ACT化可否判定プロセスの存在確認
pid_check_enable_toact=`ps --no-heading -f --user "${PREX_USER}" | grep "${PNAME_CHECK_ENABLE_TOACT}" | grep -v "${CMDNAME_GREP}" | awk '{print $2}'`

if [ ! -z ${pid_check_enable_toact} ] ; then
	# プロセスが存在したら強制終了
	pid_sleep=`ps --no-heading --ppid ${pid_check_enable_toact} | grep "${PNAME_SLEEP}" | awk '{print $1}'`
	kill ${pid_check_enable_toact} ${pid_sleep}
fi

[ "${DEBUG_LOG}" == 'ON' ] && ${PREX_BIN}/prexsyslog ${LOG_DEBUG} ${MSGID_FMSIF_DEBUG} ${0} ${LINENO} "[end]   stop.sh (ret=${errflg})"

[ ${errflg} -eq 1 ] && exit ${RET_NG}
exit ${RET_OK}
