#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2007 NTT
#/**
# * @file    prex_progchecker.sh
# * @brief   運用コマンド - トランザクション進捗遅れ確認コマンド
# *
# * ローカルサーバとリモートサーバのトランザクション進行度合いを
# * チェックし、その結果を表示形式に従って表示する。
# * 
# * 表示形式は、コマンド引数(オプション)に指定する。
# * 表示形式および対応するオプションを以下に示す。
# *
# * -F：日時、タイムラインID、WALセグメントのファイル名および簡易グラフ
# * -L：日時、タイムラインID、LSNおよび簡易グラフ
# * -S：日付、タイムラインID、LSN(CSV形式)
# * ※表示形式オプションが未指定の場合は-Lの形式で表示を行なう
# *
# * 簡易グラフは、セグメントIDの0x00を最小、0xFEを最大とし、
# * 文字「*」を使用してセグメントIDの進行具合を表示する。
# *
# * オプションに-i <数値>が指定された場合は、数値を表示間隔として、
# * CTRL+Cが実行されるまで、断続的にトランザクション進行度合いチェックおよび表示を行なう。
# *
# *   詳細は以下のとおり。
# *     -# コマンド引数(オプション)を解析し、表示形式および表示間隔を取得する。
# *       - 未定義のオプションがしてされていた場合、usageを出力して戻り値に異常終了を返却する。
# *     -# オプションに「-h」が指定されていた場合、usageを出力して戻り値に正常終了を返却する。
# *     -# 設定ファイル(WALWriter.conf)より、パラメータを取得する。取得するパラメータを以下に示す。
# *       - actwalhost (リモートサーバのホスト名 or IPアドレス)
# *       - actwaluser (リモートサーバのログインユーザ名)
# *       - actwaldir  (リモートサーバのWALセグメント格納ディレクトリ)
# *       - sbywaldir  (ローカルサーバのWALセグメント格納ディレクトリ)
# *       - sbyarchivedir  (ローカルサーバのアーカイブログ格納ディレクトリ)
# *       - actarchivedir  (リモートサーバのアーカイブログ格納ディレクトリ)
# *     -# ローカルサーバのホスト名を取得する。
# *     -# ローカルサーバに対し、状態遷移機能のACT状況確認を呼び出し、PREXの状態がACTであるかを確認する。
# *       - ACTの場合、終端LSNをSQL(SELECT pg_current_xlog_location())を発行して取得する。
# *       - ACT以外の場合、最終WALセグメントより、終端LSNを取得する。
# *     -# リモートサーバに対し、状態遷移機能のACT状況確認を呼び出し、PREXの状態がACTであるかを確認する。
# *       - ACTの場合、終端LSNをSQL(SELECT pg_current_xlog_location())を発行して取得する。
# *       - ACT以外の場合、最終WALセグメントより、終端LSNを取得する。
# *     -# ローカルサーバ、リモートサーバの各々でpg_controldataを実行し、タイムラインIDを取得する。
# *     -# 終端LSNより、ログIDおよびセグメントIDを取得する。
# *     -# 現在日時を取得する。
# *     -# 表示形式により、以下の処理を行なう。
# *     -# 表示形式「-F」の場合
# *       - タイムラインID、ログID、セグメントIDより、最終WALセグメントのファイル名を構成する。
# *       - セグメントIDより、簡易グラフを構成する。
# *       - 画面設計(タイムラインID＋ファイル名)に従った内容を標準出力に出力する。
# *       - 表示間隔が指定されている場合、指定秒数待機し、画面をクリアして5の処理に戻る。
# *     -# 表示形式「-L」または表示形式が未指定の場合
# *       - セグメントIDより、簡易グラフを構成する。
# *       - 画面設計(タイムラインID＋LSN)に従った内容を標準出力に出力する。
# *       - 表示間隔が指定されている場合、指定秒数待機し、画面をクリアして5の処理に戻る。
# *     -# 表示形式「-S」の場合
# *       - 画面設計(タイムラインID＋LSN(CSV))に従った内容を標準出力に出力する。
# *       - 表示間隔が指定されている場合、指定秒数待機し、5の処理に戻る。
# *     -# 割り込み(CTRL+C)のシグナルを受信した場合、戻り値に正常終了を返却して終了する。
# *
# * @param[in] -F|-L|-S     表示形式指定(左記のいずれか or 省略)
# * @param[in] -i interval  表示間隔指定
# * @param[in] -h           usage表示
# * @retval    RET_OK(0)    正常終了
# * @retval    RET_NG(1)    異常終了
# *
# * @version 1.0.0
# * @date    08/01/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

##
## 定数
##
MODE_FILE='-F'              # 表示形式(タイムラインID＋ファイル名)
MODE_LSN='-L'               # 表示形式(タイムラインID＋LSN)
MODE_CSV='-S'               # 表示形式(タイムラインID＋LSN(CSV))
DEFAULT_MODE=${MODE_LSN}    # 表示形式のデフォルト(タイムラインID＋LSN)
DEFAULT_INTERVAL=0          # 表示間隔のデフォルト(0は無効)
DEFAULT_IFS="${IFS}"        # IFSのデフォルト
WAL_SEGMENT_SIZE=16777216   # WALセグメントのサイズ
PROGRESS_EXPR="10.583"      # 簡易グラフで「*」を出力すべき数 (出力すべき数 = セグメントID * (254 / 24))
                            # ※254(0xFE)の場合に「*」が24個(最大個数)となる条件

#/**
# * @fn      function usage()
# * @brief   usageを表示する。
# *
# *   詳細は以下のとおり。
# *     -# 標準出力にusage文字列を表示する。
# *
# * @retval  RET_OK(0) 正常終了
# *
# * @version 1.0.0
# * @date    08/01/2007
# * @since   1.0
# */
function usage()
{
	echo 'Usage:'
	echo '  prex_progchecker.sh [-i interval] [-F|-L|-S] [-h]'
	echo ''
	echo 'Options:'
	echo '  -i interval   interval when progress degree of transaction(xlog) is checked (default only once is checked)'
	echo '  -F   output mode that displays both time line ID and file name of xlog'
	echo '  -L   output mode that displays both time line ID and LSN (default output mode)'
	echo '  -S   output mode that displays both time line ID and LSN with CSV format'
	echo '  -h   display this help and exit'
	
	return ${RET_OK}
}

#/**
# * @fn      function print_prog_file()
# * @brief   トランザクション進捗状況を「タイムラインID＋ファイル名」の形式で表示する。
# *
# *   詳細は以下のとおり。
# *     -# タイムラインID＋ログID＋セグメントIDからWALセグメントファイル名を構築する。
# *     -# 簡易グラフで表示する「*」の数を計算する。セグメントIDを10進数に変換し、254(0xFE)の場合に「*」が24個(最大個数)となるよう計算する。
# *     -# トランザクション進捗状況を画面設計に従って標準出力に出力する。
# *
# * @param[in] host_local   ホスト名(ローカルサーバ)
# * @param[in] host_remote  ホスト名(リモートサーバ)
# * @param[in] logid_local  ログID(ローカルサーバ)
# * @param[in] logid_remote ログID(リモートサーバ)
# * @param[in] segid_local  セグメントID(ローカルサーバ)
# * @param[in] segid_remote セグメントID(リモートサーバ)
# * @param[in] tid_local    タイムラインID(ローカルサーバ)
# * @param[in] tid_remote   タイムラインID(リモートサーバ)
# * @param[in] datetime     現在日時
# * @retval  RET_OK(0)      正常終了
# *
# * @version 1.0.0
# * @date    08/02/2007
# * @since   1.0
# */
function print_prog_file()
{
	##
	## 変数宣言
	##
	local host_local="${1}"   # ホスト名(ローカルサーバ)
	local host_remote="${2}"  # ホスト名(リモートサーバ)
	local logid_local="${3}"  # ログID(ローカルサーバ)
	local logid_remote="${4}" # ログID(リモートサーバ)
	local segid_local="${5}"  # セグメントID(ローカルサーバ)
	local segid_remote="${6}" # セグメントID(リモートサーバ)
	local tid_local="${7}"    # タイムラインID(ローカルサーバ)
	local tid_remote="${8}"   # タイムラインID(リモートサーバ)
	local datetime="${9}"     # 現在日時
	
	local filename_local=     # 最終WALセグメントファイル名(ローカルサーバ)
	local filename_remote=    # 最終WALセグメントファイル名(リモートサーバ)
	local prog_local=         # 簡易グラフで使用する「*」文字列(ローカルサーバ)
	local prog_remote=        # 簡易グラフで使用する「*」文字列(リモートサーバ)
	
	##
	## WALセグメントファイル名を構築
	## 「タイムラインID＋ログID＋セグメントID」
	##
	filename_local="${tid_local}${logid_local}${segid_local}"
	filename_remote="${tid_remote}${logid_remote}${segid_remote}"
	
	##
	## 簡易グラフで表示する「*」文字列を構築
	## (1)セグメントIDを10進数に変換する。
	## (2)254(0xFE)の場合に「*」が24個(最大個数)となる条件を満たす数式で
	##    出力すべき「*」の数を取得
	##
	local segid_local_decimal=`printf "%d" "0x${segid_local}"`
	local segid_remote_decimal=`printf "%d" "0x${segid_remote}"`

	local num_local=`echo "${segid_local_decimal} / ${PROGRESS_EXPR}" | bc`
	local num_remote=`echo "${segid_remote_decimal} / ${PROGRESS_EXPR}" | bc`
	local cnt=
	
	# 「*」文字列を構築 (ローカルサーバ)
	cnt=0
	while [ ${cnt} -lt ${num_local} ]
	do
		prog_local="${prog_local}*"
		cnt=`expr ${cnt} + 1`
	done
	
	# 「*」文字列を構築 (リモートサーバ)
	cnt=0
	while [ ${cnt} -lt ${num_remote} ]
	do
		prog_remote="${prog_remote}*"
		cnt=`expr ${cnt} + 1`
	done
	
	# 桁揃え
	prog_local="`printf "%-25s" "${prog_local}"`"
	prog_remote="`printf "%-25s" "${prog_remote}"`"
	
	##
	## トランザクション進捗状況を標準出力に出力
	## ・表示形式：タイムラインID＋ファイル名
	##
	echo "[DATE]"
	echo "${datetime}"
	echo "[TIME LINE ID]"
	echo "${tid_local} ... ${host_local}"
	echo "${tid_remote} ... ${host_remote}"
	echo "[WAL SEGMENT FILE]"
	echo "${filename_local} ... ${host_local}"
	echo "${filename_remote} ... ${host_remote}"
	echo "[PROGRESS]"
	echo "${prog_local} ... ${host_local}"
	echo "${prog_remote} ... ${host_remote}"
	
	return ${RET_OK}
}

#/**
# * @fn      function print_prog_lsn()
# * @brief   トランザクション進捗状況を「タイムラインID＋LSN」の形式で表示する。
# *
# *   詳細は以下のとおり。
# *     -# 簡易グラフで表示する「*」の数を計算する。セグメントIDを10進数に変換し、254(0xFE)の場合に「*」が24個(最大個数)となるよう計算する。
# *     -# トランザクション進捗状況を画面設計に従って標準出力に出力する。
# *
# * @param[in] host_local    ホスト名(ローカルサーバ)
# * @param[in] host_remote   ホスト名(リモートサーバ)
# * @param[in] logid_local   ログID(ローカルサーバ)
# * @param[in] logid_remote  ログID(リモートサーバ)
# * @param[in] segid_local   セグメントID(ローカルサーバ)
# * @param[in] segid_remote  セグメントID(リモートサーバ)
# * @param[in] offset_local  オフセット(ローカルサーバ)
# * @param[in] offset_remote オフセット(リモートサーバ)
# * @param[in] tid_local     タイムラインID(ローカルサーバ)
# * @param[in] tid_remote    タイムラインID(リモートサーバ)
# * @param[in] datetime      現在日時
# * @retval  RET_OK(0)       正常終了
# *
# * @version 1.0.0
# * @date    08/02/2007
# * @since   1.0
# */
function print_prog_lsn()
{
	##
	## 変数宣言
	##
	local host_local="${1}"    # ホスト名(ローカルサーバ)
	local host_remote="${2}"   # ホスト名(リモートサーバ)
	local logid_local="${3}"   # ログID(ローカルサーバ)
	local logid_remote="${4}"  # ログID(リモートサーバ)
	local segid_local="${5}"   # セグメントID(ローカルサーバ)
	local segid_remote="${6}"  # セグメントID(リモートサーバ)
	local offset_local="${7}"  # オフセット(ローカルサーバ)
	local offset_remote="${8}" # オフセット(リモートサーバ)
	local tid_local="${9}"     # タイムラインID(ローカルサーバ)
	local tid_remote="${10}"   # タイムラインID(リモートサーバ)
	local datetime="${11}"     # 現在日時
	
	local prog_local=           # 簡易グラフで使用する「*」文字列(ローカルサーバ)
	local prog_remote=          # 簡易グラフで使用する「*」文字列(リモートサーバ)
	
	##
	## 簡易グラフで表示する「*」文字列を構築
	## (1)セグメントIDを10進数に変換する。
	## (2)254(0xFE)の場合に「*」が24個(最大個数)となる条件を満たす数式で
	##    出力すべき「*」の数を取得
	##
	segid_local=`printf "%d" "0x${segid_local}"`
	segid_remote=`printf "%d" "0x${segid_remote}"`

	local num_local=`echo "${segid_local} / ${PROGRESS_EXPR}" | bc`
	local num_remote=`echo "${segid_remote} / ${PROGRESS_EXPR}" | bc`
	local cnt
	
	# 「*」文字列を構築 (ローカルサーバ)
	cnt=0
	while [ ${cnt} -lt ${num_local} ]
	do
		prog_local="${prog_local}*"
		cnt=`expr ${cnt} + 1`
	done
	
	# 「*」文字列を構築 (リモートサーバ)
	cnt=0
	while [ ${cnt} -lt ${num_remote} ]
	do
		prog_remote="${prog_remote}*"
		cnt=`expr ${cnt} + 1`
	done
	
	# 桁揃え
	prog_local="`printf "%-25s" "${prog_local}"`"
	prog_remote="`printf "%-25s" "${prog_remote}"`"
	
	##
	## トランザクション進捗状況を標準出力に出力
	## ・表示形式：タイムラインID＋ファイル名
	##
	echo "[DATE]"
	echo "${datetime}"
	echo "[TIME LINE ID]"
	echo "${tid_local} ... ${host_local}"
	echo "${tid_remote} ... ${host_remote}"
	echo "[LSN]"
	echo "${logid_local}/${offset_local} ... ${host_local}"
	echo "${logid_remote}/${offset_remote} ... ${host_remote}"
	echo "[PROGRESS]"
	echo "${prog_local} ... ${host_local}"
	echo "${prog_remote} ... ${host_remote}"
	
	return ${RET_OK}
}

#/**
# * @fn      function print_prog_csv()
# * @brief   トランザクション進捗状況を「タイムラインID＋LSN(CSV)」の形式で表示する。
# *
# *   詳細は以下のとおり。
# *     -# トランザクション進捗状況を画面設計に従って標準出力に出力する。
# *
# * @param[in] host_local    ホスト名(ローカルサーバ)
# * @param[in] host_remote   ホスト名(リモートサーバ)
# * @param[in] logid_local   ログID(ローカルサーバ)
# * @param[in] logid_remote  ログID(リモートサーバ)
# * @param[in] offset_local  オフセット(ローカルサーバ)
# * @param[in] offset_remote オフセット(リモートサーバ)
# * @param[in] tid_local     タイムラインID(ローカルサーバ)
# * @param[in] tid_remote    タイムラインID(リモートサーバ)
# * @param[in] datetime      現在日時
# * @retval  RET_OK(0)       正常終了
# *
# * @version 1.0.0
# * @date    08/02/2007
# * @since   1.0
# */
function print_prog_csv()
{
	##
	## 変数宣言
	##
	local host_local="${1}"    # ホスト名(ローカルサーバ)
	local host_remote="${2}"   # ホスト名(リモートサーバ)
	local logid_local="${3}"   # ログID(ローカルサーバ)
	local logid_remote="${4}"  # ログID(リモートサーバ)
	local offset_local="${5}"  # オフセット(ローカルサーバ)
	local offset_remote="${6}" # オフセット(リモートサーバ)
	local tid_local="${7}"     # タイムラインID(ローカルサーバ)
	local tid_remote="${8}"    # タイムラインID(リモートサーバ)
	local datetime="${9}"      # 現在日時
	
	##
	## トランザクション進捗状況を標準出力に出力
	## ・表示形式：タイムラインID＋ファイル名
	##
	echo "${datetime},${tid_local},${tid_remote},${logid_local}/${offset_local},${logid_remote}/${offset_remote}"
	
	return ${RET_OK}
}

#######################################
## メインルート
##

##
## 変数宣言
##
interval=            # 表示間隔の秒数
mode=                # 表示形式
host_local=          # ホスト名(ローカルサーバ)
actwalhost=          # ホスト名(リモートサーバ) ※設定ファイル
lsn_local=           # LSN(ローカルサーバ)
lsn_remote=          # LSN(リモートサーバ)
logid_local=         # ログID(ローカルサーバ)
logid_remote=        # ログID(リモートサーバ)
segid_local=         # セグメントID(ローカルサーバ)
segid_remote=        # セグメントID(リモートサーバ)
tid_local=           # タイムラインID(ローカルサーバ)
tid_remote=          # タイムラインID(リモートサーバ)
offset_local=        # オフセット(ローカルサーバ)
offset_remote=       # オフセット(リモートサーバ)
datetime=            # 現在日時

##
## trap設定
## ・割り込み(CTRL+C)シグナルを受信した場合、戻り値に正常終了を返却
##
trap "exit ${RET_OK}" 2

##
## コマンド引数(オプション)を解析
##
index=1
nextflg=0
for param in "${@}"
do
	[ ${nextflg} -eq 1 ] && nextflg=0 && continue
	
	case "${param}" in
		'-i')
			# 「-i」(表示間隔)の場合、表示間隔の値を取得
			index=`expr ${index} + 1`
			interval="`eval "echo \\${${index}}"`"
			
			# 表示間隔の値が指定されていることを確認
			if [ -z "${interval}" ] ; then
				echo "[${MSGID_UTIL_ILLEGALPARAM}]: Error invalid parameter \"${MSGWD_INVALIDPARAM}\"" 1>&2
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_INVALIDPARAM}
				usage
				exit ${RET_NG}
			fi
			
			# 値が数値であることを確認
			echo "${interval}" | grep -Pq '^\d+$' && [ ${interval} -gt 0 ]
			
			if [ ${?} -ne ${RET_OK} ] ; then
				echo "[${MSGID_UTIL_ILLEGALPARAM}]: Error invalid parameter \"${MSGWD_INVALIDPARAM}\"" 1>&2
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_INVALIDPARAM}
				usage
				exit ${RET_NG}
			fi
			nextflg=1
			;;
		'-F'|'-L'|'-S')
			# 「-F」(表示形式)の場合、表示形式(タイムラインID＋ファイル名)を設定
			# 「-L」(表示形式)の場合、表示形式(タイムラインID＋LSN)を設定
			# 「-S」(表示形式)の場合、表示形式(タイムラインID＋LSN(CSV))を設定
			# 既に表示形式が指定されていた場合、usageを表示し、異常終了を返却する
			if [ ! -z "${mode}" ] ; then
				echo "[${MSGID_UTIL_ILLEGALPARAM}]: Error invalid parameter \"${MSGWD_INVALIDPARAM}\"" 1>&2
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_INVALIDPARAM}
				usage
				exit ${RET_NG}
			fi
			mode="${param}"
			;;
		'-h')
			# usageを表示し、正常終了を返却する
			usage
			exit ${RET_OK}
			;;
		*)
			# 規定外のオプションが指定された場合、usageを表示し、異常終了を返却する
			echo "[${MSGID_UTIL_ILLEGALPARAM}]: Error invalid parameter \"${MSGWD_INVALIDPARAM}\"" 1>&2
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_INVALIDPARAM}
			usage
			exit ${RET_NG}
			;;
	esac
	index=`expr ${index} + 1`
done

# オプションが未指定の場合、デフォルト値を設定
interval=${interval:="${DEFAULT_INTERVAL}"}
mode=${mode:="${DEFAULT_MODE}"}

##
## 設定ファイル(WALWriter.conf)より、パラメータを取得
## ・prex_msgkey   (メッセージキューのキー値)
## ・actwalhost    (リモートサーバのホスト名 or IPアドレス)
## ・actwaluser    (リモートサーバのログインユーザ名)
## ・actwaldir     (リモートサーバのWALセグメント格納ディレクトリ)
## ・sbywaldir     (ローカルサーバのWALセグメント格納ディレクトリ)
## ・sbyarchivedir (ローカルサーバのアーカイブログ格納ディレクトリ)
## ・actarchivedir (リモートサーバのアーカイブログ格納ディレクトリ)
##
IFS='
'
item_list=(`${PREX_BIN}/ReadConfig "${wwconffile}"`)

if [ ${?} -ne ${RET_OK} ] ; then
	echo "[${MSGID_UTIL_FAILEDTOREADCONFIG}]: could not read configuration file" 1>&2
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_FAILEDTOREADCONFIG} ${0} ${LINENO}
	exit ${RET_NG}
fi

for item in "${item_list[@]}"
do
    eval "${item}"
done
IFS="${DEFAULT_IFS}"

##
## ローカルサーバのホスト名を取得
##
host_local="`hostname`"

while [ 1 ]
do
	##
	## ローカルサーバの終端LSNを取得
	## (1)ACT状況確認を呼び出し、PREXの状態がACTであるかを確認
	##  (1-1)ACTの場合、SQL(SELECT pg_current_xlog_location())を発行して終端LSNを取得
	##  (1-2)ACT以外の場合、最終WALセグメントより終端LSNを取得
	##
	# ACT状況確認を実行
	${PREX_BIN}/actmonitor.sh ${prex_msgkey} -N > /dev/null 2>&1
	
	if [ ${?} -eq ${RET_OK} ] ; then
		# ACTの場合、SQL(SELECT pg_current_xlog_location())を発行して終端LSNを取得
		lsn_local="`psql -A -t -c 'SELECT pg_current_xlog_location()' postgres`"
		ret=${?}
	else
		# ACT以外の場合、最終WALセグメントより終端LSNを取得
		# 終端LSNの取得に失敗した場合、規定回数のリトライを行なう
		retry_cnt=0
		ret=${RET_NG}
		while [ ${retry_cnt} -lt ${LASTWAL_RETRY} ]
		do
			lsn_local="`${PREX_BIN}/LastLSN`"
			
			if [ ${?} -eq ${RET_OK} ] ; then
				# 終端LSNの取得に成功
				ret=${RET_OK}
				break
			fi
			
			sleep ${LASTWAL_RETRY_INTERVAL}
			retry_cnt=`expr ${retry_cnt} + 1`
		done
	fi
	
	if [ ${ret} -ne ${RET_OK} ] ; then
		echo "[${MSGID_UTIL_FAILEDTOGET}]: could not get \"${MSGWD_LASTLSN_LOCAL}\"" 1>&2
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_FAILEDTOGET} ${0} ${LINENO} ${MSGWD_LASTLSN_LOCAL}
		exit ${RET_NG}
	fi
	
	##
	## リモートサーバの終端LSNを取得
	## (1)ACT状況確認を呼び出し、PREXの状態がACTであるかを確認
	##  (1-1)ACTの場合、SQL(SELECT pg_current_xlog_location())を発行して終端LSNを取得
	##  (1-2)ACT以外の場合、最終WALセグメントより終端LSNを取得
	##
	# ネットワーク接続可否を判定
	ping -w 1 -r ${actwalhost} > /dev/null 2>&1
	
	if [ ${?} -ne ${RET_OK} ] ; then
		echo "[${MSGID_UTIL_ERRORNETWORK}]: impossible to connect a remote host \"${actwalhost}\"" 1>&2
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORNETWORK} ${0} ${LINENO} "${actwalhost}"
		exit ${RET_NG}
	fi
	
	# ACT状況確認を実行
	ssh "${actwaluser}@${actwalhost}" "${PREX_BIN}/actmonitor.sh ${prex_msgkey} -N" > /dev/null 2>&1
	
	if [ ${?} -eq ${RET_OK} ] ; then
		# ACTの場合、SQL(SELECT pg_current_xlog_location())を発行して終端LSNを取得
		lsn_remote="`ssh "${actwaluser}@${actwalhost}" "psql -A -t -c 'SELECT pg_current_xlog_location()' postgres"`"
	else
		# ACT以外の場合、最終WALセグメントより終端LSNを取得
		# 終端LSNの取得に失敗した場合、規定回数のリトライを行なう
		retry_cnt=0
		ret=${RET_NG}
		while [ ${retry_cnt} -lt ${LASTWAL_RETRY} ]
		do
			lsn_remote="`ssh "${actwaluser}@${actwalhost}" "${PREX_BIN}/LastLSN"`"
			
			if [ ${?} -eq ${RET_OK} ] ; then
				# 終端LSNの取得に成功
				ret=${RET_OK}
				break
			fi
			
			sleep ${LASTWAL_RETRY_INTERVAL}
			retry_cnt=`expr ${retry_cnt} + 1`
		done
	fi
	
	if [ ${ret} -ne ${RET_OK} ] ; then
		echo "[${MSGID_UTIL_FAILEDTOGET}]: could not get \"${MSGWD_LASTLSN_REMOTE}\"" 1>&2
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_FAILEDTOGET} ${0} ${LINENO} ${MSGWD_LASTLSN_REMOTE}
		exit ${RET_NG}
	fi
	
	##
	## タイムラインIDを取得 (桁数不足を「0」で補完し8桁に整形する)
	##
	# ローカルサーバのタイムラインIDを取得
	tid_local="`pg_controldata | grep -P "^Latest checkpoint\'s TimeLineID:" | \
	sed s/^.\\\+:\\\s*\\\\\\([0-9]\\\\\\|[A-F]\\\+\\\\\\)\\\$/\\\1/`"
	
	if [ -z ${tid_local} ] ; then
		echo "[${MSGID_UTIL_FAILEDTOGET}]: could not get \"${MSGWD_TIMELINEID_LOCAL}\"" 1>&2
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_FAILEDTOGET} ${0} ${LINENO} ${MSGWD_TIMELINEID_LOCAL}
		exit ${RET_NG}
	fi
	
	# リモートサーバのタイムラインIDを取得
	tid_remote="`ssh "${actwaluser}@${actwalhost}" "pg_controldata | grep -P \\"^Latest checkpoint's TimeLineID:\\" | sed \\"s/^.\+:\s*\([0-9]\|[A-F]\+\)\$/\1/\\""`"
	
	if [ -z ${tid_remote} ] ; then
		echo "[${MSGID_UTIL_FAILEDTOGET}]: could not get \"${MSGWD_TIMELINEID_REMOTE}\"" 1>&2
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_FAILEDTOGET} ${0} ${LINENO} ${MSGWD_TIMELINEID_REMOTE}
		exit ${RET_NG}
	fi
	
	# 8桁に整形 (桁数不足を「0」で補完)
	tid_local="`printf "%08X" "0x${tid_local}"`"
	tid_remote="`printf "%08X" "0x${tid_remote}"`"
	
	##
	## 終端LSNの文字列からログIDとセグメントIDとオフセットを取得
	## ・終端LSNの左辺の値(ログID)を取得 (桁数不足を「0」で補完し、8桁に整形)
	## ・終端LSNの右辺の値(オフセット)を取得 (桁数不足を「0」で補完し、8桁に整形)
	## ・終端LSNの右辺の値を10進数に変換し、WALセグメントサイズで
	##   割算した値(セグメントID)を取得 (桁数不足を「0」で補完し、8桁に整形)
	##
	logid_local="`echo "${lsn_local}" | cut -d '/' -f 1`"
	logid_remote="`echo "${lsn_remote}" | cut -d '/' -f 1`"
	segid_local="`echo "${lsn_local}" | cut -d '/' -f 2`"
	segid_remote="`echo "${lsn_remote}" | cut -d '/' -f 2`"
	
	# ログIDを8桁に整形(桁数不足を「0」で補完)
	logid_local="`printf "%08X" "0x${logid_local}"`"
	logid_remote="`printf "%08X" "0x${logid_remote}"`"
	
	# オフセットを8桁に整形(桁数不足を「0」で補完)
	offset_local="`printf "%08X" "0x${segid_local}"`"
	offset_remote="`printf "%08X" "0x${segid_remote}"`"
	
	# セグメントIDを取得し、8桁に整形(桁数不足を「0」で補完)
	segid_local=`printf "%lu" "0x${segid_local}"`
	segid_local="`echo "${segid_local} / ${WAL_SEGMENT_SIZE}" | bc`"
	segid_local="`printf "%08X" "${segid_local}"`"
	
	segid_remote=`printf "%lu" "0x${segid_remote}"`
	segid_remote="`echo "${segid_remote} / ${WAL_SEGMENT_SIZE}" | bc`"
	segid_remote="`printf "%08X" "${segid_remote}"`"
	
	##
	## 現在の日時を取得
	##
	datetime="`date "+%F %T"`"
	
	##
	## 表示間隔が指定されていて表示形式が「-S」以外の場合は、画面クリアを実行する
	##
	[ ${interval} -ne 0 ] && [ "${mode}" != "${MODE_CSV}" ] && clear

	##
	## トランザクション進捗情報を出力する
	## (A)表示形式「-F」の場合
	##  (A-1)タイムラインID、ログID、セグメントIDより、最終WALセグメントのファイル名を構成する
	##  (A-2)セグメントIDより、簡易グラフを構成する
	##  (A-3)画面設計(タイムラインID＋ファイル名)に従った内容を標準出力に出力する
	##
	## (B)表示形式「-L」の場合
	##  (B-1)セグメントIDより、簡易グラフを構成する
	##  (B-2)画面設計(タイムラインID＋LSN)に従った内容を標準出力に出力する
	##
	## (C)表示形式「-S」の場合
	##  (C-1)画面設計(タイムラインID＋LSN(CSV))に従った内容を標準出力に出力する
	##
	case "${mode}" in
		"${MODE_FILE}")
			print_prog_file "${host_local}" "${actwalhost}" "${logid_local}" "${logid_remote}" "${segid_local}" "${segid_remote}" "${tid_local}" "${tid_remote}" "${datetime}"
			;;
		"${MODE_LSN}")
			print_prog_lsn "${host_local}" "${actwalhost}" "${logid_local}" "${logid_remote}" "${segid_local}" "${segid_remote}" "${offset_local}" "${offset_remote}" "${tid_local}" "${tid_remote}" "${datetime}"
			;;
		"${MODE_CSV}")
			print_prog_csv "${host_local}" "${actwalhost}" "${logid_local}" "${logid_remote}" "${offset_local}" "${offset_remote}" "${tid_local}" "${tid_remote}" "${datetime}"
			;;
	esac
	
	##
	## (A)表示間隔が未指定の場合、ループを抜ける
	## (B)表示間隔が指定されている場合、ループ処理を繰り返す
	##
	[ ${interval} -eq 0 ] && break
	
	# 表示間隔の秒数待機
	sleep ${interval}
done

exit ${RET_OK}
