#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2007 NTT
#/**
# * @file    prex_shutdown.sh
# * @brief   運用コマンド - 計画停止コマンド(評価用)
# *
# * 高可用化ソフトウェアに、計画停止を指示する。
# * 
# *   詳細は以下のとおり。
# *     -# コマンド引数(オプション)を解析し、XLOG同期待ちのタイムアウト時間を取得する。
# *       - 未定義のオプションがしてされていた場合、usageを出力して戻り値に異常終了を返却する。
# *     -# オプションに「-h」が指定されていた場合、usageを出力して戻り値に正常終了を返却する。
# *     -# 設定ファイル(WALWriter.conf)より、パラメータを取得する。取得するパラメータを以下に示す。
# *       - actwalhost (リモートサーバのホスト名 or IPアドレス)
# *       - actwaluser (リモートサーバのログインユーザ名)
# *     -# ACT状況確認、SBY状況確認を実行し、ACT-SBY構成であることを確認する。
# *       - ローカルサーバに対して、ACT状況確認およびSBY状況確認を実行する。
# *       - リモートサーバに対して、ACT状況確認およびSBY状況確認を実行する。
# *     -# ACTサーバに対してSQL(CHECKPOINT)を発行する。
# *     -# ACTのサーバでローカル・リモートLSN比較コマンドを実行し、XLOG同期が完了するまで待機する。待機時間がタイムアウトに設定された時間を超えた場合は戻り値に異常終了を返却する。
# *     -# SBYサーバの高可用化ソフトウェアを停止する。
# *     -# ACTサーバの高可用化ソフトウェアを停止する。
# *
# * @param[in] -t timeout XLOG同期のタイムアウト時間
# * @param[in] -h         usage表示
# * @retval    RET_OK(0)  正常終了
# * @retval    RET_NG(1)  異常終了
# *
# * @version 1.0.0
# * @date    08/02/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

##
## 定数
##
DEFAULT_TIMEOUT=0     # XLOG同期待ちタイムアウトのデフォルト値(0：タイムアウト無効)
DEFAULT_IFS="${IFS}"  # IFSのデフォルト
ACTHOST_LOCAL="ACTHOST_LOCAL"        # ACTサーバ認識フラグ (ACTサーバがローカルサーバであることを示す)
ACTHOST_REMOTE="ACTHOST_REMOTE"      # ACTサーバ認識フラグ (ACTサーバがリモートサーバであることを示す)
CHECK_INTERVAL=5      # XLOG同期チェックの間隔

#/**
# * @fn      function usage()
# * @brief   usageを表示する。
# *
# *   詳細は以下のとおり。
# *     -# 標準出力にusage文字列を表示する。
# *
# * @retval  RET_OK(0) 正常終了
# *
# * @version 1.0.0
# * @date    08/02/2007
# * @since   1.0
# */
function usage()
{
	echo 'Usage:'
	echo '  prex_shutdown.sh [OPTIONS]'
	echo ''
	echo 'Options:'
	echo '  -t timeout  set timeout seconds'
	echo '  -h  display this help and exit'
	
	return ${RET_OK}
}

#######################################
## メインルート
##

##
## 変数宣言
##
timeout=${DEFAULT_TIMEOUT}  # XLOG同期待ちのタイムアウト時間
acthost=                    # ACTサーバ認識フラグ (0:ローカルサーバ, 1:リモートサーバ)
lsn_act=                    # 終端LSN(ACT)
lsn_sby=                    # 終端LSN(SBY)
interval=${CHECK_INTERVAL}  # XLOG同期チェックの間隔

##
## コマンド引数(オプション)を解析
##
index=1
nextflg=0
for param in "${@}"
do
	[ ${nextflg} -eq 1 ] && nextflg=0 && continue
	
	case "${param}" in
		'-t')
			# 「-t」(XLOG同期待ちのタイムアウト時間)の場合、タイムアウト時間を取得
			index=`expr ${index} + 1`
			timeout="`eval "echo \\${${index}}"`"
			
			# タイムアウト時間が指定されていることを確認
			if [ -z "${timeout}" ] ; then
				echo "[${MSGID_UTIL_ILLEGALPARAM}]: Error invalid parameter \"${MSGWD_INVALIDPARAM}\"" 1>&2
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_INVALIDPARAM}
				usage
				exit ${RET_NG}
			fi
			
			# 数値であることを確認
			echo "${timeout}" | grep -Pq '^\d+$' && [ ${timeout} -gt 0 ]
			
			if [ ${?} -ne ${RET_OK} ] ; then
				echo "[${MSGID_UTIL_ILLEGALPARAM}]: Error invalid parameter \"${MSGWD_INVALIDPARAM}\"" 1>&2
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_INVALIDPARAM}
				usage
				exit ${RET_NG}
			fi
			
			# タイムアウト時間がXLOG同期チェックの間隔以下の場合は、
			# XLOG同期チェックの間隔を1秒に設定
			[ ${timeout} -le ${CHECK_INTERVAL} ] && interval=1
			
			nextflg=1
			;;
		'-h')
			# usageを表示し、正常終了を返却する
			usage
			exit ${RET_OK}
			;;
		*)
			# 規定外のオプションが指定された場合、usageを表示し、異常終了を返却する
			echo "[${MSGID_UTIL_ILLEGALPARAM}]: Error invalid parameter \"${MSGWD_INVALIDPARAM}\"" 1>&2
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_INVALIDPARAM}
			usage
			exit ${RET_NG}
			;;
	esac
	index=`expr ${index} + 1`
done

##
## 設定ファイル(WALWriter.conf)より、パラメータを取得
## ・prex_msgkey (メッセージキューのキー値)
## ・actwalhost  (リモートサーバのホスト名 or IPアドレス)
## ・actwaluser  (リモートサーバのログインユーザ名)
##
IFS='
'
item_list=(`${PREX_BIN}/ReadConfig "${wwconffile}"`)

if [ ${?} -ne ${RET_OK} ] ; then
	echo "[${MSGID_UTIL_FAILEDTOREADCONFIG}]: could not read configuration file" 1>&2
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_FAILEDTOREADCONFIG} ${0} ${LINENO}
	exit ${RET_NG}
fi

for item in "${item_list[@]}"
do
	eval "${item}"
done
IFS="${DEFAULT_IFS}"

##
## ACT-SBY構成であることを確認
## (1)ローカルサーバに対して、ACT状況確認およびSBY状況確認を実行
## (2)リモートサーバに対して、ACT状況確認およびSBY状況確認を実行
##
# ネットワーク接続可否を判定
ping -w 1 -r ${actwalhost} > /dev/null 2>&1

if [ ${?} -ne ${RET_OK} ] ; then
	echo "[${MSGID_UTIL_ERRORNETWORK}]: impossible to connect a remote host \"${actwalhost}\"" 1>&2
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORNETWORK} ${0} ${LINENO} "${actwalhost}"
	exit ${RET_NG}
fi

# ACT状況確認を実行 (ローカルサーバ)
 ${PREX_BIN}/actmonitor.sh ${prex_msgkey} -N > /dev/null 2>&1

if [ ${?} -eq ${RET_OK} ] ; then
	# ローカルサーバがACTの場合
	# リモートサーバに対してSBY状況確認を実行
	ssh "${actwaluser}@${actwalhost}" "${PREX_BIN}/sbymonitor.sh ${prex_msgkey} -N" > /dev/null 2>&1
	
	if [ ${?} -ne ${RET_OK} ] ; then
		# リモートサーバがSBY以外の場合、戻り値に異常終了を返却
		echo "[${MSGID_UTIL_ERRORSTATENOTMATCH}]: the state of PREX is incompatible to the state transition" 1>&2
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORSTATENOTMATCH} ${0} ${LINENO}
		exit ${RET_NG}
	fi
	acthost=${ACTHOST_LOCAL}
else
	# ローカルサーバがACT以外の場合
	# ローカルサーバに対してSBY状況確認を実行
	 ${PREX_BIN}/sbymonitor.sh ${prex_msgkey} -N > /dev/null 2>&1
	
	if [ ${?} -ne ${RET_OK} ] ; then
		# ローカルサーバがOUSの場合、戻り値に異常終了を返却
		echo "[${MSGID_UTIL_ERRORSTATENOTMATCH}]: the state of PREX is incompatible to the state transition" 1>&2
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORSTATENOTMATCH} ${0} ${LINENO}
		exit ${RET_NG}
	fi
	
	# ローカルサーバがSBYの場合
	# リモートサーバに対して、ACT状況確認を実行
	ssh "${actwaluser}@${actwalhost}" "${PREX_BIN}/actmonitor.sh ${prex_msgkey} -N" > /dev/null 2>&1
	
	if [ ${?} -ne ${RET_OK} ] ; then
		# リモートサーバがACT以外の場合、戻り値に異常終了を返却
		echo "[${MSGID_UTIL_ERRORSTATENOTMATCH}]: the state of PREX is incompatible to the state transition" 1>&2
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORSTATENOTMATCH} ${0} ${LINENO}
		exit ${RET_NG}
	fi
	acthost=${ACTHOST_REMOTE}
fi

##
## チェックポイントを実行
## ・ACTサーバに対してSQL(CHECKPOINT)を発行
##
case "${acthost}" in
	"${ACTHOST_LOCAL}")
		# ACTサーバがローカルサーバの場合
		psql -c 'CHECKPOINT' postgres > /dev/null
		;;
	"${ACTHOST_REMOTE}")
		# ACTサーバがリモートサーバの場合
		ssh "${actwaluser}@${actwalhost}" "psql -c 'CHECKPOINT' postgres > /dev/null"
		;;
esac

if [ ${?} -ne ${RET_OK} ] ; then
	echo "[${MSGID_UTIL_ERRORPSQL}]: psql command returned error" 1>&2
	 ${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORPSQL} ${0} ${LINENO}
	exit ${RET_NG}
fi

##
## ローカル・リモートLSN比較コマンドを実行し、XLOG同期が完了するまで待機
## ※待機時間がタイムアウトに設定された時間を超えた場合は戻り値に異常終了を返却
##
basetime=`date +%s` # タイムアウト判定用の基準時間
while [ 1 ]
do
	# ACTサーバおよびSBYサーバの終端LSNを取得
	# ・ACTサーバ：SQL(SELECT pg_current_xlog_location())を発行して終端LSNを取得
	# ・SBYサーバ：最終WALセグメントより終端LSNを取得
	case "${acthost}" in
		"${ACTHOST_LOCAL}")
			# ACTサーバがローカルサーバの場合
			lsn_act="`psql -A -t -c 'SELECT pg_current_xlog_location()' postgres`"
			
			if [ ${?} -ne ${RET_OK} ] ; then
				echo "[${MSGID_UTIL_FAILEDTOGET}]: could not get \"${MSGWD_LASTLSN_LOCAL}\"" 1>&2
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_FAILEDTOGET} ${0} ${LINENO} ${MSGWD_LASTLSN_LOCAL}
				exit ${RET_NG}
			fi
			
			retry_cnt=0
			ret=${RET_NG}
			while [ ${retry_cnt} -lt ${LASTWAL_RETRY} ]
			do
				lsn_sby="`ssh "${actwaluser}@${actwalhost}" "${PREX_BIN}/LastLSN"`"
				
				if [ ${?} -eq ${RET_OK} ] ; then
					# 終端LSNの取得に成功
					ret=${RET_OK}
					break
				fi
				
				sleep ${LASTWAL_RETRY_INTERVAL}
				retry_cnt=`expr ${retry_cnt} + 1`
			done
			
			if [ ${ret} -ne ${RET_OK} ] ; then
				echo "[${MSGID_UTIL_FAILEDTOGET}]: could not get \"${MSGWD_LASTLSN_REMOTE}\"" 1>&2
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_FAILEDTOGET} ${0} ${LINENO} ${MSGWD_LASTLSN_REMOTE}
				exit ${RET_NG}
			fi
			
			;;
		"${ACTHOST_REMOTE}")
			# ACTサーバがリモートサーバの場合
			lsn_act="`ssh "${actwaluser}@${actwalhost}" "psql -A -t -c 'SELECT pg_current_xlog_location()' postgres"`"
			
			if [ ${?} -ne ${RET_OK} ] ; then
				echo "[${MSGID_UTIL_FAILEDTOGET}]: could not get \"${MSGWD_LASTLSN_REMOTE}\"" 1>&2
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_FAILEDTOGET} ${0} ${LINENO} ${MSGWD_LASTLSN_REMOTE}
				exit ${RET_NG}
			fi
			
			retry_cnt=0
			ret=${RET_NG}
			while [ ${retry_cnt} -lt ${LASTWAL_RETRY} ]
			do
				lsn_sby="`${PREX_BIN}/LastLSN`"
				
				if [ ${?} -eq ${RET_OK} ] ; then
					# 終端LSNの取得に成功
					ret=${RET_OK}
					break
				fi
				
				sleep ${LASTWAL_RETRY_INTERVAL}
				retry_cnt=`expr ${retry_cnt} + 1`
			done
			
			if [ ${ret} -ne ${RET_OK} ] ; then
				echo "[${MSGID_UTIL_FAILEDTOGET}]: could not get \"${MSGWD_LASTLSN_LOCAL}\"" 1>&2
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_FAILEDTOGET} ${0} ${LINENO} ${MSGWD_LASTLSN_LOCAL}
				exit ${RET_NG}
			fi
			
			;;
	esac
	
	# LSNが同一であるか判定
	[ "${lsn_act}" = "${lsn_sby}" ] && break
	
	# タイムアウトを判定
	if [ ${timeout} -ne 0 ] ; then
		currtime=`date +%s`
		time=`expr ${currtime} - ${basetime}`
		
		if [ ${time} -gt ${timeout} ] ; then
			echo "[${MSGID_UTIL_XLOGSYNCTIMEOUT}]: synchronous waiting of xlog did the time-out" 1>&2
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_XLOGSYNCTIMEOUT} ${0} ${LINENO}
			exit ${RET_NG}
		fi
	fi
done

##
## 以下の順序で、高可用化ソフトウェアより停止指示を実行
## (1)SBY側サーバ
## (2)ACT側サーバ
##
case "${acthost}" in
	"${ACTHOST_LOCAL}")
		# ACTサーバがローカルサーバの場合
		# リモートサーバ、ローカルサーバの順序で停止
		ssh "${actwaluser}@${actwalhost}" "sudo ${CMDNAME_HEARTBEAT} stop"
		
		if [ ${?} -ne ${RET_OK} ] ; then
			echo "[${MSGID_UTIL_ERRORHASTOP}]: heartbeat command returned error (host = \"${acthost}\")" 1>&2
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORHASTOP} ${0} ${LINENO} "${acthost}"
			exit ${RET_NG}
		fi
		
		sudo ${CMDNAME_HEARTBEAT} stop
		
		if [ ${?} -ne ${RET_OK} ] ; then
			echo "[${MSGID_UTIL_ERRORHASTOP}]: heartbeat command returned error (host = \"localhost\")" 1>&2
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORHASTOP} ${0} ${LINENO} "localhost"
			exit ${RET_NG}
		fi
		;;
	"${ACTHOST_REMOTE}")
		# ACTサーバがリモートサーバの場合
		sudo ${CMDNAME_HEARTBEAT} stop
		
		if [ ${?} -ne ${RET_OK} ] ; then
			echo "[${MSGID_UTIL_ERRORHASTOP}]: heartbeat command returned error (host = \"localhost\")" 1>&2
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORHASTOP} ${0} ${LINENO} "localhost"
			exit ${RET_NG}
		fi
		
		ssh "${actwaluser}@${actwalhost}" "sudo ${CMDNAME_HEARTBEAT} stop"
		
		if [ ${?} -ne ${RET_OK} ] ; then
			echo "[${MSGID_UTIL_ERRORHASTOP}]: heartbeat command returned error (host = \"${acthost}\")" 1>&2
			${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORHASTOP} ${0} ${LINENO} "${acthost}"
			exit ${RET_NG}
		fi
		;;
esac

exit ${RET_OK}
