#!/bin/sh
#####################################################################
# Copyright   : Copyright (C) 2007 NTT
#/**
# * @file    prex_switch_over.sh
# * @brief   運用コマンド - スイッチオーバコマンド(評価用)
# *
# * 高可用化ソフトウェアに、スイッチオーバを指示する。
# * 
# *   詳細は以下のとおり。
# *     -# コマンド引数(オプション)を解析し、ACTサーバのノード名を取得する。
# *       - 未定義のオプションがしてされていた場合、usageを出力して戻り値に異常終了を返却する。
# *     -# オプションに「-h」が指定されていた場合、usageを出力して戻り値に正常終了を返却する。
# *     -# 設定ファイル(WALWriter.conf)より、パラメータを取得する。取得するパラメータを以下に示す。
# *       - actwalhost (リモートサーバのホスト名 or IPアドレス)
# *       - actwaluser (リモートサーバのログインユーザ名)
# *     -# ACT状況確認、SBY状況確認を実行し、ACT-SBY構成であることを確認する。
# *       - ローカルサーバに対して、ACT状況確認およびSBY状況確認を実行する。
# *       - リモートサーバに対して、ACT状況確認およびSBY状況確認を実行する。
# *     -# 引数より取得したノードがACTサーバのノード名と一致することを確認する。
# *     -# ACTサーバのノードを指定して、高可用化ソフトウェアのスイッチオーバ指示を実行する。
# *
# * @param[in] actnode   ACTサーバのノード名
# * @param[in] -h        usage表示
# * @retval    RET_OK(0) 正常終了
# * @retval    RET_NG(1) 異常終了
# *
# * @version 1.0.0
# * @date    08/02/2007
# * @since   1.0
# */

# DEFINE値取り込み
. ${HOME}/prex/bin/prexdefine.sh

##
## 定数
##
DEFAULT_IFS="${IFS}"             # IFSのデフォルト
ACT_MONITOR_INTERVAL=1           # スイッチオーバ時のACT遷移監視間隔
ACTHOST_LOCAL="ACTHOST_LOCAL"    # ACTサーバ認識フラグ (ACTサーバがローカルサーバであることを示す)
ACTHOST_REMOTE="ACTHOST_REMOTE"  # ACTサーバ認識フラグ (ACTサーバがリモートサーバであることを示す)

#/**
# * @fn      function usage()
# * @brief   usageを表示する。
# *
# *   詳細は以下のとおり。
# *     -# 標準出力にusage文字列を表示する。
# *
# * @retval  RET_OK(0) 正常終了
# *
# * @version 1.0.0
# * @date    08/02/2007
# * @since   1.0
# */
function usage()
{
	echo 'Usage:'
	echo '  prex_switch_over.sh [OPTIONS] <ACT server node name>'
	echo ''
	echo 'Options:'
	echo '  -h  display this help and exit'
	
	return ${RET_OK}
}

#######################################
## メインルート
##

##
## 変数宣言
##
actnode=      # ACTサーバのノード名
acthost=      # ACTサーバ認識フラグ
acthostname=  # ACTサーバのホスト名

##
## コマンド引数(オプション)を解析
##
index=1
for param in "${@}"
do
	case "${param}" in
		'-h')
			# usageを表示し、正常終了を返却する
			usage
			exit ${RET_OK}
			;;
		*)
			# 既に引数が指定されていた場合、usageを表示し、異常終了を返却する
			if [ ! -z "${actnode}" ] ; then
				echo "[${MSGID_UTIL_ILLEGALPARAM}]: Error invalid parameter \"${MSGWD_INVALIDPARAM}\"" 1>&2
				${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_INVALIDPARAM}
				usage
				exit ${RET_NG}
			fi
			actnode="${param}"
			;;
	esac
	index=`expr ${index} + 1`
done

if [ -z "${actnode}" ] ; then
	echo "[${MSGID_UTIL_ILLEGALPARAM}]: Error invalid parameter \"${MSGWD_INVALIDPARAM}\"" 1>&2
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ILLEGALPARAM} ${0} ${LINENO} ${MSGWD_INVALIDPARAM}
	usage
	exit ${RET_NG}
fi

##
## 設定ファイル(WALWriter.conf)より、パラメータを取得
## ・prex_msgkey (メッセージキューのキー値)
## ・actwalhost  (リモートサーバのホスト名 or IPアドレス)
## ・actwaluser  (リモートサーバのログインユーザ名)
##
IFS='
'
item_list=(`${PREX_BIN}/ReadConfig "${wwconffile}"`)

if [ ${?} -ne ${RET_OK} ] ; then
	echo "[${MSGID_UTIL_FAILEDTOREADCONFIG}]: could not read configuration file" 1>&2
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_FAILEDTOREADCONFIG} ${0} ${LINENO}
	exit ${RET_NG}
fi

for item in "${item_list[@]}"
do
    eval "${item}"
done
IFS="${DEFAULT_IFS}"

##
## ACT-SBY構成であることを確認
## (1)ローカルサーバに対して、ACT状況確認およびSBY状況確認を実行
## (2)リモートサーバに対して、ACT状況確認およびSBY状況確認を実行
##
# ネットワーク接続可否を判定
ping -w 1 -r ${actwalhost} > /dev/null 2>&1

if [ ${?} -ne ${RET_OK} ] ; then
	echo "[${MSGID_UTIL_ERRORNETWORK}]: impossible to connect a remote host \"${actwalhost}\"" 1>&2
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORNETWORK} ${0} ${LINENO} "${actwalhost}"
	exit ${RET_NG}
fi

# ACT状況確認を実行 (ローカルサーバ)
${PREX_BIN}/actmonitor.sh ${prex_msgkey} -N > /dev/null 2>&1

if [ ${?} -eq ${RET_OK} ] ; then
	# ローカルサーバがACTの場合
	# リモートサーバに対してSBY状況確認を実行
	ssh "${actwaluser}@${actwalhost}" "${PREX_BIN}/sbymonitor.sh ${prex_msgkey} -N" > /dev/null 2>&1
	
	if [ ${?} -ne ${RET_OK} ] ; then
		# リモートサーバがSBY以外の場合、戻り値に異常終了を返却
		echo "[${MSGID_UTIL_ERRORSTATENOTMATCH}]: the state of PREX is incompatible to the state transition" 1>&2
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORSTATENOTMATCH} ${0} ${LINENO}
		exit ${RET_NG}
	fi
	
	acthost=${ACTHOST_LOCAL}
	acthostname="`uname -n`"
else
	# ローカルサーバがACT以外の場合
	# ローカルサーバに対してSBY状況確認を実行
	 ${PREX_BIN}/sbymonitor.sh ${prex_msgkey} -N > /dev/null 2>&1
	
	if [ ${?} -ne ${RET_OK} ] ; then
		# ローカルサーバがOUSの場合、戻り値に異常終了を返却
		echo "[${MSGID_UTIL_ERRORSTATENOTMATCH}]: the state of PREX is incompatible to the state transition" 1>&2
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORSTATENOTMATCH} ${0} ${LINENO}
		exit ${RET_NG}
	fi
	
	# ローカルサーバがSBYの場合
	# リモートサーバに対して、ACT状況確認を実行
	ssh "${actwaluser}@${actwalhost}" "${PREX_BIN}/actmonitor.sh ${prex_msgkey} -N" > /dev/null 2>&1
	
	if [ ${?} -ne ${RET_OK} ] ; then
		# リモートサーバがACT以外の場合、戻り値に異常終了を返却
		echo "[${MSGID_UTIL_ERRORSTATENOTMATCH}]: the state of PREX is incompatible to the state transition" 1>&2
		${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORSTATENOTMATCH} ${0} ${LINENO}
		exit ${RET_NG}
	fi
	
	acthost=${ACTHOST_REMOTE}
	acthostname="`ssh "${actwaluser}@${actwalhost}" "uname -n"`"
fi

##
## 引数のノード(ホスト)がACTであることを確認
##
if [ "${acthostname}" != "${actnode}" ] ; then
	echo "[${MSGID_UTIL_ILLEGALPARAM}]: Error invalid parameter \"specified node is not ACT node\"" 1>&2
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ILLEGALPARAM} ${0} ${LINENO} "specified node is not ACT node"
	exit ${RET_NG}
fi

##
## 高可用化ソフトウェアのスイッチオーバ指示を実行
##
sudo ${CMDNAME_CRMSTANDBY} -U ${actnode} -v on

if [ ${?} -ne ${RET_OK} ] ; then
	echo "[${MSGID_UTIL_ERRORHASWITCHOVER}]: crm_standby command returned error" 1>&2
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORHASWITCHOVER} ${0} ${LINENO}
	exit ${RET_NG}
fi

# SBY->ACTに遷移するまで待機
# (待機中にスクリプトが終了した場合、STANDBY状態を解除する)
trap "sudo ${CMDNAME_CRMSTANDBY} -U ${actnode} -D &" 0

while [ 1 ]
do
	case "${acthost}" in
		"${ACTHOST_LOCAL}")
			# ACTサーバがローカルサーバの場合、
			# リモートサーバがACTに遷移するまで待機
			ssh "${actwaluser}@${actwalhost}" "${PREX_BIN}/actmonitor.sh ${prex_msgkey} -N" > /dev/null 2>&1
			;;
		"${ACTHOST_REMOTE}")
			# ACTサーバがリモートサーバの場合
			# ローカルサーバがACTに遷移するまで待機
			${PREX_BIN}/actmonitor.sh ${prex_msgkey} -N > /dev/null 2>&1
			;;
	esac

	if [ ${?} -eq ${RET_OK} ] ; then
		# SBY->ACTの遷移を確認した場合、ACT状況監視のループを終了
		break
	fi

	sleep ${ACT_MONITOR_INTERVAL}
done

trap 0

sudo ${CMDNAME_CRMSTANDBY} -U ${actnode} -D

if [ ${?} -ne ${RET_OK} ] ; then
	echo "[${MSGID_UTIL_ERRORHASWITCHOVER}]: crm_standby command returned error" 1>&2
	${PREX_BIN}/prexsyslog ${LOG_ERR} ${MSGID_UTIL_ERRORHASWITCHOVER} ${0} ${LINENO}
	exit ${RET_NG}
fi

exit ${RET_OK}
