/*****************************************************************************
   Copyright (C) 2006-2008 NTT
  *****************************************************************************/
/**
 * @file    WWCommon.h
 * @brief   WALWriterスレッド間共通関数の宣言とテーブル及びマクロ定義
 *
 * WALWriterのスレッド間において共通に使用する関数を宣言し、
 * テーブル及びマクロを定義する。
 *
 * @version 1.3.0
 * @date    06/17/2008
 * @since   1.0
 */
#ifndef WWCOMMON_H_
#define WWCOMMON_H_

#include <stdio.h>
#include <pthread.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "PREXDefine.h"
#include "ReadWWConfFile.h"

/*****************
 * 定数定義
 ****************/
/**
 * @def     WAL_BLOCK_SIZE
 * @brief   WALの１ブロック分のサイズ
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define WAL_BLOCK_SIZE    8192

/**
 * @def     PTHREAD_OK
 * @brief   pthread 関数の戻り値（正常）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define PTHREAD_OK        0

/*****************
 * テーブル定義
 ****************/
/**
 * @struct  RegWALInfo_t
 * @brief   登録WAL情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct RegWALInfo_t {
	LSNInfo_t  LSN;                                   ///< 該当するWALに関するLSN情報
	int        nRequiredBlocks;                       ///< 該当するWALが必要とするWALWriter用WALバッファ上のブロック数
} RegWALInfo_t;

/* 200807 -> */
/**
 * @struct  TLIInfo_t
 * @brief   バックアップ取得完了情報
 *
 * @version 1.3.0
 * @date    07/25/2008
 * @since   1.3
 */
typedef struct TLIInfo_t
{
	int               nFlag;                           ///< フラグ
	pthread_mutex_t   mutex;                           ///< ロック
	pthread_cond_t    cond;                            ///< 状態変数
} TLIInfo_t;
/* <- 200807 */

/**
 * @struct  GettingBackupStatusInfo_t
 * @brief   バックアップ取得完了情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct GettingBackupStatusInfo_t
{
	int               nFlag;                           ///< フラグ
	pthread_mutex_t   mutex;                           ///< ロック
	pthread_cond_t    cond;                            ///< 状態変数
} GettingBackupStatusInfo_t;

/**
 * @struct  WALBufferStatusInfo_t
 * @brief   WAL受信待ち状態情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct WALBufferStatusInfo_t {
	int               nFlag;                           ///< フラグ
	pthread_mutex_t   mutex;                           ///< ロック
	pthread_cond_t    cond;                            ///< 状態変数
} WALBufferStatusInfo_t;

/**
 * @struct  OutputWALSegmentInfo_t
 * @brief   出力先WALセグメント情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct OutputWALSegmentInfo_t {
	char    filename[XLOGFILENAME_LEN+PREX_MAX_PATH_LEN+4]; ///< ファイル名
	FILE*   fp;                                        ///< ファイルポインタ
} OutputWALSegmentInfo_t;

/**
 * @struct  CommTargetInfo_t
 * @brief   接続情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct CommTargetInfo_t {
	char*               hostname;                      ///< サーバ名（IPアドレス）
	u_short             port;                          ///< 接続先ポート番号
	struct sockaddr_in  sock_info;                     ///< ソケット情報をもつ構造体
	int                 sndrcv_fd;                     ///< ディスクリプタ（送受信用）
} CommTargetInfo_t;

/**
 * @struct  NewestLSNInfo_t
 * @brief   最新反映終端LSN情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct NewestLSNStatusInfo_t {
	LSNInfo_t                 LSN;        ///< 最新の反映終端 LSN
	int                       nFlag;      ///< フラグ
	pthread_mutex_t           mutex;      ///< 最新の反映終端 LSN のアクセス用 Mutex
	pthread_cond_t            cond;       ///< 状態変数
} NewestLSNStatusInfo_t;

/**
 * @struct  CheckEnableToactInfo_t
 * @brief   ACT化可否コマンド情報
 *
 * @version 1.1.0
 * @date    10/30/2007
 * @since   1.1
 */
typedef struct CheckEnableToactInfo_t {
	int                       nFlag;          ///< ACT化可否コマンド実行中フラグ
	int                       messageEndFlag; ///< messageスレッド終了フラグ
	pthread_mutex_t           mutex;          ///< ACT化可否コマンド実行中フラグのアクセス用 Mutex
	pthread_cond_t            cond;           ///< 状態変数
} CheckEnableToactInfo_t;

/**
 * @struct  WWCommonInfo_t
 * @brief   スレッド間共有情報
 *
 * @version 1.3.0
 * @date    06/06/2008
 * @since   1.0
 */
typedef struct WWCommonInfo_t {
	char*                     pWALBuffer;             ///< WALWriter用WALバッファの先頭アドレス
	int                       ww_blocks;              ///< WALWriter用WALバッファ上のブロック数
	pthread_mutex_t*          pBufferMutexList;       ///< WALWriter用WALバッファへのアクセス用ロックリスト
	RegWALInfo_t*             pWALInfoList;           ///< 登録WAL情報リスト
	int                       prex_msgkey;            ///< メッセージキューのキー値
	int                       nFlgInit;               ///< 初期起動モードフラグ
	int                       nFlgInitOrg;            ///< 初期起動モードフラグの初期値（for バックアップ取得スレッド）
	int                       nFlgToACT;              ///< ACT化処理中フラグ
	int                       nFlgToSBY;              ///< SBY化処理中フラグ
	int                       nFlgSendFilesCompleted; ///< ファイル転送情報通知完了フラグ
	GettingBackupStatusInfo_t GettingBackupInfo;      ///< バックアップ取得完了情報
	WALBufferStatusInfo_t     WALBufferInfo;          ///< WAL受信待ち状態情報
	NewestLSNStatusInfo_t     NewestLSNInfo;          ///< 最新反映終端LSN情報
	CommTargetInfo_t          commTarget;             ///< 接続情報
	CheckEnableToactInfo_t    CheckEnableToactInfo;   ///< ACT化可否コマンド情報
	char*                     pBlank;                 ///< 0 クリア領域を作成する際のテンプレート領域

	pthread_mutex_t           lockTermWW;             ///< 終了処理の排他制御に使用するmutex
	/* 200806 -> */
	uint32                    nTimeLineID;            ///< タイムライン番号
	CommHistoryInfo_t         recvHistoryInfo;        ///< history情報
	TLIInfo_t                 TLIInfo;                ///< TLI通知完了情報
	/* <- 200806 */
	
} WWCommonInfo_t;

/**
 * @struct  WWCommThreadInfo_t
 * @brief   通信スレッド情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct WWCommThreadInfo_t {
	int            nTargetPos;              ///< 格納先ブロック位置
	LSNInfo_t      SendStartLSN;            ///< 送信開始LSN
} WWCommThreadInfo_t;

/**
 * @struct  WWWriterThreadInfo_t
 * @brief   ディスク書き込みスレッド情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct WWWriterThreadInfo_t {
	int                         nTargetPos;         ///< 読み込みブロック位置（初期値は（ww_buffers-1））
	OutputWALSegmentInfo_t      pWALSegmentInfo;    ///< 出力先最終WALセグメント情報
	int                         nMsgqID;            ///< メッセージキューID
} WWWriterThreadInfo_t;

/**
 * @struct  WWGetterThreadInfo_t
 * @brief   バックアップ取得スレッド情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct WWGetterThreadInfo_t {
	int              nFlgInit;              ///< 初期起動フラグ
	LSNInfo_t        SendEndLSN;            ///< ファイル転送終了LSN
	LSNInfo_t        SendStartLSN;          ///< 送信開始LSN
} WWGetterThreadInfo_t;

/**
 * @struct  WWMessageThreadInfo_t
 * @brief   メッセージスレッド情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct WWMessageThreadInfo_t {
	int              nFlgInit;              ///< 初期起動フラグ
	int              nMsgKey;               ///< メッセージキューのキー値
	int              nMsgqID;               ///< メッセージキューID
} WWMessageThreadInfo_t;

/**
 * @struct  WALBlockInfo_t
 * @brief   WALブロック情報
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
typedef struct WALBlockInfo_t
{
	uint16     xlp_magic;                     ///< 未使用
	uint16     xlp_info;                      ///< 未使用
	uint32     nTimeLineID;                   ///< タイムライン番号
	LSNInfo_t  LSN;                           ///< LSN情報
} WALBlockInfo_t;

/************************************************
 * グローバル変数定義（実体は WWCommon.c で定義）
 ***********************************************/
#ifndef SET_GLOBAL_VAR_
	extern WWSettingInfo_t g_WWEnvInfo;
	extern WWCommonInfo_t  g_WWCmnInfo;
#endif /* SET_GLOBAL_VAR_ */

/*******************
 * プロトタイプ宣言
 ******************/
void TermWALWriter();
void LockWALBuffer(int posFrom, int nCount);
void UnlockWALBuffer(int posFrom, int nCount);
void HandleSIGTERM(int nSig);
int SendMessageReplyToACTToQueue(int nMsgQueID, int nResult);
int SendMessageReplyToSBYToQueue(int nMsgQueID, int nResult);
int SendPITRMessageToQueue(int nMsgQueID);

/*****************
 * マクロ関数定義
 ****************/
/**
 * @def     ExitWALWriter(int nCode__)
 * @brief   WALWriterプロセスの終了
 *
 *   プロセスを終了する。
 *
 *   詳細は以下のとおり。
 *     -# プロセスのリソースを解放する（TermWALWriter 関数）。
 *     -# exit()を呼び出し、プロセスを終了する。
 *
 * @param[in]   nCode__        WALWriterコマンドとしての戻り値。
 *
 * @return      なし
 *
 * @note  exit()を呼び出した際に、呼出元以外のスレッドには SIGTERM が送られる。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define ExitWALWriter(nCode__) \
{\
	TermWALWriter();\
	exit(nCode__);\
}

/**
 * @def     GetXLogPathName(char* path__, uint32 tli__, LSNInfo_t* pLSN__)
 * @brief   WALセグメント名の取得
 *
 *   指定されたLSNに対応するWALセグメント名を生成する。
 *
 *   詳細は以下のとおり。
 *     -# 指定されたLSNに対応するWALセグメント名を取得する。
 *
 * @param[out] path__ 生成したパスの格納先
 * @param[in]  tli__  タイムライン番号
 * @param[in]  pLSN__ LSN情報
 * @return     なし
 * @attention  呼出元によって、path__がさす領域が確保されていること。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define GetXLogPathName(path__, tli__, pLSN__) \
{\
	snprintf(path__, XLOGFILENAME_LEN+1, "%08X%08X%08X", \
		tli__, (pLSN__)->uXLogID, ((pLSN__)->uXRecOff / WAL_SEG_FILESIZE));\
}

/**
 * @def     GetXLogPathNameUnknownTLI(char* path__, LSNInfo_t* pLSN__)
 * @brief   WALセグメント名の取得（タイムライン部ワイルドカード表現）
 *
 *   指定されたLSNに対応するWALセグメント名を生成する。
 *
 *   詳細は以下のとおり。
 *     -# 指定されたLSNに対応するWALセグメント名を取得する。
 *
 * @param[out] path__ 生成したパスの格納先
 * @param[in]  pLSN__ LSN情報
 * @return     なし
 * @attention  生成するWALセグメント名において、タイムラインの部分はワイルドカード表現（????????）とする。
 * @attention  呼出元によって、path__がさす領域が確保されていること。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define GetXLogPathNameUnknownTLI(path__, pLSN__) \
{\
	snprintf(path__, XLOGFILENAME_LEN+1, "????????%08X%08X", \
		(pLSN__)->uXLogID, ((pLSN__)->uXRecOff / WAL_SEG_FILESIZE));\
}

/* 200806 -> */
/**
 * @def     GetHistoryPathName(char* path__, uint32 tli__)
 * @brief   historyファイル名の取得
 *
 *   指定されたTLIに対応するhistory名を生成する。
 *
 *   詳細は以下のとおり。
 *     -# 指定されたLSNに対応するhistory名を取得する。
 *
 * @param[out] path__ 生成したパスの格納先
 * @param[in]  tli__  タイムライン番号
 * @return     なし
 * @attention  呼出元によって、path__がさす領域が確保されていること。
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
*/
#define GetHistoryPathName(path__, tli__) \
{\
	snprintf(path__, XLOGFILENAME_LEN+1, "%08X.history", tli__);\
}
/* <- 200806 */

/**
 * @def   IsXLogInPrevSeg(LSNInfo_t* xlrp__, uint32 logID__, uint32 logSeg__)
 * @brief セグメント番号の更新確認
 *
 *   指定されたLSNが、指定されたログID及びセグメント番号の組合せに該当するか
 *   どうかを判定する。
 *
 * @param[in]   xlrp__    LSN情報
 * @param[in]   logID__   ログID
 * @param[in]   logSeg__  セグメント番号
 *
 * @note    このマクロは、PostgreSQL における XLByteInPrevSeg() に等しい
 * @retval  ０。xlrp__ が logID__ 及び logSeg__ に該当しない。
 * @retval  非０。xlrp__ が logID__ 及び logSeg__ に該当する。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define IsXLogInPrevSeg(xlrp__, logID__, logSeg__) \
	(((xlrp__)->uXLogID == (logID__)) && \
	 (((xlrp__)->uXRecOff - 1) / WAL_SEG_FILESIZE) == (logSeg__))

/**
 * @def     GetWALInfo(int pos__)
 * @brief   登録WAL情報の取得
 *
 *   指定されたブロック位置に登録されているWAL情報を取得する。
 *
 *   詳細は以下のとおり。
 *     -# 指定されたブロック位置に登録されているWAL情報を取得する。
 *
 * @param[in]   pos__  読み込みブロック位置
 * @note    登録WAL情報リストにおける pos__ 番目のアドレスを返却する。
 * @return  登録WAL情報リストの pos__ 番目のアドレス
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define GetWALInfo(pos__)  ((g_WWCmnInfo.pWALInfoList + pos__))

/**
 * @def     SetBlocksToWALInfo(int pos__, int nBlocks__)
 * @brief   登録WAL情報への使用ブロック数の登録
 *
 *   WAL情報における使用ブロック数を登録WAL情報リストに登録する。
 *
 *   詳細は以下のとおり。
 *     -# 登録WAL情報リスト上の pos__番目のブロック位置に関するWAL情報を
 *        取得する（GetWALInfo 関数）。
 *     -# 1 で取得した登録WAL情報がさす領域内に、使用ブロック数を登録する。
 *
 * @param[in]   pos__      格納先ブロック位置
 * @param[in]   nBlocks__  使用するブロック数
 * @return  なし
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define SetBlocksToWALInfo(pos__, nBlocks__)  { \
	RegWALInfo_t*  pWALInfo__; \
	pWALInfo__ = GetWALInfo(pos__); \
	pWALInfo__->nRequiredBlocks = nBlocks__; \
}

/**
 * @def     SetLSNToWALInfo(int pos__, unsigned int xlogid__, unsigned int xrecoff__)
 * @brief   登録WAL情報へのLSN情報の登録
 *
 *   WAL情報におけるLSN情報を登録WAL情報リストに登録する。
 *
 *   詳細は以下のとおり。
 *     -# 登録WAL情報リスト上の pos__番目のブロック位置に関するWAL情報を
 *        取得する（GetWALInfo 関数）。
 *     -# 1 で取得した登録WAL情報がさす領域内に、LSN情報を登録する。
 *
 * @param[in]   pos__      格納先ブロック位置
 * @param[in]   xlogid__   ログID
 * @param[in]   xrecoff__  オフセット
 * @return  なし
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define SetLSNToWALInfo(pos__, xlogid__, xrecoff__)  { \
	RegWALInfo_t*  pWALInfo__; \
	pWALInfo__ = GetWALInfo(pos__); \
	pWALInfo__->LSN.uXLogID  = xlogid__; \
	pWALInfo__->LSN.uXRecOff = xrecoff__; \
}

/* 200807 -> */
/**
 * @def     InitTLIInfo()
 * @brief   TLI通知完了情報の初期化
 *
 *   TLI通知完了情報の内容を初期化する。
 *
 *   詳細は以下のとおり。
 *     -# TLI通知完了情報のフラグを FALSE で初期化する。
 *     -# TLI通知完了情報の Mutex を初期化する。
 *     -# TLI通知完了情報の状態変数を初期化する。
 *
 * @return  なし
 *
 * @version 1.3.0
 * @date    07/25/2008
 * @since   1.3
*/
#define InitTLIInfo()  { \
	g_WWCmnInfo.TLIInfo.nFlag = FALSE; \
	pthread_mutex_init(&(g_WWCmnInfo.TLIInfo.mutex), NULL); \
	pthread_cond_init(&(g_WWCmnInfo.TLIInfo.cond), NULL); \
}

/**
 * @def     TermTLIInfo()
 * @brief   TLI通知完了情報の破棄
 *
 *   TLI通知完了情報の内容を破棄する。
 *
 *   詳細は以下のとおり。
 *     -# TLI通知完了情報の Mutex を破棄する。
 *     -# TLI通知完了情報の状態変数を破棄する。
 *
 * @return  なし
 *
 * @version 1.3.0
 * @date    07/25/2008
 * @since   1.3
*/
#define TermTLIInfo()  { \
	pthread_mutex_destroy(&(g_WWCmnInfo.TLIInfo.mutex)); \
	pthread_cond_destroy(&(g_WWCmnInfo.TLIInfo.cond)); \
}
/* <- 200807 */

/**
 * @def     InitGettingBackupInfo()
 * @brief   バックアップ取得完了情報の初期化
 *
 *   バックアップ取得完了情報の内容を初期化する。
 *
 *   詳細は以下のとおり。
 *     -# バックアップ取得完了情報のフラグを FALSE で初期化する。
 *     -# バックアップ取得完了情報の Mutex を初期化する。
 *     -# バックアップ取得完了情報の状態変数を初期化する。
 *
 * @return  なし
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define InitGettingBackupInfo()  { \
	g_WWCmnInfo.GettingBackupInfo.nFlag = FALSE; \
	pthread_mutex_init(&(g_WWCmnInfo.GettingBackupInfo.mutex), NULL); \
	pthread_cond_init(&(g_WWCmnInfo.GettingBackupInfo.cond), NULL); \
}

/**
 * @def     TermGettingBackupInfo()
 * @brief   バックアップ取得完了情報の破棄
 *
 *   バックアップ取得完了情報の内容を破棄する。
 *
 *   詳細は以下のとおり。
 *     -# バックアップ取得完了情報の Mutex を破棄する。
 *     -# バックアップ取得完了情報の状態変数を破棄する。
 *
 * @return  なし
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define TermGettingBackupInfo()  { \
	pthread_mutex_destroy(&(g_WWCmnInfo.GettingBackupInfo.mutex)); \
	pthread_cond_destroy(&(g_WWCmnInfo.GettingBackupInfo.cond)); \
}

/**
 * @def     InitWALBufferInfo()
 * @brief   WAL受信待ち状態情報の初期化
 *
 *   WAL受信待ち状態情報の内容を初期化する。
 *
 *   詳細は以下のとおり。
 *     -# WAL受信待ち状態情報のフラグを FALSE で初期化する。
 *     -# WAL受信待ち状態情報の Mutex を初期化する。
 *     -# WAL受信待ち状態情報の状態変数を初期化する。
 *
 * @return  なし
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define InitWALBufferInfo()  { \
	g_WWCmnInfo.WALBufferInfo.nFlag = FALSE; \
	pthread_mutex_init(&(g_WWCmnInfo.WALBufferInfo.mutex), NULL); \
	pthread_cond_init(&(g_WWCmnInfo.WALBufferInfo.cond), NULL); \
}

/**
 * @def     TermWALBufferInfo()
 * @brief   WAL受信待ち状態情報の破棄
 *
 *   WAL受信待ち状態情報の内容を破棄する。
 *
 *   詳細は以下のとおり。
 *     -# WAL受信待ち状態情報の Mutex を破棄する。
 *     -# WAL受信待ち状態情報の状態変数を破棄する。
 *
 * @return  なし
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define TermWALBufferInfo()  { \
	pthread_mutex_destroy(&(g_WWCmnInfo.WALBufferInfo.mutex)); \
	pthread_cond_destroy(&(g_WWCmnInfo.WALBufferInfo.cond)); \
}

/**
 * @def     InitNewestLSNInfo()
 * @brief   最新反映終端LSN情報の初期化
 *
 *   最新反映終端LSN情報の内容を初期化する。
 *
 *   詳細は以下のとおり。
 *     -# 最新反映終端LSN情報のフラグを FALSE で初期化する。
 *     -# 最新反映終端LSN情報の Mutex を初期化する。
 *     -# 最新反映終端LSN情報の状態変数を初期化する。
 *     -# 最新反映終端LSN情報の LSN を初期化する。
 *
 * @return  なし
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define InitNewestLSNInfo()  { \
	g_WWCmnInfo.NewestLSNInfo.nFlag = FALSE; \
	pthread_mutex_init(&(g_WWCmnInfo.NewestLSNInfo.mutex), NULL); \
	pthread_cond_init(&(g_WWCmnInfo.NewestLSNInfo.cond), NULL); \
	InitLSNInfo(&(g_WWCmnInfo.NewestLSNInfo.LSN)); \
}

/**
 * @def     TermNewestLSNInfo()
 * @brief   最新反映終端LSN情報の破棄
 *
 *   最新反映終端LSN情報の内容を破棄する。
 *
 *   詳細は以下のとおり。
 *     -# 最新反映終端LSN情報の Mutex を破棄する。
 *     -# 最新反映終端LSN情報の状態変数を破棄する。
 *
 * @return  なし
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define TermNewestLSNInfo()  { \
	pthread_mutex_destroy(&(g_WWCmnInfo.NewestLSNInfo.mutex)); \
	pthread_cond_destroy(&(g_WWCmnInfo.NewestLSNInfo.cond)); \
}

/**
 * @def  UpdateNewestLSNInfo (LSNInfo_t* pLSN__)
 * @brief   最新反映終端LSN情報の更新
 *
 *   最新反映終端LSN情報における LSN を更新する。
 *
 *   詳細は以下のとおり。
 *     -# 最新反映終端 LSN アクセス用ロックを取得する。
 *     -# 最新反映終端 LSN を、指定された LSN で更新する。
 *     -# 最新反映終端 LSN アクセス用ロックを解放する。
 *
 * @param[in]   pLSN__   更新する LSN
 * @return  なし
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define UpdateNewestLSNInfo(pLSN__)  { \
	pthread_mutex_lock (&(g_WWCmnInfo.NewestLSNInfo.mutex)); \
	g_WWCmnInfo.NewestLSNInfo.LSN = *pLSN__; \
	pthread_mutex_unlock (&(g_WWCmnInfo.NewestLSNInfo.mutex)); \
}

/**
 * @def     InitCheckEnableToactInfo()
 * @brief   ACT化可否コマンド情報の初期化
 *
 *   ACT化可否コマンド情報の内容を初期化する。
 *
 *   詳細は以下のとおり。
 *     -# ACT化可否コマンド情報のフラグを FALSE で初期化する。
 *     -# messageスレッド終了フラグを FALSE で初期化する。
 *     -# ACT化可否コマンド情報の Mutex を初期化する。
 *     -# ACT化可否コマンド情報の状態変数を初期化する。
 *
 * @return  なし
 *
 * @version 1.1.0
 * @date    10/30/2007
 * @since   1.1
*/
#define InitCheckEnableToactInfo()  { \
	g_WWCmnInfo.CheckEnableToactInfo.nFlag = FALSE; \
	g_WWCmnInfo.CheckEnableToactInfo.messageEndFlag = FALSE; \
	pthread_mutex_init(&(g_WWCmnInfo.CheckEnableToactInfo.mutex), NULL); \
	pthread_cond_init(&(g_WWCmnInfo.CheckEnableToactInfo.cond), NULL); \
}

/**
 * @def     TermCheckEnableToactInfo()
 * @brief   ACT化可否コマンド情報の破棄
 *
 *   ACT化可否コマンド情報の内容を破棄する。
 *
 *   詳細は以下のとおり。
 *     -# ACT化可否コマンド情報の Mutex を破棄する。
 *     -# ACT化可否コマンド情報の状態変数を破棄する。
 *
 * @return  なし
 *
 * @version 1.1.0
 * @date    10/30/2007
 * @since   1.1
*/
#define TermCheckEnableToactInfo()  { \
	pthread_mutex_destroy(&(g_WWCmnInfo.CheckEnableToactInfo.mutex)); \
	pthread_cond_destroy(&(g_WWCmnInfo.CheckEnableToactInfo.cond)); \
}

/**
 * @def     CheckEnableToactTrue()
 * @brief   ACT化可否判断コマンド実行中フラグ設定(真)
 *
 *   ACT化可否判断コマンド実行中フラグをTRUEにする。
 *
 *   詳細は以下のとおり。
 *     -# ACT化可否判断コマンド実行中フラグをTRUEにする。
 *
 * @version 1.1.0
 * @date    10/30/2007
 * @since   1.1
*/
#define CheckEnableToactTrue() {\
	pthread_mutex_lock(&(g_WWCmnInfo.CheckEnableToactInfo.mutex));\
	g_WWCmnInfo.CheckEnableToactInfo.nFlag = TRUE;\
	pthread_mutex_unlock(&(g_WWCmnInfo.CheckEnableToactInfo.mutex));\
}

/**
 * @def     CheckEnableToactfalse()
 * @brief   ACT化可否判断コマンド実行中フラグ設定(偽)
 *
 *   ACT化可否判断コマンド実行中フラグをFALSEにする。
 *
 *   詳細は以下のとおり。
 *     -# ACT化可否判断コマンド実行中フラグをFALSEにする。
 *
 * @version 1.1.0
 * @date    10/30/2007
 * @since   1.1
*/
#define CheckEnableToactfalse() {\
	g_WWCmnInfo.CheckEnableToactInfo.nFlag = FALSE;\
	pthread_cond_signal(&(g_WWCmnInfo.CheckEnableToactInfo.cond)); \
}

/**
 * @def     CheckEnableToactfalseAndMessageEndtrue()
 * @brief   ACT化可否判断コマンド実行中フラグ(偽)messageスレッド終了フラグ(真)設定
 *
 *   ACT化可否判断コマンド実行中フラグをFALSEに、
 *   messageスレッド終了フラグをTRUEにする。
 *
 *   詳細は以下のとおり。
 *     -# ACT化可否判断コマンド実行中フラグをFALSEにする。
 *     -# messageスレッド終了フラグをTRUEにする。
 *
 * @version 1.1.0
 * @date    10/30/2007
 * @since   1.1
*/
#define CheckEnableToactfalseAndMessageEndtrue() {\
	g_WWCmnInfo.CheckEnableToactInfo.messageEndFlag = TRUE;\
	g_WWCmnInfo.CheckEnableToactInfo.nFlag = FALSE;\
	pthread_cond_signal(&(g_WWCmnInfo.CheckEnableToactInfo.cond)); \
}

/* 200807 -> */
/**
 * @def     WaitForTLICompleted()
 * @brief   TLI通知完了待ち
 *
 *   書き出し取得スレッドにおけるTLI通知が完了するまで待機する。
 *
 *   詳細は以下のとおり。
 *     -# TLI通知完了フラグが「偽」の場合、状態変数を用いて、
 *        TLI通知完了フラグが「真」に更新されるまで待機する。
 *
 * @version 1.3.0
 * @date    07/25/2008
 * @since   1.3
*/
#define WaitForTLICompleted() {\
	pthread_mutex_lock(&(g_WWCmnInfo.TLIInfo.mutex));\
	while (g_WWCmnInfo.TLIInfo.nFlag == FALSE) {\
		pthread_cond_wait(&(g_WWCmnInfo.TLIInfo.cond), &(g_WWCmnInfo.TLIInfo.mutex));\
	}\
	pthread_mutex_unlock(&(g_WWCmnInfo.TLIInfo.mutex));\
}
/* <- 200807 */

#endif /* WWCOMMON_H_ */

