/*****************************************************************************
   Copyright (C) 2006-2008 NTT
  *****************************************************************************/
/**
 * @file    WWGetterThread.h 
 * @brief   WALWriterのバックアップ取得スレッドに関する関数の宣言
 *
 * WALWriterのバックアップ取得スレッドにおいて使用する関数を宣言する。
 *
 * @version 1.3.0
 * @date    06/17/2008
 * @since   1.0
 */
#ifndef WWGETTERTHREAD_H_
#define WWGETTERTHREAD_H_

/*****************
 * 定数定義
 ****************/
/**
 * @def     WWGET_POLLING_INTERVAL
 * @brief   アーカイブログ転送再試行時の待ち時間間隔
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define WWGET_POLLING_INTERVAL   10

/**
 * @def     WWGET_RETRYMAX
 * @brief   アーカイブログ転送再試行回数の上限
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define WWGET_RETRYMAX   6

/**
 * @def     WWGET_CMDFMT_RSYNCFORFILE
 * @brief   rsyncコマンド生成用フォーマット（ファイル転送用）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define WWGET_CMDFMT_RSYNCFORFILE	"rsync -e ssh -pqI %s@%s:%s/%s %s"

/**
 * @def     WWGET_CMDFMT_RSYNCFORDIR
 * @brief   rsyncコマンド生成用フォーマット（ディレクトリ転送用）
 *
 * @version 1.1.0
 * @date    08/01/2007
 * @since   1.0
 */
#define WWGET_CMDFMT_RSYNCFORDIR	"rsync -e ssh -aqI --exclude 'recovery.done' --exclude 'recovery.conf' --exclude 'postmaster.pid' --exclude='pg_xlog/*' --delete %s@%s:%s/ %s"

/**
 * @def     WWGET_BACKUPHISTORYFILE
 * @brief   .backupファイル名（ワイルドカード使用）
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define WWGET_BACKUPHISTORYFILE		"*.backup"

/**
 * @def     WWGET_CMDFMT_CHECKFILESIZE
 * @brief   ファイルサイズチェックコマンド生成用フォーマット
 *
 * @version 1.3.0
 * @date    06/17/2008
 * @since   1.0
 */
/* 200806 複数ファイルに対応するため、statを文字列比較にしてファイル名を表示し、ソートするように修正 */
#define WWGET_CMDFMT_CHECKFILESIZE	"test \"`ssh %s@%s \"stat --format='%%n %%s\' %s/%s|sort\"`\" = \"`stat --format='%%n %%s' %s/%s|sort`\""
/* <- 200806 */

/**
 * @def     WWGET_COMMANDMAX_RSYNCARCHIVE
 * @brief   アーカイブ転送rsyncコマンド最大長
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define WWGET_COMMANDMAX_RSYNCARCHIVE	PREX_MAX_USERNAME_LEN + PREX_MAX_HOSTNAME_LEN + PREX_MAX_PATH_LEN + XLOGFILENAME_LEN + PREX_MAX_PATH_LEN + 32

/**
 * @def     WWGET_COMMANDMAX_RSYNCBACKUP
 * @brief   ベースバックアップ転送rsyncコマンド最大長
 *
 * @attention 128という値は、コマンド生成用フォーマット内の可変部(%s)以外の文字列の長さを考慮した値である。
 *            コマンドを修正する場合は、値に注意すること。
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
 */
#define WWGET_COMMANDMAX_RSYNCBACKUP	PREX_MAX_USERNAME_LEN + PREX_MAX_HOSTNAME_LEN + PREX_MAX_PATH_LEN + PREX_MAX_PATH_LEN + 128

/********************
 * プロトタイプ宣言
 *******************/
void* WWGetterThreadMain(void* arg);
int RsyncWALSegment(LSNInfo_t* SendEndLSN);
int RsyncBackup();
int SendReplyOfCompletingBackup(void);

/*****************
 * マクロ関数定義
 ****************/
 /**
 * @def     NotifyGetterIsCompleted()
 * @brief   バックアップ取得完了情報を通知
 *
 *   バックアップ取得が完了したことを、他のスレッドに通知する。
 *
 *   詳細は以下のとおり。
 *     -# バックアップ取得完了情報のロック及び状態変数を用いて通知する。
 *
 * @note   マクロにて実装する。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define NotifyGetterIsCompleted() { \
	g_WWCmnInfo.GettingBackupInfo.nFlag = TRUE; \
	pthread_cond_signal (&(g_WWCmnInfo.GettingBackupInfo.cond)); \
}

/**
 * @def     WaitForWritingWALToFile()
 * @brief   WALセグメントへのWAL書き込み待ち
 *
 *   WALセグメントにWALが書き込まれるまで待機する。
 *
 *   詳細は以下のとおり。
 *     -# WAL書き込みフラグが「偽」の場合、状態変数を用いて、
 *        WAL書き込みフラグが「真」に更新されるまで待機する。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define WaitForWritingWALToFile() {\
	pthread_mutex_lock(&(g_WWCmnInfo.NewestLSNInfo.mutex));\
	while (g_WWCmnInfo.NewestLSNInfo.nFlag == FALSE) {\
		pthread_cond_wait(&(g_WWCmnInfo.NewestLSNInfo.cond), &(g_WWCmnInfo.NewestLSNInfo.mutex));\
	}\
	pthread_mutex_unlock(&(g_WWCmnInfo.NewestLSNInfo.mutex));\
}

#endif /* WWGETTERTHREAD_H_ */
