/*****************************************************************************
   Copyright (C) 2006-2008 NTT
  *****************************************************************************/
/**
 * @file    WWWriterThread.h
 * @brief   WALWriterのディスク書き込みスレッドに関する関数の宣言
 *
 * WALWriterのディスク書き込みスレッドにおいて使用する関数を宣言する。
 *
 * @version 1.3.0
 * @date    06/17/2008
 * @since   1.0
 */
#ifndef WWWRITERTHREAD_H_
#define WWWRITERTHREAD_H_

/**
 * @def     FILE_NAME_MAX_LEN
 * @brief   ファイル名の最大バイトサイズ
 *
 * @version 1.1.0
 * @date    10/12/2007
 * @since   1.0
 */
#define FILE_NAME_MAX_LEN (256 + 1)

/*******************
 * プロトタイプ宣言
 ******************/
void* WWWriterThreadMain(void* arg);
int InitWWWriterThread();
int OpenWALSegment(char* filepath);
void CloseWALSegment();
int WriteWAL();
int WriteWALToSegmentFile(char* pWAL, int nSize);
int LockNextBlockAndUnlockCurrentBlock();
/* 200806 -> */
int WriteHistory();
int WriteHistoryToFile(char* pHistoryFilePath);
/* <- 200806 */

/*****************
 * マクロ関数定義
 ****************/
/**
 * @def     UpdateCurrentReadPosition(WWWriterThreadInfo_t* pRc__)
 * @brief   読み込みブロック位置更新
 *
 *   現在の読み込みブロック位置を更新する。
 *
 *   詳細は以下のとおり。
 *     -# 登録WAL情報リスト上の読み込みブロック位置に関する登録WAL情報を取得する
 *        （GetWALInfo 関数）。
 *     -# 読み込みブロック位置を更新する。
 *        読み込みブロック位置 =
 *            （読み込みブロック位置 +（1 で取得した登録WAL情報のブロック数））% 
 *            WALWriter用WALバッファにおけるブロック数(ww_buffers)
 *
 * @param[out] pRc__ ディスク書き込みスレッド情報のアドレス
 *
 * @note    登録WAL情報リストから、現在の読み込み対象WALの使用ブロック数を取得し、
 *          次の読み込みブロック位置を算出する。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define UpdateCurrentReadPosition(pRc__) { \
	RegWALInfo_t*  pWALInfo__; \
	pWALInfo__ = GetWALInfo((pRc__)->nTargetPos); \
	(pRc__)->nTargetPos = \
		((pRc__)->nTargetPos + pWALInfo__->nRequiredBlocks) % g_WWEnvInfo.ww_buffers; \
}

/**
 * @def     GetRequiredBlocks(int nPos__)
 * @brief   読み込みブロックにおけるWALの使用ブロック数取得
 *
 *   指定されたブロック位置に設定された WALの使用ブロック数を取得する。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define GetRequiredBlocks(pPos__) \
	g_WWCmnInfo.pWALInfoList[(pPos__)].nRequiredBlocks

/**
 * @def     GetNextReadPosition()
 * @brief   次の読み込みブロックの先頭位置取得
 *
 *   次の読み込みブロックの先頭ブロックの位置を取得する。
 *
 *   詳細は以下のとおり。
 *     -# 次の読み込みブロックの先頭ブロック位置の算出を行なう。
 *        次の読み込みブロックの先頭ブロック位置 =
 *          （読み込みブロック位置 + 登録WAL情報リスト[読み込みブロック位置].ブロック数）% ww_buffers
 *
 * @note    登録WAL情報リストから、現在の書き込み対象
 *          WALの使用ブロック数を取得し、次の読み込みブロック位置を算出する。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define GetNextReadPosition() \
	((WrtThRc.nTargetPos + g_WWCmnInfo.pWALInfoList[WrtThRc.nTargetPos].nRequiredBlocks) %\
	g_WWEnvInfo.ww_buffers)

/**
 * @def     NotifyWALBufferIsEmpty()
 * @brief   WAL受信待ち状態を通知
 *
 *   WALWriter用WALバッファ内に未書き込みWALが存在しない状態であることを、
 *   他のスレッドに通知する。
 *
 *   詳細は以下のとおり。
 *     -# WAL受信待ち状態フラグの値を TRUE に更新する。
 *     -# WAL受信待ち状態情報の状態変数を用いて通知する。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define NotifyWALBufferIsEmpty() { \
	g_WWCmnInfo.WALBufferInfo.nFlag = TRUE; \
	pthread_cond_signal(&(g_WWCmnInfo.WALBufferInfo.cond)); \
}

/**
 * @def     NotifyWritingWALToFile()
 * @brief   WALセグメントへのWAL書き込みを通知
 *
 *   最新反映終端LSNを更新し、WALセグメントにWALを書き込んだことを
 *   他のスレッドに通知する。
 *
 *   詳細は以下のとおり。
 *     -# 最新反映終端LSN更新フラグの値を TRUE に更新する。
 *     -# 最新反映終端LSN情報の状態変数を用いて通知する。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define NotifyWritingWALToFile() { \
	g_WWCmnInfo.NewestLSNInfo.nFlag = TRUE; \
	pthread_cond_signal(&(g_WWCmnInfo.NewestLSNInfo.cond)); \
}

/**
 * @def     TermWWWriterThread()
 * @brief   ディスク書き込みスレッドのリソース解放
 *
 *   ディスク書き込みスレッドで使用したリソースを解放する。
 *
 *   詳細は以下のとおり。
 *     -# 最終WALセグメントをオープンしている場合は、クローズする（CloseWALSegment 関数）。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
#define TermWWWriterThread() { \
	CloseWALSegment(); \
}

/**
 * @def     freeEntries(entries__, num__)
 * @brief   WALファイル名格納領域の解放
 *
 *   WALファイル名格納領域の解放する。
 *
 *   詳細は以下のとおり。
 *     -# scandirで取得したWALファイル名格納領域の解放する。
 *
 * @version 1.1.0
 * @date    10/12/2007
 * @since   1.0
*/
#define freeEntries(entries__, num__) { \
    int counter; \
    if((entries__) != NULL) { \
        for(counter = 0; counter < (num__); counter++) { \
            if((entries__)[counter] != NULL) { \
                free((entries__)[counter]); \
            } \
        } \
        free((entries__)); \
    } \
}

#endif /* WWWRITERTHREAD_H_ */
