/*****************************************************************************
   Copyright (C) 2006-2008 NTT
  *****************************************************************************/
/**
 * @file    WWCommThread.c
 * @brief   WALWriterの通信スレッドに関する関数の定義
 *
 * WALWriterの通信スレッドにおいて使用する関数を定義する。
 *
 * @version 1.3.0
 * @date    06/17/2008
 * @since   1.0
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <errno.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#ifdef PG82
#include "WALDecompress.h"
#endif
#include "PREXMessage.h"
#include "PREXUtil.h"
#include "WWCommon.h"
#include "WWCommThread.h"
#include "WWMessageThread.h"
#include "WWWriterThread.h"
#include "WWGetterThread.h"
#include "ReadWWConfFile.h"

WWCommThreadInfo_t  WWCommThRc;     ///< 通信スレッド情報

static char*        g_recvBuffer;   ///< LSN復元後のWALレコードの展開領域
static int          g_cmpBufSize;   ///< 圧縮済みイメージの展開領域サイズ
static char*        g_cmpBuffer;    ///< 圧縮済みイメージの展開領域
static char g_Command[CMN_COMMANDMAX]; /* コマンド文字列作成用 */


static int StartCheckEnableToact();

#define RECV_OPT    0


/**
 * @fn int main(int argc, char **argv)
 * @brief   WALWriter通信スレッドメイン
 *
 *   各スレッドを起動した後、WALSenderから送信されるWALの受信を行なう。
 *
 *   詳細は以下のとおり。
 *     -# 通信スレッドの初期設定を行なう（InitWWCommThread 関数）。エラー発生時は、
 *        確保したリソースをすべて解放し、プロセスを終了する（ExitWALWriter 関数）。
 *     -# メッセージスレッドを起動する（StartWWMessageThread 関数）。エラー発生時は、
 *        確保したリソースをすべて解放し、プロセスを終了する（ExitWALWriter 関数）。
 *     -# WALSenderとの接続を確立する（ConnectToWALSender 関数）。エラー発生時は、
 *        確保したリソースをすべて解放し、プロセスを終了する（ExitWALWriter 関数）。
 *     -# WALSenderから「TLI通知」を受信し、WALSenderへ「TLI通知応答」を送信する
 *        （ReceiveTLI 関数）。エラー発生時は、確保したリソースをすべて解放し、
 *        プロセスを終了する（ExitWALWriter 関数）。
 *     -# WALSenderから「history情報通知」を受信し、WALSenderへ「history情報通知応答」
 *        を送信する（ReceiveTLI 関数）。エラー発生時は、確保したリソースをすべて解放し、
 *        プロセスを終了する（ExitWALWriter 関数）。
 *     -# ディスク書き込みスレッドを起動する（StartWWWriterThread 関数）。エラー発生時
 *        は、確保したリソースをすべて解放し、プロセスを終了する（ExitWALWriter 関数）。
 *     -# バックアップ取得スレッドを起動する（StartWWGetterThread 関数）。エラー発生時
 *        は、確保したリソースをすべて解放し、プロセスを終了する（ExitWALWriter 関数）。
 *     -# 7 を繰り返す。
 *     -# WALSenderから「WAL転送」を受信する（ReceiveWAL 関数）。エラー発生時は、
 *        通信断（戻り値が RET_ECONNREFUSED）の場合、通信スレッドに関するリソースを
 *        すべて解放し（TermWWCommThread 関数）、スレッドを終了する。
 *        その他エラー（戻り値が RET_NG）の場合、プロセスに関するリソースをすべて解放し、
 *        プロセスを終了する（ExitWALWriter 関数）。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @attention   InitWWCommThread 関数を呼び出す際、引数として argc 及び
 *              argv を加工せずそのまま指定すること。
 * @note        通信スレッドは、WALWriterプロセスにおけるメインスレッドである。
 * @note        本関数から起動する各スレッドは、pthread_join() による回収を行なわない
 *              ため、すべてデタッチモードで起動する。
 *
 * @param[in]   argc          実行時引数の数
 * @param[in]   argv          実行時引数のリスト
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.3.0
 * @date    06/17/2008
 * @since   1.0
*/
int main(int argc, char **argv)
{
	int rcode;

	/* 起動メッセージの出力 */
	OutputSyslog(LOG_INFO, MSGID_WWCOM_STARTWALWRITER, PREX_MODNAME_WW);

	/* 通信スレッドの初期設定 */
	rcode = InitWWCommThread(argc, argv);
	if (rcode != RET_OK) {
		ExitWALWriter(EXIT_FAILURE);
	}

	/* メッセージスレッドの起動 */
	rcode = StartWWMessageThread();
	if (rcode != RET_OK) {
		ExitWALWriter(EXIT_FAILURE);
	}

	/* WALSenderとの接続 */
	rcode = ConnectToWALSender();
	if (rcode != RET_OK) {
		ExitWALWriter(EXIT_FAILURE);
	}
/* 200806 TLI通知待ち、TLI通知応答の送信、history情報通知待ち、history情報通知応答の通知 */
	/* TLI通知待ち */
	rcode = ReceiveTLI();
	if (rcode != RET_OK) {
		ExitWALWriter(EXIT_FAILURE);
	}

	/* history情報通知待ち */
	rcode = ReceiveHistory();
	if (rcode != RET_OK) {
		ExitWALWriter(EXIT_FAILURE);
	}
/* <- 200806 */

	/* ディスク書き込みスレッドの起動 */
	rcode = StartWWWriterThread();
	if (rcode != RET_OK) {
		ExitWALWriter(EXIT_FAILURE);
	}

	/* バックアップ取得スレッドの起動 */
	rcode = StartWWGetterThread();
	if (rcode != RET_OK) {
		ExitWALWriter(EXIT_FAILURE);
	}

	/* WAL受信 */
	rcode = ReceiveWAL();
	if (rcode == RET_ECONNREFUSED) {
		TermWWCommThread();
		pthread_exit(NULL);
	} else if (rcode != RET_OK) {
		ExitWALWriter(EXIT_FAILURE);
	}

	return RET_OK;
}

/**
 * @fn int InitWWCommThread(int argc, char **argv)
 * @brief   通信スレッド初期設定
 *
 *   通信スレッド及び複数スレッド間で共有する変数及びリソースの初期化を行なう。さらに、
 *   WALWriter用WALバッファの先頭ブロック及び末尾ブロックに関するロックを取得する。
 *
 *   詳細は以下のとおり。
 *     -# SIGTERM に関するシグナルハンドラを登録する。
 *     -# パラメータチェックを実施する（CheckConfiguration 関数）。
 *        エラー発生時は、呼出元に RET_NG を返却する。
 *     -# バックアップ取得完了情報を初期化する（InitGettingBackupInfo 関数）。
 *     -# TLI通知完了情報を初期化する（InitTLIInfo 関数）。
 *     -# WAL受信待ち状態情報を初期化する（InitWALBufferInfo 関数）。
 *     -# 最新反映終端WAL情報を初期化する（InitNewestLSNInfo 関数）。
 *     -# ACT化可否コマンド情報を初期化する（InitCheckEnableToactInfo 関数）。
 *     -# ACT化処理中フラグを FALSE で初期化する。
 *     -# SBY化処理中フラグを FALSE で初期化する。
 *     -# ファイル転送情報通知取得完了フラグを FALSE で初期化する。
 *     -# WALWriter用WALバッファの領域（WALWriter用WALバッファにおけるブロック数
 *        （ww_buffers）* １ブロックのサイズ（WAL_BLOCK_SIZE））を確保する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 受信したWALレコードの展開領域確保（WALWriter用WALバッファにおけるブロック数
 *        （ww_buffers）* １ブロックのサイズ（WAL_BLOCK_SIZE））を確保する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 登録WAL情報リストの領域（WALWriter用WALバッファにおけるブロック数(ww_buffers) *
 *        登録WAL情報のサイズ（sizeof(RegWALInfo_t)））を確保する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# WALWriter用WALバッファへのアクセス用ロックリストの領域（WALWriter用WALバッファ
 *        におけるブロック数(ww_buffers) * Mutexのサイズ（sizeof(pthread_mutex_t)））を
 *        確保する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# WALWriter用WALバッファへのアクセス用ロックリストにおいて、すべてのロックを
 *        初期化する。
 *     -# 通信スレッド情報における格納先ブロック位置を 0に初期化する。
 *     -# WALWriter用WALバッファの先頭ブロックから１ブロック分のロック（通信スレッド用）
 *        を取得する
 *        （LockWALBuffer 関数）。
 *     -# WALWriter用WALバッファの末尾ブロックから１ブロック分のロック（ディスク書き込み
 *        スレッド用）を取得する（LockWALBuffer 関数）。
 *     -# 登録WAL情報リストの末尾において、使用ブロック数を 1 で初期化する
 *        （SetBlocksToWALInfo 関数）。
 *     -# ACT化可否コマンド文字列の構築し、グローバル変数に格納する。
 *     -# 呼出元に RET_OKを返却する。
 *
 * @attention   CheckConfiguration 関数を呼び出す際、引数として argc 及び argv
 *              を加工せずそのまま指定すること。
 * @note        詳細説明の 14は、WALWriter用WALバッファ上でディスクにまだ書き込んでいない
 *              WALが、通信スレッドによりバッファ上で上書きされてしまわないようにするため
 *              である。
 * @note        詳細説明の 15は、ディスク書き込みスレッドの読み込みブロック位置の初期値を
 *              （ww_buffers-1）とするためである。
 *
 * @param[in]   argc          実行時引数の数
 * @param[in]   argv          実行時引数のリスト
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.3.0
 * @date    07/25/2008
 * @since   1.0
*/
int InitWWCommThread(int argc, char **argv)
{
	int    rcode;
	int    nAllocSize;
	int    nPos;

	/* シグナルハンドラの登録 */
	signal(SIGTERM, HandleSIGTERM);

	/* 終了処理排他制御に使用するmutexの初期化 */
	pthread_mutex_init(&(g_WWCmnInfo.lockTermWW), NULL);

	/* パラメータチェック */
	rcode = CheckConfiguration(argc, argv);
	if (rcode != RET_OK) {
		return RET_NG;
	}

	/* 200807 -> */
	/* TLI通知完了情報の初期化 */
	InitTLIInfo();
	/* <- 200807 */
	
	/* バックアップ取得完了情報の初期化 */
	InitGettingBackupInfo();

	/* WAL受信待ち状態情報の初期化 */
	InitWALBufferInfo();

	/* 最新反映終端WAL情報の初期化 */
	InitNewestLSNInfo();
	
	/* ACT化可否コマンド情報の初期化 */
	InitCheckEnableToactInfo();
	
	/* 各フラグの初期化 */
	g_WWCmnInfo.nFlgToACT               = FALSE;
	g_WWCmnInfo.nFlgToSBY               = FALSE;
	g_WWCmnInfo.nFlgSendFilesCompleted  = FALSE;

	/* ブランク領域の確保 */
	g_WWCmnInfo.pBlank = (char*)calloc(WAL_SEG_FILESIZE, sizeof(char));
	if (g_WWCmnInfo.pBlank == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOALLOCMEM);
		return RET_NG;
	}

	/* WALWriter用WALバッファの領域確保 */
	nAllocSize = g_WWEnvInfo.ww_buffers * WAL_BLOCK_SIZE;
	g_WWCmnInfo.pWALBuffer = (char*)malloc(nAllocSize);
	if (g_WWCmnInfo.pWALBuffer == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOALLOCMEM);
		return RET_NG;
	}
	
	/* 受信したWALレコードの展開領域確保 */
	g_recvBuffer = (char*)malloc(WAL_SEG_FILESIZE);
	if (g_recvBuffer == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOALLOCMEM);
		return RET_NG;
	}
	

	/* 登録WAL情報の領域確保 */
	nAllocSize = g_WWEnvInfo.ww_buffers * sizeof(RegWALInfo_t);
	g_WWCmnInfo.pWALInfoList = (RegWALInfo_t*)malloc(nAllocSize);
	if (g_WWCmnInfo.pWALInfoList == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOALLOCMEM);
		return RET_NG;
	}

	/* WALWriter用WALバッファへのアクセス用ロックリストの領域確保 */
	nAllocSize = g_WWEnvInfo.ww_buffers * sizeof(pthread_mutex_t);
	g_WWCmnInfo.pBufferMutexList = (pthread_mutex_t*)malloc(nAllocSize);
	if (g_WWCmnInfo.pBufferMutexList == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOALLOCMEM);
		return RET_NG;
	}

	/* ロックリストにおける各Mutexの初期化 */
	for (nPos = 0; nPos < g_WWEnvInfo.ww_buffers; nPos++) {
		pthread_mutex_init(g_WWCmnInfo.pBufferMutexList + nPos, NULL);
	}

	/* 通信スレッド情報における格納先ブロック位置の初期化 */
	WWCommThRc.nTargetPos = 0;

	/* 通信スレッド用のロックの取得 */
	LockWALBuffer(0, 1);

	/* ディスク書き込みスレッド用のロックの取得 */
	LockWALBuffer(g_WWEnvInfo.ww_buffers - 1, 1);

	/* 登録WAL情報リストの更新 */
	SetBlocksToWALInfo(g_WWEnvInfo.ww_buffers - 1, 1);

	/* ACT化可否判断スクリプト実行文字列の構築 */
	sprintf(g_Command, WWCOMM_CMDFMT_CHECKENABLETOACT, g_WWEnvInfo.prex_msgkey, g_WWEnvInfo.acthahost, g_WWEnvInfo.actwalhost, g_WWEnvInfo.enabletoactretrycount, g_WWEnvInfo.enabletoactretryinterval );

	return RET_OK;
}

/**
 * @fn void TermWWCommThread()
 * @brief   通信スレッド情報の破棄
 *
 *   通信スレッド及び複数スレッド間で共有する変数及びリソースを破棄する。
 *
 *   詳細は以下のとおり。
 *     -# WALSenderとの接続を切断する（DisconnectWALSender関数）。
 *
 * @return      なし
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
void TermWWCommThread()
{
	DisconnectWALSender();
}

/**
 * @fn int CheckConfiguration(int argc, char **argv)
 * @brief   パラメータチェック
 *
 *   外部から指定されるパラメータに関するチェックを行なう。
 *
 *   詳細は以下のとおり。
 *     -# 関数パラメータのチェック（argv が非 NULLであること、及び argc が
 *        WW_MIN_LEGAL_ARGC 以上、かつ WW_MAX_LEGAL_ARGC 以下であることの確認）を行なう。
 *        エラーの場合は、syslog にエラーメッセージを出力し（OutputSyslog 関数）、
 *        呼出元に RET_NG を返却する。
 *     -# 実行時引数のチェックを行なう。
 *        初期起動パラメータが指定された場合、初期起動フラグの値を TRUE に更新する。
 *        初期起動パラメータ以外の文字列が指定された場合、syslog にエラーメッセージを
 *        出力し（OutputSyslog 関数）、呼出元に RET_NG を返却する。
 *     -# 実行ユーザ名を取得する。
 *     -# 設定ファイル情報に、デフォルト値が規定されている項目にはその値を設定し、規定
 *        されていない場合には、文字列はNULL、数値は-1で初期化する。 設定値が文字列で、
 *        デフォルト値が規定されている設定項目の場合は、設定値の格納用の領域を確保する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# WALWriter用設定ファイルから、設定ファイル項目を取得する（ReadConfigFile 関数）。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 呼出元にRET_OKを返却する。
 *
 * @note WALWriter用設定ファイルのパスは「${HOME}/prex/WALWriter.conf」である。
 * @note
 * @note 以下に、WALWriterの実行時に指定可能なパラメータについて示す。
 *       - -i：初期起動パラメータ。指定されると、WALWriterは初期起動モードで動作する。
 *
 * @param[in]   argc          実行時引数の数
 * @param[in]   argv          実行時引数のリスト
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int CheckConfiguration(int argc, char **argv)
{
	int         rcode;
	char*       strUsername;
	char        strConfFilePath[PATH_MAX+1];
	char*       strConfFilePathTmp;
	char*       pParam;

	/* 関数パラメータのチェック */
	if ((argc > WW_MAX_LEGAL_ARGC) || (argc < WW_MIN_LEGAL_ARGC) || (argv == NULL)) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_ILLEGALPARAM, NUM_OF_ARGUMENTS);
		return RET_NG;
	}

	/* 実行時引数のチェック */
	g_WWCmnInfo.nFlgInit    = FALSE;
	if (argc == WW_MAX_LEGAL_ARGC) {
		pParam = *(argv + 1);
		/* 有効なパラメータ（初期起動パラメータ）かどうかのチェック */
		rcode = strcmp(pParam, WWPRM_INIT);
		if (rcode == 0) {
			g_WWCmnInfo.nFlgInit    = TRUE;
		} else {
			OutputSyslog(LOG_ERR, MSGID_WWCOM_ILLEGALPARAM, pParam);
			return RET_NG;
		}
	}
	/* 初期起動フラグの初期値を保持*/
	g_WWCmnInfo.nFlgInitOrg = g_WWCmnInfo.nFlgInit;

	/* 実行ユーザ名の取得 */
	strUsername = getenv(ENVITEM_USER);
	if (strUsername == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOGETENV, ENVVAL_USER);
		return RET_NG;
	}

	/* 接続情報の初期化 */
	g_WWCmnInfo.commTarget.sndrcv_fd = FD_INACTIVE;

	/* 設定ファイル項目の初期化 */
	rcode = InitSettingInfo(&g_WWEnvInfo);
	if (rcode != RET_OK) {
		return RET_NG;
	}

	/* WALWriter用設定ファイルのオープン */
	/*   - 環境変数 HOME の取得 */
	strConfFilePathTmp = getenv(ENVITEM_HOME);
	if (strConfFilePathTmp == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOGETENV, ENVVAL_HOME);
		return RET_NG;
	}

	/*   - WALWriter用設定ファイルパスの決定 */
	sprintf(strConfFilePath, "%s/%s/%s",
		strConfFilePathTmp, PREX_HOMENAME, WW_CONF_FILE);

	/* WALWriter用設定ファイルからの設定項目値取得 */
	rcode = ReadConfigFile(strConfFilePath, &g_WWEnvInfo);
	if (rcode != RET_OK) {
		return RET_NG;
	}

	return RET_OK;
}

/**
 * @fn int ConnectToWALSender()
 * @brief   WALSenderとの接続
 *
 *   WALSenderへの接続要求を行ない、接続を確立する。
 *
 *   詳細は以下のとおり。
 *     -# WALSenderへの接続に必要な情報（ws_host, prex_port, ws_polling_interval）を
 *        設定ファイル情報から取得する。
 *     -# ソケットを生成する。エラー発生時は、syslogにエラーメッセージを出力し、呼出元に
 *        RET_NG を返却する。
 *     -# ソケットにキープアライブの設定、およびsend/recvのタイムアウト設定を行う。
 *        - SO_KEEPALIVE
 *        - TCP_KEEPIDLE(keepalives_idleが1以上の場合)
 *        - TCP_KEEPINTVL(keepalives_intervalが1以上の場合)
 *        - TCP_KEEPCNT(keepalives_countが1以上の場合)
 *        - SO_RCVTIMEO(keepalives_idleが1以上の場合、keepalives_idleの値を設定)
 *        - SO_SNDTIMEO(keepalives_idleが1以上の場合、keepalives_idleの値を設定)
 *     -# WALSenderへの接続要求を行なう。エラー発生時は、ws_polling_interval
 *        秒間スリープする。
 *     -# WALSenderとの接続を確立するまで 3 を繰り返す。接続を確立した場合は、呼出元に
 *        RET_OK を返却する。
 *
 * @note    ww_polling_interval は、設定ファイルから取得した値で、WALSenderへの接続要求を
 *          行なう時間間隔（単位：秒）である。
 *
 * @retval      RET_OK(0)     正常終了（接続成功）
 * @retval      RET_NG(1)     異常終了（ソケット生成失敗）
 *
 * @version 1.1.0
 * @date    10/30/2007
 * @since   1.0
*/
int ConnectToWALSender()
{
	int                rcode;
	int                ws_polling_interval;
	CommTargetInfo_t*  pInfo;
	size_t             addressLen;
	struct hostent*    wsHostInfo;
	int                optval;
    struct timeval     timeout;

	pInfo = &(g_WWCmnInfo.commTarget);

	/* 情報の取得 */
	pInfo->hostname     = g_WWEnvInfo.ws_host;
	pInfo->port         = g_WWEnvInfo.prex_port;
	ws_polling_interval = g_WWEnvInfo.ws_polling_interval;

	/* ソケット生成の準備 */
	addressLen = sizeof(pInfo->sock_info);
	memcpy(&(pInfo->sock_info), g_WWCmnInfo.pBlank, addressLen);
	pInfo->sock_info.sin_family      = AF_INET;
	pInfo->sock_info.sin_addr.s_addr = INADDR_ANY;
	pInfo->sock_info.sin_port        = htons((in_port_t)(pInfo->port));

	/* ソケットの生成 */
	pInfo->sndrcv_fd = socket(AF_INET, SOCK_STREAM, 0);
	if (pInfo->sndrcv_fd < 0) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOCREATESOCKET);
		return RET_NG;
	}

	/* keepAlive関連のソケットオプションを設定する */
	optval = 1;
	
	/* SO_KEEPALIVE */
	rcode = setsockopt(pInfo->sndrcv_fd, SOL_SOCKET, SO_KEEPALIVE, 
	                            (const void *)&optval, sizeof(optval));
	if (rcode == SOCKET_ERROR) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOCREATESOCKET);
		return RET_NG;
	}
	
	/* TCP_KEEPIDLE */
	if(g_WWEnvInfo.keepalives_idle > 0) {
        rcode = setsockopt(pInfo->sndrcv_fd, IPPROTO_TCP, TCP_KEEPIDLE,
            (const void *)&(g_WWEnvInfo.keepalives_idle), sizeof(int));
    	if (rcode == SOCKET_ERROR) {
	    	OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOCREATESOCKET);
		    return RET_NG;
	    }
    }

	/* TCP_KEEPINTVL */
	if(g_WWEnvInfo.keepalives_interval > 0) {
        rcode = setsockopt(pInfo->sndrcv_fd, IPPROTO_TCP, TCP_KEEPINTVL,
            (const void *)&(g_WWEnvInfo.keepalives_interval), sizeof(int));
    	if (rcode == SOCKET_ERROR) {
	    	OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOCREATESOCKET);
		    return RET_NG;
	    }
    }

	/* TCP_KEEPCNT */
	if(g_WWEnvInfo.keepalives_count > 0) {
        rcode = setsockopt(pInfo->sndrcv_fd, IPPROTO_TCP, TCP_KEEPIDLE,
            (const void *)&(g_WWEnvInfo.keepalives_count), sizeof(int));
    	if (rcode == SOCKET_ERROR) {
	    	OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOCREATESOCKET);
		    return RET_NG;
	    }
    }


	/* SO_SNDTIMEO、SO_RCVTIMEO */
	if(g_WWEnvInfo.keepalives_idle > 0) {

        timeout.tv_sec = g_WWEnvInfo.keepalives_idle;
        timeout.tv_usec = 0;

        rcode = setsockopt(pInfo->sndrcv_fd, SOL_SOCKET, SO_SNDTIMEO,
            (const void *)&timeout, sizeof(struct timeval));
    	if (rcode == SOCKET_ERROR) {
	    	OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOCREATESOCKET);
		    return RET_NG;
	    }
        rcode = setsockopt(pInfo->sndrcv_fd, SOL_SOCKET, SO_RCVTIMEO,
            (const void *)&timeout, sizeof(struct timeval));
    	if (rcode == SOCKET_ERROR) {
	    	OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOCREATESOCKET);
		    return RET_NG;
	    }
    }




	/* 接続先サーバ情報の取得 */
	wsHostInfo = gethostbyname(pInfo->hostname);
	if (wsHostInfo == NULL) {
		in_addr_t addr;
		addr = inet_addr(pInfo->hostname);
		wsHostInfo = gethostbyaddr((char*)&addr, sizeof(addr), AF_INET);
		if (wsHostInfo == NULL) {
			OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOGETHOST, WS_HOST_INFO);
			shutdown(pInfo->sndrcv_fd, SHUT_RDWR);
			close(pInfo->sndrcv_fd);
			pInfo->sndrcv_fd = FD_INACTIVE;
			return RET_NG;
		}
	}
	memcpy(&(pInfo->sock_info.sin_addr), wsHostInfo->h_addr, sizeof(struct in_addr));

	/* WALSender のソケット監視に対応するための一時停止 */
	usleep(WAIT_BEFORE_CONNECT_USECVAL);

	/* WALSender への接続 */
	do {
		rcode = connect(pInfo->sndrcv_fd,
			(struct sockaddr*)&(pInfo->sock_info), addressLen);
		if (rcode == SOCKET_ERROR) {
			sleep(ws_polling_interval);
		}
	} while (rcode == SOCKET_ERROR);

	DebugOutputMesSyslog(LOG_ERR, "connect WALSender rcode = %d", rcode);

	/* 初期起動フラグを更新（for メッセージスレッド）*/
	g_WWCmnInfo.nFlgInit = FALSE;

	return RET_OK;
}

/**
 * @fn      void DisconnectWALSender()
 * @brief   WALSenderからの切断
 *
 *   WALSenderとの通信の切断、及び使用しているソケットの解放を行なう。
 *
 *   詳細は以下のとおり。
 *     -# ソケットが破棄されていない場合、2～3 を実施する。
 *     -# WALSenderとの接続を切断する。
 *     -# ソケットを破棄する。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
void DisconnectWALSender()
{
	if (g_WWCmnInfo.commTarget.sndrcv_fd != FD_INACTIVE) {
		shutdown(g_WWCmnInfo.commTarget.sndrcv_fd, SHUT_RDWR);
		close(g_WWCmnInfo.commTarget.sndrcv_fd);
		g_WWCmnInfo.commTarget.sndrcv_fd = FD_INACTIVE;
	}

	return;
}

/**
 * @fn int StartWWMessageThread()
 * @brief   メッセージスレッド起動
 *
 *   メッセージスレッドを起動する。
 *
 *   詳細は以下のとおり。
 *     -# メッセージスレッド情報の領域を確保する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NG を返却する。
 *     -# メッセージスレッド情報を初期化する。
 *     -# 生成するスレッドの属性を、デタッチ状態に設定する。
 *     -# 2 で初期化したメッセージスレッド情報を引数として、メッセージスレッドを起動する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NG を返却する。
 *     -# 呼出元にRET_OKを返却する。
 *
 * @post        通信スレッドによってメッセージスレッドを停止させる場合は、
 *              ExitWALWriter 関数によってプロセスごと停止させること。
 * @attention   メッセージスレッドの終了時は、他のスレッドがpthread_join関数を
 *              呼び出すことによる同期を行なわない。そのため、メッセージスレッドは
 *              デタッチ状態で起動すること。
 * @attention   詳細説明 1 で確保した領域は、メッセージスレッドにおいて解放すること。
 * @note        メッセージスレッドのメインは、WWMessageThreadMain関数である。
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int StartWWMessageThread()
{
	int                     rcode;
	pthread_t               ptid;
	pthread_attr_t          threadAttr;
	WWMessageThreadInfo_t*  pWWMsgRc;

	/* メッセージスレッド情報の生成 */
	pWWMsgRc = (WWMessageThreadInfo_t*)malloc(sizeof(WWMessageThreadInfo_t));
	if (pWWMsgRc == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOALLOCMEM);
		return RET_NG;
	}

	pWWMsgRc->nFlgInit = g_WWCmnInfo.nFlgInit;
	pWWMsgRc->nMsgKey  = g_WWEnvInfo.prex_msgkey;
	pWWMsgRc->nMsgqID  = 0;

	/* スレッド属性の初期化 */
	rcode = pthread_attr_init(&threadAttr);
	if (rcode != PTHREAD_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSTARTTHREAD, WW_MSGTHREAD);
		free(pWWMsgRc);
		return RET_NG;
	}

	/* スレッドの属性設定 */
	rcode = pthread_attr_setdetachstate(&threadAttr, PTHREAD_CREATE_DETACHED);
	if (rcode != PTHREAD_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSTARTTHREAD, WW_MSGTHREAD);
		free(pWWMsgRc);
		return RET_NG;
	}

	/* メッセージスレッドの生成 */
	rcode = pthread_create(&ptid, &threadAttr, WWMessageThreadMain, (void*)pWWMsgRc);
	if (rcode != PTHREAD_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSTARTTHREAD, WW_MSGTHREAD);
		free(pWWMsgRc);
		return RET_NG;
	}

	return RET_OK;
}

/**
 * @fn int StartWWWriterThread()
 * @brief   ディスク書き込みスレッド起動
 *
 *   ディスク書き込みスレッドを起動する。
 *
 *   詳細は以下のとおり。
 *     -# 生成するスレッドの属性を、デタッチ状態に設定する。
 *     -# ディスク書き込みスレッドを起動する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 呼出元にRET_OKを返却する。
 *
 * @post        通信スレッドによってディスク書き込みスレッドを停止させる場合は、
 *              ExitWALWriter 関数によってプロセスごと停止させること。
 * @attention   ディスク書き込みスレッドの終了時は、他のスレッドがpthread_join関数を
 *              呼び出すことによる同期を行なわない。そのため、ディスク書き込みスレッドは
 *              デタッチ状態で起動すること。
 * @note        ディスク書き込みスレッドのメインは、WWWriterThreadMain関数である。
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int StartWWWriterThread()
{
	int                     rcode;
	pthread_t               ptid;
	pthread_attr_t          threadAttr;

	/* スレッド属性の初期化 */
	rcode = pthread_attr_init(&threadAttr);
	if (rcode != PTHREAD_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSTARTTHREAD, WW_WRTTHREAD);
		return RET_NG;
	}

	/* スレッドの属性設定 */
	rcode = pthread_attr_setdetachstate(&threadAttr, PTHREAD_CREATE_DETACHED);
	if (rcode != PTHREAD_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSTARTTHREAD, WW_WRTTHREAD);
		return RET_NG;
	}

	/* ディスク書き込みスレッドの生成 */
	rcode = pthread_create(&ptid, &threadAttr, WWWriterThreadMain, NULL);
	if (rcode != PTHREAD_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSTARTTHREAD, WW_WRTTHREAD);
		return RET_NG;
	}

	return RET_OK;
}

/**
 * @fn int StartWWGetterThread()
 * @brief   バックアップ取得スレッド起動
 *
 *   バックアップ取得スレッドが必要とする情報を取得し、バックアップ取得スレッドを
 *   起動する。
 *
 *   詳細は以下のとおり。
 *     -# WALSenderへ「送信開始要求」を送信する（SendRequestToStartSending 関数）。
 *        エラー発生時は、呼出元に RET_NGを返却する。
 *     -# WALSenderから「ファイル転送情報通知」を受信する（ReceiveLastLSNFromWALSender 関数）。
 *        エラー発生時は、呼出元に RET_NGを返却する。
 *     -# 「ファイル転送情報通知」で受信した LSN を最新反映終端 LSN の初期値として登録する。
 *     -# ファイル転送情報通知完了フラグを TRUE に更新する。
 *     -# バックアップ取得スレッド情報の領域を確保する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NG を返却する。
 *     -# バックアップ取得スレッド情報を初期化する。
 *     -# 生成するスレッドの属性を、デタッチ状態に設定する。
 *     -# バックアップ取得スレッドを起動する。この時、バックアップ取得スレッド情報の
 *        メンバ変数として、ファイル転送終了LSN,送信開始LSNをパラメータで渡す。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 呼出元にRET_OKを返却する。
 *
 * @pre         WALSenderとの接続が確立されていること。
 * @post        通信スレッドによってバックアップ取得スレッドを停止させる場合は、
 *              ExitWALWriter 関数によってプロセスごと停止させること。
 * @attention   バックアップ取得スレッドの終了時は、他のスレッドがpthread_join関数を
 *              呼び出すことによる同期を行なわない。そのため、バックアップ取得スレッドは
 *              デタッチ状態で起動すること。
 * @attention   詳細説明 4 で確保した領域は、バックアップ取得スレッドにおいて解放すること。
 * @note        バックアップ取得スレッドのメインは、WWGetterThreadMain関数である。
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int StartWWGetterThread()
{
	int                     rcode;
	pthread_t               ptid;
	pthread_attr_t          threadAttr;
	WWGetterThreadInfo_t*   pWWGtrRc;
	LSNInfo_t               lastLSN;

	/* WALSenderへの「送信開始要求」の送信 */
	rcode = SendRequestToStartSending();
	if (rcode != RET_OK) {
		return RET_NG;
	}

	/* WALSenderからの「ファイル転送情報通知」の受信 */
	rcode = ReceiveLastLSNFromWALSender(&lastLSN);
	if (rcode != RET_OK) {
		return RET_NG;
	}

	/* 「ファイル転送情報通知」で受信した LSN を最新反映終端 LSN の初期値として登録 */
	UpdateNewestLSNInfo(&lastLSN);

	/* ファイル転送情報通知完了フラグの更新 */
	g_WWCmnInfo.nFlgSendFilesCompleted = TRUE;

	/* バックアップ取得スレッド情報の生成 */
	pWWGtrRc = (WWGetterThreadInfo_t*)malloc(sizeof(WWGetterThreadInfo_t));
	if (pWWGtrRc == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOALLOCMEM);
		return RET_NG;
	}

	pWWGtrRc->nFlgInit = g_WWCmnInfo.nFlgInitOrg;
	InitLSNInfo(&(pWWGtrRc->SendEndLSN));
	SetLSNInfo(&(pWWGtrRc->SendEndLSN), lastLSN.uXLogID, lastLSN.uXRecOff);

	/* スレッド属性の初期化 */
	rcode = pthread_attr_init(&threadAttr);
	if (rcode != PTHREAD_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSTARTTHREAD, WW_GTRTHREAD);
		free(pWWGtrRc);
		return RET_NG;
	}

	/* スレッドの属性設定 */
	rcode = pthread_attr_setdetachstate(&threadAttr, PTHREAD_CREATE_DETACHED);
	if (rcode != PTHREAD_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSTARTTHREAD, WW_GTRTHREAD);
		free(pWWGtrRc);
		return RET_NG;
	}

	/* バックアップ取得スレッドの生成 */
	rcode = pthread_create(&ptid, &threadAttr, WWGetterThreadMain, (void*)pWWGtrRc);
	if (rcode != PTHREAD_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSTARTTHREAD, WW_GTRTHREAD);
		free(pWWGtrRc);
		return RET_NG;
	}

	return RET_OK;
}

/**
 * @fn      int ReceiveWAL()
 * @brief   WAL受信
 *
 *   WALSenderから「WAL転送」を受信し、受信したWALをWALWriter用WALバッファに登録する。
 *
 *   詳細は以下のとおり。
 *     -# WALSenderから「WAL転送」のヘッダ部（メッセージID及びメッセージサイズ）
 *        及び最終書き込み LSN を受信する（ReceiveHeaderAndLSNFromWALSender 関数）。
 *        通信断によるエラーが発生した場合は、syslog にエラーメッセージを出力し、
 *        呼出元に RET_ECONNREFUSED を返却する。エラーが通信断以外の場合は、
 *        転送結果を RSLT_SENDWAL_NG(=-1) とする「WAL転送応答」を WALSenderに返信する
 *        （SendReplyOfSendingWAL 関数）。
 *        その後 syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 次の計算式によって、使用ブロック数ｎを算出する。
 *        （計算式：n =（メッセージサイズ - sizeof(LSNInfo_t)）/ WAL_BLOCK_SIZE）
 *     -# 使用ブロック数ｎを、登録WAL情報リストの格納先ブロック位置に登録する
 *        （SetBlocksToWALInfo 関数）。
 *     -# 格納先ブロックの２つ目以降に関する（（ｎ－１）個分の）ロックを取得する
 *        （LockWALBuffer 関数）。取得対象のすべてのロックを取得するまで待機する。
 *     -# WALSenderから WAL を受信する（ReceiveWALFromWALSender 関数）。
 *        通信断によるエラーが発生した場合は、syslog にエラーメッセージを出力し、
 *        呼出元に RET_ECONNREFUSED を返却する。エラーが通信断以外の場合は、転送結果を
 *        RSLT_SENDWAL_NG(=-1) とする「WAL転送応答」を WALSenderに返信する
 *        （SendReplyOfSendingWAL 関数）。その後 syslogにエラーメッセージを出力し、呼出元に
 *        RET_NGを返却する。
 *     -# 最終書き込みLSNを登録WAL情報リストの格納先ブロック位置に登録する
 *        （SetLSNToWALInfo 関数）。
 *     -# 次の格納先ブロック位置に関するロック（１つ）を取得する（LockWALBuffer 関数）。
 *        次の格納先ブロック位置は、次の計算式によって算出される。
 *        （次の格納先ブロック位置 =（格納先ブロック位置 +
 *            登録WAL情報リスト[格納先ブロック位置].使用ブロック数）% ww_buffers）
 *     -# 格納先ブロック位置から使用ブロック数分のロックを解放する
 *        （UnlockWALBuffer 関数）。
 *     -# 転送結果を RSLT_SENDWAL_OK(=0) とする「WAL転送応答」を、WALSender に送信する
 *        （SendReplyOfSendingWAL 関数）。通信断によるエラーが発生した場合は、
 *        呼出元に RET_ECONNREFUSED を返却する。エラーが通信断以外の場合は、
 *        呼出元に RET_NGを返却する。
 *     -# 格納先ブロック位置を更新する（UpdateCurrentWritePosition 関数）。
 *     -# 1～10 を繰り返す。
 *
 * @pre 以下のすべての条件を満たしていること。
 *      - WALSenderとの接続が確立されていること。
 *      - WALWriter用WALバッファが作成済みであること。
 *      - １つのブロックに関するロックが取得済みであること。
 * @attention   データ部を受信する際、受信情報データ部のデータ領域へのアドレス（pWAL）に
 *              WALWriter用WALバッファにおけるｎ個分の領域の先頭アドレスを設定すること。
 * @attention   通信断が発生した場合、通信スレッドは登録WAL情報に終了を示すための情報(*1)を登録する。
 *              ディスク書き込みスレッドは、この終了を示すための情報を取得すると、
 *              WAL受信待ち状態待ちフラグの値を「真」に更新する。
 * @note        (*1)「終了を示すための情報」とは、uXLogID 及び uXRecOff の両方の値が 0の
 *              LSN情報の事である。
 *
 * @retval      RET_OK(0)               正常
 * @retval      RET_NG(1)               異常（通信断以外）
 * @retval      RET_ECONNREFUSED(2)     異常（通信断）
 *
 * @version 1.0.1
 * @date    12/22/2006
 * @since   1.0
*/
int ReceiveWAL()
{
	int             rcode;
	int             nBlocksNeed;
	int             nNextTargetPos;
	CommWALInfo_t   commInfo;

	while(1) {
		/* 「WAL転送」ヘッダ部及び最終書き込みLSNの受信 */
		rcode = ReceiveHeaderAndLSNFromWALSender(&commInfo);
		if (rcode != RET_OK) {
			if (rcode == RET_ECONNREFUSED) {
				OutputSyslog(LOG_ERR, MSGID_WWCOM_CONNECTIONCLOSED);
				return RET_ECONNREFUSED;
			} else {
				/* 「WAL転送応答（NG）」の送信 */
				SendReplyOfSendingWAL(RSLT_SENDWAL_NG);
				OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTORECVFROMWS, MSGITEM_SENDWAL);
				return RET_NG;
			}
		}
		
		/* 使用ブロック数の算出                                       */
		/* orgMessageSizeにはメッセージサイズ(圧縮前)が格納されている */
		nBlocksNeed = commInfo.orgMessageSize / WAL_BLOCK_SIZE;

		/* 登録WAL情報リスト[格納先ブロック位置]への使用ブロック数の登録 */
		SetBlocksToWALInfo(WWCommThRc.nTargetPos, nBlocksNeed);
 
		/* 格納先ブロックの２つ目以降に関するロックの取得 */
		LockWALBuffer(
			(WWCommThRc.nTargetPos + 1) % g_WWEnvInfo.ww_buffers, nBlocksNeed - 1);
 
		/* TCP送受信情報への WAL格納先アドレスの設定 */
		commInfo.pWAL =
			g_WWCmnInfo.pWALBuffer + (WWCommThRc.nTargetPos * WAL_BLOCK_SIZE);

		/* 「WAL転送」における WAL の受信 */
		rcode = ReceiveWALFromWALSender(nBlocksNeed, &commInfo);
		if (rcode != RET_OK) {
			if (rcode == RET_ECONNREFUSED) {
				return RET_ECONNREFUSED;
			} else {
				/* 「WAL転送応答（NG）」の送信 */
				SendReplyOfSendingWAL(RSLT_SENDWAL_NG);
				return RET_NG;
			}
		}

		/* 登録WAL情報リスト[格納先ブロック位置]への最終書き込みLSNの登録 */
		SetLSNToWALInfo(WWCommThRc.nTargetPos,
			commInfo.LSN.uXLogID, commInfo.LSN.uXRecOff);

		/* 次の格納先ブロックの先頭位置を算出 */
		nNextTargetPos =
			(WWCommThRc.nTargetPos + nBlocksNeed) % g_WWEnvInfo.ww_buffers;
 
		/* 次の格納先ブロックの先頭位置に関するロックの取得 */
		LockWALBuffer(nNextTargetPos, 1);

		/* 受信済みの格納先ブロックに関するロックの解放 */
		UnlockWALBuffer(WWCommThRc.nTargetPos, nBlocksNeed);

		/* 「WAL転送応答（OK）」の送信 */
		rcode = SendReplyOfSendingWAL(RSLT_SENDWAL_OK);
		if (rcode != RET_OK) {
			return rcode;
		}

		/* 格納先ブロック位置の更新 */
		UpdateCurrentWritePosition();
	} /* while(1) */

	return RET_OK;
}

/**
 * @fn      int SendRequestToStartSending()
 * @brief   WALSenderへの「送信開始要求」の送信
 *
 *   WALSenderに対して「送信開始要求」をTCP通信により送信する。
 *
 *   詳細は以下のとおり。
 *     -# 送信する TCP送受信情報のヘッダ部に、メッセージID、メッセージサイズ
 *        を設定する。
 *     -# TCP送受信情報のヘッダ部の内容を、WALSenderに送信する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @pre         WALSenderとの接続が確立されていること。
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int SendRequestToStartSending()
{
	ssize_t            rcode;
	CommTargetInfo_t*  pSockInfo;
	CommHeader_t       sendInfo;

	pSockInfo = &(g_WWCmnInfo.commTarget);

	sendInfo.nMessageID   = MSGID_REQSTARTSEND;
	sendInfo.uMessageSize = 0;

	rcode = sendall(pSockInfo->sndrcv_fd,
			(char*)&sendInfo, sizeof(CommHeader_t), MSG_NOSIGNAL);
	DebugOutputMesSyslog(LOG_ERR, "send 'RequestToStartSending' rcode = %ld, Id = %d", rcode, sendInfo.nMessageID);
	if (rcode == SOCKET_ERROR) {
        /* 送信開始要求の送信に失敗した場合は原因によらず異常終了させる        */
        /* Sender/Writer間の同期がとれていないためF/OによるACT化を拒否するため */
    	DebugOutputMesSyslog(LOG_ERR, "send 'RequestToStartSending' was failed (errno = %d)", errno);
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSENDTOWS, MSGITEM_REQUESTTOSTARTSEND);
		return RET_NG;
	}

	return RET_OK;
};

/**
 * @fn      int SendReplyOfSendingWAL(int nResult)
 * @brief   WALSenderへの「WAL転送応答」の送信
 *
 *   WALSenderに対して「WAL転送応答」をTCP通信により送信する。
 *
 *   詳細は以下のとおり。
 *     -# 送信する WAL転送応答情報に、メッセージID、メッセージサイズ、
 *        転送結果を設定する。
 *     -# WAL転送応答情報の内容を、WALSenderに送信する。
 *        エラー発生時は、シグナルを受信していた（errno が EINTR）場合は、
 *        再送信を行い、他の場合は syslog にエラーメッセージを出力し、
 *        エラーが通信断であれば呼出元に RET_ECONNREFUSED を、
 *        エラーが通信断以外であれば呼出元に RET_NGを返却する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @param[in]   nResult  WAL転送結果（RSLT_SENDWAL_OK または RSLT_SENDWAL_NG）。
 * @pre         WALSenderとの接続が確立されていること。
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)               異常（通信断以外）
 * @retval      RET_ECONNREFUSED(2)     異常（WALSender停止による通信断）
 *
 * @version 1.1.0
 * @date    10/02/2007
 * @since   1.0
*/
int SendReplyOfSendingWAL(int nResult)
{
	ssize_t                     rcode;
	CommTargetInfo_t*           pSockInfo;
	CommReplyOfSendingWALInfo_t sendInfo;
	size_t                      nSendSize;
	int                         errRetry;  /* sendをリトライした回数     */
	int ret;							/* ACT化可否判定コマンドの戻り値 */
	
	pSockInfo = &(g_WWCmnInfo.commTarget);

	sendInfo.header.nMessageID   = MSGID_REPLYSENDWAL;
	sendInfo.header.uMessageSize = sizeof(nResult);
	sendInfo.result              = nResult;

	nSendSize = sizeof(CommHeader_t) + sendInfo.header.uMessageSize;

    errRetry = 0;
	do {
		rcode = sendall(pSockInfo->sndrcv_fd,
				(char*)&sendInfo, nSendSize, MSG_NOSIGNAL);
		if (rcode == SOCKET_ERROR) {
			switch (errno) {
                /* 相手起因と判断した場合、通信スレッドを終了させる */
                case ENONET:
                case ECOMM:
                case ESTRPIPE:
                case ENETRESET:
                case ECONNABORTED:
                case ESHUTDOWN:
                case EHOSTDOWN:
                case EHOSTUNREACH:
	    		case ECONNRESET:
    			case EPIPE:
    			case ENETDOWN:
    			case ENETUNREACH:
    			case ETIMEDOUT:    /* ネットワーク切断 */
					DebugOutputSyslog(LOG_ERR, "send 'ReplyOfSendingWAL' was failed at remote error (errno = %d)", errno)
					/* ACT化可否判断コマンド実行中フラグ真 */
					CheckEnableToactTrue()
	    			/* ACT化可否判断 */
					ret = StartCheckEnableToact();
					if (ret == RET_OK) {
						/* ACT化可否判断コマンド実行中フラグ偽 */
						CheckEnableToactfalse()
						return RET_ECONNREFUSED;
					} else {
						/* ACT化可否判断コマンド実行中フラグ(偽)messageスレッド終了フラグ(真) */
						CheckEnableToactfalseAndMessageEndtrue()
						return RET_NG;
					}
                /* 自己起因と判断した場合(リトライを実施するエラー) */
				case EINTR:
				case ENOBUFS:
				case EAGAIN:
                    DebugOutputMesSyslog(LOG_ERR, "send 'ReplyOfSendingWAL' was failed at local error (errno = %d) (retry = %d)", errno, errRetry);

				    errRetry++;
				    if(errRetry < ERR_RETRY_MAX_COUNT) {
                        break;
                    }

                /* 自己起因と判断した場合 */
				default:
                    DebugOutputMesSyslog(LOG_ERR, "send 'ReplyOfSendingWAL' was failed at local error (errno = %d)", errno);
					OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSENDTOWS, MSGITEM_REPLYOFSENDWAL);
					return RET_NG;
			}
		}
	} while (rcode == SOCKET_ERROR);

    DebugOutputMesSyslog(LOG_ERR, "send 'ReplyOfSendingWAL' size = %ld, Id = %d", rcode, sendInfo.header.nMessageID);

	return RET_OK;
};


/**
 * @fn int ReceiveLastLSNFromWALSender(LSNInfo_t* pLastLSN)
 * @brief   WALSenderからの「ファイル転送情報通知」の受信
 *
 *   TCP通信によって、WALSenderから「ファイル転送情報通知」を受信する。
 *
 *   詳細は以下のとおり。
 *     -# WALSenderから、「ファイル転送情報通知」を受信する。エラー発生時は、
 *        syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 受信した「ファイル転送情報通知」から、ファイル転送終了LSNを取得し、
 *        *pLastLSN に設定する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @param[out]  pLastLSN  WALSenderから受信したファイル転送終了 LSN
 *                        を格納する領域のアドレス
 * @pre         以下のすべての条件を満たしていること。
 *              - WALSenderとの接続が確立されていること。
 *              - 呼出元において *pLastLSN の領域が確保済みであること。
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.1.0
 * @date    10/02/2007
 * @since   1.0
*/
int ReceiveLastLSNFromWALSender(LSNInfo_t* pLastLSN)
{
	ssize_t                  rcode;
	CommTargetInfo_t*        pSockInfo;
	CommLastLSNInfo_t        recvInfo;
	int                      errRetry;  /* recvをリトライした回数     */

	pSockInfo = &(g_WWCmnInfo.commTarget);

	/* 「ファイル転送情報通知」の受信 */
	errRetry = 0;
	while(1) {
		rcode = recvall(
			pSockInfo->sndrcv_fd, (char*)&recvInfo, sizeof(CommLastLSNInfo_t), RECV_OPT);
        DebugOutputMesSyslog(LOG_ERR,
            "recv 'LastLSNToWALWriter' size = %ld, Id = %d", rcode, recvInfo.header.nMessageID);


		if (rcode > TRGTSOCK_SHUTDOWN) {
			if (recvInfo.header.nMessageID == MSGID_SENDLASTLSN) {
				break;
			} else {
				OutputSyslog(LOG_ERR,
						MSGID_WWCOM_RECVILLEGALMSG, MSGITEM_LASTLSNINFO);
				return RET_NG;
			}
		} else if (rcode == SOCKET_ERROR) {
            /* ファイル転送情報通知の受信に失敗した場合は原因によらず異常終了させる*/
            /* Sender/Writer間の同期がとれていないためF/OによるACT化を拒否するため */
			if (errno == EINTR || errno == EAGAIN) {
                DebugOutputSyslog(LOG_ERR, "recv 'LastLSNToWALWriter' was failed at local error (errno = %d) (retry = %d)", errno, errRetry);
                
			    errRetry++;
			    if(errRetry < ERR_RETRY_MAX_COUNT) {
                    continue;
                }
			}
			OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTORECVFROMWS, MSGITEM_LASTLSNINFO);
            DebugOutputSyslog(LOG_ERR, "recv 'LastLSNToWALWriter' was failed (errno = %d)", errno);
			return RET_NG;
		} else if (rcode == TRGTSOCK_SHUTDOWN) {
            DebugOutputSyslog(LOG_ERR, "detected: WALSender was down");
			OutputSyslog(LOG_ERR, MSGID_WWCOM_CONNECTIONCLOSED);
			return RET_NG;
		}
	}

	/* ファイル転送終了LSNの取得 */
	*pLastLSN = recvInfo.LSN;

	return RET_OK;
}

/**
 * @fn int ReceiveHeaderAndLSNFromWALSender(CommWALInfo_t* pCommInfo)
 * @brief   WALSenderからのヘッダ部及びLSNの受信
 *
 *   TCP通信によって、WALSenderから「WAL転送」情報のヘッダ部及び最終書き込み
 *   LSNを受信する。
 *
 *   詳細は以下のとおり。
 *     -# WALSenderから、送受信情報のヘッダ部及び最終書き込み LSNを受信する。
 *        エラー発生時は、シグナルを受信していた（errno が EINTR）場合は、
 *        受信待ちを継続し、他の場合は syslog にエラーメッセージを出力し、
 *        エラーが通信断であれば呼出元に RET_ECONNREFUSED を、
 *        エラーが通信断以外であれば呼出元に RET_NGを返却する。
 *
 * @param[out]  pCommInfo   WALSender から受信した内容を格納する領域のアドレス
 * @pre         以下のすべての条件を満たしていること。
 *              - WALSenderとの接続が確立されていること。
 *              - 呼出元において、*pCommInfo の領域が確保済みであること。
 * @retval      RET_OK(0)               正常
 * @retval      RET_NG(1)               異常（通信断以外）
 * @retval      RET_ECONNREFUSED(2)     異常（WALSender停止による通信断）
 *
 * @version 1.1.0
 * @date    14/08/2007
 * @since   1.0
*/
int ReceiveHeaderAndLSNFromWALSender(CommWALInfo_t* pCommInfo)
{
	ssize_t            rcode;
	CommTargetInfo_t*  pSockInfo;
	CommWALInfo_t      recvInfo;
	int            errRetryLocal;      /* recvをリトライした回数(自己起因エラー) */
	int            errRetryRemote;     /* recvをリトライした回数(相手起因エラー) */
	int ret;							/* ACT化可否判定コマンドの戻り値 */
	
	pSockInfo = &(g_WWCmnInfo.commTarget);

	/* ヘッダ部+データ部のLSN+メッセージサイズ(圧縮前) の受信 */
    errRetryLocal  = 0;
    errRetryRemote = 0;
	while (1) {
		rcode = recvall(pSockInfo->sndrcv_fd,
				(char*)&recvInfo, sizeof(CommHeader_t) + sizeof(LSNInfo_t) + sizeof(unsigned int), RECV_OPT);

        DebugOutputMesSyslog(LOG_ERR, "recv 'WAL(header)' rcode = %ld, Id = %d, sendSize = %u, (%u/%u)", rcode, recvInfo.header.nMessageID, recvInfo.header.uMessageSize, recvInfo.LSN.uXLogID, recvInfo.LSN.uXRecOff);
        

		if (rcode > TRGTSOCK_SHUTDOWN) {
			if (   recvInfo.header.nMessageID == MSGID_SENDWAL
			    || recvInfo.header.nMessageID == MSGID_SENDCOMPWAL) {
				break;
			} else {
				OutputSyslog(LOG_ERR, MSGID_WWCOM_RECVILLEGALMSG, MSGITEM_SENDWAL);
                DebugOutputSyslog(LOG_ERR, "recv 'WAL(header)' was failed (nMessageID = %d)", recvInfo.header.nMessageID);
				return RET_NG;
			}
		} else if (rcode == SOCKET_ERROR) {
			switch (errno) {

                /* 相手起因と判断した場合(リトライを実施するエラー) */
				case EAGAIN:
                    DebugOutputSyslog(LOG_ERR, "recv 'WAL(header)' was failed at remote error (errno = %d) (retry = %d)", errno, errRetryRemote);

                    errRetryRemote++;
                    if(errRetryRemote < ERR_RETRY_MAX_COUNT) {
                        break;
                    }

                /* 相手起因と判断した場合 */
                case ENONET:
                case ECOMM:
                case ESTRPIPE:
                case ENETRESET:
                case ECONNABORTED:
                case ESHUTDOWN:
                case EHOSTDOWN:
                case EHOSTUNREACH:
                case ENOTCONN:
	    		case ECONNRESET:
		    	case ECONNREFUSED:
    			case ENETDOWN:
    			case ENETUNREACH:
			    case EPIPE:
    			case ETIMEDOUT:    /* ネットワーク切断 */
					DebugOutputSyslog(LOG_ERR, "recv 'WAL(header)' was failed at remote error (errno = %d)", errno)
					/* ACT化可否判断コマンド実行中フラグ真 */
					CheckEnableToactTrue()
	    			/* ACT化可否判断 */
					ret = StartCheckEnableToact();
					if (ret == RET_OK) {
						/* ACT化可否判断コマンド実行中フラグ偽 */
						CheckEnableToactfalse()
						return RET_ECONNREFUSED;
					} else {
						/* ACT化可否判断コマンド実行中フラグ(偽)messageスレッド終了フラグ(真) */
						CheckEnableToactfalseAndMessageEndtrue()
						return RET_NG;
					}
                /* 自己起因と判断した場合(リトライを実施するエラー) */
                case EINTR:
                case ERESTART:
                case ENOMEM:
                    DebugOutputSyslog(LOG_ERR, "recv 'WAL(header)' was failed at local error (errno = %d) (retry = %d)", errno, errRetryLocal);

                    errRetryLocal++;
                    if(errRetryLocal < ERR_RETRY_MAX_COUNT) {
                        break;
                    }

                /* 自己起因と判断した場合 */
				default:
				    /* WALSender停止以外のrecv異常は全て異常とみなす */
                    DebugOutputSyslog(LOG_ERR, "recv 'WAL(header)' was failed at local error (errno = %d)", errno);
					return RET_NG;
			}
			continue;

		} else if (rcode == TRGTSOCK_SHUTDOWN) {
            /* WALSender停止による通信断 */
            DebugOutputSyslog(LOG_ERR, "detected: WALSender was down");
			/* ACT化可否判断コマンド実行中フラグ真 */
			CheckEnableToactTrue()
			/* ACT化可否判断 */
			ret = StartCheckEnableToact();
			if (ret == RET_OK) {
				/* ACT化可否判断コマンド実行中フラグ偽 */
				CheckEnableToactfalse()
				return RET_ECONNREFUSED;
			} else {
				/* ACT化可否判断コマンド実行中フラグ(偽)messageスレッド終了フラグ(真) */
				CheckEnableToactfalseAndMessageEndtrue()
				return RET_NG;
			}
		}
	}

	pCommInfo->header = recvInfo.header;
	pCommInfo->LSN    = recvInfo.LSN;
	pCommInfo->orgMessageSize = recvInfo.orgMessageSize;

	return RET_OK;
}

/**
 * @fn      int ReceiveWALFromWALSender(int nTotalBlock, CommWALInfo_t* pCommInfo)
 * @brief   WALSenderからのデータ部の受信
 *
 *   TCP通信によって、WALSenderから送受信情報のデータ部を受信する。
 *
 *   詳細は以下のとおり。
 *     -# 受信したデータ部が圧縮済みイメージであるか判定し、圧縮済みイメージの
 *        場合はLSN復元処理にてWALを復元して取得する（DecompressWAL 関数）。
 *     -# 受信したデータ部が非圧縮の場合はWALを取得する（RecvCurrentWAL 関数）。
 *     -# WALWriter用WALバッファにおいて、現在の格納先ブロック位置と、そこから
 *        （nTotalBlock - 1）個後方のブロック位置
 *        （（現在の格納先ブロック位置 + nTotalBlock - 1）% ww_buffers
 *        ：格納予定ブロックの末尾位置）を比較する。
 *     -# 1 の比較の結果、現在の格納先ブロック位置の方が大きい場合は、
 *        （ww_buffers - 現在の格納先ブロック位置）分のWALを、
 *        （pCommBody->pWAL）がさす領域に格納する。その後残りのWAL
 *        （（nTotalBlock - (ww_buffers - 現在の格納先ブロック位置)）分）
 *        を、WALWriter用WALバッファ上の先頭ブロックから格納する。
 *     -# 1 の比較の結果、現在の格納先ブロック位置の方が小さいまたは等しい場合は、
 *        nTotalBlock分のWALを、（pCommBody->pWAL）がさす領域に格納する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @param[in]   nTotalBlock  受信するWALの格納に必要なブロックの数
 * @param[out]  pCommInfo    WALSenderから受信した内容を格納する領域のアドレス
 * @pre         以下のすべての条件を満たしていること。
 *              - WALSenderとの接続が確立されていること。
 *              - 呼出元は、*pCommInfoの領域が確保済みであること。
 *              - 呼出元は、pCommInfo->body.pWAL にWALの格納先となる領域の先頭アドレス
 *                が設定済みであること。
 * @retval      RET_OK(0)               正常
 * @retval      RET_NG(1)               異常
 * @retval      RET_ECONNREFUSED(2)     異常（WALSender停止による通信断）
 *
 * @version 1.1.0
 * @date    08/08/2007
 * @since   1.0
*/
int ReceiveWALFromWALSender(int nTotalBlock, CommWALInfo_t* pCommInfo)
{
	ssize_t            rcode;
	int                nTermPos;
	int                nRecvSize;
	CommTargetInfo_t*  pSockInfo;
	int                nSizeDone;
	char*              pWriteAddr;
	char*              recvAddr;

    /* 受信したデータ部が圧縮済みイメージであるか判定 */
    if(pCommInfo->header.nMessageID == MSGID_SENDCOMPWAL) {
        
        /* 圧縮済みイメージの場合はWALを復元する */
        rcode = DecompressWAL(pCommInfo, &recvAddr);
        if(rcode != RET_OK) {
            return rcode;
        }
    } else {
        /* 通常のWALデータの場合はそのまま受信する */
        rcode = RecvCurrentWAL(pCommInfo, &recvAddr);
        if(rcode != RET_OK) {
            return rcode;
        }
    }

	/* 現在の格納先ブロック位置から（nTotalBlock - 1）個後方の位置を算出 */
	nTermPos = (WWCommThRc.nTargetPos + nTotalBlock - 1) % g_WWEnvInfo.ww_buffers;

	pSockInfo = &(g_WWCmnInfo.commTarget);
	pWriteAddr = pCommInfo->pWAL;
	nSizeDone = 0;

	if (WWCommThRc.nTargetPos > nTermPos) {
		/* 格納先ブロック位置からWALバッファの末尾までのサイズを算出 */
		nRecvSize =
			(g_WWEnvInfo.ww_buffers - WWCommThRc.nTargetPos) * WAL_BLOCK_SIZE;

		/* WAL（前半部）の格納（WALバッファの末尾まで格納） */
		memcpy(pWriteAddr, recvAddr, nRecvSize);
		recvAddr  += nRecvSize;

		/* 残りの受信サイズを算出 */
		nRecvSize =
			(nTotalBlock - (g_WWEnvInfo.ww_buffers - WWCommThRc.nTargetPos)) * WAL_BLOCK_SIZE;
		pWriteAddr = g_WWCmnInfo.pWALBuffer;

		/* WAL（後半部）の格納（WALバッファの末尾まで格納） */
		memcpy(pWriteAddr , recvAddr, nRecvSize);
	} else {
		nRecvSize = nTotalBlock * WAL_BLOCK_SIZE;

		/* WAL（全部）の格納 */
		memcpy(pWriteAddr, recvAddr, nRecvSize);
	}

	return RET_OK;
}


/**
 * @fn int DecompressWAL(CommWALInfo_t* pCommInfo, char **decompressedWALBuffer)
 * @brief   受信した圧縮済みイメージからのWALの復元
 *
 *   受信した圧縮済みイメージに対してLSN復元処理を行い、WALを復元する。
 *
 *   詳細は以下のとおり。
 *     -# 圧縮済みイメージを展開する領域サイズが小さい場合、拡張する。
 *     -# 圧縮済みイメージをTCP通信によって一括受信する。
 *     -# 圧縮済みイメージにLSN復元処理を実行し、WALを復元する。
 *     -# 復元したWAL領域のアドレスをdecompressedWALBufferに設定する。
 *     -# 呼出元へRE_OKを返却する。
 *
 * @param[in]  pCommInfo   WALSender から受信した内容を格納する領域のアドレス
 * @param[out] decompressedWALBuffer 復元したWAL領域のアドレス
 * @retval      RET_OK(0)               正常
 * @retval      RET_NG(1)               異常（通信断以外）
 * @retval      RET_ECONNREFUSED(2)     異常（WALSender停止による通信断）
 *
 * @version 1.1.0
 * @date    08/08/2007
 * @since   1.1
*/
int DecompressWAL(CommWALInfo_t* pCommInfo, char **decompressedWALBuffer)
{
	ssize_t            rcode;
	CommTargetInfo_t*  pSockInfo;
	unsigned int       recvSize;
	int            errRetryLocal;      /* recvをリトライした回数(自己起因エラー) */
	int            errRetryRemote;     /* recvをリトライした回数(相手起因エラー) */
	int ret;							/* ACT化可否判定コマンドの戻り値 */

	pSockInfo = &(g_WWCmnInfo.commTarget);
	
	/* ReceiveHeaderAndLSNFromWALSender 関数で読み込み済みのデータサイズ */
	/* を除いた圧縮データのサイズを求める                                */
	recvSize = pCommInfo->header.uMessageSize 
	         - sizeof(LSNInfo_t) - sizeof(unsigned int);

    /* 圧縮済みイメージを展開する領域サイズが小さい場合、拡張する */
    /* この領域は使いまわすため、グローバル領域に格納する         */
    if(g_cmpBufSize < recvSize) {
        char *tmpBuffer;
        tmpBuffer = realloc(g_cmpBuffer, recvSize);
        if(tmpBuffer == NULL) {
            OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOALLOCMEM);
            free(g_cmpBuffer);
            return RET_NG;
        }
        g_cmpBuffer = tmpBuffer;
        g_cmpBufSize = recvSize;
    }

    /* 圧縮済みイメージを一括して受信する */
    errRetryLocal  = 0;
    errRetryRemote = 0;
    while(1) {
		rcode = recvall(pSockInfo->sndrcv_fd, g_cmpBuffer, recvSize, RECV_OPT);
        DebugOutputMesSyslog(LOG_ERR, "recv 'WAL(body)' rcode = %ld", rcode);

		if (rcode > TRGTSOCK_SHUTDOWN) {
            break;
        } else if (rcode == SOCKET_ERROR) {
			switch (errno) {
                /* 相手起因と判断した場合(リトライを実施するエラー) */
	   			case EAGAIN:
                    DebugOutputSyslog(LOG_ERR, "recv 'WAL(body)' was failed at remote error (errno = %d) (retry = %d)", errno, errRetryRemote);

                    errRetryRemote++;
                    if(errRetryRemote < ERR_RETRY_MAX_COUNT) {
                        break;
                    }

                /* 相手起因と判断した場合 */
                case ENONET:
                case ECOMM:
                case ESTRPIPE:
                case ENETRESET:
                case ECONNABORTED:
                case ESHUTDOWN:
                case EHOSTDOWN:
                case EHOSTUNREACH:
                case ENOTCONN:
	    		case ECONNRESET:
		    	case ECONNREFUSED:
    			case ENETDOWN:
    			case ENETUNREACH:
			    case EPIPE:
    			case ETIMEDOUT:    /* ネットワーク切断 */
					DebugOutputSyslog(LOG_ERR, "recv 'WAL(body)' was failed at remote error (errno = %d)", errno)
					/* ACT化可否判断コマンド実行中フラグ真 */
					CheckEnableToactTrue()
	    			/* ACT化可否判断 */
					ret = StartCheckEnableToact();
					if (ret == RET_OK) {
						/* ACT化可否判断コマンド実行中フラグ偽 */
						CheckEnableToactfalse()
						return RET_ECONNREFUSED;
					} else {
						/* ACT化可否判断コマンド実行中フラグ(偽)messageスレッド終了フラグ(真) */
						CheckEnableToactfalseAndMessageEndtrue()
						return RET_NG;
					}
                /* 自己起因と判断した場合(リトライを実施するエラー) */
                case EINTR:
                case ERESTART:
                case ENOMEM:
                    DebugOutputSyslog(LOG_ERR, "recv 'WAL(body)' was failed at local error (errno = %d) (retry = %d)", errno, errRetryLocal);

                    errRetryLocal++;
                    if(errRetryLocal < ERR_RETRY_MAX_COUNT) {
                        break;
                    }

                /* 自己起因と判断した場合 */
				default:
				    /* WALSender停止以外のrecv異常は全て異常とみなす */
                    DebugOutputSyslog(LOG_ERR, "recv 'WAL(body)' was failed at local error (errno = %d)", errno);
					OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTORECVFROMWS, MSGITEM_SENDWAL);
					return RET_NG;
			} 
			continue;
		} else if (rcode == TRGTSOCK_SHUTDOWN) {
            /* WALSender停止による通信断 */
    		OutputSyslog(LOG_ERR, MSGID_WWCOM_CONNECTIONCLOSED);
            DebugOutputSyslog(LOG_ERR, "detected: WALSender was down");
								/* ACT化可否判断コマンド実行中フラグ真 */
			CheckEnableToactTrue()
			/* ACT化可否判断 */
			ret = StartCheckEnableToact();
			if (ret == RET_OK) {
				/* ACT化可否判断コマンド実行中フラグ偽 */
				CheckEnableToactfalse()
				return RET_ECONNREFUSED;
			} else {
				/* ACT化可否判断コマンド実行中フラグ(偽)messageスレッド終了フラグ(真) */
				CheckEnableToactfalseAndMessageEndtrue()
				return RET_NG;
			}
		}
	}
	
	/* 圧縮済みイメージにLSN復元処理を実行し、WALを復元する */
#ifdef PG82
	memset(g_recvBuffer, 0, pCommInfo->orgMessageSize);
    rcode = create_wal_image(g_recvBuffer, g_cmpBuffer, recvSize);
    if(rcode == 0) {
        OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTODECMPFUNC);
        return RET_NG;
    }
#else
	memcpy(g_recvBuffer, g_cmpBuffer, recvSize);
#endif

#ifdef DEBUG_PROF
    DebugOutputSyslog(LOG_ERR, "PROF:WALDeCompress size\tbefore =\t%u\tafter =\t%ld\t (%x/%x)", recvSize, rcode, pCommInfo->LSN.uXLogID, pCommInfo->LSN.uXRecOff);
#endif
    
    /* 復元したWALを返却 */
    *decompressedWALBuffer = g_recvBuffer;

	return RET_OK;
}


/**
 * @fn int RecvCurrentWAL(CommWALInfo_t* pCommInfo, char **recvWALBuffer)
 * @brief   WALデータの受信
 *
 *   WALデータを受信する。
 *
 *   詳細は以下のとおり。
 *     -# TCP通信によってデータを一括受信する。
 *     -# 受信したデータを格納する領域のアドレスをrecvWALBufferに設定する。
 *     -# 呼出元へRE_OKを返却する。
 *
 * @param[in]  pCommInfo     WALSender から受信した内容を格納する領域のアドレス
 * @param[out] recvWALBuffer 受信したWAL領域のアドレス
 * @retval      RET_OK(0)               正常
 * @retval      RET_NG(1)               異常（通信断以外）
 * @retval      RET_ECONNREFUSED(2)     異常（WALSender停止による通信断）
 *
 * @version 1.1.0
 * @date    08/08/2007
 * @since   1.1
*/
int RecvCurrentWAL(CommWALInfo_t* pCommInfo, char **recvWALBuffer)
{
	ssize_t            rcode;
	CommTargetInfo_t*  pSockInfo;
	unsigned int       recvSize;
	int            errRetryLocal;      /* recvをリトライした回数(自己起因エラー) */
	int            errRetryRemote;     /* recvをリトライした回数(相手起因エラー) */
	int ret;							/* ACT化可否判定コマンドの戻り値 */
	
	pSockInfo = &(g_WWCmnInfo.commTarget);

	/* ReceiveHeaderAndLSNFromWALSender 関数で読み込み済みのデータサイズ */
	/* を除いたデータのサイズを求める                                    */
	recvSize = pCommInfo->header.uMessageSize 
	         - sizeof(LSNInfo_t) - sizeof(unsigned int);

    /* 非圧縮イメージを一括して受信する */
    errRetryLocal  = 0;
    errRetryRemote = 0;
    while(1) {
		rcode = recvall(pSockInfo->sndrcv_fd, g_recvBuffer, recvSize, RECV_OPT);
        DebugOutputMesSyslog(LOG_ERR, "recv 'WAL(body)' rcode = %ld,", rcode);
		if (rcode > TRGTSOCK_SHUTDOWN) {
            break;
        } else if (rcode == SOCKET_ERROR) {
			switch (errno) {
                /* 相手起因と判断した場合(リトライを実施するエラー) */
	   			case EAGAIN:
                    DebugOutputSyslog(LOG_ERR, "recv 'WAL(body)' was failed at remote error (errno = %d) (retry = %d)", errno, errRetryRemote);

                    errRetryRemote++;
                    if(errRetryRemote < ERR_RETRY_MAX_COUNT) {
                        break;
                    }

                /* 相手起因と判断した場合 */
                case ENONET:
                case ECOMM:
                case ESTRPIPE:
                case ENETRESET:
                case ECONNABORTED:
                case ESHUTDOWN:
                case EHOSTDOWN:
                case EHOSTUNREACH:
                case ENOTCONN:
	    		case ECONNRESET:
		    	case ECONNREFUSED:
    			case ENETDOWN:
    			case ENETUNREACH:
			    case EPIPE:
    			case ETIMEDOUT:    /* ネットワーク切断 */
					DebugOutputSyslog(LOG_ERR, "recv 'WAL(body)' was failed at remote error (errno = %d)", errno)
					/* ACT化可否判断コマンド実行中フラグ真 */
					CheckEnableToactTrue()
	    			/* ACT化可否判断 */
					ret = StartCheckEnableToact();
					if (ret == RET_OK) {
						/* ACT化可否判断コマンド実行中フラグ偽 */
						CheckEnableToactfalse()
						return RET_ECONNREFUSED;
					} else {
						/* ACT化可否判断コマンド実行中フラグ(偽)messageスレッド終了フラグ(真) */
						CheckEnableToactfalseAndMessageEndtrue()
						return RET_NG;
					}
                /* 自己起因と判断した場合(リトライを実施するエラー) */
                case EINTR:
                case ERESTART:
                case ENOMEM:
                    DebugOutputSyslog(LOG_ERR, "recv 'WAL(body)' was failed at local error (errno = %d) (retry = %d)", errno, errRetryLocal);

                    errRetryLocal++;
                    if(errRetryLocal < ERR_RETRY_MAX_COUNT) {
                        break;
                    }

                /* 自己起因と判断した場合 */
				default:
				    /* WALSender停止以外のrecv異常は全て異常とみなす */
                    DebugOutputSyslog(LOG_ERR, "recv 'WAL(body)' was failed at local error (errno = %d)", errno);
					OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTORECVFROMWS, MSGITEM_SENDWAL);
					return RET_NG;
			} 
			continue;
		} else if (rcode == TRGTSOCK_SHUTDOWN) {
            /* WALSender停止による通信断 */
    		OutputSyslog(LOG_ERR, MSGID_WWCOM_CONNECTIONCLOSED);
            DebugOutputSyslog(LOG_ERR, "detected: WALSender was down");
			/* ACT化可否判断コマンド実行中フラグ真 */
			CheckEnableToactTrue()
   			/* ACT化可否判断 */
			ret = StartCheckEnableToact();
			if (ret == RET_OK) {
				/* ACT化可否判断コマンド実行中フラグ偽 */
				CheckEnableToactfalse()
				return RET_ECONNREFUSED;
			} else {
				/* ACT化可否判断コマンド実行中フラグ(偽)messageスレッド終了フラグ(真) */
				CheckEnableToactfalseAndMessageEndtrue()
				return RET_NG;
			}
		}
	}

#ifdef DEBUG_PROF
    DebugOutputSyslog(LOG_ERR, "PROF:normal size\tbefore =\t%u\tafter =\t%ld\t (%x/%x)", recvSize, rcode, pCommInfo->LSN.uXLogID, pCommInfo->LSN.uXRecOff);
#endif

    /* 受信したWALを返却 */
    *recvWALBuffer = g_recvBuffer;
    

	return RET_OK;
}

/**
 * @fn      static int StartCheckEnableToact()
 * @brief   ACT可否判断を実行する
 *
 *  詳細は以下の通り。
 *   -# ACT可否判断をpopenにて実行。
 *   -# 結果をマクロにて取得し、戻り値とする。
 *
 * @retval      ret  ACT可否実行結果
 *
 * @version 1.0.0
 * @date    08/07/2007
 * @since   1.0
 */
static int StartCheckEnableToact() {
	int ret;		/* ACT化可否判定コマンドの戻り値 */
	FILE *fp = NULL;
	
	DebugOutputSyslog(LOG_ERR, "StartCheckEnableToact START")
	
	/* ACT化可否判定コマンド実行 */
	fp = popen(g_Command, "r");
	if ( fp == NULL) {
		DebugOutputSyslog(LOG_ERR, "popen error")
		return RET_NG;
	}
	
	ret = pclose(fp);
	if (ret < 0) {
		DebugOutputSyslog(LOG_ERR, "pclose error")
		return RET_NG;
	}
	
	DebugOutputSyslog(LOG_ERR, "StartCheckEnableToact END")
	
	return WEXITSTATUS(ret);

}

/**
 * @fn      int ReceiveTLI()
 * @brief   TLI通知受信
 *
 *   WALSenderから「TLI通知」を受信し、受信したTLIをWALWriter用構造体に登録する。
 *   WALSenderへ「TLI通知応答」を送信する。
 *
 *   詳細は以下のとおり。
 *     -# WALSenderから TLI を受信する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 受信した「TLI」を、スレッド間共有情報構造体に格納する。
 *     -# 「TLI通知応答」を、WALSender に送信する（SendReplyOfSendingTLI 関数）。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *
 * @pre WALSenderとの接続が確立されていること。
 *
 * @retval      RET_OK(0)               正常
 * @retval      RET_NG(1)               異常
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
*/
int ReceiveTLI()
{
	ssize_t            rcode;
	CommTargetInfo_t*  pSockInfo;
	CommSendTLIInfo_t  recvTLIInfo;
	int                retreply;
	
	pSockInfo = &(g_WWCmnInfo.commTarget);

	rcode = recvall(pSockInfo->sndrcv_fd,
			(char*)&recvTLIInfo, sizeof(CommSendTLIInfo_t), RECV_OPT);

    DebugOutputMesSyslog(LOG_ERR, "recv 'TLI' rcode = %ld, Id = %d, sendSize = %u, TLI = %u", rcode, recvTLIInfo.header.nMessageID, recvTLIInfo.header.uMessageSize, recvTLIInfo.TLI);

	if (rcode > TRGTSOCK_SHUTDOWN) {
		if ( recvTLIInfo.header.nMessageID == MSGID_SENDTLI ) {
		} else {
			OutputSyslog(LOG_ERR, MSGID_WWCOM_RECVILLEGALMSG, MSGITEM_SENDTLI);
            DebugOutputSyslog(LOG_ERR, "recv 'TLI' was failed (nMessageID = %d)", recvTLIInfo.header.nMessageID);
			return RET_NG;
		}
	} else {
        /* TLI通知受信に失敗した場合は原因によらず異常終了させる        */
        /* Sender/Writer間の同期がとれていないためF/OによるACT化を拒否するため */
    	DebugOutputMesSyslog(LOG_ERR, "recv 'TLI' was failed (errno = %d)", errno);
		OutputSyslog(LOG_ERR, MSGID_WWCOM_RECVILLEGALMSG, MSGITEM_SENDTLI);
		return RET_NG;
	}
	
	/* 「TLI」の格納 */
	g_WWCmnInfo.nTimeLineID = recvTLIInfo.TLI;
	
	/* 「TLI通知応答」の送信 */
	retreply = SendReplyOfSendingTLI();
	if (retreply != RET_OK) {
		return RET_NG;
	}

	return RET_OK;
}


/**
 * @fn      int SendReplyOfSendingTLI()
 * @brief   WALSenderへの「TLI通知応答」の送信
 *
 *   WALSenderに対して「TLI通知応答」をTCP通信により送信する。
 *
 *   詳細は以下のとおり。
 *     -# 送信する TCP送受信情報のヘッダ部に、メッセージID、メッセージサイズ
 *        を設定する。
 *     -# TCP送受信情報のヘッダ部の内容を、WALSenderに送信する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @pre         WALSenderとの接続が確立されていること。
 * @retval      RET_OK(0)     正常
 * @retval      RET_NG(1)     異常
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
*/
int SendReplyOfSendingTLI()
{
	ssize_t            rcode;
	CommTargetInfo_t*  pSockInfo;
	CommHeader_t       sendInfo;

	pSockInfo = &(g_WWCmnInfo.commTarget);

	sendInfo.nMessageID   = MSGID_REPLYSENDTLI;
	sendInfo.uMessageSize = 0;

	rcode = sendall(pSockInfo->sndrcv_fd,
			(char*)&sendInfo, sizeof(CommHeader_t), MSG_NOSIGNAL);
	DebugOutputMesSyslog(LOG_ERR, "send 'SendReplyOfSendingTLI' rcode = %ld, Id = %d", rcode, sendInfo.nMessageID);
	if (rcode == SOCKET_ERROR) {
        /* TLI通知応答の送信に失敗した場合は原因によらず異常終了させる        */
        /* Sender/Writer間の同期がとれていないためF/OによるACT化を拒否するため */
    	DebugOutputMesSyslog(LOG_ERR, "send 'SendReplyOfSendingTLI' was failed (errno = %d)", errno);
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSENDTOWS, MSGITEM_REPLYOFSENDTLI);
		return RET_NG;
	}

	return RET_OK;
};

/**
 * @fn      int ReceiveHistory()
 * @brief   history情報通知受信
 *
 *   WALSenderから「history情報通知」を受信し、受信したhistory情報をWALWriter用history情報バッファに登録する。
 *   WALSenderへ「history情報通知応答」を送信する。
 *
 *   詳細は以下のとおり。
 *     -# WALSenderから送受信情報のヘッダ部を受信する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 受信したヘッダ情報から、WALWriter用history情報バッファの領域の大きさを指定し、バッファを確保する。
 *     -# WALSenderから  history情報 を受信する。
 *     -# 受信した「history情報」を、WALWriter用history情報バッファに登録する。
 *     -# 「history情報通知応答」を、WALSender に送信する（SendReplyOfSendingHistory 関数）。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *
 * @pre WALSenderとの接続が確立されていること。
 *
 * @retval      RET_OK(0)               正常
 * @retval      RET_NG(1)               異常
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
*/
int ReceiveHistory()
{
	ssize_t            rcode;
	CommTargetInfo_t*  pSockInfo;
	int                retreply;
	
	pSockInfo = &(g_WWCmnInfo.commTarget);

    /* ヘッダ部 の受信 */
	rcode = recvall(pSockInfo->sndrcv_fd,
			(char*)&(g_WWCmnInfo.recvHistoryInfo.header), sizeof(CommHeader_t), RECV_OPT);

    DebugOutputMesSyslog(LOG_ERR, "recv 'Histroy(header)' rcode = %ld, Id = %d, sendSize = %u", rcode, g_WWCmnInfo.recvHistoryInfo.header.nMessageID, g_WWCmnInfo.recvHistoryInfo.header.uMessageSize);
    

	if (rcode > TRGTSOCK_SHUTDOWN) {
		if ( g_WWCmnInfo.recvHistoryInfo.header.nMessageID == MSGID_SENDHISTORY ) {
		} else {
			OutputSyslog(LOG_ERR, MSGID_WWCOM_RECVILLEGALMSG, MSGITEM_SENDHISTORY);
            DebugOutputSyslog(LOG_ERR, "recv 'Histroy(header)' was failed (nMessageID = %d)", g_WWCmnInfo.recvHistoryInfo.header.nMessageID);
			return RET_NG;
		}
	} else {
        /* history情報通知受信に失敗した場合は原因によらず異常終了させる        */
        /* Sender/Writer間の同期がとれていないためF/OによるACT化を拒否するため */
    	DebugOutputMesSyslog(LOG_ERR, "recv 'Histroy(header)' was failed (errno = %d)", errno);
		OutputSyslog(LOG_ERR, MSGID_WWCOM_RECVILLEGALMSG, MSGITEM_SENDHISTORY);
		return RET_NG;
	}
	
	/* WALWriter用history情報バッファの領域確保 */
	g_WWCmnInfo.recvHistoryInfo.pHistory = (char*)malloc(g_WWCmnInfo.recvHistoryInfo.header.uMessageSize);
	if (g_WWCmnInfo.recvHistoryInfo.pHistory == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOALLOCMEM);
		return RET_NG;
	}
	
	/* ボディ部 の受信 */
	rcode = recvall(pSockInfo->sndrcv_fd,
			g_WWCmnInfo.recvHistoryInfo.pHistory, g_WWCmnInfo.recvHistoryInfo.header.uMessageSize, RECV_OPT);
			
	DebugOutputMesSyslog(LOG_ERR, "recv 'Histroy(body)' rcode = %ld,", rcode);
    
	if (rcode <= TRGTSOCK_SHUTDOWN) {
        /* history情報通知受信に失敗した場合は原因によらず異常終了させる        */
        /* Sender/Writer間の同期がとれていないためF/OによるACT化を拒否するため */
    	DebugOutputMesSyslog(LOG_ERR, "recv 'Histroy(body)' was failed (errno = %d)", errno);
		OutputSyslog(LOG_ERR, MSGID_WWCOM_RECVILLEGALMSG, MSGITEM_SENDHISTORY);
		return RET_NG;
	}
	
	/* 「history情報通知応答」の送信 */
	retreply = SendReplyOfSendingHistory();
	if (retreply != RET_OK) {
		return RET_NG;
	}

	return RET_OK;
}


/**
 * @fn      int SendReplyOfSendingHistory()
 * @brief   WALSenderへの「history情報通知応答」の送信
 *
 *   WALSenderに対して「history情報通知応答」をTCP通信により送信する。
 *
 *   詳細は以下のとおり。
 *     -# 送信する TCP送受信情報のヘッダ部に、メッセージID、メッセージサイズ
 *        を設定する。
 *     -# TCP送受信情報のヘッダ部の内容を、WALSenderに送信する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @pre         WALSenderとの接続が確立されていること。
 * @retval      RET_OK(0)     正常
 * @retval      RET_NG(1)     異常
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
*/
int SendReplyOfSendingHistory()
{
	ssize_t            rcode;
	CommTargetInfo_t*  pSockInfo;
	CommHeader_t       sendInfo;

	pSockInfo = &(g_WWCmnInfo.commTarget);

	sendInfo.nMessageID   = MSGID_REPLYSENDHISTORY;
	sendInfo.uMessageSize = 0;

	rcode = sendall(pSockInfo->sndrcv_fd,
			(char*)&sendInfo, sizeof(CommHeader_t), MSG_NOSIGNAL);
	DebugOutputMesSyslog(LOG_ERR, "send 'SendReplyOfSendingHistory' rcode = %ld, Id = %d", rcode, sendInfo.nMessageID);
	if (rcode == SOCKET_ERROR) {
        /* history情報通知応答の送信に失敗した場合は原因によらず異常終了させる        */
        /* Sender/Writer間の同期がとれていないためF/OによるACT化を拒否するため */
    	DebugOutputMesSyslog(LOG_ERR, "send 'SendReplyOfSendingHistory' was failed (errno = %d)", errno);
		OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSENDTOWS, MSGITEM_REPLYOFSENDTLI);
		return RET_NG;
	}

	return RET_OK;
};
