/*****************************************************************************
   Copyright (C) 2006-2008 NTT
  *****************************************************************************/
/**
 * @file    WWCommon.c
 * @brief   WALWriterスレッド間共通関数定義
 *
 * WALWriterのスレッド間において共通に使用する関数を定義する。
 *
 * @version 1.3.0
 * @date    06/17/2008
 * @since   1.0
 */
#define SET_GLOBAL_VAR_
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include "PREXMessage.h"
#include "WWCommon.h"
#include "WWCommThread.h"
#include "WWWriterThread.h"

/*********************
 * グローバル変数定義
 ********************/
WWSettingInfo_t g_WWEnvInfo;          ///< 設定ファイル項目情報
WWCommonInfo_t  g_WWCmnInfo;          ///< スレッド間共通情報

/**
 * @fn      void TermWALWriter()
 * @brief   WALWriterプロセスのリソース解放
 *
 *   プロセスのリソースを解放する。
 *
 *   詳細は以下のとおり。
 *     -# リソース解放処理実行ロックの取得を試みる（trylock）。取得に成功した
 *        場合は、2～8 を実行する。失敗した場合は、スレッドを終了する。
 *     -# 通信スレッドのリソースを解放する。（TermWWCommThread 関数）。
 *     -# history用バッファを解放する。
 *     -# ディスク書き込みスレッドのリソースを解放する（TermWWWriterThread関数）。
 *     -# プロセスで使用しているグローバルリソース（ブランク領域、WALWriter用
 *        WALバッファ、WALWriter用WALバッファへのアクセス用ロックリスト、
 *        登録WAL情報リスト、設定ファイル情報）を解放する（TermSettingInfo 関数）。
 *     -# TLI通知完了情報を破棄する（TermTLIInfo 関数）。
 *     -# バックアップ取得完了情報を破棄する（TermGettingBackupInfo 関数）。
 *     -# WAL受信待ち状態情報を破棄する（TermWALBufferInfo 関数）。
 *     -# 最新反映終端LSN情報を破棄する（TermNewestLSNInfo 関数）。
 *     -# ACT化可否コマンド情報を破棄する（TermCheckEnableToactInfo 関数）。
 *     -# リソース解放処理実行ロックを解放する。
 *
 * @version 1.3.0
 * @date    07/25/2008
 * @since   1.0
*/
void TermWALWriter()
{
	int pos;                              /* リスト上の位置        */

	int rcode;                            /* 諸関数の戻り値        */

	/* 解放処理実行ロックの取得 */
	rcode = pthread_mutex_trylock(&g_WWCmnInfo.lockTermWW);
	if (rcode != PTHREAD_OK) {
		/* 既に他のスレッドが終了処理を実施している場合、スレッドを終了する */
		pthread_exit(NULL);
	}

	/* 通信スレッド関連リソースの解放 */
	TermWWCommThread();

	/* WALWriter用WALバッファの解放 */
	if (g_WWCmnInfo.pWALBuffer != NULL) {
		free(g_WWCmnInfo.pWALBuffer);
		g_WWCmnInfo.pWALBuffer = NULL;
	}
	/*200806 history用バッファの解放 */
	if (g_WWCmnInfo.recvHistoryInfo.pHistory != NULL) {
		free(g_WWCmnInfo.recvHistoryInfo.pHistory);
		g_WWCmnInfo.recvHistoryInfo.pHistory = NULL;
	}
	/* <- 200806 */
	
	/* ディスク書き込みスレッド関連リソースの解放 */
	TermWWWriterThread();

	/* ブランク領域の解放 */
	if (g_WWCmnInfo.pBlank != NULL) {
		free(g_WWCmnInfo.pBlank);
		g_WWCmnInfo.pBlank = NULL;
	}

	/* WALWriter用WALバッファへのアクセス用ロックリストの解放 */
	if (g_WWCmnInfo.pBufferMutexList != NULL) {
		for (pos = 0; pos < g_WWEnvInfo.ww_buffers; pos++) {
			pthread_mutex_destroy(g_WWCmnInfo.pBufferMutexList + pos);
		}
		free(g_WWCmnInfo.pBufferMutexList);
		g_WWCmnInfo.pBufferMutexList = NULL;
	}

	/* 登録WAL情報リストの解放 */
	if (g_WWCmnInfo.pWALInfoList != NULL) {
		free(g_WWCmnInfo.pWALInfoList);
		g_WWCmnInfo.pWALInfoList = NULL;
	}

    /*
	 * 設定ファイル情報における各項目領域の解放
     */
	TermSettingInfo(&g_WWEnvInfo);

	/* 200807 -> */
	/* TLI通知完了情報の破棄 */
	TermTLIInfo();
	/* <- 200807 */

	/* バックアップ取得完了情報の破棄 */
	TermGettingBackupInfo();

	/* WAL受信待ち状態情報の破棄 */
	TermWALBufferInfo();

	/* 最新反映終端LSN情報の破棄 */
	TermNewestLSNInfo();

	/* ACT化可否コマンド情報の破棄 */
	TermCheckEnableToactInfo();
	
	/* 解放処理実行ロックの解放 */
	pthread_mutex_unlock(&(g_WWCmnInfo.lockTermWW));
}

/**
 * @fn      void LockWALBuffer(int posFrom, int nCount)
 * @brief   WALWriter用WALバッファアクセス用ロック取得
 *
 *   WALWriter用WALバッファにおける指定された位置のブロックから、指定された
 *   個数分のブロックに関するロックを取得する。
 *
 *   詳細は以下のとおり。
 *     -# 指定されたブロック位置から指定された数のロックを取得する。
 *        取得できないロックがある場合、取得できるまで処理を待ち合わせる。
 *
 * @param[in]  posFrom WALWriter用WALバッファ上での、ロック取得対象となるブロック位置。
 *                     複数のロックを取得する場合は、その先頭ブロック位置。
 * @param[in]  nCount  取得するロックの数。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
void LockWALBuffer(int posFrom, int nCount)
{
	int  nPos;

	for (nPos = posFrom; nPos < posFrom + nCount; nPos++) {
		pthread_mutex_lock(g_WWCmnInfo.pBufferMutexList + (nPos % g_WWEnvInfo.ww_buffers));
	}

	return;
}

/**
 * @fn      void UnlockWALBuffer(int posFrom, int nCount)
 * @brief   WALWriter用WALバッファアクセス用ロック解放
 *
 *   WALWriter用WALバッファにおける指定された位置のブロックから、指定された
 *   個数分のブロックに関するロックを解放する。
 *
 *   詳細は以下のとおり。
 *     -# 指定されたブロック位置から指定された数のロックを解放する。
 *
 * @param[in]  posFrom   WALWriter用WALバッファ上での、ロック解放対象となるブロック位置。
 *                       複数のロックを解放する場合は、その先頭ブロック位置。
 * @param[in]  nCount    解放するロックの数。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
void UnlockWALBuffer(int posFrom, int nCount)
{
	int  nPos;

	for (nPos = posFrom; nPos < posFrom + nCount; nPos++) {
		pthread_mutex_unlock(g_WWCmnInfo.pBufferMutexList + (nPos % g_WWEnvInfo.ww_buffers));
	}

	return;
}

/**
 * @fn      void HandleSIGTERM(int nSig)
 * @brief   SIGTERM 用のシグナルハンドラ
 *
 *   SIGTERM 用のシグナルハンドラを定義する。
 *
 *   詳細は以下のとおり。
 *     -# ACT化処理中（ACT化処理中フラグが TRUE）あるいは
 *        SBY化処理中（SBY化処理中フラグが TRUE）の場合、メッセージキューID を取得し、
 *        2～3 を実行する。その他の場合は、4 を実行する。
 *     -# ACT化処理中（ACT化処理中フラグが TRUE）の場合は、メッセージキューに
 *        「ACT化応答（失敗）」メッセージを送信し（SendMessageReplyToACTToQueue 関数）、
 *        4 を実行する。
 *     -# SBY化処理中（SBY化処理中フラグが TRUE）の場合は、メッセージキューに
 *        「SBY化応答（失敗）」メッセージを送信し（SendMessageReplyToSBYToQueue 関数）、
 *        4 を実行する。
 *     -# プロセスのリソースを解放する（TermWALWriter 関数）。
 *     -# プロセスを終了する。
 *
 * @pre  SIGTERMを受信した場合に動作するシグナルハンドラとして、この関数を登録すること。
 *
 * @param[in]   nSig       シグナルID。
 * @return      なし
 *
 * @version 1.0.1
 * @date    12/22/2006
 * @since   1.0
*/
void HandleSIGTERM(int nSig)
{
	int rcode;
	int nMsgQueID;

	if ((g_WWCmnInfo.nFlgToACT == TRUE) || (g_WWCmnInfo.nFlgToSBY == TRUE)) {
		/* メッセージキューIDの取得 */
		nMsgQueID = msgget((key_t)(g_WWEnvInfo.prex_msgkey), MSGGET_FLG_RW);
		if (nMsgQueID == MSGQ_ERROR) {
			OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOGETMSGQUE);
		} else {
			if (g_WWCmnInfo.nFlgToACT == TRUE) {
				/* 「ACT化応答」メッセージの送信 */
				rcode = SendMessageReplyToACTToQueue(nMsgQueID, MSNID_ACTNG);
				if (rcode != RET_OK) {
					OutputSyslog(LOG_ERR,
							MSGID_WWCOM_FAILEDTOSENDMSGTOQUE, MSGQITEM_REPLYOFTOACT);
				}
			} else if (g_WWCmnInfo.nFlgToSBY == TRUE) {
				/* 「SBY化応答」メッセージの送信 */
				rcode = SendMessageReplyToSBYToQueue(nMsgQueID, MSNID_SBYNG);
				if (rcode != RET_OK) {
					OutputSyslog(LOG_ERR,
							MSGID_WWCOM_FAILEDTOSENDMSGTOQUE, MSGQITEM_REPLYOFTOSBY);
				}
			}
		}
	}

	TermWALWriter();

	exit(EXIT_FAILURE);
}

/**
 * @fn      SendMessageReplyToACTToQueue(int nMsgQueID, int nResult)
 * @brief   「ACT化応答」メッセージの送信
 *
 *   メッセージキューに「ACT化応答」メッセージを送信する。
 *
 *   詳細は以下のとおり。
 *     -# nResult が MSNID_ACTOK、MSNID_ACTNG 以外である場合、syslogにエラーメッセージを
 *        出力し、呼出元に RET_NG を返却する。
 *     -# メッセージキューに Clusterコマンド宛の「ACT化応答」メッセージを送信する。
 *        エラー発生時は、呼出元に RET_NG を返却する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @note    宛先IDは MSGTO_CCTOACT（CCコマンド（ACT化要求））、指示IDは nResultである。
 *
 * @param[in]   nMsgQueID     メッセージキューID。
 * @param[in]   nResult       ACT化結果（成功：MSNID_ACTOK、失敗：MSNID_ACTNG）
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int SendMessageReplyToACTToQueue(int nMsgQueID, int nResult)
{
	int              rcode;                /* 諸関数の戻り値 */
	MessageQueItem_t msgInfo;              /* 送信メッセージ格納用領域 */

	/* ACT化結果（nResult）の判定 */
	if ((nResult != MSNID_ACTOK) && (nResult != MSNID_ACTNG)) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_ILLEGALPARAM, MSGQITEM_REPLYOFTOACT);
		return RET_NG;
	}

	/* 「ACT化応答」メッセージ情報の作成*/
	msgInfo.nAddressID = MSGTO_CCTOACT;
	msgInfo.nMissionID = nResult;

	/* メッセージキューへのメッセージ送信 */
	rcode = msgsnd(nMsgQueID, &msgInfo, MESSAGE_LEN_NO_LSN, 0);
	if (rcode == MSGQ_ERROR) {
		return RET_NG;
	}

	return RET_OK;
}

/**
 * @fn      SendMessageReplyToSBYToQueue(int nMsgQueID, int nResult)
 * @brief   「SBY化応答」メッセージの送信
 *
 *   メッセージキューに「SBY化応答」メッセージを送信する。
 *
 *   詳細は以下のとおり。
 *     -# nResult が MSNID_SBYOK、MSNID_SBYNG 以外である場合、syslogにエラーメッセージを
 *        出力し、呼出元に RET_NG を返却する。
 *     -# メッセージキューにClusterコマンド宛の「SBY化応答」メッセージを送信する。
 *        エラー発生時は、呼出元に RET_NGを返却する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @note    宛先IDは MSGTO_CCTOSBY（CCコマンド（SBY化要求））、指示IDは nResultである。
 *
 * @param[in]   nMsgQueID     メッセージキューID。
 * @param[in]   nResult       SBY化結果（成功：MSNID_SBYOK、失敗：MSNID_SBYNG）
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int SendMessageReplyToSBYToQueue(int nMsgQueID, int nResult)
{
	int               rcode;                /* 諸関数の戻り値 */
	MessageQueItem_t  msgInfo;              /* 送信メッセージ格納用領域 */

	/* SBY化結果（nResult）の判定 */
	if ((nResult != MSNID_SBYOK) && (nResult != MSNID_SBYNG)) {
		OutputSyslog(LOG_ERR, MSGID_WWCOM_ILLEGALPARAM, MSGQITEM_REPLYOFTOSBY);
		return RET_NG;
	}

	/* 「SBY化応答」メッセージ情報の作成*/
	msgInfo.nAddressID = MSGTO_CCTOSBY;
	msgInfo.nMissionID = nResult;

	/* メッセージキューへのメッセージ送信 */
	rcode = msgsnd(nMsgQueID, &msgInfo, MESSAGE_LEN_NO_LSN, 0);
	if (rcode == MSGQ_ERROR) {
		return RET_NG;
	}

	return RET_OK;
}

/**
 * @fn     int SendPITRMessageToQueue(int nMsgQueID)
 * @brief   「WAL反映」メッセージの送信
 *
 *   メッセージキューに「WAL反映」メッセージを送信する。
 *
 *   詳細は以下のとおり。
 *     -# メッセージキューに残存する psotgres（リカバリ処理）宛の
 *        メッセージ（「WAL反映」メッセージ）を全て排除する。
 *     -# 最新反映終端 LSN アクセス用ロックを取得する。
 *     -# 現在の反映終端LSNを取得する。
 *     -# 最新反映終端 LSN アクセス用ロックを解放する。
 *     -# 送信メッセージ情報を生成する。
 *     -# メッセージキューに「WAL反映」メッセージを送信する。
 *        エラー発生時は、呼出元に RET_NG を返却する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @param[in]   nMsgQueID     メッセージキューID。
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int SendPITRMessageToQueue(int nMsgQueID)
{
	int               rcode;        /* 諸関数の戻り値 */
	MessageQueItem_t  msgInfo;      /* 送信メッセージ格納用領域 */
	LSNInfo_t         NewestLSN;    /* 現在の反映終端 LSN */

	ssize_t           scode;        /* 諸関数の戻り値 */
	MessageQueItem_t  oldMsgInfo;   /* 古い「WAL反映」メッセージ格納用領域 */

	/*WALSenderとの接続前か判定*/
	if (!g_WWCmnInfo.nFlgInit){
		WaitForTLICompleted()
	}
	/* メッセージキューに残存する「WAL反映」メッセージの排除 */
	scode = MSGQ_ERROR;
	do {
		scode = msgrcv(
			nMsgQueID, &oldMsgInfo, MESSAGE_LEN, MSGTO_PGRCV, IPC_NOWAIT);
	} while (scode != MSGQ_ERROR);

	/* 最新の反映終端LSNの取得 */
	pthread_mutex_lock(&(g_WWCmnInfo.NewestLSNInfo.mutex));
	NewestLSN = g_WWCmnInfo.NewestLSNInfo.LSN;
	pthread_mutex_unlock(&(g_WWCmnInfo.NewestLSNInfo.mutex));
	
	/* 「WAL反映」メッセージ情報の作成*/
	msgInfo.nAddressID = MSGTO_PGRCV;
	msgInfo.nMissionID = MSNID_PITR;
	msgInfo.LSN        = NewestLSN;

	/* メッセージキューへのメッセージ送信 */
	rcode = msgsnd(nMsgQueID, &msgInfo, MESSAGE_LEN, 0);
	if (rcode == MSGQ_ERROR) {
		return RET_NG;
	}

	return RET_OK;
}
