/*****************************************************************************
   Copyright (C) 2006-2007 NTT
  *****************************************************************************/
/**
 * @file    WWGetterThread.c
 * @brief   WALWriterのバックアップ取得スレッドに関する関数の定義
 *
 * WALWriterのバックアップ取得スレッドにおいて使用する関数を定義する。
 *
 * @version 1.1.0
 * @date    08/08/2007
 * @since   1.0
 */
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <unistd.h>
#include <errno.h>
#include "WWCommon.h"
#include "WWGetterThread.h"
#include "PREXDefine.h"
#include "PREXUtil.h"
#include "PREXMessage.h"
#include "GetCheckPointLSN.h"
#include "GetCheckPointLSNFromBackupLabel.h" 

/**
* @fn      void* WWGetterThreadMain(void* arg)
 * @brief   WALWriterバックアップ取得スレッドメイン
 *
 *   他サーバからアーカイブログ、ベースバックアップを取得する。
 *
 *   詳細は以下のとおり。
 *     -# パラメータチェックを実施する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、プロセスを終了する（ExitWALWriter 関数）。
 *     -# バックアップ取得スレッド情報より以下の情報を取り出す。
 *        - ファイル転送終了LSN
 *     -# アーカイブログの転送を行う（RsyncWALSegment 関数）。
 *        エラー発生時は、syslogにエラーメッセージを出力し、プロセスを終了する（ExitWALWriter 関数）。
 *     -# ディスク書き込みスレッドにる最終WALセグメントの書き込み待ちを行う。
 *     -# 初期起動フラグが「偽」の場合、postmasterプロセスの存在確認を行う。postmasterプロセスが存在しない場合、以下の6～8の処理を行う。
 *     -# postmasterの起動を行う。
 *     -# postmasterの起動に成功した場合、メッセージキューにSBY化成功の「SBY化応答」メッセージを送信する（SendMessageReplyToSBYToQueue 関数）。
 *        エラー発生時は、syslogにエラーメッセージを出力し、プロセスを終了する（ExitWALWriter 関数）。
 *     -# postmasterの起動に失敗した場合、メッセージキューにSBY化失敗の「SBY化応答」メッセージを送信する（SendMessageReplyToSBYToQueue 関数）。
 *        エラー発生時は、syslogにエラーメッセージを出力し、プロセスを終了する（ExitWALWriter 関数）。
 *     -# メッセージキューに「WAL反映」メッセージを送信する（SendPITRMessageToQueue 関数）。
 *        エラー発生時は、syslogにエラーメッセージを出力し、プロセスを終了する（ExitWALWriter 関数）。
 *     -# バックアップ取得完了フラグの値を「真」に更新し、他スレッドに通知する（NotifyGetterIsCompleted 関数）。
 *     -# WALSenderに「バックアップ取得完了通知」を送信する（SendReplyOfCompletingBackup 関数）。
 *     -# スレッドを終了する。
 *
 * @param[in]   arg    バックアップ取得スレッド情報：WWGetterThreadInfo_t（ファイル転送終了LSNと送信開始LSN）を示す構造体へのアドレス
 *                     
 * @retval      NULL
 *
 * @version 1.1.0
 * @date    08/08/2007
 * @since   1.0
*/
void* WWGetterThreadMain(void* arg)
{
	int rcode;							/* 諸関数の戻り値 */
	WWGetterThreadInfo_t* pGetterThRc;	/* バックアップ取得スレッド情報 */
	int nMsgqID;						/* メッセージキューID */
	
	/* 引数チェック */
	if (arg == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWGET_ILLEGALPARAM, MSGWD_WWGETTERTHREADINFO);
		ExitWALWriter(EXIT_FAILURE);
	}
	pGetterThRc = (WWGetterThreadInfo_t*)arg;

	/* アーカイブログ転送 */
	rcode = RsyncWALSegment(&(pGetterThRc->SendEndLSN));
	if (rcode == RET_NG) {
		OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTORSYNC, MSGWD_ARCHIVELOG);
		free(pGetterThRc);
		ExitWALWriter(EXIT_FAILURE);
	}
	
	/* メッセージキューIDの取得 */
	nMsgqID = msgget((key_t)(g_WWEnvInfo.prex_msgkey), MSGGET_FLG_RW);
	if (nMsgqID == MSGQ_ERROR) {
		OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTOGETMSGQ);
		free(pGetterThRc);
		ExitWALWriter(EXIT_FAILURE);
	}
	
	/* 最終WALセグメント書込み待ち(1回でも書込みを行うまで待つ) */
	WaitForWritingWALToFile();	
	
	/* 初期起動以外の場合、postmasterの起動を行い「SBY化応答」メッセージを送信する */
	if (pGetterThRc->nFlgInit != TRUE) {
		rcode = system("pg_ctl start");
		if (WIFEXITED(rcode) == FALSE || WEXITSTATUS(rcode) != RET_OK) {
			/* 起動失敗の場合、SBY化失敗の「SBY化応答」メッセージ送信 */
			OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTOBOOTPROCESS, MSGWD_POSTMASTER);
			rcode = SendMessageReplyToSBYToQueue(nMsgqID, MSNID_SBYNG);
			if (rcode == RET_NG) {
				OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTOSEND, MSGWD_QMSG_REPLYTOSBY);
			}
			free(pGetterThRc);
			ExitWALWriter(EXIT_FAILURE);
		} else {
			/* 起動成功の場合、SBY化成功の「SBY化応答」メッセージ送信 */
			rcode = SendMessageReplyToSBYToQueue(nMsgqID, MSNID_SBYOK);
			if (rcode == RET_NG) {
				OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTOSEND, MSGWD_QMSG_REPLYTOSBY);
				free(pGetterThRc);
				ExitWALWriter(EXIT_FAILURE);
			}
		}
	}
	
	/* 「WAL反映」メッセージを送信 */
	rcode = SendPITRMessageToQueue(nMsgqID);
	if (rcode == RET_NG) {
		OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTOSEND, MSGWD_QMSG_REQUESTTOPITR);
		free(pGetterThRc);
		ExitWALWriter(EXIT_FAILURE);
	}

	/* バックアップ取得完了フラグの値を「真」に更新し、他のスレッドに通知 */
	NotifyGetterIsCompleted();
	
	/* 「バックアップ取得完了通知」を送信 */
	rcode = SendReplyOfCompletingBackup();
	if(rcode != RET_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTOSEND, MSGWD_QMSG_BKUPFINISHED);
		free(pGetterThRc);
		ExitWALWriter(EXIT_FAILURE);
    }
	
	free(pGetterThRc);
	return NULL;
}

/**
 * @fn	  int RsyncWALSegment(LSNInfo_t* SendEndLSN)
 * @brief   アーカイブログ転送処理
 *
 *  送信開始LSNからファイル転送終了LSNまでのアーカイブログを転送する。
 *
 *   詳細は以下のとおり。
 *     -# パラメータチェックを実施する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、戻り値としてRET_NGを呼出元に返却する。
 *     -# pg_controlよりチェックポイントLSNを取得する(GetCheckPoint 関数)。
 *     -# チェックポイントLSNから送信開始LSNを導き出す。
 *     -# 送信開始LSNから転送するファイル名を導き出す（GetXLogPathNameUnknownTLI 関数）。
 *        タイムライン番号の部分は「????????」というワイルドカード表現とする。
 *     -# 引数で与えられた文字列と送信開始LSNから導き出したファイル名からrsyncのコマンド文字列を生成する
 *        (rsync -e ssh -pq [actuser]@[acthost]:[actdir]/[ファイル名] [sbydir])。
 *     -# rsyncコマンドを実行し転送を行う。
 *     -# 転送結果が正常の場合、送信開始LSNの値を1セグメントサイズ分進めて、10以降の処理を行う。
 *     -# 転送結果がエラー（ファイルが見つからない）であれば、ベースバックアップの取得を行い（RsyncBackup 関数）、
 *        送信開始LSNの値を取得し直し、10以降の処理を行う。
 *     -# 転送結果がエラー（上記以外）であればエラーとし、syslogにエラーメッセージを出力後、呼出元に RET_NGを返却する。
 *     -# 送信開始LSNがファイル転送終了LSN以下の間、以下11～17の処理を繰り返す。
 *     -# 引数で与えられた文字列と送信開始LSNから導き出したファイル名からrsyncのコマンド文字列を生成する
 *        (rsync -e ssh -pq [actuser]@[acthost]:[actdir]/[ファイル名] [sbydir])。
 *     -# rsyncコマンドを実行し転送を行う。
 *     -# 転送結果が正常の場合、送信開始LSNの値を1セグメントサイズ分進める。
 *     -# 転送結果がエラー（ファイルが見つからない）であればスリープした後、再度転送を行う。
 *     -# 転送結果がエラー（上記以外）であればエラーとし、syslogにエラーメッセージを出力後、呼出元に RET_NGを返却する。
 *     -# 転送したファイルのサイズ確認を行う。サイズ確認は、ログサイズ削減により圧縮されたアーカイブログである場合を
 *        考慮する必要があるため、リモートホスト上でのファイルサイズとの比較を行う。
 *     -# サイズがWALセグメントの規定されたファイルサイズと一致しない場合、スリープした後、再度転送を行う。
 *     -# 戻り値としてRET_OKを呼出元に返却する。
 *
 * @param[in]   SendEndLSN    ファイル転送終了LSN
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.1.0
 * @date    09/01/2007
 * @since   1.0
*/
int RsyncWALSegment(LSNInfo_t* SendEndLSN)
{
	int rcode;
	int nGetBackup = 0;
	int nRetryCount;
	char sCommand[WWGET_COMMANDMAX_RSYNCARCHIVE]; /* コマンド文字列作成用 */
	char strFileName[XLOGFILENAME_LEN + 1];
	LSNInfo_t SendStartLSN;		/* 送信開始LSN */
	LSNInfo_t TempLSN;
	
	/* 引数チェック */
	if (SendEndLSN == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWGET_ILLEGALPARAM, MSGWD_SENDENDLSN);
		return RET_NG;
	}

	/* 転送の必要性をチェック */
	if ((SendEndLSN->uXLogID  == INVALID_XLOGID) &&
		(SendEndLSN->uXRecOff == INVALID_XRECOFF)) {
		return RET_OK;
	}
	
	/* pg_controlよりチェックポイントLSNを取得 */
	rcode = GetCheckPointLSN(&SendStartLSN, NULL);
	if (rcode == RET_NG) {
		OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTOGET, MSGWD_CHECKPOINTLSN);
		return RET_NG;
	}
	/* ファイルの先頭を指す値にし送信開始LSNとする */
	SendStartLSN.uXRecOff = SendStartLSN.uXRecOff / WAL_SEG_FILESIZE * WAL_SEG_FILESIZE;

	/* 送信開始LSNが、ファイル転送終了LSNの値より大きい場合 */
	if (CompLSN_IsLessThan(*SendEndLSN, SendStartLSN)) {
		/* ファイル転送終了LSNをひとつ進めて送信開始LSNと同じになった場合は、転送不要（両系でアーカイブログの差分なし） */
		/* 上記以外はエラー */
		TempLSN = *SendEndLSN;
		ADDSegSizeToLSNInfo(TempLSN);
		TempLSN.uXRecOff = TempLSN.uXRecOff / WAL_SEG_FILESIZE * WAL_SEG_FILESIZE;
		if (CompLSN_IsLessThan(TempLSN, SendStartLSN)) {
			OutputSyslog(LOG_ERR, MSGID_WWGET_ILLEGALPARAM, MSGWD_SENDENDLSN);
			return RET_NG;
		} else {
			return RET_OK;
		}
	}

	/* LSNが示すWALセグメントのファイル名 */
	GetXLogPathNameUnknownTLI(strFileName, &SendStartLSN);
	

	nRetryCount = 0;
	while (1) {
		/* アーカイブログディレクトリからWALセグメントを取得するコマンド文字列の作成 */
		sprintf(sCommand, WWGET_CMDFMT_RSYNCFORFILE, g_WWEnvInfo.actarchiveuser, g_WWEnvInfo.actarchivehost,
					g_WWEnvInfo.actarchivedir, strFileName, g_WWEnvInfo.sbyarchivedir);
		/* rsync実行 */
		rcode = system(sCommand);
		if (WIFEXITED(rcode) != FALSE && WEXITSTATUS(rcode) == RET_OK) {
			/* 転送したファイルのサイズ確認 */
			sprintf(sCommand, WWGET_CMDFMT_CHECKFILESIZE, g_WWEnvInfo.actarchiveuser, g_WWEnvInfo.actarchivehost, g_WWEnvInfo.actarchivedir, strFileName, g_WWEnvInfo.sbyarchivedir, strFileName);
			rcode = system(sCommand);
			if (WIFEXITED(rcode) == FALSE || WEXITSTATUS(rcode) != RET_OK) {
				/* サイズが不正な場合はスリープ後、再試行する */
				if (nRetryCount < WWGET_RETRYMAX) {
					OutputSyslog(LOG_WARNING, MSGID_WWGET_WARNINGRETRYRSYNC, WWGET_POLLING_INTERVAL);
					sleep(WWGET_POLLING_INTERVAL);
					nRetryCount++;
					continue;
				}
				/* 再試行上限回数を越えていたらベースバックアップを取得する */
				nGetBackup = 1;
				break;
			}
			
			/* 送信開始LSNの値を1セグメントサイズ分進める */
			ADDSegSizeToLSNInfo(SendStartLSN);
			break;
		} else {
			if (WIFEXITED(rcode) == FALSE || WEXITSTATUS(rcode) != RET_RSYNC_ERR_FILE_NOT_FOUND) {
				OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTORSYNC, MSGWD_ARCHIVELOG);
				return RET_NG;
			}
			
			/* 見つからない場合はスリープ後、再試行する */
			if (nRetryCount < WWGET_RETRYMAX) {
				OutputSyslog(LOG_WARNING, MSGID_WWGET_WARNINGRETRYRSYNC, WWGET_POLLING_INTERVAL);
				sleep(WWGET_POLLING_INTERVAL);
				nRetryCount++;
				continue;
			}

			/* 再試行上限回数を越えていたらベースバックアップを取得する */
			nGetBackup = 1;
			break;
		}
	}

	if (nGetBackup == 1) {
		/* ベースバックアップの取得 */
		rcode = RsyncBackup();
		if (rcode == RET_NG) {
			OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTORSYNC, MSGWD_BASEBACKUP);
			return RET_NG;
		}
		
		
		/* 送信開始LSNをbackup_labelから取得し直す */
		rcode = GetCheckPointLSNFromBackupLabel(&SendStartLSN);
		if (rcode == RET_NG) {
			/* 送信開始LSNをpg_controlから取得し直す */
			rcode = GetCheckPointLSN(&SendStartLSN, NULL);
			if (rcode == RET_NG) {
				OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTOGET, MSGWD_CHECKPOINTLSN);
				return RET_NG;
			}
		}

		/* ファイルの先頭を指す値にする */
		SendStartLSN.uXRecOff = SendStartLSN.uXRecOff / WAL_SEG_FILESIZE * WAL_SEG_FILESIZE;
	}
	
	nRetryCount = 0;
	while (CompLSN_IsLessEqual(SendStartLSN, *SendEndLSN)) {
		/* LSNが示すWALセグメントのファイル名 */
		GetXLogPathNameUnknownTLI(strFileName, &SendStartLSN);
		
		/* アーカイブログディレクトリからWALセグメントを取得するコマンド文字列の作成 */
		sprintf(sCommand, WWGET_CMDFMT_RSYNCFORFILE, g_WWEnvInfo.actarchiveuser, g_WWEnvInfo.actarchivehost,
					g_WWEnvInfo.actarchivedir, strFileName, g_WWEnvInfo.sbyarchivedir);

		/* rsync実行 */
		rcode = system(sCommand);
		if (WIFEXITED(rcode) != FALSE && WEXITSTATUS(rcode) == RET_RSYNC_ERR_FILE_NOT_FOUND) {
			/* 見つからない場合はスリープ後、再試行する */
			if (nRetryCount < WWGET_RETRYMAX) {
				OutputSyslog(LOG_WARNING, MSGID_WWGET_WARNINGRETRYRSYNC, WWGET_POLLING_INTERVAL);
				sleep(WWGET_POLLING_INTERVAL);
				nRetryCount++;
				continue;
			}
			/* 再試行上限回数を越えていたらエラー */
			OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTORSYNC, MSGWD_ARCHIVELOG);
			return RET_NG;
			
		}else if (WIFEXITED(rcode) == FALSE || WEXITSTATUS(rcode) != RET_OK) {
			OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTORSYNC, MSGWD_ARCHIVELOG);
			return RET_NG;
		}

		/* 転送したファイルのサイズ確認 */
		sprintf(sCommand, WWGET_CMDFMT_CHECKFILESIZE, g_WWEnvInfo.actarchiveuser, g_WWEnvInfo.actarchivehost, g_WWEnvInfo.actarchivedir, strFileName, g_WWEnvInfo.sbyarchivedir, strFileName);
		rcode = system(sCommand);
		if (WIFEXITED(rcode) == FALSE || WEXITSTATUS(rcode) != RET_OK) {
			/* サイズが不正な場合はスリープ後、再試行する */
			if (nRetryCount < WWGET_RETRYMAX) {
				OutputSyslog(LOG_WARNING, MSGID_WWGET_WARNINGRETRYRSYNC, WWGET_POLLING_INTERVAL);
				sleep(WWGET_POLLING_INTERVAL);
				nRetryCount++;
				continue;
			}
			/* 再試行上限回数を越えていたらエラー */
			OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTORSYNC, MSGWD_ARCHIVELOG);
			return RET_NG;
		}
		
		/* 送信開始LSNの値を1セグメントサイズ分進める */
		ADDSegSizeToLSNInfo(SendStartLSN);
	}

	return RET_OK;
}

/**
 * @fn      int RsyncBackup()
 * @brief   ベースバックアップ転送処理
 *
 *   指定されたベースバックアップディレクトリをディレクトリごと転送する。
 *
 *   詳細は以下のとおり。
 *     -# 設定ファイル情報より以下の情報を取り出す。
 *        - 転送対象アーカイブログディレクトリを持つACT側ホスト：actarchivehost
 *        - actarchivehostへのログインユーザ名：actarchiveuser
 *        - ACT側アーカイブディレクトリ：actarchivedir
 *        - SBY側アーカイブディレクトリ：sbyarchivedir
 *        - 転送対象ベースバックアップディレクトリを持つACT側ホスト：actbackuphost
 *        - actbackuphostへのログインユーザ名：actbackupuser
 *        - ACT側ベースバックアップディレクトリ：actbackupdir
 *        - SBY側ベースバックアップディレクトリ：sbybackupdir
 *     -# 取り出した情報からベースバックアップの転送を行うrsyncのコマンド文字列を生成する。
 *        (rsync -e ssh -pq [actbackupuser]@[actbackuphost]:[actbackupdir] [sbybackupdir])。
 *     -# rsyncコマンドを実行し転送を行う。
 *     -# 転送結果が正常の場合、6以降の処理を行う。
 *     -# 転送結果がエラーの場合、エラーとし、syslogにエラーメッセージを出力後、呼出元に戻り値としてRET_NGを返却する。
 *     -# ACT側アーカイブディレクトリからSBY側アーカイブディレクトリへ「*.backup」というワイルドカードで示されるファイルの転送を行うrsyncのコマンド文字列を生成する。
 *        (rsync -e ssh -pq [actarchiveuser]@[actarchivehost]:[actarchivedir]/ *.backup [sbyarchivedir])。
 *     -# rsyncコマンドを実行し転送を行う。
 *     -# 転送結果がエラーの場合、エラーとし、syslogにエラーメッセージを出力する。
 *     -# 呼出元に戻り値としてRET_OKを返却する。
 *
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int RsyncBackup()
{
	int rcode;
	char sCommand[WWGET_COMMANDMAX_RSYNCBACKUP]; /* コマンド文字列作成用 */

	/* ベースバックアップ取得用コマンド文字列の作成 */
	sprintf(sCommand, WWGET_CMDFMT_RSYNCFORDIR, g_WWEnvInfo.actbackupuser, g_WWEnvInfo.actbackuphost,
				g_WWEnvInfo.actbackupdir, g_WWEnvInfo.sbybackupdir);
	
	rcode = system(sCommand);
	if (WIFEXITED(rcode) == FALSE || WEXITSTATUS(rcode) != RET_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTORSYNC, MSGWD_BASEBACKUP);
		return RET_NG;
	}

	/* .backupファイル(アーカイブディレクトリ)取得用コマンド文字列の作成 */
	sprintf(sCommand, WWGET_CMDFMT_RSYNCFORFILE, g_WWEnvInfo.actarchiveuser, g_WWEnvInfo.actarchivehost,
				g_WWEnvInfo.actarchivedir, WWGET_BACKUPHISTORYFILE, g_WWEnvInfo.sbyarchivedir);

	rcode = system(sCommand);
	if (WIFEXITED(rcode) == FALSE || WEXITSTATUS(rcode) != RET_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTORSYNC, MSGWD_BACKUPHISTORYFILE);
	}
	
	return RET_OK;
}


/**
 * @fn      int SendReplyOfCompletingBackup(void)
 * @brief   WALSenderへの「バックアップ取得完了通知」の送信
 *
 *   WALSenderに対して「バックアップ取得完了通知」をTCP通信により送信する。
 *
 *   詳細は以下のとおり。
 *     -# 送信する Wバックアップ取得完了通知情報に、メッセージID、メッセージサイズ、
 *        転送結果を設定する。
 *     -# バックアップ取得完了通知情報の内容を、WALSenderに送信する。
 *        エラー発生時は、シグナルを受信していた（errno が EINTR）場合は、
 *        再送信を行い、他の場合は syslog にエラーメッセージを出力し、
 *        エラーが通信断であれば呼出元に RET_ECONNREFUSED を、
 *        エラーが通信断以外であれば呼出元に RET_NGを返却する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @pre         WALSenderとの接続が確立されていること。
 *
 * @param[in]   pGetterThRc    バックアップ取得スレッド情報
 *
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)               異常（通信断以外）
 * @retval      RET_ECONNREFUSED(2)     異常（通信断）
 *
 * @version 1.1.0
 * @date    10/30/2007
 * @since   1.0
*/
int SendReplyOfCompletingBackup(void)
{
	ssize_t                     rcode;
	CommTargetInfo_t*           pSockInfo;
	CommReplyOfSendingWALInfo_t sendInfo;
	size_t                      nSendSize;
	int                         errRetry;  /* sendをリトライした回数     */

    /* WAL転送応答と競合する場合があるため、転送サイズを同一にする */
	sendInfo.header.nMessageID   = MSGID_BKUPFINISHED;
	sendInfo.header.uMessageSize = sizeof(int);
	sendInfo.result              = 0;
	nSendSize = sizeof(CommHeader_t) + sendInfo.header.uMessageSize;

	pSockInfo = &(g_WWCmnInfo.commTarget);

    errRetry = 0;
	do {
		rcode = sendall(pSockInfo->sndrcv_fd,
				(char*)&sendInfo, nSendSize, MSG_NOSIGNAL);
		if (rcode == SOCKET_ERROR) {
			switch (errno) {
                /* 相手起因と判断した場合 */
                case ENONET:
                case ECOMM:
                case ESTRPIPE:
                case ENETRESET:
                case ECONNABORTED:
                case ESHUTDOWN:
                case EHOSTDOWN:
                case EHOSTUNREACH:
	    		case ECONNRESET:
    			case EPIPE:
    			case ENETDOWN:
    			case ENETUNREACH:
    			case ETIMEDOUT:    /* ネットワーク切断 */
    			    DebugOutputMesSyslog(LOG_ERR, "send 'ReplyOfCompletingBackup' was failed at remote error (errno = %d)", errno);
					return RET_ECONNREFUSED;
					
                /* 自己起因と判断した場合(リトライを実施するエラー) */
				case EINTR:
				case ENOBUFS:
				case EAGAIN:
                    DebugOutputMesSyslog(LOG_ERR, "send 'ReplyOfCompletingBackup' was failed at local error (errno = %d) (retry = %d)", errno, errRetry);

				    errRetry++;
				    if(errRetry < ERR_RETRY_MAX_COUNT) {
                        break;
                    }

                /* 自己起因と判断した場合 */
				default:
                    DebugOutputMesSyslog(LOG_ERR, "send 'ReplyOfCompletingBackup' was failed at local error (errno = %d)", errno);
					OutputSyslog(LOG_ERR, MSGID_WWCOM_FAILEDTOSENDTOWS, MSGQITEM_REPLYOFBKUPFINISHED);
					return RET_NG;
			}
		}
	} while (rcode == SOCKET_ERROR);
    DebugOutputMesSyslog(LOG_ERR,
         "send 'ReplyOfCompletingBackup' rcode = %ld, Id = %d", rcode, sendInfo.header.nMessageID);
	return RET_OK;
};



