/*****************************************************************************
   Copyright (C) 2006-2008 NTT
  *****************************************************************************/
/**
 * @file    WWMessageThread.c
 * @brief   WALWriterのメッセージスレッドに関する関数の定義
 *
 * WALWriterのメッセージスレッドにおいて使用する関数を定義する。
 *
 * @version 1.3.0
 * @date    07/25/2008
 * @since   1.0
 */
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <errno.h>
#include "PREXMessage.h"
#include "WWCommon.h"
#include "WWMessageThread.h"

/**
 * @fn      void* WWMessageThreadMain(void* arg)
 * @brief   WALWriterメッセージスレッドメイン
 *
 *   メッセージキューからのメッセージの受信及び「ACT化要求」メッセージの送信を
 *   行なう。
 *
 *   詳細は以下のとおり。
 *     -# メッセージスレッドの初期設定を行なう（InitWWMessageThread 関数）。
 *        エラー発生時は、プロセスを終了する（ExitWALWriter 関数）。
 *     -# 「ACT化要求」メッセージを受信する（ReceiveMessageToACTFromQueue 関数）。
 *        また、「ACT化要求」メッセージより反映終端LSNを取得する。
 *        エラー発生時は、プロセスを終了する（ExitWALWriter 関数）。
 *     -# 初期起動モードの場合、7 の処理を実行する。
 *     -# 初期起動モードではない場合、ファイル転送情報通知取得完了フラグが FALSE
 *        であればエラーとみなして、プロセスを終了する（ExitWALWriter 関数）。
 *     -# 書き込み取得スレッドによるTLI通知が完了するまで待機する
 *        （WaitForTLICompleted 関数）。
 *     -# バックアップ取得スレッドによるファイル転送が完了するまで待機する
 *        （WaitForGettingBackupCompleted 関数）。
 *     -# ディスク書き込みスレッドがWAL受信待ち状態となるまで待機する
 *        （WaitForWALBufferEmpty 関数）。
 *     -# 「ACT化要求」メッセージを送信する（SendMessageToACTToQueue 関数）。
 *        エラー発生時は、プロセスを終了する（ExitWALWriter 関数）。
 *     -# プロセスを終了する（ExitWALWriter 関数）。
 *
 * @note      pthread_create() によって呼び出される。
 *            ReceiveMessageToACTFromQueue 関数を呼び出すと、「ACT化要求」メッセージを受信
 *            するか、あるいはエラーが発生するまで処理はブロックされる。
 *            2 の処理で取得した反映終端LSNは、SendMessageToACTToQueue 関数内で使用される。
 *
 * @param[in]   arg    メッセージスレッド情報構造体へのアドレス。
 *                     WWMessageThreadInfo_t* 型にキャストして使用する。
 *
 * @return      NULL
 *
 * @version 1.3.0
 * @date    07/25/2008
 * @since   1.0
*/
void* WWMessageThreadMain(void* arg)
{
	int                    rcode;         // 諸関数の戻り値
	WWMessageThreadInfo_t* pMsgThRc;      // メッセージスレッド情報
    LSNInfo_t              LSN;           // 「ACT化要求」メッセージに付与された
                                          // 反映終端LSN
	
	/* 引数チェック */
	if(arg == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWGET_ILLEGALPARAM, MSGWD_WWMESSAGETHREADINFO);
		ExitWALWriter(EXIT_FAILURE);
	}

	/* メッセージスレッド情報を取得 */
	pMsgThRc = (WWMessageThreadInfo_t*)arg;

	/* メッセージスレッドの初期設定 */
	rcode = InitWWMessageThread(pMsgThRc);
	if (rcode != RET_OK) {
		free(pMsgThRc);
		ExitWALWriter(EXIT_FAILURE);
	}

	/* 「ACT化要求」メッセージの受信 */
	rcode = ReceiveMessageToACTFromQueue(pMsgThRc->nMsgqID, &LSN);
	if (rcode != RET_OK) {
		free(pMsgThRc);
		ExitWALWriter(EXIT_FAILURE);
	}
	
	if (g_WWCmnInfo.nFlgInit == FALSE) {
		if (g_WWCmnInfo.nFlgSendFilesCompleted == FALSE) {
			OutputSyslog(LOG_ERR, MSGID_WWMSG_ACTBEFORESYNCBACKUP);
			free(pMsgThRc);
			ExitWALWriter(EXIT_FAILURE);
		}

		/* ファイル転送完了の待機 */
		WaitForGettingBackupCompleted();

		/* WALWriter用WALバッファが空になるのを待機 */
		WaitForWALBufferEmpty();
	}

	/* 「ACT化要求」メッセージの送信 */

	rcode = SendMessageToACTToQueue(pMsgThRc->nMsgqID, &LSN);
	if (rcode != RET_OK) {
		free(pMsgThRc);
		ExitWALWriter(EXIT_FAILURE);
	}

	/* プロセスの終了 */
	free(pMsgThRc);
	ExitWALWriter(EXIT_SUCCESS);
}

/**
 * @fn      int InitWWMessageThread(WWMessageThreadInfo_t* pMsgThRc)
 * @brief   メッセージスレッドの初期設定
 *
 *   メッセージスレッドで使用するメッセージキューの確認を行なう。
 *
 *   詳細は以下のとおり。
 *     -# 引数 pMsgThRc が非NULLであることをチェックする。NULL である場合、
 *        syslog にエラーメッセージを出力し（OutputSyslog 関数）、呼出元に RET_NG を返却する。
 *     -# メッセージキューのキー値が、正数値であるかどうかを確認する。正数値でない場合は、
 *        syslog にエラーメッセージを出力し（OutputSyslog 関数）、呼出元に RET_NG を返却する。
 *     -# メッセージキューの存在確認を行なう。
 *        存在する場合は、メッセージスレッド情報構造体のメンバにメッセージキューIDを登録する。
 *        存在しない場合は、syslog にエラーメッセージを出力し（OutputSyslog 関数）、
 *        呼出元に RET_NG を返却する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @param[out]   pMsgThRc  メッセージスレッド情報構造体へのアドレス。
 *
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int InitWWMessageThread(WWMessageThreadInfo_t* pMsgThRc)
{
	/* パラメータのチェック */
	if (pMsgThRc == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWMSG_ILLEGALPARAM, INTERNAL_PARAMETERS);
		return RET_NG;
	}

	/* メッセージキューのキー値の検査 */
	if (pMsgThRc->nMsgKey <= (key_t)0) {
		OutputSyslog(LOG_ERR, MSGID_WWMSG_ILLEGALPARAM, ENVITEM_MSGKEY);
		return RET_NG;
	}

	/* メッセージキューIDの取得 */
	pMsgThRc->nMsgqID = msgget((key_t)(pMsgThRc->nMsgKey), MSGGET_FLG_RW);
	if (pMsgThRc->nMsgqID == MSGQ_ERROR) {
		OutputSyslog(LOG_ERR, MSGID_WWMSG_FAILEDTOGETMSGQUE);
		return RET_NG;
	}

	return RET_OK;
}

/**
 * @fn      int ReceiveMessageToACTFromQueue(int nMsgQueID, LSNInfo_t *lsn)
 * @brief   「ACT化要求」メッセージの受信
 *
 *   メッセージキューから「ACT化要求」メッセージを受信する。
 *
 *   詳細は以下のとおり。
 *     -# メッセージキューからメッセージを受信する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 受信したメッセージが「ACT化要求」メッセージの場合、ACT化処理中フラグを TRUE
 *        に更新し、反映終端LSNを退避し、呼出元に RET_OKを返却する。
 *     -# 受信したメッセージが「ACT化要求」メッセージ以外の場合、またはシグナルを受信した場合は、
 *        1 に戻る。
 *
 * @param[in]   nMsgQueID  メッセージキューID。
 * @param[out]  lsn        「ACT化要求」メッセージに付与された反映終端LSN
 *
 * @retval      RET_OK(0)     正常終了（「ACT化要求」メッセージを受信）
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.1.0
 * @date    08/01/2008
 * @since   1.0
*/
int ReceiveMessageToACTFromQueue(int nMsgQueID, LSNInfo_t *lsn)
{
	ssize_t           scode;                /* 諸関数の戻り値 */
	MessageQueItem_t  msgInfo;              /* 受信メッセージ格納用領域 */
	int               flgTLI = FALSE;       /* TLI受信メッセージフラグ */

	do {
		scode = msgrcv(nMsgQueID, &msgInfo,
			MESSAGE_LEN, MSGTO_WW, 0);
		if (scode >= (ssize_t)MESSAGE_LEN_NO_LSN) {
			if (msgInfo.nMissionID == MSNID_TOACT) {
				g_WWCmnInfo.nFlgToACT = TRUE;
				/* ACT化可否判断コマンド完了待ち(ACT化可否判断コマンドが実行中の場合) */
				WaitForCheckEnableToact();
				/* ACT化可否判断コマンドで停止と判定された場合スレッドを終了 */
				if (g_WWCmnInfo.CheckEnableToactInfo.messageEndFlag) {
					pthread_exit(NULL);
				}
				/*WALSenderとの接続前か判定*/
				if (g_WWCmnInfo.nFlgInit){
					flgTLI = TRUE;
				}
			} else if (msgInfo.nMissionID == MSNID_RECEIVETLI) {
				/* 2008 add TLI通知フラグの値を「真」に更新し、他のスレッドに通知 */
				NotifyTLIIsCompleted();
				flgTLI = TRUE;
			} else {
				OutputSyslog(LOG_ERR, MSGID_WWMSG_NOTACTMSGRECVED);
			}
		} else {
			if (errno == EINTR) {
				continue;
			}
			OutputSyslog(LOG_ERR,
					MSGID_WWMSG_FAILEDTORECVMSGFROMQUE, MSGQITEM_TOACT);
			return RET_NG;
		}
	} while (g_WWCmnInfo.nFlgToACT != TRUE || flgTLI != TRUE);
	
	/* 「ACT化要求」メッセージに付与された反映終端LSNを設定 */
	*lsn = msgInfo.LSN;

	return RET_OK;
}

/**
 * @fn      int SendMessageToACTToQueue(int nMsgQueID, LSNInfo_t *lsn)
 * @brief   「ACT化要求」メッセージの送信
 *
 *   メッセージキューにpostgres（リカバリ処理）宛の「ACT化要求」メッセージを送信する。
 *
 *   詳細は以下のとおり。
 *     -# メッセージキューに残存する postgres（リカバリ処理）宛の
 *        メッセージ（「WAL反映」メッセージ）を全て排除する。
 *     -# 最新反映終端 LSN アクセス用ロックを取得する。
 *     -# 現在の反映終端 LSN を取得する。
 *        通信スレッドにより更新された反映終端LSNと、「ACT化要求」に付与された反映終端
 *        LSNを比較し、より最新のLSNを現在の反映終端LSNとする。
 *     -# 最新反映終端 LSN アクセス用ロックを解放する。
 *     -# 送信メッセージ情報を生成する。
 *     -# メッセージキューに「ACT化要求」メッセージを送信する。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に
 *        RET_NG(1)を返却する。
 *     -# 呼出元にRET_OK(0)を返却する。
 *
 * @note        宛先 ID は MSGTO_PGRCV（postgres（リカバリ処理））、指示 ID は
 *              MSNID_TOACT（ACT化要求）である。
 *
 * @param[in]   nMsgQueID  メッセージキューID。
 * @param[in]   lsn        「ACT化要求」メッセージに付与された反映終端LSN
 *
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.1.0
 * @date    08/01/2007
 * @since   1.0
*/
int SendMessageToACTToQueue(int nMsgQueID, LSNInfo_t *lsn)
{
	int               rcode;        /* 諸関数の戻り値 */
	MessageQueItem_t  msgInfo;      /* 送信メッセージ格納用領域 */
	LSNInfo_t         NewestLSN;    /* 現在の反映終端 LSN */

	ssize_t           scode;        /* 緒関数の戻り値 */
	MessageQueItem_t  oldMsgInfo;   /* 古い「WAL反映」メッセージ格納用領域 */

	/* メッセージキューに残存する「WAL反映」メッセージの排除 */
	scode = MSGQ_ERROR;
	do {
		scode = msgrcv(
			nMsgQueID, &oldMsgInfo, MESSAGE_LEN, MSGTO_PGRCV, IPC_NOWAIT);
	} while (scode != MSGQ_ERROR);

	/* 最新の反映終端LSNの取得 */
    pthread_mutex_lock(&(g_WWCmnInfo.NewestLSNInfo.mutex));
	DebugOutputMesSyslog(LOG_ERR, "MessageThread WWCmnInfo=(%u/%u)  LastLSN=(%u/%u)", 
	    g_WWCmnInfo.NewestLSNInfo.LSN.uXLogID,g_WWCmnInfo.NewestLSNInfo.LSN.uXRecOff,
	    lsn->uXLogID, lsn->uXRecOff);

    NewestLSN = *lsn;
    pthread_mutex_unlock(&(g_WWCmnInfo.NewestLSNInfo.mutex));

	/* 「ACT化要求」メッセージ情報の作成*/
	msgInfo.nAddressID = MSGTO_PGRCV;
	msgInfo.nMissionID = MSNID_TOACT;
	msgInfo.LSN        = NewestLSN;

	/* メッセージキューへのメッセージ送信 */
	rcode = msgsnd(nMsgQueID, &msgInfo, MESSAGE_LEN, 0);
	if (rcode == MSGQ_ERROR) {
		OutputSyslog(LOG_ERR, MSGID_WWMSG_FAILEDTOSENDMSGTOQUE, MSGQITEM_TOACT);
		return RET_NG;
	}

	return RET_OK;
}

