/*****************************************************************************
   Copyright (C) 2006-2008 NTT
  *****************************************************************************/
/**
 * @file    WWWriterThread.c
 * @brief   WALWriterのディスク書き込みスレッドに関する関数の定義
 *
 * WALWriterのディスク書き込みスレッドにおいて使用する関数を定義する。
 *
 * @version 1.3.0
 * @date    07/25/2008
 * @since   1.0
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include "PREXMessage.h"
#include "WWCommon.h"
#include "WWWriterThread.h"
#include <errno.h>
#include <dirent.h>

WWWriterThreadInfo_t WrtThRc;     ///< ディスク書き込みスレッド情報

static int filter_walfile(const struct dirent*);

/**
 * @fn      void* WWWriterThreadMain(void* arg)
 * @brief   WALWriterディスク書き込みスレッドメイン
 *
 *   WALWriter用WALバッファに格納されているWALを、最終WALセグメントに書き込む。
 *
 *   詳細は以下のとおり。
 *     -# ディスク書き込みスレッドの初期設定を行なう（InitWWWriterThread 関数）。
 *        エラー発生時は、プロセスを終了する（ExitWALWriter 関数）。
 *     -# historyファイル（postgresql用とLastLSN用）の作成（WriteHistory 関数）。
 *     -# メッセージキューへ「TLI更新」メッセージを送信する。
 *     -# あることを他スレッドに通知する（NotifyTLIIsCompleted 関数）。
 *     -# WALWriter用WALバッファの先頭ブロックに関するアクセス用ロックの取得を
 *        試行（trylock）する。
 *     -# 2 の実行時にロックの取得に成功した場合、6～11 を実行する。
 *     -# 2 の実行時にロックの取得に失敗した場合、WALWriter用WALバッファが空で
 *        あることを他スレッドに通知する（NotifyWALBufferIsEmpty 関数）。
 *     -# WALWriter用WALバッファの先頭ブロックに関するアクセス用ロックを取得する
 *        （LockWALBuffer 関数）。
 *     -# WAL受信待ち状態フラグを FALSE に更新する。
 *     -# 読み込みブロック位置のロックを解放する。（UnlockWALBuffer 関数）。
 *     -# 読み込みブロック位置を更新する（UpdateCurrentReadPosition 関数）。
 *     -# 読み込み位置の WALWriter用WALバッファから、（ブロックヘッダにある）
 *        書き込み先のタイムライン番号及び LSNを取り出し、出力先となる最終
 *        WALセグメント名を決定する（GetXLogPathName 関数）。
 *     -# WALWriter用WALバッファが解放されるまで 11 を繰り返す。
 *     -# 現在の読み込みブロック位置に格納されている WALを最終WALセグメントに
 *        書き込む。次の読み込みブロック位置に関するロックを取得し、
 *        書き込みを済ませたブロックに関するロックを解放する（WriteWAL 関数）。
 *
 * @param[in]   arg    ディスク書き込みスレッド固有パラメータへのアドレス。
 *                     実際には使用しないため、呼び出し側は NULL を指定する。
 * @return      NULL
 *
 * @version 1.3.0
 * @date    07/25/2008
 * @since   1.0
*/
void* WWWriterThreadMain(void* arg)
{
	int             rcode;
	char            strSegFileName[XLOGFILENAME_LEN + 1];
	char            strSegFilePath[MAX_WW_PATH_LEN + XLOGFILENAME_LEN + 1];
	WALBlockInfo_t* pWalBlkInfo;
	char strFileName[FILE_NAME_MAX_LEN];
	/* 200806 -> */
	MessageQueItemTLI_t msgInfo;
	/* <- 200806 */
	
	struct dirent** entries = NULL;
	int num = 0;
	/* 配列の初期化 */
	memset(strFileName, '\0', FILE_NAME_MAX_LEN);
	
	/* ディスク書き込みスレッドの初期設定 */
	rcode = InitWWWriterThread();
	if (rcode != RET_OK) {
		ExitWALWriter(EXIT_FAILURE);
	}
/* 200806 historyファイル（postgresql用とLastLSN用）の作成、postgresqlへの「TLI更新」メッセージ送信 処理の追加 */
	/* historyファイルの書き込み */
	rcode = WriteHistory();
	if (rcode != RET_OK) {
		ExitWALWriter(EXIT_FAILURE);
	}

	/* 「TLI更新」メッセージ情報の作成*/
	msgInfo.nAddressID = MSGTO_PGRCV;
	msgInfo.nMissionID = MSNID_TLI;
	msgInfo.TLI        = g_WWCmnInfo.nTimeLineID;

	/* メッセージキューへのメッセージ送信 */
	rcode = msgsnd(WrtThRc.nMsgqID, &msgInfo, MESSAGE_TLI_LEN, 0);
	if (rcode == MSGQ_ERROR) {
		ExitWALWriter(EXIT_FAILURE);
	}
	
/* <- 200806 */
	/* ロックの取得（読み込みブロックにおける先頭の１つ） */
	rcode = pthread_mutex_trylock(&(g_WWCmnInfo.pBufferMutexList[0]));

	if (rcode != PTHREAD_OK) {
		/* WAL受信待ち状態であることの通知 */
		NotifyWALBufferIsEmpty();

		/* ロックの取得（再試行。先頭の１つ） */
		LockWALBuffer(0, 1);
	}

	/* WAL受信待ち状態フラグの更新 */
	g_WWCmnInfo.WALBufferInfo.nFlag = FALSE;

	/* 読み込みブロック位置に関するロックの解放 */
	UnlockWALBuffer(WrtThRc.nTargetPos, 1);

	/* 読み込みブロック位置の更新 */
	UpdateCurrentReadPosition(&WrtThRc);

	/* 読み込みブロックの先頭から、WALブロック情報を取得 */
	pWalBlkInfo =
		(WALBlockInfo_t*)(g_WWCmnInfo.pWALBuffer + (WrtThRc.nTargetPos * WAL_BLOCK_SIZE));

	/* 取得した WALブロック情報からの出力先最終WALセグメント名の決定 */
	GetXLogPathName(strSegFileName, g_WWCmnInfo.nTimeLineID, &(pWalBlkInfo->LSN));

	/* 出力先最終WALセグメントのフルパスの決定 */
	sprintf(strSegFilePath, "%s/%s", g_WWEnvInfo.sbywaldir, strSegFileName);

	/*
	 * WALセグメントファイルの一覧をファイル名でソートした状態で取得
	 */
	if ( (num = scandir(g_WWEnvInfo.sbywaldir, &entries, filter_walfile, alphasort)) < 0 ) {
		OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTOGET, strerror(errno));
		return NULL;
	}

	/*
	 * WALセグメント格納ディレクトリ配下にファイルが存在する場合、以下の処理を行なう
	 * ・ファイル一覧の順に、最終WAL以外のWALファイルを削除する
	 */
	if ( num > 0 ) {
		/* ファイルが存在する場合 */
		int i;
		for ( i = (num - 1) ; i >= 0 ; i-- ) {
			/*
			 * ファイル名を作成する
			 */
			snprintf(strFileName, FILE_NAME_MAX_LEN, "%s/%s", g_WWEnvInfo.sbywaldir, entries[i]->d_name);
			/* 最終WALファイル名との比較 */
			if ( strcmp(strSegFilePath, strFileName) != 0 ){
				/* WALファイルの削除 */
				if ( remove(strFileName) < 0 ) {
					freeEntries(entries, num);
					OutputSyslog(LOG_ERR, MSGID_WWGET_FAILEDTOGET, strerror(errno));
					return NULL;
				}
			}
			
		}
		
		freeEntries(entries, num);
		memset(strFileName, '\0', FILE_NAME_MAX_LEN);
	}

	/* WALバッファが解放済みされるまで繰り替えす */
	while(g_WWCmnInfo.pWALBuffer != NULL) {
		/* 最終WALセグメントへの書き込み */
		rcode = WriteWAL();
		if (rcode != RET_OK) {
			ExitWALWriter(EXIT_FAILURE);
		}
	}

	return NULL;
}

/**
 * @fn      int InitWWWriterThread()
 * @brief   ディスク書き込みスレッド初期設定
 *
 *   ディスク書き込みスレッドで使用する変数及びリソースの初期化を行なう。
 *
 *   詳細は以下のとおり。
 *     -# 読み込みブロック位置を（ww_buffers-1：WALWriter用WALバッファの末尾）に設定する。
 *     -# 出力先WALセグメント情報のファイルポインタを NULL で初期化する。
 *     -# 設定ファイル情報のメッセージキューキー値（prex_msgkey）を用いて、
 *        メッセージキューIDを取得する。エラー発生時は、syslog にエラーメッセージを
 *        出力し、呼出元に RET_NG を返却する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @note        読み込みブロック位置の初期値を（ww_buffers-1）とするのは、
 *              次のローテーションで先頭ブロックから処理を行うためである。
 *
 * @retval      RET_OK(0)  正常
 * @retval      RET_NG(1)  異常
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int InitWWWriterThread()
{
	/* 読み込みブロック位置の初期化 */
	WrtThRc.nTargetPos = g_WWEnvInfo.ww_buffers - 1;

	/* 出力先WALセグメント情報の初期化 */
	WrtThRc.pWALSegmentInfo.fp = NULL;

	/* メッセージキューIDの取得 */
	WrtThRc.nMsgqID = msgget((key_t)(g_WWEnvInfo.prex_msgkey), MSGGET_FLG_RW);
	if (WrtThRc.nMsgqID == MSGQ_ERROR) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOGETMSGQUE);
		return RET_NG;
	}

	return RET_OK;
}

/**
 * @fn      int OpenWALSegment(char* filepath)
 * @brief   最終WALセグメントのオープン
 *
 *   WALの書き込み先となる最終WALセグメントをオープンする。
 *
 *   詳細は以下のとおり。
 *     -# 指定されたファイルパスが、現在の出力先WALセグメントと一致する場合、
 *        呼出元に RET_OK を返却する。
 *     -# 最終WALセグメントを書き込みモードでオープンする。
 *        指定されたファイルが存在しない場合は、作成してからオープンする。
 *        エラー発生時は、syslogにエラーメッセージを出力し（OutputSyslog
 *        関数）、呼出元に RET_NGを返却する。
 *     -# スレッド間共通情報のブランク領域から WAL_SEG_FILESIZE バイト分を
 *        最終WALセグメントに出力する。エラー発生時は、syslogにエラーメッセージを
 *        出力し（OutputSyslog 関数）、呼出元に RET_NG を返却する。
 *     -# オープンした最終WALセグメントのファイルディスクリプタを取得し、fsync
 *        を実行する。
 *     -# オープンしたファイルポインタを出力先WALセグメント情報に登録する。
 *     -# 呼出元に RET_OK を返却する。
 *
 * @param[in]   filepath      WALの書き込み先となる最終WALセグメントのパス
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int OpenWALSegment(char* filepath)
{
	int      rcode;
	ssize_t  scode;
	FILE*    fpWALSeg;
	int      fdWALSeg;

	/* 指定されたパスが、現在登録されている出力先ファイルパスと一致する場合、 */
	/* 何もしない                                                             */
	if (filepath == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_ILLEGALPARAM, INTERNAL_PARAMETERS);
		return RET_NG;
	}
	if ((WrtThRc.pWALSegmentInfo.fp != NULL) &&
		(strcmp(WrtThRc.pWALSegmentInfo.filename, filepath) == 0)) {
		return RET_OK;
	}

	/* 出力先最終WALセグメントのオープン */
	fpWALSeg = fopen(filepath, "wb");
	if (fpWALSeg == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOOPENFILE, WALSEGMENTFILE);
		return RET_NG;
	}

	/* 出力先最終WALセグメントのサイズ調整                       */
	/*   - 16MB に拡張するため、ブランク領域をファイルに出力する */
	scode = fwrite(g_WWCmnInfo.pBlank, sizeof(char), WAL_SEG_FILESIZE, fpWALSeg);
	if (scode < WAL_SEG_FILESIZE) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOWRITEFILE, WALSEGMENTFILE);
		return RET_NG;
	}

	/* fsync を行なうためファイルディスクリプタを取得 */
	fdWALSeg = fileno(fpWALSeg);
	if (fdWALSeg == -1) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOWRITEFILE, WALSEGMENTFILE);
		return RET_NG;
	}

	/* fsync の実行 */
	rcode = fsync(fdWALSeg);
	if (rcode == -1) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOWRITEFILE, WALSEGMENTFILE);
		return RET_NG;
	}

	/* 出力WALセグメント情報への情報設定 */
	memcpy(WrtThRc.pWALSegmentInfo.filename, filepath, strlen(filepath)+1);
	WrtThRc.pWALSegmentInfo.fp = fpWALSeg;

	return RET_OK;
}

/**
 * @fn      void CloseWALSegment()
 * @brief   最終WALセグメントのクローズ
 *
 *   最終WALセグメントをクローズする。
 *
 *   詳細は以下のとおり。
 *     -# 最終WALセグメントをクローズする。
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
void CloseWALSegment()
{
	if (WrtThRc.pWALSegmentInfo.fp != NULL) {
		fclose(WrtThRc.pWALSegmentInfo.fp);
		WrtThRc.pWALSegmentInfo.fp = NULL;
	}
}

/**
 * @fn      int WriteWAL()
 * @brief   WAL書き込み及び「WAL反映」メッセージの送信
 *
 *   書き込み対象のWALを最終WALセグメントに出力し、必要に応じて「WAL反映」
 *   メッセージの送信も行なう。
 *
 *   詳細は以下のとおり。
 *     -# 登録WAL情報から、書き込み対象WALの使用ブロック数を取得し、書き込み対象
 *        WALのサイズを計算する。
 *     -# WALWriter用WALバッファのブロック数が、
 *        （読み込みブロック位置 + 書き込み対象WALの使用ブロック数）より大きい
 *        （リングバッファの末端ブロックと先頭ブロックを跨がない）場合、
 *        4 の処理を行なう。
 *     -# WALWriter用WALバッファのブロック数が、
 *        （読み込みブロック位置 + 書き込み対象WALの使用ブロック数）以下
 *        （リングバッファの末端ブロックと先頭ブロックを跨ぐ）の場合、5 及び 6
 *        の処理を行なう。
 *     -# 最終WALセグメントへ、WALWriter用WALバッファの読み込みブロック位置以降に
 *        格納されている WALを出力する（WriteWALToSegmentFile 関数）。
 *        エラー発生時は、呼出元に RET_NG を返却する。次に 7 を実施する。
 *     -# 最終WALセグメントへ、WALWriter用WALバッファの読み込みブロック位置から
 *        （（ww_buffers - 現在のブロック位置）＊8KB）分だけWALを出力する
 *        （WriteWALToSegmentFile 関数）。エラー発生時は、呼出元に RET_NGを返却する。
 *     -# 最終WALセグメントへ、残りの書き込み対象WAL（WALWriter用WALバッファの
 *        先頭ブロックから
 *        （書き込み対象WALのサイズ -（（ww_buffers - 現在のブロック位置）＊8KB））分）
 *        を出力する（WriteWALToSegmentFile 関数）。エラー発生時は、呼出元に RET_NG
 *        を返却する。
 *     -# スレッド間共有情報における最新反映終端 LSN を、
 *        「登録WAL情報[読み込みブロック位置].書き込みWALに関するLSN情報」で更新する
 *        （UpdateNewestLSNInfo 関数）。
 *     -# バックアップ取得完了フラグが TRUE の場合、「WAL反映」メッセージを送信する
 *        （SendPITRMessageToQueue 関数）。エラー発生時は、syslog にエラーメッセージを
 *        出力し、呼出元に RET_NG を返却する。
 *     -# 次の読み込みブロック位置に関するロックの取得及び取得済みロックの解放を行なう
 *        （LockNextBlockAndUnlockCurrentBlock 関数）。エラー発生時は、呼出元に RET_NG
 *        を返却する。
 *     -# 読み込みブロック位置を更新する（UpdateCurrentReadPosition 関数）。
 *     -# 呼出元に RET_OKを返却する。
 *
 * @pre        以下のすべての条件を満たしていること。
 *               - 最終WALセグメントが書き込みモードでオープンされていること。
 *               - 読み込みブロック位置に関するロックを取得していること。
 * @attention  書き込み対象のWALは、以下のいずれかの状態でメモリ上に
 *             格納されている。
 *               - WALWriter用WALバッファ上の連続したブロックに格納されている。
 *               - WALWriter用WALバッファ上で、末尾ブロックから先頭ブロックに
 *                 跨って格納されている。
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.1
 * @date    12/22/2006
 * @since   1.0
*/
int WriteWAL()
{
	int           rcode;
	RegWALInfo_t* pWALInfo;
	int           nTotalSize;
	int           nTargetSize;
	char*         pWAL;

	/* 書き込み対象WALに関する登録WAL情報の取得 */
	pWALInfo   = GetWALInfo(WrtThRc.nTargetPos);

	/* 書き込み対象WALの総サイズの算出 */
	nTotalSize = pWALInfo->nRequiredBlocks * WAL_BLOCK_SIZE;

	/* 書き込み対象WALの先頭アドレスの取得 */
	pWAL = g_WWCmnInfo.pWALBuffer + (WrtThRc.nTargetPos * WAL_BLOCK_SIZE);

	if (g_WWEnvInfo.ww_buffers >= (WrtThRc.nTargetPos + pWALInfo->nRequiredBlocks)) {
		/* 最終WALセグメントへのWALの書き込み */
		rcode = WriteWALToSegmentFile(pWAL, nTotalSize);
		if (rcode != RET_OK) {
			return RET_NG;
		}
	} else {
		/* 書き込みサイズ（WALバッファの末尾ブロックまで）の算出 */
		nTargetSize = (g_WWEnvInfo.ww_buffers - WrtThRc.nTargetPos) * WAL_BLOCK_SIZE;

		/* 最終WALセグメントへのWALの部分書き込み */
		rcode = WriteWALToSegmentFile(pWAL, nTargetSize);
		if (rcode != RET_OK) {
			return RET_NG;
		}

		/* 書き込み対象部分WALの先頭アドレスの取得 */
		pWAL = g_WWCmnInfo.pWALBuffer;

		/* 書き込みサイズ（WALバッファの先頭ブロックから対象WALの末尾まで）の算出 */
		nTargetSize = nTotalSize - nTargetSize;

		/* 最終WALセグメントへのWALの部分書き込み */
		rcode = WriteWALToSegmentFile(pWAL, nTargetSize);
		if (rcode != RET_OK) {
			return RET_NG;
		}
	}

	/* 最新反映終端 LSN の更新 */
	UpdateNewestLSNInfo(&(pWALInfo->LSN));
	NotifyWritingWALToFile();

	if (g_WWCmnInfo.GettingBackupInfo.nFlag == TRUE) {
		/* 「WAL反映」メッセージの送信 */
		rcode = SendPITRMessageToQueue(WrtThRc.nMsgqID);
		if (rcode != RET_OK) {
			OutputSyslog(LOG_ERR,
				MSGID_WWWRT_FAILEDTOSENDMSGTOQUE, MSGWD_QMSG_REQUESTTOPITR);
			return RET_NG;
		}
	}

	/* 次の読み込みブロック位置に関するロックの取得及び取得済みロックの解放 */
	rcode = LockNextBlockAndUnlockCurrentBlock();
	if (rcode != RET_OK) {
		return RET_NG;
	}

	/* 読み込みブロック位置の更新 */
	UpdateCurrentReadPosition(&WrtThRc);

	return RET_OK;
}

/**
 * @fn      int WriteWALToSegmentFile(char* pWAL, int nSize)
 * @brief   最終WALセグメントへのWAL書き込み
 *
 *   指定されたアドレスに格納されているWALを、指定されたサイズ分だけ最終 WAL
 *   セグメントに上書き出力する。
 *
 *   詳細は以下のとおり。
 *     -# 出力するWALブロックのブロックヘッダにあるブロック先頭LSNを取得する。
 *     -# 1 で取得した LSN に該当するセグメント番号が、直前の xrecoff に関する
 *         セグメント番号と一致するかどうかを確認する（IsXLogInPrevSeg 関数）。
 *         セグメント番号が一致する場合は、7～18を実行する。異なる場合、3～18
 *         を実行する。
 *     -# 新しい出力先最終WALセグメント名を決定する（GetXLogPathName 関数）。
 *     -# 3 で決定した出力先最終WALセグメント名と、現在の出力先最終WALセグメント名
 *        が一致する場合、7～18を実行する。異なる場合、5～18を実行する。
 *     -# オープン済みの出力先WALセグメントをクローズする（CloseWALSegment 関数）。
 *     -# 3 で名前を決定したWALセグメントをオープンし（OpenWALSegment 関数）、
 *        出力先ファイルとして設定した後に8を実行する。
 *        エラー発生時は、呼出元に RET_NGを返却する。
 *     -# オープン済みの出力先WALセグメントを、出力先ファイルとして設定する。
 *     -# 出力するWALブロックのブロックヘッダにあるブロック先頭LSNを元に、シークサイズを
 *        求める。シークサイズ = ブロック先頭LSNの xrecoff % WAL_SEG_FILESIZE（16MB）
 *     -# オープン済みの出力先最終WALセグメントを、シークサイズ分シークする。
 *     -# シークサイズ＋nSizeが 16M(WAL_SEG_FILESIZE)を超えている場合、12～16を実行する。
 *     -# シークサイズ＋nSizeが 16M(WAL_SEG_FILESIZE)を超えていない場合、17を実行する。
 *     -# （16M(WAL_SEG_FILESIZE) - シークサイズ）分を最終WALセグメントに書き込む。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 最終WALセグメントをクローズする（CloseWALSegment 関数）。
 *     -# 次の出力先となる最終WALセグメントを決定する（GetXLogPathName 関数）。
 *     -# 最終WALセグメントをオープンする（OpenWALSegment 関数）。
 *        エラー発生時は、呼出元に RET_NGを返却する。
 *     -# 残りのWAL（nSize-(16M(WAL_SEG_FILESIZE) - シークサイズ)分）を、
 *        最終WALセグメントの先頭から書き込む。その後 18を行う。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# nSize分を最終WALセグメントに書き込む。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 呼出元に RET_OKを返却する。
 *
 * @note        最終WALセグメントのサイズは、最大で16MBである。そのため、WALの書き込みに
 *              よって最終 WAL セグメントのサイズが16MBに達した場合は、一旦現在の書き込み
 *              先最終 WAL セグメントをクローズし、新たな書き込み先となる最終 WAL
 *              セグメントを生成して、続きを書き込むことが必要となる。
 *
 * @param[in]   pWAL  出力対象のWALが格納されている領域の先頭アドレス
 * @param[in]   nSize 出力するWALのサイズ（バイト）
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int WriteWALToSegmentFile(char* pWAL, int nSize)
{
	int          rcode;
	FILE*        pTargetFp;
	LSNInfo_t*   pStartLSN;
	int          nSeekSize;
	int          nLastPos;        /* 出力後の見込み位置（nSeekSize + nSize） */
	int          nWriteSize;      /* 書き込むサイズ */
	char         strSegFileName[XLOGFILENAME_LEN + 1];
	char         strSegFilePath[MAX_WW_PATH_LEN + XLOGFILENAME_LEN + 1];
	WALBlockInfo_t* pWalBlkInfo;
	uint32       nTimeLineID;
	uint32       nTrgtSegID;
	int          bNeedNotNewSeg;
	char*        pRestWAL;
	int          fdWALSeg;

	/* 読み込みブロックの先頭から、WALブロック情報を取得 */
	pWalBlkInfo = (WALBlockInfo_t*)pWAL;

	pStartLSN   = &(pWalBlkInfo->LSN);
	nTimeLineID = g_WWCmnInfo.nTimeLineID;
	nTrgtSegID   = pStartLSN->uXRecOff / WAL_SEG_FILESIZE;
	bNeedNotNewSeg = IsXLogInPrevSeg(pStartLSN, pStartLSN->uXLogID, nTrgtSegID);

	if (! bNeedNotNewSeg) {
		/* 次の出力先最終WALセグメント名の決定 */
		GetXLogPathName(strSegFileName, nTimeLineID, pStartLSN);

		/* 出力先最終WALセグメントのフルパスの決定 */
		sprintf(strSegFilePath, "%s/%s", g_WWEnvInfo.sbywaldir, strSegFileName);

		/* 現在の出力先WALセグメント名との比較 */
		rcode = strcmp(strSegFilePath, WrtThRc.pWALSegmentInfo.filename);
		if (rcode != 0) {

			/* 出力先WALセグメントのクローズ（オープンしていない場合は何もしない） */
			CloseWALSegment();

			/* 出力先最終WALセグメントのオープン */
			rcode = OpenWALSegment(strSegFilePath);
			if (rcode != RET_OK) {
				return RET_NG;
			}
		}

		pTargetFp = WrtThRc.pWALSegmentInfo.fp;
	} else {
		/* 出力先ファイルポインタの取得 */
		pTargetFp = WrtThRc.pWALSegmentInfo.fp;
	}

	/* シークサイズの算出 */
	nSeekSize = (pStartLSN->uXRecOff % WAL_SEG_FILESIZE);

	/* 出力先最終WALセグメントにおいて出力位置をシーク */
	rcode = fseek(pTargetFp, nSeekSize, SEEK_SET);
	if (rcode != 0) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOSEEKFILE, WALSEGMENTFILE);
		return RET_NG;
	}

	/* 出力後の見込みファイルポインタ位置を算出 */
	nLastPos = nSeekSize + nSize;
	if (nLastPos > WAL_SEG_FILESIZE) {
		/* 出力先最終WALセグメントへの書き込み */
		nWriteSize = WAL_SEG_FILESIZE - nSeekSize;

		rcode = fwrite(pWAL, sizeof(char), nWriteSize, pTargetFp);
		if (rcode < nWriteSize) {
			OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOWRITEFILE, WALSEGMENTFILE);
			return RET_NG;
		}

		/* 出力先最終WALセグメントのクローズ */
		CloseWALSegment();

		nWriteSize = nSize - (WAL_SEG_FILESIZE - nSeekSize);
		if (nWriteSize > 0) {
			pRestWAL = pWAL + (WAL_SEG_FILESIZE - nSeekSize);

			/* 読み込みブロックの先頭から、WALブロック情報を取得 */
			pWalBlkInfo = (WALBlockInfo_t*)pRestWAL;
			pStartLSN   = &(pWalBlkInfo->LSN);
			nTimeLineID = pWalBlkInfo->nTimeLineID;

			/* 次の出力先最終WALセグメント名の決定 */
			GetXLogPathName(strSegFileName, nTimeLineID, pStartLSN);

			/* 出力先最終WALセグメントのフルパスの決定 */
			sprintf(strSegFilePath, "%s/%s", g_WWEnvInfo.sbywaldir, strSegFileName);

			/* 出力先最終WALセグメントのオープン */
			rcode = OpenWALSegment(strSegFilePath);
			if (rcode != RET_OK) {
				return RET_NG;
			}

			/* 残りのWALの書き込み */
			pTargetFp = WrtThRc.pWALSegmentInfo.fp;
			rcode = fwrite(pRestWAL, sizeof(char), nWriteSize, pTargetFp);
			if (rcode < nWriteSize) {
				OutputSyslog(LOG_ERR,
					MSGID_WWWRT_FAILEDTOWRITEFILE, WALSEGMENTFILE);
				return RET_NG;
			}
		}
	} else {
		/* 出力先最終WALセグメントへの書き込み */
		rcode = fwrite(pWAL, sizeof(char), nSize, pTargetFp);
		if (rcode < nSize) {
			OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOWRITEFILE, WALSEGMENTFILE);
			return RET_NG;
		}
	}

	/* fsync を行なうためファイルディスクリプタを取得 */
	fdWALSeg = fileno(pTargetFp);
	if (fdWALSeg == -1) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOWRITEFILE, WALSEGMENTFILE);
		return RET_NG;
	}

	/* fsync の実行 */
	rcode = fsync(fdWALSeg);
	if (rcode == -1) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOWRITEFILE, WALSEGMENTFILE);
		return RET_NG;
	}

	return RET_OK;
}

/**
 * @fn      int LockNextBlockAndUnlockCurrentBlock()
 * @brief   次のロック取得及び現在のロック解放
 *
 *   次の読み込みブロック位置に関するロックを取得した後、書き込みを終えたブロックの
 *   ロックを解放する。
 *
 *   詳細は以下のとおり。
 *     -# 現在の読み込みブロックにおける WAL の使用ブロック数を取得する
 *        （GetRequiredBlocks 関数）。
 *     -# 次の読み込みブロックの先頭ブロック位置を算出する（GetNextReadPosition 関数）。
 *     -# 次の読み込みブロックの先頭ブロックに関するアクセス用ロックの取得を試行
 *        （trylock）する。
 *     -# 3 の実行時にロックの取得に成功した場合、7～11 を実行する。
 *     -# 3 の実行時にロックの取得に失敗した場合、WALWriter用WALバッファが空であることを
 *        他スレッドに通知する（NotifyWALBufferIsEmpty 関数）。
 *     -# 次の読み込み先の先頭ブロックに関するアクセス用ロックを取得するまで待機する
 *        （LockWALBuffer 関数）。
 *     -# WAL受信待ち状態フラグを FALSE に更新する。
 *     -# ロックを取得した後、WALの出力を終えたブロックに関するロック（読み込みブロック
 *        位置から使用ブロック数分）を解放する（UnlockWALBuffer 関数）。
 *     -# 次の読み込みブロックにおける WAL の使用ブロック数を取得する
 *        （GetRequiredBlocks 関数）。
 *     -# 次の読み込みブロックにおける WAL の使用ブロック数に関する、残りのロックを
 *        取得するまで待機する（LockWALBuffer 関数）。
 *     -# 呼出元に RET_OKを返却する。
 *
 * @pre         １つの読み込みブロック位置に関するロックを取得していること。
 * @post        呼び出し前に取得していたロックとは異なるブロック位置のロックを
 *              取得している。
 * @attention   WALWriter用WALバッファにおいて、通信スレッドによる未書き込み
 *              WAL の上書きを防ぐために、ディスク書き込みスレッドは、次の読み込み
 *              ブロックに関するロックを取得してから、書き込みを済ませたブロックの
 *              ロック解放を行なうようにすること。
 *
 * @retval      RET_OK(0)     正常終了
 *
 * @version 1.0.0
 * @date    09/01/2006
 * @since   1.0
*/
int LockNextBlockAndUnlockCurrentBlock()
{
	int   rcode;
	int   nCurrentRequiredBlocks;
	int   nNextRequiredBlocks;
	int   nNextReadBlockPos;

	/* 現在の読み込みブロックにおけるWALの使用ブロック数を取得 */
	nCurrentRequiredBlocks = GetRequiredBlocks(WrtThRc.nTargetPos);

	/* 次の読み込みブロックの先頭ブロック位置を算出 */
	nNextReadBlockPos = GetNextReadPosition();

	/* ロックの取得（読み込みブロックにおける先頭の１つ） */
	rcode = pthread_mutex_trylock(&(g_WWCmnInfo.pBufferMutexList[nNextReadBlockPos]));

	if (rcode != PTHREAD_OK) {
		/* WAL受信待ち状態であることの通知 */
		NotifyWALBufferIsEmpty();

		/* ロックの取得（再試行。先頭の１つ） */
		LockWALBuffer(nNextReadBlockPos, 1);
	}

	/* WAL受信待ち状態フラグの更新 */
	g_WWCmnInfo.WALBufferInfo.nFlag = FALSE;

	/* 書き込み済みの読み込みブロックに関するロックの解放 */
	UnlockWALBuffer(WrtThRc.nTargetPos, nCurrentRequiredBlocks);

	/* 次の読み込みブロックにおけるWALの使用ブロック数を取得 */
	nNextRequiredBlocks = GetRequiredBlocks(nNextReadBlockPos);

	/* 残りのロックの取得 */
	LockWALBuffer(nNextReadBlockPos + 1, nNextRequiredBlocks - 1);

	return RET_OK;
}

/**
 * @fn      static int filter_walfile(const struct dirent* entry)
 * @brief   WALセグメントファイルを抽出するためのフィルタリング
 *
 *  詳細は以下の通り。
 *   -# エントリ種別が「ファイル」であることを確認する。
 *   -# ファイル名が24文字であることを確認する。
 *
 * @param[in]   entry     エントリ
 * @retval      TRUE(1)   WALセグメントファイルである
 * @retval      FALSE(0)  WALセグメントファイルではない
 *
 * @version 1.0.0
 * @date    08/07/2007
 * @since   1.0
 */
static int filter_walfile(const struct dirent* entry) {
	/*
	 * 以下の条件に一致することを確認
	 * ・ファイル種別が「ファイル」であること
	 * ・ファイル名の文字列長が24文字であること
	 */
	if ( entry->d_type == DT_REG && strlen(entry->d_name) == 24 ) {
		return TRUE;
	}
	return FALSE;
}

/* 200806 -> */
/**
 * @fn      int WriteHistory()
 * @brief   history file書き込み
 *
 *   指定されたアドレスに格納されているhistory情報を、PostgreSQL用とLastLSN用にそれぞれ出力する。
 *
 *   詳細は以下のとおり。
 *     -# PostgreSQL用出力先ファイル名を決定する（GetHistoryPathName 関数）。
 *     -# 1 で名前を決定したファイルをオープンしてhistory情報を書き込み、
 *        ファイルをクローズする（WriteHistoryToFile 関数）。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# LastLSN用ファイル先ファイル名でファイルをオープンしてhistory情報を書き込み、
 *        ファイルをクローズする（WriteHistoryToFile 関数）。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 呼出元に RET_OKを返却する。
 *
 *
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
*/
int WriteHistory()
{
	int          rcode;
	char         strPGFileName[XLOGFILENAME_LEN + 1];
	char         strHistoryFilePath[MAX_WW_PATH_LEN + XLOGFILENAME_LEN + 1];
	uint32       nTimeLineID;
	char         strLastLSNHistoryFilePath[MAX_WW_PATH_LEN + XLOGFILENAME_LEN + 1];

	nTimeLineID = g_WWCmnInfo.nTimeLineID;

	/* 次の出力先historyファイル名の決定 */
	GetHistoryPathName(strPGFileName, nTimeLineID);

	/* 出力先historyファイルのフルパスの決定 */
	sprintf(strHistoryFilePath, "%s/%s", g_WWEnvInfo.sbywaldir, strPGFileName);

	/* PostgreSQL用historyファイルへの書き出し */
	rcode = WriteHistoryToFile(strHistoryFilePath);
	if (rcode != RET_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOWRITEFILE, strHistoryFilePath);
		return RET_NG;
	}

	/* 出力先historyファイルのフルパスの決定 */
	sprintf(strLastLSNHistoryFilePath, "%s/%s", g_WWEnvInfo.sbywaldir, HISTORYFILENAME);

	/* LastLSN用historyファイルへの書き出し */
	rcode = WriteHistoryToFile(strLastLSNHistoryFilePath);
	if (rcode != RET_OK) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOWRITEFILE, strLastLSNHistoryFilePath);
		return RET_NG;
	}
	

	/*history用バッファの解放 */
	if (g_WWCmnInfo.recvHistoryInfo.pHistory != NULL) {
		free(g_WWCmnInfo.recvHistoryInfo.pHistory);
		g_WWCmnInfo.recvHistoryInfo.pHistory = NULL;
	}
	
	return RET_OK;
}

/**
 * @fn      int WriteHistoryToFile(char* pHistoryFilePath)
 * @brief   history file書き込み
 *
 *   history file情報を指定されたファイルに上書き出力する。
 *
 *   詳細は以下のとおり。
 *     -# 出力対象のファイルをオープンする。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# history情報を書き出す。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# ファイルをクローズする。
 *        エラー発生時は、syslogにエラーメッセージを出力し、呼出元に RET_NGを返却する。
 *     -# 呼出元に RET_OKを返却する。
 *
 * @param[in]   pHistoryFilePath  出力対象のファイル名
 * @retval      RET_OK(0)     正常終了
 * @retval      RET_NG(1)     異常終了
 *
 * @version 1.3.0
 * @date    06/04/2008
 * @since   1.3
*/
int WriteHistoryToFile(char* pHistoryFilePath)
{
	int          rcode;
	FILE*        pHistoryFp;
	int          fdHistory;
	int          nSize;
	
	/* history情報のサイズを格納 */
	nSize = g_WWCmnInfo.recvHistoryInfo.header.uMessageSize;
	
	/* 出力先history fileのオープン */
	pHistoryFp = fopen(pHistoryFilePath, "wb");
	if (pHistoryFp == NULL) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOOPENFILE, pHistoryFilePath);
		return RET_NG;
	}

	/* 出力先history fileへの書き込み */

	rcode = fwrite(g_WWCmnInfo.recvHistoryInfo.pHistory, sizeof(char), nSize, pHistoryFp);
	if (rcode < nSize) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOWRITEFILE, pHistoryFilePath);
		return RET_NG;
	}

	/* fsync を行なうためファイルディスクリプタを取得 */
	fdHistory = fileno(pHistoryFp);
	if (fdHistory == -1) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOWRITEFILE, pHistoryFilePath);
		return RET_NG;
	}

	/* fsync の実行 */
	rcode = fsync(fdHistory);
	if (rcode == -1) {
		OutputSyslog(LOG_ERR, MSGID_WWWRT_FAILEDTOWRITEFILE, pHistoryFilePath);
		return RET_NG;
	}
	
	
	/* 出力先history fileのクローズ */
	fclose(pHistoryFp);

	return RET_OK;
}
/* <- 200806 */

